@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -81,8 +82,10 @@ is meant to be scriptable.
 <pre>
 Usage: ergo [args...]
 args can be: input file name
-             -e "input line"
-             -m molecule file name
+             -e  "input line"
+             -d  VARIABLE_NAME variable to describe
+             -m  molecule file name
+             -c  extra-charges molecule file name
              -h help message that lists also all the available variables.
 </pre>
 The statements in the input file are divided in two classes: variable
@@ -157,19 +160,16 @@ get_polarisability "HF" "Y" 0.01
  Each file should begin with a header in the following style:
 
 @verbatim
- /** @file filename.h Brief description
-  *  @author Author1 <em>responsible</em>
-  *  @author Author2
-  *  @version 1.0
-  *  @date    March 2006
+ /** @file filename.h
+     @brief Brief description
+     @author Author1 <em>responsible</em>
+     @author Author2
   */
 @endverbatim
 which will result in the following: @n
 Brief description
 @author Author1   <em>responsible</em>
 @author Author2
-@version 1.0
-@date    March 2006
 
 @htmlonly <hr> @endhtmlonly
 
@@ -190,7 +190,7 @@ make check VERBOSE=1
   
  @subsection examples Configuration examples 
   
-How to install on different machines and libraries (this should probably be updated)
+How to install on different machines and libraries
 
 @verbinclude config_examples.txt
 
