@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -30,8 +31,9 @@
 
     \brief Code for setting up basis functions starting from shells.
 
-    @author: Elias Rudberg <em>responsible</em>. 
+    @author: Elias Rudberg <em>responsible</em>.
 */
+
 /* -*-mode:c; c-style:k&r; indent-tabs-mode: nil -*- */
 /* Written by Elias Rudberg, KTH, Stockholm */
 #include <stdlib.h>
@@ -96,7 +98,7 @@ output_basisinfo(const BasisInfoStruct & basisInfo)
 }
 
 
-static int
+static void
 define_basis_func_poly(BasisFuncStruct* basisFunc, int polyIndex, const IntegralInfo& b)
 {
   if(polyIndex >= b.no_of_basis_func_polys)
@@ -105,89 +107,75 @@ define_basis_func_poly(BasisFuncStruct* basisFunc, int polyIndex, const Integral
   basisFunc->noOfTermsInPolynomial = poly->noOfTerms;
   for(int i = 0; i < poly->noOfTerms; i++)
     memcpy(&basisFunc->poly[i], &poly->termList[i], sizeof(basis_func_term_struct));
-  return 0;
 }
 
 
-/* FIXME: is it a way to make this routine shorter, cleaner? */
+static void
+define_basis_func_poly_special_6dfuncs(BasisFuncStruct* basisFunc, const IntegralInfo & b)
+{
+  // SPECIAL CASE: USE 6 d-type functions instead of 5.
+  // Use the following 6 functions: x^2 y^2 z^2 xy xz yz
+  basisFunc->noOfTermsInPolynomial = 1;
+  int i0 = 0, i1 = 0, i2 = 0;
+  ergo_real coeff = 0;
+  const ergo_real coeff_a = 1;
+  const ergo_real coeff_b = template_blas_sqrt((ergo_real)3);
+  switch(basisFunc->functionNumber) {
+  case 0: i0 = 2; i1 = 0; i2 = 0; coeff = coeff_a; break; // x^2
+  case 1: i0 = 0; i1 = 2; i2 = 0; coeff = coeff_a; break; // y^2
+  case 2: i0 = 0; i1 = 0; i2 = 2; coeff = coeff_a; break; // y^2
+  case 3: i0 = 1; i1 = 1; i2 = 0; coeff = coeff_b; break; // xy
+  case 4: i0 = 1; i1 = 0; i2 = 1; coeff = coeff_b; break; // xz
+  case 5: i0 = 0; i1 = 1; i2 = 1; coeff = coeff_b; break; // yz
+  default: throw "Error: default reached when defining d-type basis function.";
+  }
+  basisFunc->poly[0].coeff = coeff;
+  basisFunc->poly[0].monomialInts[0] = i0;
+  basisFunc->poly[0].monomialInts[1] = i1;
+  basisFunc->poly[0].monomialInts[2] = i2;
+  basisFunc->poly[0].monomialID = b.monomial_info.monomial_index_list[i0][i1][i2];
+}
+
+
 static int 
-get_simple_primitives(
-		      BasisFuncStruct* currBasisFunc,
+get_simple_primitives(BasisFuncStruct* currBasisFunc,
 		      DistributionSpecStruct* list,
 		      int nInput,
 		      int nListMax,
 		      const IntegralInfo& b,
                       int use_6_d_funcs)
 {
-  /* make sure there is enough space left in list */
-  if((nListMax - nInput) < MAX_NO_OF_PRIMITIVES_PER_BASIS_FUNC)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_simple_primitives: "
-		"not enough space left in list");
-      return -1;
-    }
-
   /* first setup polynomial */
-
   int spd = currBasisFunc->shellType;
-
   // Note: special case for use_6_d_funcs comes in here.
   if(spd == 2 && use_6_d_funcs == 1) {
     // SPECIAL CASE: USE 6 d-type functions instead of 5.
     // Use the following 6 functions: x^2 y^2 z^2 xy xz yz
-    currBasisFunc->noOfTermsInPolynomial = 1;
-    int i0 = 0, i1 = 0, i2 = 0;
-    ergo_real coeff = 0;
-    const ergo_real coeff_a = 1;
-    const ergo_real coeff_b = 1.73205080757;
-    switch(currBasisFunc->functionNumber) {
-      case 0: i0 = 2; i1 = 0; i2 = 0; coeff = coeff_a; break; // x^2
-      case 1: i0 = 0; i1 = 2; i2 = 0; coeff = coeff_a; break; // y^2
-      case 2: i0 = 0; i1 = 0; i2 = 2; coeff = coeff_a; break; // y^2
-      case 3: i0 = 1; i1 = 1; i2 = 0; coeff = coeff_b; break; // xy
-      case 4: i0 = 1; i1 = 0; i2 = 1; coeff = coeff_b; break; // xz
-      case 5: i0 = 0; i1 = 1; i2 = 1; coeff = coeff_b; break; // yz
-      default: throw "Error: default reached when defining d-type basis function."; 
-    }
-    currBasisFunc->poly[0].coeff = coeff;
-    currBasisFunc->poly[0].monomialInts[0] = i0;
-    currBasisFunc->poly[0].monomialInts[1] = i1;
-    currBasisFunc->poly[0].monomialInts[2] = i2;
-    currBasisFunc->poly[0].monomialID = b.monomial_info.monomial_index_list[i0][i1][i2];
+    define_basis_func_poly_special_6dfuncs(currBasisFunc, b);
   }
   else {
     int baseIndex = spd*spd;
     define_basis_func_poly(currBasisFunc, baseIndex + currBasisFunc->functionNumber, b);
   }
-
-
-  
   int n = nInput;
   int contr = currBasisFunc->noOfContr;
-  
   for(int kk = 0; kk < contr; kk++) {
     for(int ii = 0; ii < currBasisFunc->noOfTermsInPolynomial; ii++) {
-      list[n].coeff = currBasisFunc->coeffList[kk] 
-	* currBasisFunc->poly[ii].coeff;
+      list[n].coeff = currBasisFunc->coeffList[kk] * currBasisFunc->poly[ii].coeff;
       for(int coordNo = 0; coordNo < 3; coordNo++)
-	list[n].monomialInts[coordNo] =
-	  currBasisFunc->poly[ii].monomialInts[coordNo];
+	list[n].monomialInts[coordNo] = currBasisFunc->poly[ii].monomialInts[coordNo];
       list[n].exponent = currBasisFunc->exponentList[kk];
       for(int coordNo = 0; coordNo < 3; coordNo++)
-	list[n].centerCoords[coordNo] =
-	  currBasisFunc->centerCoords[coordNo];
+	list[n].centerCoords[coordNo] = currBasisFunc->centerCoords[coordNo];
       n++;
-      if(n >= nListMax)
-	{
-	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_simple_primitives: "
-		    "(n >= nListMax)");
-	  return -1;
-	}
+      if(n >= nListMax) {
+	do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_simple_primitives: "
+		  "(n >= nListMax)");
+	return -1;
+      }
     } /* END FOR ii */
   } /* END FOR kk */
-
   return n - nInput;
-  
 } /* END get_simple_primitives */
 
 
@@ -243,7 +231,7 @@ sort_shells(ShellSpecStruct* list, ShellSpecStruct* listTemp, int n)
    SquareFuncIntegrator stuff? */
 namespace {
 
-}; /* end of anonymous name space */
+} /* end of anonymous name space */
 
 
 ergo_real 
@@ -279,7 +267,7 @@ SquareFuncIntegrator::computeIntegralOfSquareOfBasisFunc
   } // END FOR ii
   ergo_real sum = 0;
   for(int ii = 0; ii < productCount; ii++)
-    sum += compute_integral_of_simple_prim(&productlist[ii]);
+    sum += compute_integral_of_simple_prim(productlist[ii]);
   if(sum < 0)
     throw "Error in computeIntegralOfSquareOfBasisFunc, norm factor sum < 0.";
   return sum;
@@ -301,7 +289,7 @@ SquareFuncIntegrator::getShellFactor(const IntegralInfo& integralInfo,
   basisFunc.functionNumber = 0;
   // Compute integral of this basis function squared.
   ergo_real integralValue = computeIntegralOfSquareOfBasisFunc(integralInfo, &basisFunc, use_6_d_funcs);
-  ergo_real shellFactor = (ergo_real)1.0 / std::sqrt(integralValue);
+  ergo_real shellFactor = (ergo_real)1.0 / template_blas_sqrt(integralValue);
   return shellFactor;
 }
 
@@ -525,7 +513,7 @@ void BasisInfoStruct::addBasisfuncsForAtomList(const Atom* atomList,
     for(int kk = 0; kk < shell_list_new[i].noOfContr; kk++) {
       // calculate size
       shell_list_new[i].sizeList[kk] = 
-	std::fabs(std::pow((ergo_real)pi/shell_list_new[i].exponentList[kk], (ergo_real)1.5) * shell_list_new[i].coeffList[kk]);
+	template_blas_fabs(template_blas_pow((ergo_real)pi/shell_list_new[i].exponentList[kk], (ergo_real)1.5) * shell_list_new[i].coeffList[kk]);
     }
   } /* END FOR i */
   
@@ -606,8 +594,10 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
   };
   basisset_info* basissetDefault = new basisset_info();
   
-  basis_set_range_struct rangeListTemp[noOfRanges];
-  memset(rangeListTemp, 0, noOfRanges*sizeof(basis_set_range_struct));
+  std::vector<basis_set_range_struct> rangeListTemp(noOfRanges);
+  // Call memset only if nonzero size to avoid problem if -D_GLIBCXX_ASSERTIONS is used
+  if(noOfRanges > 0)
+    memset(&rangeListTemp[0], 0, noOfRanges*sizeof(basis_set_range_struct));
   
   if(noOfRanges > 0 && rangeList == NULL)
     {
@@ -640,11 +630,16 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
     }
   }
 
+  basis_set_range_struct* rangeListPtr = NULL;
+  // Use &rangeListTemp[0] only if nonzero size to avoid problem if -D_GLIBCXX_ASSERTIONS is used
+  if(noOfRanges > 0)
+    rangeListPtr = &rangeListTemp[0];
+
   addBasisfuncsForAtomList(molecule.getAtomListPtr(),
 			   molecule.getNoOfAtoms(),
 			   *basissetDefault,
 			   noOfRanges,
-			   rangeListTemp,
+			   rangeListPtr,
 			   integralInfo,
 			   print_raw,
 			   do_normalization,
@@ -656,6 +651,24 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
 }
 
 
+int
+BasisInfoStruct::getNoOfBasisFuncsForAtomType(const basisset_info & basisset,
+					      const IntegralInfo & integralInfo,
+					      int atomCharge,
+					      int use_6_d_funcs) {
+  BasisInfoStruct b(use_6_d_funcs);
+  const int do_basis_normalization = 1;
+  const int skip_sort_shells = 0;
+  // Create dummy atom with the given charge
+  Atom atom;
+  atom.charge = atomCharge;
+  for(int i = 0; i < 3; i++)
+    atom.coords[i] = 0;
+  b.addBasisfuncsForAtomList(&atom, 1, basisset, 0, NULL, integralInfo, 0, do_basis_normalization, skip_sort_shells);
+  return b.noOfBasisFuncs;
+}
+
+
 /** a factory method generating new BasisInfo struct with permuted
     shells and basis functions.
     
@@ -705,7 +718,7 @@ int BasisInfoStruct::normalizeShells(const IntegralInfo& integralInfo)
   ergo_real normFactorTot_max = 0;
 
   // Adapt tolerance to machine accuracy to be able to run with different precision.
-  ergo_real max_allowed_difference = std::sqrt(get_machine_epsilon());
+  ergo_real max_allowed_difference = template_blas_sqrt(get_machine_epsilon());
   SquareFuncIntegrator sfi;
   for(int i = 0; i < this->noOfShells; i++) {
     ShellSpecStruct* currShell = &this->shellList[i];
@@ -734,7 +747,7 @@ int BasisInfoStruct::normalizeShells(const IntegralInfo& integralInfo)
       // Compute integral of this basis function squared, for normalization.
       ergo_real integralValue =
         sfi.computeIntegralOfSquareOfBasisFunc(integralInfo, &basisFunc, use_6_d_funcs);
-      ergo_real normalizationFactor = (ergo_real)1.0 / std::sqrt(integralValue);
+      ergo_real normalizationFactor = (ergo_real)1.0 / template_blas_sqrt(integralValue);
 
       if(normalizationFactor < normFactorShell_min)
 	normFactorShell_min = normalizationFactor;
@@ -742,10 +755,10 @@ int BasisInfoStruct::normalizeShells(const IntegralInfo& integralInfo)
 	normFactorShell_max = normalizationFactor;          
     } /* END FOR j */
       
-    ergo_real absdiff = std::fabs(normFactorShell_max - normFactorShell_min);
+    ergo_real absdiff = template_blas_fabs(normFactorShell_max - normFactorShell_min);
     if(absdiff > max_allowed_difference) {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in BasisInfoStruct::normalizeShells: different norm factors within shell, absdiff = %22.11f.", (double)absdiff);
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "normFactorShell_min = %22.11g  normFactorShell_max = %22.11g", normFactorShell_min, normFactorShell_max);
+      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in BasisInfoStruct::normalizeShells: different norm factors within shell, absdiff = %g.", (double)absdiff);
+      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "normFactorShell_min = %22.15g  normFactorShell_max = %22.15g", (double)normFactorShell_min, (double)normFactorShell_max);
       return -1;
     }
       
@@ -819,6 +832,7 @@ int BasisInfoStruct::getSimplePrimitivesAll(const IntegralInfo& integralInfo)
 {
   BasisFuncStruct* currBasisFunc;
   int nbast = this->noOfBasisFuncs;
+  const int MAX_NO_OF_PRIMITIVES_PER_BASIS_FUNC = 44; // Guess here to be able to preallocate large enough buffer
   int maxNoOfSimplePrimsTot = nbast * MAX_NO_OF_PRIMITIVES_PER_BASIS_FUNC;
   DistributionSpecStruct* list = new DistributionSpecStruct[maxNoOfSimplePrimsTot];
   
@@ -884,7 +898,7 @@ void BasisInfoStruct::write_to_buffer ( char * dataBuffer, size_t const bufferSi
   // First store the 4 int numbers.
   char* p = dataBuffer;
   if(bufferSize < get_size())
-    throw std::runtime_error("Error: bufferSize too small.");
+    throw std::runtime_error("Error in BasisInfoStruct::write_to_buffer: bufferSize too small.");
   memcpy(p, &noOfShells, sizeof(int));
   p += sizeof(int);
   memcpy(p, &noOfBasisFuncs, sizeof(int));
@@ -919,7 +933,7 @@ void BasisInfoStruct::assign_from_buffer ( char const * dataBuffer, size_t const
   // First get the 4 int numbers.
   const char* p = dataBuffer;
   if(bufferSize < 4 * sizeof(int))
-    throw std::runtime_error("Error: bufferSize too small.");
+    throw std::runtime_error("Error in BasisInfoStruct::assign_from_buffer: bufferSize too small.");
   memcpy(&noOfShells, p, sizeof(int));
   p += sizeof(int);
   memcpy(&noOfBasisFuncs, p, sizeof(int));
@@ -983,5 +997,5 @@ ergo_real getSafeMaxDistance(const BasisInfoStruct & basisInfo)
       ergo_real dx = maxCoords[coordNo] - minCoords[coordNo];
       sum += dx*dx;
     }
-  return std::sqrt(sum);
+  return template_blas_sqrt(sum);
 }
