@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,23 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file basisinfo.h
+
+    \brief Code for setting up basis functions starting from shells.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #ifndef BASISINFO_HEADER
 #define BASISINFO_HEADER
 
@@ -39,19 +47,15 @@
 
 #include "basisset.h"
 
-#define MAX_NO_OF_PRIMITIVES_PER_BASIS_FUNC 44
-
-struct DistributionSpecStruct_{
+struct DistributionSpecStruct {
   ergo_real coeff;           /**< Coefficient A */
   ergo_real exponent;        /**< exponent alfa */
-  ergo_real extent;
+  ergo_real extent;          /* FIXME ELIAS: remove this here since the "extent" you want to use depends on what you are doing? */
   ergo_real centerCoords[3]; /**< x0, y0, z0    */
   char monomialInts[4];  /**< nx, ny, nz    */
 };
-typedef struct DistributionSpecStruct_ DistributionSpecStruct;
 
-typedef struct
-{
+struct DistributionSpecStructLabeled {
   int basisFuncIndex_1;
   int basisFuncIndex_2;
   int pairIndex;
@@ -59,12 +63,12 @@ typedef struct
   ergo_real limitingFactor; // squareroot of repulsion integral of this distr with itself.
   ergo_real dmatElement;
   DistributionSpecStruct distr;
-} DistributionSpecStructLabeled;
+};
 
 
 #define MAX_NO_OF_CONTR_GAUSSIANS 20
 
-struct ShellSpecStruct_{
+struct ShellSpecStruct {
   ergo_real coeffList[MAX_NO_OF_CONTR_GAUSSIANS];
   ergo_real exponentList[MAX_NO_OF_CONTR_GAUSSIANS];
   ergo_real sizeList[MAX_NO_OF_CONTR_GAUSSIANS];
@@ -77,9 +81,8 @@ struct ShellSpecStruct_{
   int startIndexInMatrix; /* start index in density matrix  */
   int dummy; /* padding to make sure the size of this structure is a multiple of 8 bytes */
 };
-typedef struct ShellSpecStruct_ ShellSpecStruct;
 
-struct BasisFuncStruct_{
+struct BasisFuncStruct {
   int noOfContr;
   ergo_real coeffList[MAX_NO_OF_CONTR_GAUSSIANS];
   ergo_real exponentList[MAX_NO_OF_CONTR_GAUSSIANS];
@@ -92,24 +95,21 @@ struct BasisFuncStruct_{
   int noOfTermsInPolynomial;
   basis_func_term_struct poly[MAX_NO_OF_TERMS_IN_BASIS_FUNC_POLY];
 };
-typedef struct BasisFuncStruct_ BasisFuncStruct; 
 
 
-typedef struct
-{
+struct basis_set_range_struct {
   int startAtomIndex;
   int count;
   basisset_info* basisset;
-} basis_set_range_struct;
+};
 
-typedef struct
-{
+struct BasissetNameRange {
   int startAtomIndex;
   int count;
   char* basisSetFileName;
-} BasissetNameRange;
+};
 
-struct BasisInfoStruct{
+struct BasisInfoStruct {
   int use_6_d_funcs; /**< Whether to use 6 d-type basis functions
 			instead of the usual 5 functions. This option
 			exists to make it possible to get results
@@ -159,6 +159,11 @@ struct BasisInfoStruct{
 				  int do_normalization,
 				  int skip_sort_shells);
 
+  static int getNoOfBasisFuncsForAtomType(const basisset_info & basisset,
+					  const IntegralInfo & integralInfo,
+					  int atomCharge,
+					  int use_6_d_funcs);
+
   BasisInfoStruct *permuteShells(const int *shellMap,
                                  const IntegralInfo& ii) const;
 
