@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,21 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /** @file basisset.cc
 
-    \brief Code for parsing a text file specifying a basisset.
+    \brief Code for representing basis set information for Gaussian
+    basis sets, and for parsing a text file specifying such a
+    basisset.
 
-    @author: Elias Rudberg <em>responsible</em>. 
+    @author: Elias Rudberg <em>responsible</em>.
 */
+
 /* -*-mode:c; indent-tabs-mode: nil -*- */
 /* basisset.c: provides read_basisset_file() which creates a
    basisset_struct from a data contained in a file */
@@ -117,7 +121,7 @@ read_basisset_file(basisset_info & result,
   enum { END_PARSING, ATOM_EXPECTED, SHELL_EXPECTED,
          SHELL_OR_ATOM_EXPECTED, CONTRACTION_BLOCK_EXPECTED } state;
   int uncontracted = 0;
-  char line[256];
+  char line[512];
   basisset_atom_struct* currAtom = NULL;
   int spdf = -1;
   int shellBaseIndex = -1;
@@ -162,10 +166,18 @@ read_basisset_file(basisset_info & result,
   do {
     int dummy;
     if(lineConsumed) {
+      for(unsigned int cc = 0; cc < sizeof(line); cc++)
+	line[cc] = '\0';
       if(fgets(line, sizeof(line), f) == NULL) {
         state = END_PARSING;
         break;
       }
+      // This only works if sizeof(line) is large enough. Here we check that sizeof(line) is really large enough.
+      if(strlen(line) >= sizeof(line)-1) {
+	do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "Error in read_basisset_file: (strlen(line) >= sizeof(line)-1). "
+		  "This indicates that the basis set file '%s' included a line that was too long, longer than %d characters.", fileName, (int)(sizeof(line)-1));
+	return -1;
+      }
       lineConsumed = 0;
       lineNo++;
     }
