@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file basisset.h
+
+    \brief Code for representing basis set information for Gaussian
+    basis sets, and for parsing a text file specifying such a
+    basisset.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; indent-tabs-mode: nil -*- */
 #ifndef BASISSET_HEADER
 #define BASISSET_HEADER
@@ -62,11 +72,6 @@ typedef struct
   basisset_shell_struct shells[MAX_NO_OF_SHELLS_PER_ATOM];
 } basisset_atom_struct;
 
-/* ELIAS NOTE 2014-08-02: FIXME: The basisset_struct struct is too
-   big, causing problems when used as a
-   chttl::Basic<basisset_struct>. The easiest fix may be to make the
-   list dynamically allocated, and implement a simple chunk type for
-   it instead of using chttl::Basic.  */
 struct basisset_info {
   std::vector<basisset_atom_struct> atoms;
   basisset_info();
