@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,21 +17,23 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /** @file ci.cc
 
-    \brief Configuration Interaction (CI) code.
+    @brief Configuration Interaction (CI) code.
 
-    @author: Elias Rudberg <em>responsible</em>. 
+    @author: Elias Rudberg <em>responsible</em>
 */
+
 #include <cstdlib>
 #include <cstdio>
 #include <memory.h>
@@ -102,7 +104,7 @@ static ergo_real get_vector_norm(int n, const ergo_real* v)
   ergo_real sqSum = 0;
   for(int i = 0; i < n; i++)
     sqSum += v[i] * v[i];
-  return std::sqrt(sqSum);
+  return template_blas_sqrt(sqSum);
 }
 
 static void normalize_vector(int n, ergo_real* v)
@@ -1341,7 +1343,7 @@ int get_1e_density_matrix(int nSOs,
   for(p = 0; p < nSOs; p++)
     for(q = 0; q < nSOs; q++)
       {
-	if(std::fabs(D->x[p][q] - D->x[q][p]) > 1e-9)
+	if(template_blas_fabs(D->x[p][q] - D->x[q][p]) > 1e-9)
 	  printf("ERROR 11!\n");
       }
   ergo_real sum = 0;
@@ -1467,17 +1469,17 @@ int get_2e_density_matrix(int nSOs,
       for(r = 0; r < nSOs; r++)
 	for(s = 0; s < nSOs; s++)
 	  {
-	    if(std::fabs(d->x[p][q][r][s] - d->x[r][s][p][q]) > 1e-9)
+	    if(template_blas_fabs(d->x[p][q][r][s] - d->x[r][s][p][q]) > 1e-9)
 	      printf("ERROR 1!");
-	    if(std::fabs(d->x[p][q][r][s] + d->x[r][q][p][s]) > 1e-9)
+	    if(template_blas_fabs(d->x[p][q][r][s] + d->x[r][q][p][s]) > 1e-9)
 	      printf("ERROR 2!");
-	    if(std::fabs(d->x[p][q][r][s] + d->x[p][s][r][q]) > 1e-9)
+	    if(template_blas_fabs(d->x[p][q][r][s] + d->x[p][s][r][q]) > 1e-9)
 	      printf("ERROR 3!");
-	    if(std::fabs(d->x[p][q][p][s]) > 1e-9)
+	    if(template_blas_fabs(d->x[p][q][p][s]) > 1e-9)
 	      printf("ERROR 4!");
-	    if(std::fabs(d->x[p][q][r][q]) > 1e-9)
+	    if(template_blas_fabs(d->x[p][q][r][q]) > 1e-9)
 	      printf("ERROR 5!");
-	    if(std::fabs(d->x[p][q][p][q]) > 1e-9)
+	    if(template_blas_fabs(d->x[p][q][p][q]) > 1e-9)
 	      printf("ERROR 6!");
 	  }
   
@@ -2504,7 +2506,7 @@ int get_Lowdin_orbitals(int n, const ergo_real* S, ergo_real* MOs)
       {
 	ergo_real sum = 0;
 	for(int k = 0; k < n; k++)
-	  sum += A[k*n+i] * A[k*n+j] * (1.0 / std::sqrt(eigvalList[k]));
+	  sum += A[k*n+i] * A[k*n+j] * (1.0 / template_blas_sqrt(eigvalList[k]));
 	MOs[i*n+j] = sum;
       }
 
