@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,21 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /** @file densfromf_full.cc
 
-    \brief Routine get_dens_from_fock_full() for getting density matrix from a given Fock matrix using diagonalization.
+    @brief Routine get_dens_from_fock_full() for getting density
+    matrix from a given Fock matrix using diagonalization.
 
-    @author: Elias Rudberg <em>responsible</em>. 
+    @author: Elias Rudberg <em>responsible</em>
 */
+
 #include "densfromf_full.h"
 #include "output.h"
 #include <memory.h>
@@ -49,7 +52,7 @@
 #include "units.h"
 
 #include "mat_gblas.h"
-
+#include "matInclude.h"
 
 
 
@@ -122,10 +125,10 @@ get_dens_from_cmo_zeroT(int n,
 
 
 static ergo_real x_times_ln_x(ergo_real x) {
-  ergo_real eps = std::numeric_limits<ergo_real>::epsilon();
-  if(x < std::sqrt(eps))
+  ergo_real eps = mat::getMachineEpsilon<ergo_real>();
+  if(x < template_blas_sqrt(eps))
     return 0;
-  return x * std::log(x);
+  return x * template_blas_log(x);
 }
 
 static void 
@@ -156,7 +159,7 @@ get_dens_from_cmo_FermiDiracDistr(int n,
       niter++;
       mu = (mu_hi + mu_lo) / 2;
       for(int i = 0; i < n; i++)
-	occupationNumbers[i] = (ergo_real)1 / (std::exp((eigv[i] - mu) / (k_B * T)) + 1);
+	occupationNumbers[i] = (ergo_real)1 / (template_blas_exp((eigv[i] - mu) / (k_B * T)) + 1);
       trace = 0;
       for(int i = 0; i < n; i++)
 	trace += occupationNumbers[i];
@@ -166,15 +169,15 @@ get_dens_from_cmo_FermiDiracDistr(int n,
 	mu_hi = mu;
       // Emanuel comment: use some kind of relative error here,
       // otherwise we do not converge if mu is small or large
-      if(mu_hi - mu_lo <= (std::fabs(mu_hi)+std::fabs(mu_lo))*100*std::numeric_limits<ergo_real>::epsilon())
+      if(mu_hi - mu_lo <= (template_blas_fabs(mu_hi)+template_blas_fabs(mu_lo))*100*mat::getMachineEpsilon<ergo_real>())
 	break;
       if (niter >= maxiter)
 	throw "Reached maxiter in Fermi function.";
     }
     // Check relative occupation error, if large we try to expand the
     // search interval
-    if(std::fabs(noOfOccupiedOrbs - trace)  < 
-       std::abs(noOfOccupiedOrbs)*10000*std::numeric_limits<ergo_real>::epsilon())
+    if(template_blas_fabs(noOfOccupiedOrbs - trace)  < 
+       std::abs(noOfOccupiedOrbs)*10000*mat::getMachineEpsilon<ergo_real>())
       break;
     ergo_real mu_min_max_diff = mu_max - mu_min;
     mu_min = mu_min - mu_min_max_diff;
@@ -213,13 +216,22 @@ get_dens_from_fock_full(int n,
 			ergo_real & resultEntropyTerm,
 			ergo_real & resultHomoLumoGap,
 			int store_all_eigenvalues_to_file, 
-			ergo_real * const lumoVec,
-			ergo_real * const homoVec)
+			int number_of_occ_eigenvectors, 
+			int number_of_unocc_eigenvectors, 
+			std::vector<std::vector<ergo_real> > &eigVecOCC, 
+			std::vector<std::vector<ergo_real> > &eigVecUNOCC, 
+			std::vector<ergo_real> &eigValOCC, 
+			std::vector<ergo_real> &eigValUNOCC
+			)
 {
   if(noOfOccupiedOrbs == 0) {
     memset(result_P, 0, n*n*sizeof(ergo_real));
     return 0;
   }
+	
+	assert(number_of_occ_eigenvectors <= noOfOccupiedOrbs);
+	assert(number_of_unocc_eigenvectors <= n-noOfOccupiedOrbs);
+	
   Util::TimeMeter timeMeter;
   ergo_real* cmo  = (ergo_real*)ergo_malloc(n*n*sizeof(ergo_real));
   ergo_real* eigv = (ergo_real*)ergo_malloc(n*sizeof(ergo_real));
@@ -244,16 +256,34 @@ get_dens_from_fock_full(int n,
 	throw "Error opening file for storing eigenvalues.";
 
       for (int ind = 0; ind < n; ind++) 
-	file << std::fixed << std::setprecision(12) << eigv[ind] << std::endl;
+	file << std::fixed << std::setprecision(12) << (double)eigv[ind] << std::endl;
       file.close();  
     } // end of store_all_eigenvalues_to_file
 
-  if ( lumoVec ) 
-    for (int i = 0; i < n; i++) 
-      lumoVec[i] = cmo[n*(noOfOccupiedOrbs)+i];
-  if ( homoVec ) 
-    for (int i = 0; i < n; i++) 
-      homoVec[i] = cmo[n*(noOfOccupiedOrbs-1)+i];
+
+  if(number_of_occ_eigenvectors > 0)
+	{
+		assert((int)eigVecOCC.size() >= number_of_occ_eigenvectors);
+		assert((int)eigValOCC.size() >= number_of_occ_eigenvectors);
+		for (int k = 0; k < number_of_occ_eigenvectors; k++) {
+			eigVecOCC[k].resize(n);
+			memcpy(&eigVecOCC[k][0], &cmo[n*(noOfOccupiedOrbs-1-k)], 
+			n*sizeof(ergo_real));
+			eigValOCC[k] = eigv[noOfOccupiedOrbs-1-k];
+		}
+	}
+	if(number_of_unocc_eigenvectors > 0)
+	{
+		assert((int)eigVecUNOCC.size() >= number_of_unocc_eigenvectors);
+		assert((int)eigValUNOCC.size() >= number_of_occ_eigenvectors);
+		for (int k = 0; k < number_of_unocc_eigenvectors; k++) {
+			eigVecUNOCC[k].resize(n);
+			memcpy(&eigVecUNOCC[k][0], &cmo[n*(noOfOccupiedOrbs+k)], 
+			n*sizeof(ergo_real));
+			eigValUNOCC[k] = eigv[noOfOccupiedOrbs+k];
+		}
+	}			
+			
   ergo_free(cmo);
   ergo_free(eigv);  
 
