@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,18 +17,28 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file densfromf_full.h
+
+    @brief Routine get_dens_from_fock_full() for getting density
+    matrix from a given Fock matrix using diagonalization.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef DENSFROMFFULLHEADER
 #define DENSFROMFFULLHEADER
 
+#include <vector>
 #include "realtype.h"
 
 int get_F_orbs(int n, 
@@ -47,7 +57,11 @@ int get_dens_from_fock_full(int n,
 			    ergo_real & resultEntropyTerm,
 			    ergo_real & resultHomoLumoGap,
 			    int store_all_eigenvalues_to_file, 
-			    ergo_real * const lumoVec = 0,
-			    ergo_real * const homoVec = 0);
+					int number_of_occ_eigenvectors, 
+					int number_of_unocc_eigenvectors, 
+					std::vector<std::vector<ergo_real> > &eigVecOCC, 
+					std::vector<std::vector<ergo_real> > &eigVecUNOCC,
+					std::vector<ergo_real> &eigValOCC, 
+					std::vector<ergo_real> &eigValUNOCC );
 
 #endif
