@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,22 +17,29 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file constants.h
+
+    @brief File contataining definitions of constants used in the recursive expansion. 
+
+    @author Anastasia Kruchinina <em>responsible</em>
+*/
 
 
 #ifndef HEADER_CONSTANTS
 #define HEADER_CONSTANTS
 
-#define ORDER	  1.8      /**< Order of convergence of recursive expansion (We take smaller value due to numerical errors). */
-#define C_SP2     4.4092   
+#define ORDER	  2         /**< Order of convergence of recursive expansion. */
+#define C_SP2     6.8872    /**< Constant used on the stopping criterion for the SP2 method. */
 
 #endif //HEADER_CONSTANTS
 
