@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -70,7 +71,7 @@ get_bf_vals(int nvclen, ergo_real *gao,
     ergo_real fac = currShell.coeffList[i];
     ergo_real malpha = -currShell.exponentList[i];
     for(k=0; k < nvclen; k++)
-      ga[k] += fac * std::exp(malpha*pa2[k]);
+      ga[k] += fac * template_blas_exp(malpha*pa2[k]);
   }
 
   /* multiply by the angular part */
@@ -135,7 +136,7 @@ get_bf_vals_derivs(int nvclen, ergo_real *gao,
   for(i=0; i<currShell.noOfContr; i++) {
       for(k=0; k < nvclen; k++) {
           ergo_real fac = currShell.coeffList[i] * 
-            std::exp(-currShell.exponentList[i]*pa2[k]);
+            template_blas_exp(-currShell.exponentList[i]*pa2[k]);
           ga[k] += fac;
           gu[k] -= 2*currShell.exponentList[i]*fac;
       }
