@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,17 +17,23 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /*-*-mode: C; c-indentation-style: "bsd"; c-basic-offset: 4; -*-*/
-/** @file dft_common.cc Common DFT routines. Mostly functional mixing.
+
+/** @file dft_common.cc
+
+    @brief Common DFT routines. Mostly functional mixing.
+
+    @author: Pawel Salek <em>responsible</em>
 
    (c) Pawel Salek, pawsa@theochem.kth.se, 2001-08-02
    NOTES: Adding new functionals:
@@ -71,7 +77,7 @@ dal_malloc_(size_t sz, const char *place, unsigned line)
 }
 
 #if 0
-static const ergo_real GET_BLOCKS_FUDGE_FACTOR = 0.5*std::sqrt(3.0);
+static const ergo_real GET_BLOCKS_FUDGE_FACTOR = 0.5*template_blas_sqrt(3.0);
 static const ergo_real SET_SHELL_RADII_ORBITAL_THR = 2e-12;
 #else
 static const ergo_real GET_BLOCKS_FUDGE_FACTOR = 1.0;
@@ -312,23 +318,23 @@ ErgoMolInfo::setShellRadii(real *rshell) const
 {
     static const real facl[] = {
         1.0, 1.33, 1.6, 1.83, 2.03, 2.22, 2.39, 2.55, 2.70, 2.84 };
-    real thlog = std::log(SET_SHELL_RADII_ORBITAL_THR);
+    real thlog = template_blas_log(SET_SHELL_RADII_ORBITAL_THR);
     int ishela, iprim;
 
     for(ishela=0; ishela <bis.noOfShells; ishela++) {
         const ShellSpecStruct& currShell = bis.shellList[ishela];
         rshell[ishela] =  0.0;
         for(iprim=0; iprim<currShell.noOfContr; iprim++) {
-            ergo_real absCoeff = std::fabs(currShell.coeffList[iprim]);
+            ergo_real absCoeff = template_blas_fabs(currShell.coeffList[iprim]);
             if(absCoeff>0) {
-              real r2 = (std::log(absCoeff)-thlog)/
+              real r2 = (template_blas_log(absCoeff)-thlog)/
                     currShell.exponentList[iprim];
                 if(rshell[ishela]<r2) rshell[ishela] = r2;
             }
         }
     }
     for(ishela=0; ishela <bis.noOfShells; ishela++) {
-        rshell[ishela] = std::sqrt(rshell[ishela])
+        rshell[ishela] = template_blas_sqrt(rshell[ishela])
             *facl[bis.shellList[ishela].shellType];
     }
 }
@@ -356,13 +362,13 @@ ErgoMolInfo::getBlocks1(const real *center, real cellsz, const real *rshell,
 
     *nblcnt = -1;
     iprev = -1111;
-    celldg = cellsz*0.5*std::sqrt(3.0);
+    celldg = cellsz*0.5*template_blas_sqrt(3.0);
     for(ishela=0; ishela <bis.noOfShells; ishela++) {
         const ShellSpecStruct& currShell = bis.shellList[ishela];        
         px = center[0]-currShell.centerCoords[0];
         py = center[1]-currShell.centerCoords[1];
         pz = center[2]-currShell.centerCoords[2];
-        dst = std::sqrt(px*px + py*py + pz*pz);
+        dst = template_blas_sqrt(px*px + py*py + pz*pz);
         if(dst<rshell[ishela]+celldg) {
             /* accepted... */
             if(ishela != iprev+1)
@@ -524,7 +530,7 @@ distance(const ergo_real *a, const ergo_real *b)
         ergo_real d = a[i]-b[i];
         r2 += d*d;
     }
-    return std::sqrt(r2);
+    return template_blas_sqrt(r2);
 }
 
 void
@@ -588,7 +594,7 @@ ErgoMolInfo::getBlocks(const real *center, real cellsz, const real *rshell,
                         int *nblcnt, int (*iblcks)[2]) const
 {
     std::map<int,int> activeShellList;
-    /* The factor 0.5*std::sqrt(3.0) does not fit here but is present
+    /* The factor 0.5*template_blas_sqrt(3.0) does not fit here but is present
        for compatibility with the old code. */
     shellTree->getOverlappingWith(center, cellsz *GET_BLOCKS_FUDGE_FACTOR,
                                   activeShellList);
