@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -74,10 +75,10 @@ static __inline__
 void test_var(real comp, real refer, const char* fun, const char* drv, 
               int* counter)
 {
-      if(std::fabs(comp-refer)>2e-7+5e-5*(std::fabs(comp)+std::fabs(refer))) { 
+      if(template_blas_fabs(comp-refer)>2e-7+5e-5*(template_blas_fabs(comp)+template_blas_fabs(refer))) { 
         if(*counter<95)
             printf("%s %s: fin.diff: %12g found: %12g, diff=%g\n", 
-                   fun, drv, (double)refer, (double)comp, (double)std::fabs(comp-refer));
+                   fun, drv, (double)refer, (double)comp, (double)template_blas_fabs(comp-refer));
 	++*counter;
 	  }/*  else printf("Test '%s:%s' passed (expected: %g found: %g).\n",
                fun,drv, refer, comp); */
@@ -119,7 +120,7 @@ test_first(const char* fun_name, EnergyFunc func, FirstOrderFun first_func)
 		test_var(gga.df0010,num, fun_name, "df0010", &failed);
 		
 		/* TEST df00001 */
-		if(std::fabs(gracos)<1e-5) continue;
+		if(template_blas_fabs(gracos)<1e-5) continue;
 		dgra = gracos*1e-7;
 		dt = dp; dt.gradab -= dgra; resm = func(&dt);
 		dt = dp; dt.gradab += dgra; resp = func(&dt);
