@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -43,6 +44,13 @@
 #ifndef _FUNCTIONALS_H_
 #define _FUNCTIONALS_H_
 
+/* We need to include config.h to get macro PRECISION_QUAD_FLT128 */
+#include "config.h"
+
+#ifdef PRECISION_QUAD_FLT128
+#include <quadmath.h>
+#endif
+
 #include "realtype.h"
 
 #ifdef __cplusplus
@@ -54,6 +62,8 @@
 typedef ergo_real real;
 
 #if defined(FUNC_PRECISION) && FUNC_PRECISION == 1
+
+/* Use single-precision math functions */
 #define ASINH asinhf
 #define ATAN atanf
 #define ERF erff
@@ -62,7 +72,10 @@ typedef ergo_real real;
 #define LOG logf
 #define POW powf
 #define SQRT sqrtf
+
 #elif defined(FUNC_PRECISION) && FUNC_PRECISION == 2
+
+/* Use double-precision math functions */
 #define ASINH asinh
 #define ATAN atan
 #define ERF erf
@@ -71,9 +84,23 @@ typedef ergo_real real;
 #define LOG log
 #define POW pow
 #define SQRT sqrt
+
 #else
+
+#ifdef PRECISION_QUAD_FLT128
+/* Use quad-precision math functions */
+#define ASINH asinhq
+#define ATAN atanq
+#define ERF erfq
+#define EXP expq
+#define FABS fabsq
+#define LOG logq
+#define POW powq
+#define SQRT sqrtq
+#else
+/* Use "long double"-precision math functions */
 #define ASINH asinhl
-#define ATAN atanl 
+#define ATAN atanl
 #define ERF erfl
 #define EXP expl
 #define FABS fabsl
@@ -82,6 +109,8 @@ typedef ergo_real real;
 #define SQRT sqrtl
 #endif
 
+#endif
+
 /* FirstDrv: matrix of first order derivatives with respect to two
  * parameters: density rho and SQUARE of the gradient of density grho.  
  * zeta_i = |\nabla\rho_i|
@@ -445,6 +474,7 @@ extern int fun_true(void);
 extern int fun_false(void);
 /* fortran (and not only) functional stub routines */
 EXTERN_C void dftlistfuncs_(void);
+EXTERN_C void dftlistfuncs_using_printf_(void);
 int dft_isgga_(void);
 int dft_isgga__(void);
 
