@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -99,8 +100,8 @@ RadialSchemeGC2::init(int myNumber, int Z, real thrl)
     else if(Z<=86) ta=5;
     else ta=6;
 
-    thrl = 1e-1*std::sqrt(thrl); /* Fudge factor. */
-    ri = int( -5.0*(3*std::log10(thrl)-ta+8) );
+    thrl = 1e-1*template_blas_sqrt(thrl); /* Fudge factor. */
+    ri = int( -5.0*(3*template_blas_log10(thrl)-ta+8) );
     gridSize = ri>MIN_RAD_PT ? ri : MIN_RAD_PT;
 }
 
@@ -113,7 +114,7 @@ RadialSchemeGC2::generate(real *rad, real *wght)
     /* constants */
     static const real pi_2 = 2.0/M_PI;  
     static const real sfac = 2.0/3.0;
-    const real rfac = 1.0/std::log(static_cast<real>(2.0));
+    const real rfac = 1.0/template_blas_log(static_cast<real>(2.0));
     real n_one, n_pi, wfac;
     /* variables */
     real x = 0.0, angl = 0.0, w = 0.0;
@@ -127,10 +128,10 @@ RadialSchemeGC2::generate(real *rad, real *wght)
         real sinangl, sinangl2, r;
         x = (gridSize-1-2*i)/n_one;
         angl = n_pi*(i+1);
-        sinangl = std::sin(angl); 
+        sinangl = template_blas_sin(angl); 
         sinangl2 = sinangl*sinangl;
-        x += pi_2*(1.0+sfac*sinangl2)*std::cos(angl)*sinangl;
-        r = rfac*std::log( static_cast<real>(2.0/(1.0-x)) );
+        x += pi_2*(1.0+sfac*sinangl2)*template_blas_cos(angl)*sinangl;
+        r = rfac*template_blas_log( static_cast<real>(2.0/(1.0-x)) );
         w = wfac*sinangl2*sinangl2;
         wght[gridSize-i-1] = w*rfac/(1.0-x)*r*r;
         rad[gridSize-i-1] = r;
@@ -167,7 +168,7 @@ RadialSchemeTurbo::init(int myNumber, int Z, real thrl)
   else ta=6;
 
   /* thrl = 1e-5 maps to 0, 1e-13 -> 25, following Table III */
-  accuracy_correction = int( (-std::log10(thrl)-5.0)*3.0 );
+  accuracy_correction = int( (-template_blas_log10(thrl)-5.0)*3.0 );
   if(accuracy_correction<0) accuracy_correction = 0;
   z_correction = ta*5;
 
@@ -191,14 +192,14 @@ RadialSchemeTurbo::generate(real *rad, real *wght)
   /* radial points */ 
   for (int i=0; i<gridSize; i++) {
     real angle = (i+0.5)*piOverN;
-    real x = std::cos(angle);
-    real s = std::sin(angle);
+    real x = template_blas_cos(angle);
+    real s = template_blas_sin(angle);
     real w = piOverN * s;
-    real aPlusX06 = std::pow(a+x, (ergo_real)0.6);
-    real logAPlus1Over1MinusX = std::log( (a+1.0)/(1.0-x) );
+    real aPlusX06 = template_blas_pow(a+x, (ergo_real)0.6);
+    real logAPlus1Over1MinusX = template_blas_log( (a+1.0)/(1.0-x) );
     real r = rfac*aPlusX06*logAPlus1Over1MinusX;
     real rdiff = rfac*(aPlusX06/(1.0-x) +
-                       0.6*logAPlus1Over1MinusX/std::pow(a+x, (ergo_real)0.4));
+                       0.6*logAPlus1Over1MinusX/template_blas_pow(a+x, (ergo_real)0.4));
     wght[gridSize-i-1] = w*rdiff*r*r;
     rad[gridSize-i-1] = r;
   }
@@ -249,14 +250,14 @@ diserr(int l, real rd)
 
     fac = fac/ifac;
     lm = l>4 ? 4 : l;
-    rdlog = std::log(rd);
+    rdlog = template_blas_log(rd);
     //res = polval(3,CF(1,LM),RDLOG);
     res = cf[lm][0]; x = rdlog;
     for(i=1; i<4; i++) {
         res += cf[lm][i]*x;
         x *= rdlog;
     }
-    htlog = std::log(res);
+    htlog = template_blas_log(res);
     // Newton-Raphson search
     for(it = 0; it<20; it++) {
         long_real u0, u1, f0, f1, dx;
@@ -265,14 +266,14 @@ diserr(int l, real rd)
         long_real piex = M_PI*pih*0.5;
         for(i = 0; i<l; i++)
             pihl = pihl*pih;
-        u0   = fac*pihl*std::exp(-piex);
+        u0   = fac*pihl*template_blas_exp(-piex);
         u1   = u0*((piex/res)-(l+1)/pih);
-        f0   = std::log(u0)-rdlog;
+        f0   = template_blas_log(u0)-rdlog;
         f1   = res*u1/u0;
         dx = f0/f1;
         htlog = htlog - dx;
-        res = std::exp(htlog);
-        if(std::fabs(dx)<ACC) return res;
+        res = template_blas_exp(htlog);
+        if(template_blas_fabs(dx)<ACC) return res;
     }
     puts("diserr never reached"); return 0.1;
 }
@@ -302,16 +303,16 @@ outerr(real al, int l, real rd)
     }
     expl = 0.5*(2*l+1);
     a = 2*fac/(tolen*M_2_SQRTPI);
-    aln = std::log(a);
-    rln = std::log(rd);
+    aln = template_blas_log(a);
+    rln = template_blas_log(rd);
     u = 35.0;
     //Newton-Raphson search
     for(it = 0; it<8; it++) {
-        long_real f0hln = aln+expl*std::log(u)-u-rln;
+        long_real f0hln = aln+expl*template_blas_log(u)-u-rln;
         long_real f1hln = expl/u-1.0;
         long_real dx = f0hln/f1hln;
         u = u - dx;
-        if(std::fabs(dx)<1e-8) return std::sqrt(u/al);
+        if(template_blas_fabs(dx)<1e-8) return template_blas_sqrt(u/al);
     }
     puts("outerr never reached");
     return 10.0;
@@ -344,10 +345,10 @@ RadialSchemeLMG::init(int myNumber, int charge, real thrl)
         if(lAA[l][1]>ah) ah = lAA[l][1];
         //printf("ah=%g %g\n", ah, aa[l][1]);
     }
-    eph = std::exp(h);
+    eph = template_blas_exp(h);
     ah *= 2;
-    rl = (1.9+std::log(thrl))/3.0 - 0.5*std::log(ah);
-    rl = std::exp(rl);
+    rl = (1.9+template_blas_log(thrl))/3.0 - 0.5*template_blas_log(ah);
+    rl = template_blas_exp(rl);
     //printf("* Inner grid point: %10.5g ah=%f\n", rl, ah);
     //...  Outer point
     rh = 0.0;
@@ -358,7 +359,7 @@ RadialSchemeLMG::init(int myNumber, int charge, real thrl)
         }
     }
     grdc = rl/(eph-1.0);
-    gridSize = (int) (std::log(1.0 + rh/grdc)/h);
+    gridSize = (int) (template_blas_log(1.0 + rh/grdc)/h);
 }
 
 /** Generates grid point positions and associated weights using LMG
