@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -293,7 +294,7 @@ BoxPartitioner::prepare(const std::vector<AtomicGrid>& atoms,
       real jradius = 
         BraggRadii[(atoms[atno2].charge()>=(signed)BraggSize
                     ? BraggSize-1 : atoms[atno2].charge())];
-      real chi = std::sqrt(iradius/jradius);
+      real chi = template_blas_sqrt(iradius/jradius);
       real temp = (chi-1)/(chi+1);
       temp = temp/(temp*temp-1);
       if(temp>0.5) temp = 0.5;      
@@ -309,7 +310,7 @@ BoxPartitioner::prepare(const std::vector<AtomicGrid>& atoms,
       real dy = atoms[atno].center[1] - gridPoints[ptno][1];
       real dz = atoms[atno].center[2] - gridPoints[ptno][2];
 
-      rji  [ptno] = std::sqrt(dx*dx + dy*dy + dz*dz);
+      rji  [ptno] = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
       p_kgi[ptno] = 1.0;
     }
   }
@@ -614,10 +615,15 @@ Stream::saveAtomGridInBox(unsigned iAtom, const Box& box,
      decreased from 8192 to 4096 because the 8192 was apparently too
      large when running on a Mac, it seemed like the threads ran out
      of stack space then. */
+  /* Elias note 2017-12-21: The MAX_BATCH_LENGTH value here was
+     decreased again, this time from 4096 to 2048, because the 4096
+     value was apparently too large when running on a Mac with long
+     double precision, there was a "bus error" problem then that
+     seemed to be fixed by reducing MAX_BATCH_LENGTH. */
   /* FIXME: there does not seem to be proper checking if the
      MAX_BATCH_LENGTH value is too small, it just gives wrong results
      without detecting the error. */
-  static const unsigned MAX_BATCH_LENGTH = 4096;
+  static const unsigned MAX_BATCH_LENGTH = 2048;
   const AtomicGrid& atom = atoms[relevantAtoms[iAtom]];
   real dist = box.getDistanceTo(atom.center.v);
   const RadialGrid& rGrid = atom.getRadialGrid();
@@ -883,7 +889,7 @@ Stream::saveToFile(const char *fname, int noOfThreads)
     for(int i=0; i<3; ++i) { 
       real center = 0.5* (box.lo[i] + box.hi[i]);
       real dim = 0.5* (box.hi[i] - box.lo[i]);
-      int x, dimi = int(std::ceil(dim/boxSize));
+      int x, dimi = int(std::ceil((long double)(dim/boxSize)));
       for(x=1; x<dimi; x *=2);
       dim = x*boxSize;
       box.lo[i] = center-dim;
