@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,17 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file lin_trans.cc
+
+    @brief Blocked DFT Linear Response contribution evaluator.
+
+    @author: Pawel Salek <em>responsible</em>
+*/
+
 /* -*-mode:c; c-basic-offset:4; -*- */
-/**  @file lin_trans.cc Blocked DFT Linear Response contribution evaluator. */
 
 #include "config.h"
 
@@ -74,7 +81,7 @@ lin_resp_cb_b_lda(DftIntegratorBl* grid, real * restrict tmp,
 {
   const real * restrict aos = grid->atv;
   real * restrict excmat = data->res;
-  real (* restrict vt) = data->vt; /* [bllen][4] */
+  real * restrict vt = data->vt; /* [bllen][4] */
   int ibl, i, jbl, j, k, isym, ivec;
   int nbast = data->nbast;
   int n2basx = nbast*nbast;
@@ -157,9 +164,9 @@ lin_resp_cb_b_gga(DftIntegratorBl* grid, real * restrict tmp,
       SecondDrv vxc;
       real facr, facg;
       real weight = grid->weight[grid->curr_point+i];
-      real ngrad  = std::sqrt(grid->g.grad[i][0]*grid->g.grad[i][0]+
-                              grid->g.grad[i][1]*grid->g.grad[i][1]+
-                              grid->g.grad[i][2]*grid->g.grad[i][2]);
+      real ngrad  = template_blas_sqrt(grid->g.grad[i][0]*grid->g.grad[i][0]+
+				       grid->g.grad[i][1]*grid->g.grad[i][1]+
+				       grid->g.grad[i][2]*grid->g.grad[i][2]);
       real brg, brz, b0 = vt3[i][0];
       if(ngrad<1e-15|| grid->r.rho[i]<1e-15) {
         vt3[i][0] = vt3[i][1] = vt3[i][2] = vt3[i][3] = 0;
