@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,21 +17,28 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /*-*-mode: C; c-indentation-style: "bsd"; c-basic-offset: 4; -*-*/
-/** @file xc_matrix_sparse.cc The sparse XC matrix evaluator.
-   (c) Pawel Salek, pawsa@theochem.kth.se.
-   2002.04.05
 
-   This module evaluates DFT contribution KS matrix.
+/** @file xc_matrix_sparse.cc
+
+    @brief The sparse XC matrix evaluator.
+
+    @author: Pawel Salek <em>responsible</em>
+
+    (c) Pawel Salek, pawsa@theochem.kth.se.
+    2002.04.05
+
+    This module evaluates DFT contribution KS matrix.
 */
 
 #define WITH_PTHREAD 1
@@ -327,7 +334,7 @@ getXC_seq(const BasisInfoStruct& bis, const IntegralInfo& integralInfo,
 {
   XCEvaluatorRestricted xcEvr(bis, integralInfo, mol, gss,
                               permutationHML, dens);
-  printf("%p\n", &xcEvr);
+  printf("%p\n", (void*)(&xcEvr));
   return xcEvr.getXC(nElectrons, xcm, xcEnergy, 1);
 }
 
