@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,27 +17,28 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /** @file tdhf_dynamics.cc
 
- Implementation of TDHF electron dynamics, explicitly propagating the
- electron density in time in the precence of a time-dependent electric
- field.
+    @brief Implementation of TDHF electron dynamics, explicitly
+    propagating the electron density in time in the precence of a
+    time-dependent electric field.
 
- This implementation was done to reproduce the results in the article
- by Li et al: Phys. Chem. Chem. Phys., 2005, 7, 233–239.
+    @author: Elias Rudberg <em>responsible</em>
 
- DOI link for that article: http://dx.doi.org/10.1039/B415849K
+    This implementation was done to reproduce the results in the article
+    by Li et al: Phys. Chem. Chem. Phys., 2005, 7, 233–239.
 
- @author: Elias Rudberg <em>responsible</em>.
+    DOI link for that article: http://dx.doi.org/10.1039/B415849K
 */
 
 #include <cstring>
@@ -341,12 +342,12 @@ static void get_curr_electric_field(ergo_real* electricField, ergo_real t) {
     E = 0;
   electricField[0] = 0;
   electricField[1] = 0;
-  electricField[2] = E * std::sin(omega*t + phasePhi);
+  electricField[2] = E * template_blas_sin(omega*t + phasePhi);
 }
 
 
 static ergo_real vectorLength(ergo_real x, ergo_real y, ergo_real z) {
-  return std::sqrt(x*x + y*y + z*z);
+  return template_blas_sqrt(x*x + y*y + z*z);
 }
 
 
@@ -457,7 +458,9 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
     H_core_Matrix_curr.resetSizesAndBlocks(matOpts.size_block_info, matOpts.size_block_info);
     ergo_real threshold_integrals_1el = 1e-13;
     int no_of_threads_for_V = 1;
+    ergo_real box_size_for_V_and_T = 7.3;
     Util::TimeMeter timeMeterHCore;
+    ergo_real nuclearRepulsionEnergy_dummy = 0;
     if(compute_h_core_matrix_sparse(integralInfo, 
 				    molecule, 
 				    extraCharges,
@@ -468,6 +471,8 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
 				    H_core_Matrix_curr,
 				    threshold_integrals_1el,
 				    no_of_threads_for_V,
+				    box_size_for_V_and_T,
+				    nuclearRepulsionEnergy_dummy,
 				    matOpts.size_block_info,
 				    matOpts.permutationHML) != 0)
       {
@@ -519,10 +524,10 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
     // We want to output some information every "whole" time unit. The
     // step size is typically smaller than 1 a.u. so now we figure out
     // if this timestep is closest to the nearest "whole" time unit.
-    ergo_real nearestWholeTime = round(t);
-    ergo_real currDiff = std::fabs(t - nearestWholeTime);
-    ergo_real prevDiff = std::fabs(t - timestep - nearestWholeTime);
-    ergo_real nextDiff = std::fabs(t + timestep - nearestWholeTime);
+    ergo_real nearestWholeTime = round((long double)t);
+    ergo_real currDiff = template_blas_fabs(t - nearestWholeTime);
+    ergo_real prevDiff = template_blas_fabs(t - timestep - nearestWholeTime);
+    ergo_real nextDiff = template_blas_fabs(t + timestep - nearestWholeTime);
     if(currDiff <= prevDiff && currDiff <= nextDiff) {
       // Compute instantaneous dipole.
       ergo_real instantaneousDipole_re, instantaneousDipole_im;
