@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,23 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /** @file ergo_scripted.cc
 
-    \brief The main program for the ergo project.  It enables
+    @brief The main program for the ergo project. It enables
     scripting and more complex input forms.
 
     @author: Pawel Salek <em>responsible</em>. But feel free to modify
     the file if you are humbly convinced your ideas are correct.
 */
+
 /* Copyright(c) Pawel Salek 2006. */
 
 #include <dirent.h>
@@ -55,6 +57,7 @@
 
 #ifdef USE_CHUNKS_AND_TASKS
 #include "chunks_and_tasks.h"
+#include "registration.h"
 #endif
 
 #include "atom_labels.h"
@@ -66,8 +69,8 @@
 #include "integrals_2el.h"
 #include "integrals_2el_explicit.h"
 #include "integrals_2el_boxed.h"
-#include "integrals_2el_exchange.h"
-#include "integrals_2el_coulomb.h"
+#include "integrals_2el_K.h"
+#include "integrals_2el_J.h"
 #include "integrals_general.h"
 #include "operator_matrix.h"
 #include "memorymanag.h"
@@ -156,9 +159,9 @@ static BasisInfoStruct* Basis_info = NULL;
     precision if sizeof(ergo_real) > sizeof(double) but this we can
     hopefully live with for input variables, can we? */
 #define KW(kl,vname, type, defval, desc)                          \
-  kl = variable_new_ ##type(kl, (#vname), (desc), (type), (defval))
+  kl = variable_new_ ##type(kl, (type), (#vname), (desc), (defval))
 #define variable_new_VAR_STRING variable_new
-#define variable_new_VAR_FLOAT(kl,n,h,t,v) variable_new(kl,n,h,t,double(v))
+#define variable_new_VAR_FLOAT(kl,t,n,h,v) variable_new(kl,t,n,h,double(v))
 #define variable_new_VAR_INT    variable_new
 #define variable_new_VAR_LIST   variable_new
 
@@ -167,18 +170,21 @@ static BasisInfoStruct* Basis_info = NULL;
 
     @param tail is a tail of the variable list, allowing easy variable
     list creation.
+    @param type is the variable type (string, int, or float).
     @param name is the variable name.
     @param description is a string with a few sentences describing what the variable is for.
-    @param type is the variable type (string, int, or float).
 */
 static struct variable*
-variable_new(struct variable* tail, const char *name, const char *description,
-             enum VarType type, ...)
+variable_new(struct variable* tail, enum VarType type, const char *name, const char *description, ...)
 {
   struct variable * v = ergo_new(1, struct variable);
   va_list ap;
 
-  va_start(ap, type);
+  /* The second argument to va_start is the name of the last argument
+     before the variable argument list, that is, the last argument of
+     which the calling function knows the type. In this case, that is
+     the variable called "description". */
+  va_start(ap, description);
   v->next = tail;
   v->name = name;
   v->description = description;
@@ -558,12 +564,12 @@ es_set_nthreads_string(const char *str)
 static void
 jkparams_set_from_vars(JK::Params& jkp)
 {
-  memset(&jkp, 0, sizeof(jkp));
+  JK::Params empty;
+  jkp = empty;
   jkp.threshold_J = var_get_realJK("threshold_2el_J");
   jkp.threshold_K = var_get_realJK("threshold_2el_K");
   jkp.multipole_threshold_factor = var_get_realJK("multipole_threshold_factor");
 #ifndef SKIP_UNOFFICIAL_INPUT_PARAMS
-  jkp.use_densfit_for_J          = var_get_intJK("use_densfit_for_J");
   jkp.use_differential_density   = var_get_intJK("use_differential_density");
 #endif
   jkp.use_fmm                    = var_get_intJK("use_fmm");
@@ -577,6 +583,7 @@ jkparams_set_from_vars(JK::Params& jkp)
   jkp.exchange_box_size         = var_get_realJK("exchange_box_size");
   jkp.noOfThreads_J             = var_get_intJK("threads_J");
   jkp.noOfThreads_K             = var_get_intJK("threads_K");
+  jkp.use_naive_fockmat_constr  = var_get_intJK("use_naive_fockmat_constr");
 }
 
 void
@@ -592,6 +599,9 @@ es_print_help()
          "run \"METHOD\", METHOD=HF or a DFT functional\n"
          "system \"CMD\"\n"
          "warranty\n"
+         "precision\n"
+         "list_dft_funcs\n"
+         "is_cht_used\n"
          "quit\n"
          "get_excited_state \"METHOD\" NO_OF_STATES\n"
          "get_polarisability \"METHOD\" \"[XYZ]\" FREQUENCY\n"
@@ -607,6 +617,24 @@ es_print_help_var(const struct variable *var)
   printf("%s: %s\n", var->name, var->description);
 }
 
+void
+es_print_list_dft_funcs()
+{
+  dftlistfuncs_();
+  dftlistfuncs_using_printf_();
+}
+
+void
+es_print_is_cht_used()
+{
+#ifdef USE_CHUNKS_AND_TASKS
+  const char * messageString = "chunks_and_tasks_is_used";
+#else
+  const char * messageString = "chunks_and_tasks_is_not_used";
+#endif
+  puts(messageString);
+}
+
 /** Print precision that was selected for building the program. */
 void
 es_print_precision()
@@ -615,6 +643,8 @@ es_print_precision()
   const char *precision = "single";
 #elif defined(PRECISION_LONG_DOUBLE)
   const char *precision = "long_double";
+#elif defined(PRECISION_QUAD_FLT128)
+  const char *precision = "quad_flt128";
 #else
   const char *precision = "double";
 #endif
@@ -682,6 +712,10 @@ es_run(const char *mode, int save_pot)
     int use_6_d_funcs = var_get_int("use_6_d_functions");
     Basis_info = new BasisInfoStruct(use_6_d_funcs);
     const int do_basis_normalization = 1;
+    // ELIAS NOTE 2019-04-28: Alexey Akimov asked about this, it seems
+    // like it would be a good idea to add an input option allowing
+    // the user to choose skip_sort_shells value (0 or 1).
+    // ELIAS FIXME TODO: add input option for skip_sort_shells
     const int skip_sort_shells = 0;
 
     if(ergo.Basis != NULL) {
@@ -749,9 +783,7 @@ es_run(const char *mode, int save_pot)
   scf.error_maxabs_for_diis = var_get_realSCF("error_maxabs_for_diis");
   scf.starting_guess_disturbance  = var_get_realSCF("starting_guess_disturbance");
   scf.purification_subspace_err_limit = var_get_realSCF("purification_subspace_err_limit");
-  scf.purification_eigvalue_err_limit = var_get_realSCF("purification_eigvalue_err_limit");
   scf.purification_with_acceleration = var_get_intSCF("purification_with_acceleration");
-  scf.use_new_stopping_criterion = var_get_intSCF("use_new_stopping_criterion");
   scf.puri_eig_acc_factor_for_guess = var_get_realSCF("puri_eig_acc_factor_for_guess");
   scf.gap_expected_lower_bound = var_get_realSCF("gap_expected_lower_bound");
   scf.shift_using_prev_density_matrix = var_get_realSCF("shift_using_prev_density_matrix");
@@ -760,9 +792,16 @@ es_run(const char *mode, int save_pot)
   scf.purification_stop_crit_norm  = mat::getNormType( var_get_stringSCF("purification_stop_crit_norm") ); 
   scf.break_on_energy_increase     = var_get_intSCF("break_on_energy_increase");
   scf.create_basis_func_coord_file = var_get_intSCF("create_basis_func_coord_file");
+  scf.use_prev_vector_as_initial_guess  = var_get_intSCF("use_prev_vector_as_initial_guess");
   scf.output_homo_and_lumo_eigenvectors = var_get_intSCF("output_homo_and_lumo_eigenvectors");
+  scf.number_of_occupied_eigenvectors = var_get_intSCF("number_of_occupied_eigenvectors");
+  scf.number_of_unoccupied_eigenvectors = var_get_intSCF("number_of_unoccupied_eigenvectors");
+  scf.go_back_X_iter_proj_method = var_get_intSCF("go_back_X_iter_proj_method");
+  scf.jump_over_X_iter_proj_method = var_get_intSCF("jump_over_X_iter_proj_method");  
   scf.eigensolver_accuracy        = var_get_realSCF("eigensolver_accuracy");
   scf.eigensolver_maxiter         = var_get_intSCF("eigensolver_maxiter");
+  scf.eigenvectors_method         = var_get_stringSCF("eigenvectors_method");
+  scf.purification_maxmul         = var_get_intSCF("purification_maxmul");
   scf.create_mtx_file_S           = var_get_intSCF("create_mtx_file_S");
   scf.create_mtx_file_H_core      = var_get_intSCF("create_mtx_file_H_core");
   scf.create_mtx_files_F          = var_get_intSCF("create_mtx_files_F");
@@ -780,14 +819,17 @@ es_run(const char *mode, int save_pot)
   scf.do_report_density_diff      = var_get_intSCF("do_report_density_diff");
   scf.no_of_impr_req_for_diis     = var_get_intSCF("no_of_impr_req_for_diis");
   scf.no_of_threads_for_V         = var_get_intSCF("no_of_threads_for_V");
+  scf.box_size_for_V_and_T        = var_get_realSCF("box_size_for_V_and_T");
   scf.output_density_at_every_step = var_get_intSCF("output_density_at_every_step");
   scf.save_final_potential        = var_get_intSCF("save_final_potential")
     || save_pot;
   scf.use_diagonalization         = var_get_intSCF("use_diagonalization");
   scf.use_diag_on_error           = var_get_intSCF("use_diag_on_error");
   scf.use_diag_on_error_guess     = var_get_intSCF("use_diag_on_error_guess");
+  scf.purification_ignore_failure    = var_get_intSCF("purification_ignore_failure");
   scf.store_all_eigenvalues_to_file = var_get_intSCF("store_all_eigenvalues_to_file");
   scf.output_mulliken_pop         = var_get_intSCF("output_mulliken_pop");
+  scf.output_expected_values_pos_operator = var_get_intSCF("output_expected_values_pos_operator");
   scf.output_density_images       = var_get_intSCF("output_density_images");
   scf.output_density_images_only  = var_get_intSCF("output_density_images_only");
   scf.output_density_images_boxwidth = var_get_realSCF("output_density_images_boxwidth");
@@ -795,13 +837,7 @@ es_run(const char *mode, int save_pot)
   scf.verify_gradient_fixeddens   = var_get_intSCF("verify_gradient_fixeddens");
 #ifndef SKIP_UNOFFICIAL_INPUT_PARAMS
   scf.do_f_thresh_verification    = var_get_intSCF("do_f_thresh_verification");
-  scf.do_comparison_to_simple_purification   = var_get_intSCF("do_comparison_to_simple_purification");
-  scf.do_puri_mmul_tests          = var_get_intSCF("do_puri_mmul_tests");
   scf.output_statistics_mfiles    = var_get_intSCF("output_statistics_mfiles");
-  scf.do_sparsity_investigation   = var_get_intSCF("do_sparsity_investigation");
-  scf.do_sparsity_investigation_reppuri   = var_get_intSCF("do_sparsity_investigation_reppuri");
-  scf.sparsity_plots_resolution_r = var_get_intSCF("sparsity_plots_resolution_r");
-  scf.sparsity_plots_resolution_m = var_get_intSCF("sparsity_plots_resolution_m");
   scf.do_acc_scan_J               = var_get_intSCF("do_acc_scan_J");
   scf.do_acc_scan_K               = var_get_intSCF("do_acc_scan_K");
   scf.do_acc_scan_Vxc             = var_get_intSCF("do_acc_scan_Vxc");
@@ -813,15 +849,20 @@ es_run(const char *mode, int save_pot)
   scf.no_of_core_electrons        = var_get_intSCF("no_of_core_electrons");
   scf.skip_H_core                 = var_get_intSCF("skip_H_core");
   scf.purification_create_m_files = var_get_intSCF("purification_create_m_files");
+  scf.purification_use_rand_perturbation_for_alleigsint = var_get_intSCF("purification_use_rand_perturbation_for_alleigsint");
+  #ifdef USE_CHUNKS_AND_TASKS
+    scf.cht_leavesSizeMax            = var_get_intSCF("cht_leavesSizeMax");
+    scf.cht_blocksize                = var_get_intSCF("cht_blocksize");
+  #endif
+  scf.purification_eigvalue_err_limit = var_get_realSCF("purification_eigvalue_err_limit");
   scf.create_checkpoints  = var_get_intSCF("create_checkpoints");
   scf.checkpoint_IDstr        = var_get_stringSCF("checkpoint_IDstr");
-  scf.eigenvectors_method         = var_get_stringSCF("eigenvectors_method");
+  scf.use_new_stopping_criterion = var_get_intSCF("use_new_stopping_criterion");
+  scf.try_eigv_on_next_iteration_if_fail = var_get_intSCF("try_eigv_on_next_iteration_if_fail");
+  scf.puri_compute_eigv_in_each_iteration = var_get_intSCF("puri_compute_eigv_in_each_iteration");
+  scf.run_shift_and_square_method_on_F = var_get_intSCF("run_shift_and_square_method_on_F");
+  scf.save_permuted_F_matrix_in_bin = var_get_intSCF("save_permuted_F_matrix_in_bin");
   scf.eigenvectors_iterative_method  = var_get_stringSCF("eigenvectors_iterative_method");
-  scf.purification_use_rand_perturbation_for_alleigsint = var_get_intSCF("purification_use_rand_perturbation_for_alleigsint");
-  scf.use_stochastic_orbs               = var_get_intSCF("use_stochastic_orbs");
-  scf.stochastic_orbs_no_of_vectors     = var_get_intSCF("stochastic_orbs_no_of_vectors");
-  scf.stochastic_orbs_use_unit_vectors  = var_get_intSCF("stochastic_orbs_use_unit_vectors");
-  scf.stochastic_orbs_rand_param        = var_get_intSCF("stochastic_orbs_rand_param");
 #endif
   scf.write_overlap_matrix        = var_get_intSCF("write_overlap_matrix");
   scf.use_simple_dense_H_core     = var_get_intSCF("use_simple_dense_H_core");
@@ -867,6 +908,8 @@ es_run(const char *mode, int save_pot)
     var_get_intMA("sparse_matrix_block_factor_3");
   ergo.matOptions.threads = var_get_intMA("threads");
   ergo.matOptions.parallelLevel = var_get_intMA("parallelLevel");
+  ergo.matOptions.use_allocator_manager =
+    var_get_intMA("use_allocator_manager");
   ergo.matOptions.no_of_buffers_per_allocator = var_get_intMA("no_of_buffers_per_allocator");
   ergo.matOptions.prepare(*Basis_info);
 
@@ -962,8 +1005,6 @@ es_run(const char *mode, int save_pot)
     ergo.extraChargesMolecule.replaceAtom(i, atom);
   }
 
-  BasisInfoStruct basisInfoDensFit(0); /* FIXME */
-
   int noOfElectrons = ergo.molecule.getNumberOfElectrons();
   try {
     if (noOfElectrons % 2 == 1 || 
@@ -973,7 +1014,6 @@ es_run(const char *mode, int save_pot)
       SCF_unrestricted SCF(ergo.molecule, 
 			   ergo.extraChargesMolecule,
 			   *Basis_info, 
-			   basisInfoDensFit, /* FIXME */
 			   *ergoIntegralInfo,
 			   initial_density_fname,
 			   ergo.jkOptions,
@@ -1047,7 +1087,6 @@ es_run(const char *mode, int save_pot)
       SCF_restricted SCF(ergo.molecule, 
 			 ergo.extraChargesMolecule,
 			 *Basis_info, 
-			 basisInfoDensFit, /* FIXME */
 			 *ergoIntegralInfo,
 			 initial_density_fname,
 			 ergo.jkOptions,
@@ -1170,34 +1209,28 @@ public:
 	do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "error in compute_J_by_boxes_nosymm");
 	return false;
       }
-
     if(CAM_params.alpha != 0.0 || CAM_params.computeRangeSeparatedExchange)
       {
-	do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "calling compute_K_by_boxes");
+	do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "ErgoE2Evaluator::transform(): calling compute_K_by_boxes");
 	int n = bi->noOfBasisFuncs;
 	ergo_real* K = new ergo_real[n*n];
 	memset(K, 0, n*n*sizeof(ergo_real));
 	int symmetryFlag = 0;
-
-	if(compute_K_by_boxes(*bi,
-			      *ergoIntegralInfo,
-			      CAM_params,
-			      ergo.jkOptions,
-			      K,
-			      NULL,
-			      dmat,
-			      NULL,
-			      symmetryFlag) != 0)
-	  {
-	    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "error in compute_K_by_boxes");
-	    return false;
-	  }
-	int i;
+	if(compute_K_by_boxes_dense(*bi,
+				    *ergoIntegralInfo,
+				    CAM_params,
+				    ergo.jkOptions,
+				    K,
+				    dmat,
+				    symmetryFlag) != 0) {
+	  do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "error in compute_K_by_boxes_dense.");
+	  return false;
+	}
 	ergo_real CAMhf_weight =
 	  CAM_params.computeRangeSeparatedExchange ? 1.0 : CAM_params.alpha;
-	for(i = 0; i < n*n; i++)
+	for(int i = 0; i < n*n; i++)
 	  fmat[i] += CAMhf_weight * K[i];
-	delete []K;
+	delete [] K;
       }
     res = true;
 #else
@@ -1314,10 +1347,10 @@ es_getexc(const char *mode, int no_exc)
     solver.computeMoments(dx, dy, dz);
     for(int i=0; i<no_exc; i++) {
       printf("Eigenvalue %2i: %15.9f Tran.Mom.: %15.9g\n", i+1,
-             (double)solver.getFreq(i), (double)sqrt(solver.getTransitionMoment2(i)));
+             (double)solver.getFreq(i), (double)sqrt((double)solver.getTransitionMoment2(i)));
       do_output(LOG_CAT_ERROR, LOG_AREA_MAIN,
                 "Eigenvalue %2i: %15.9f Tran.Mom.: %15.9g", i+1,
-                (double)solver.getFreq(i),  (double)sqrt(solver.getTransitionMoment2(i)));
+                (double)solver.getFreq(i),  (double)sqrt((double)solver.getTransitionMoment2(i)));
     }
   } catch(const char*s){
     do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "Error encountered: %s\n", s);
@@ -1490,13 +1523,13 @@ Ergo::registerInputVariables()
   KWJK(use_fmm,                      VAR_INT,"Use multipole method for Coulomb matrix construction. This also enables linear scaling HF-exchange matrix construction.");
 #ifndef SKIP_UNOFFICIAL_INPUT_PARAMS
   KWJK(use_differential_density,     VAR_INT,"Use \"differential density\" procedure to try to speed up Fock matrix construction.");
-  KWJK(use_densfit_for_J,            VAR_INT,"Use \"density fitting\" for Coulomb matrix construction.");
 #endif
   KW(KL,threshold_2el_J,             VAR_FLOAT,jkOptions.threshold_J,"Threshold value for Coulomb matrix construction.");
   KW(KL,threshold_2el_K,             VAR_FLOAT,jkOptions.threshold_K,"Threshold value for HF exchange matrix construction.");
   KW(KL,threshold_1el,               VAR_FLOAT, 1e-12,"Threshold value for one-electron (core) Hamiltonian matrix construction.");
   KW(KL,threads_K,                   VAR_INT,   defThreads,"Number of threads to use in Coulomb matrix construction.");
   KW(KL,threads_J,                   VAR_INT,   defThreads,"Number of threads to use in HF exchange matrix construction.");
+  KWJK(use_naive_fockmat_constr,     VAR_INT,"Use naive implementation of Fock matrix construction.");
   KWJK(multipole_threshold_factor,   VAR_FLOAT,"Factor to apply to threshold value in multipole part (far field) of Coulomb matrix construction.");
   KWJK(fmm_no_of_branches,           VAR_INT,"Number of branches to use in Coulomb matrix construction. If 0, default branch settings are used based on box size and max extent.");
   KWJK(fmm_branch_splitter_extent_5, VAR_FLOAT,"Extent splitter value 5 for branch division in Coulomb matrix construction.");
@@ -1540,6 +1573,7 @@ Ergo::registerInputVariables()
   KWMAT(sparse_matrix_block_factor_2, VAR_INT,"Block size factor determining block size at third lowest level.");
   KWMAT(sparse_matrix_block_factor_1, VAR_INT,"Block size factor determining block size at second lowest level: sparse_matrix_block_size * sparse_matrix_block_factor_1.");
   KWMAT(sparse_matrix_block_size,     VAR_INT,"Lowest level submatrix block size.");
+  KWMAT(use_allocator_manager,        VAR_INT,"Use allocator manager (value 1) or new operator ( value 0) for memory managment.");
   KWMAT(parallelLevel,                VAR_INT,"Level in matrix hierarchy for parallelization.");
   KWMAT(no_of_buffers_per_allocator,  VAR_INT,"Number of buffers per allocator to use in matrix memory allocation manager. A large value means that large chunks of memory will be allocated at a time.");
 #undef KWMAT
@@ -1562,21 +1596,12 @@ Ergo::registerInputVariables()
 #undef KWED
 #undef KL
 
+
 #define KL ergo.scf_params
 #define KWSCF(n,type,d) KW(ergo.scf_params,n,type,scfOptions.n,d)
 #ifndef SKIP_UNOFFICIAL_INPUT_PARAMS
   KWSCF(do_f_thresh_verification,     VAR_INT,"When truncating Fock matrix, verify that the error matrix norm is below the requested threshold.");
-  KWSCF(do_comparison_to_simple_purification,     VAR_INT,"In each density matrix construction, do a comparison to a simple purification scheme.");
-  KWSCF(do_puri_mmul_tests,           VAR_INT,"In each density matrix construction, do some extra purification test runs comparing different matrix-matrix multiplication and truncation approaches.");
   KWSCF(output_statistics_mfiles,     VAR_INT,"Output m-file with statistics (timings etc) each SCF iteration.");
-  KWSCF(do_sparsity_investigation,    VAR_INT,"Create distance-vs-magnitude plot files.");
-  KWSCF(do_sparsity_investigation_reppuri,    VAR_INT,"Perform purification 3 times using different norms, and create corresponding distance-vs-magnitude plot files.");
-  KWSCF(sparsity_plots_resolution_r,  VAR_INT,"Distance resolution parameter for sparsity plots.");
-  KWSCF(sparsity_plots_resolution_m,  VAR_INT,"Magnitude resolution parameter for sparsity plots.");
-  KWSCF(use_stochastic_orbs,              VAR_INT,"Use stochastic orbitals as in the Stochastic DFT paper [PRL 111, 106402 (2013)].");
-  KWSCF(stochastic_orbs_no_of_vectors,    VAR_INT,"Number of stochastic orbitals to use, if use_stochastic_orbs is turned on.");
-  KWSCF(stochastic_orbs_use_unit_vectors, VAR_INT,"If use_stochastic_orbs is turned on, use unit vectors instead of random vectors.");
-  KWSCF(stochastic_orbs_rand_param,       VAR_INT,"If use_stochastic_orbs is turned on, this value affects the way random numbers are generated. Setting this parameter to 2 means that a more elaborate, hopefully better, method is used to generate random numbers.");
   KWSCF(write_guess_density_only,     VAR_INT,"Only generate starting guess density, write it to file and exit.");
   KWSCF(compute_core_density,     VAR_INT,"Each time the density matrix is computed, also compute a 'core density matrix' using only the core electrons.");
   KWSCF(no_of_core_electrons,     VAR_INT,"If compute_core_density is set, use this number of core electrons when coputing the 'core density matrix'.");
@@ -1588,20 +1613,31 @@ Ergo::registerInputVariables()
   KWSCF(scan_start_thresh,            VAR_FLOAT,"Scan start threshold value to use for \"accuracy scans\" of J, K, Vxc matrices.");
   KWSCF(scan_step_factor,             VAR_FLOAT,"Step factor to use for \"accuracy scans\" of J, K, Vxc matrices.");
   KWSCF(purification_create_m_files,  VAR_INT,"Create m files with information about purification."),
-  KWSCF(purification_use_rand_perturbation_for_alleigsint, VAR_INT,"Use random perturbation to attempt getting easier Lanczos convergence when determining min/max eigenvalues of Fock matrix before purification."),
+  KWSCF(purification_use_rand_perturbation_for_alleigsint, VAR_INT,"Use random perturbation to attempt getting easier Lanczos convergence when determining min/max eigenvalues of Fock matrix before purification.");
   KW(ergo.scf_params, checkpoint_IDstr, VAR_STRING, "", "ID for file with saved state (parameters in the GetDensFromFock class) before recursive expansion on every SCF cycle.");
   KWSCF(create_checkpoints,           VAR_INT,"Save state (parameters in the GetDensFromFock class) before recursive expansion on every SCF cycle.");
-  KW(ergo.scf_params, eigenvectors_method, VAR_STRING, "square", "Method for computation of HOMO and LUMO molecular orbital coefficients. Value: square or projection");
+  KWSCF(purification_eigvalue_err_limit,        VAR_FLOAT,"Requested accuracy in eigenvalues of the density matrix resulting from purification.");
+  KWSCF(use_new_stopping_criterion,    VAR_INT,"Use new parameterless stopping criterion.");
+  KWSCF(try_eigv_on_next_iteration_if_fail,  VAR_INT,"If fail to compute eigenvector in iteration i, try to compute it in iteration i+1.");
+  KWSCF(puri_compute_eigv_in_each_iteration, VAR_INT,"Compute eigenvectors in each iteration of the recursive expansion.");
+  KWSCF(run_shift_and_square_method_on_F,    VAR_INT,"Run shift_and_square method to get eigenvectors of the matrix F for various shifts.");
+  KWSCF(save_permuted_F_matrix_in_bin,    VAR_INT,"Save sparse matrix F into binary file in the current permutation of rows and columns.");
+  KWSCF(store_all_eigenvalues_to_file, VAR_INT,"Store eigenvalues of the Hamiltonian matrix when using diagonalization (use_diagonalization flag must be 1).");
+#ifdef USE_CHUNKS_AND_TASKS
+  KWSCF(cht_leavesSizeMax,     VAR_INT, "Size of the leave matrices in CHTMatrix");
+  KWSCF(cht_blocksize,         VAR_INT, "Size of the block matrices in CHTMatrix if block sparse leave matrix is used");
+#endif
   KW(ergo.scf_params, eigenvectors_iterative_method, VAR_STRING, "lanczos", "Iterative method for computation of HOMO and LUMO molecular orbital coefficients. Value: power or lanczos");
 #endif
   KWSCF(use_simple_dense_H_core,      VAR_INT,"Use simple dense matrix computation of 1-electron (core) Hamiltonian matrix.");
   KWSCF(use_diis_always,              VAR_INT,"Always use DIIS even if the energy goes up.");
   KWSCF(write_overlap_matrix,         VAR_INT,"Write overlap matrix to file.");
+  KWSCF(purification_ignore_failure,  VAR_INT,"Ignore failure of the purification.");
   KWSCF(use_diagonalization,          VAR_INT,"Use diagonalization instead of purification.");
   KWSCF(use_diag_on_error,            VAR_INT,"Use diagonalization if purification fails.");
   KWSCF(use_diag_on_error_guess,      VAR_INT,"Use diagonalization if purification fails during starting guess projection.");    
-  KWSCF(store_all_eigenvalues_to_file, VAR_INT,"Store eigenvalues of the Hamiltonian matrix when using diagonalization (use_diagonalization flag must be 1).");
   KWSCF(output_mulliken_pop,          VAR_INT,"Output Mulliken population analysis stuff (atomic charges and atomic spin densities) after SCF finished.");
+  KWSCF(output_expected_values_pos_operator, VAR_INT,"Output expected value and standard deviation of position operator for eigenvectors after SCF finished (flag for eigenvectors must be set).");
   KWSCF(output_density_images,        VAR_INT,"Output density image files (including Gabedit gcube density files) after SCF finished.");
   KWSCF(output_density_images_only,   VAR_INT,"Output density image files (including Gabedit gcube density files) directly, without any SCF procedure, using the given starting guess density.");
   KWSCF(output_density_images_boxwidth, VAR_FLOAT, "Box width to use for density image files (including Gabedit gcube density files).");
@@ -1613,6 +1649,7 @@ Ergo::registerInputVariables()
   KWSCF(save_final_potential,         VAR_INT,"Save final Fock/Kohn-Sham matrix to file after SCF finished.");
   KWSCF(output_density_at_every_step, VAR_INT,"Write current density matrix to file in each iteration.");
   KW(KL,no_of_threads_for_V,          VAR_INT, defThreads,"Number of threads to use in V (1el electron-nuclear interaction) matrix construction.");
+  KWSCF(box_size_for_V_and_T,         VAR_FLOAT, "Box size to use during construction of V and T matrices (parts of H_core matrix).");
   KWSCF(no_of_impr_req_for_diis,      VAR_INT,"Number of consecutive energy improvements required before attempting to use DIIS.");
   KWSCF(min_number_of_iterations,     VAR_INT,"Minimum number of SCF iterations.");
   KWSCF(max_restart_count,            VAR_INT,"Max number of times to attempt to restart SCF procedure when it appears stuck.");
@@ -1624,9 +1661,7 @@ Ergo::registerInputVariables()
   KWSCF(force_restricted,             VAR_INT,"\"force_restricted\" parameter used for restricted open-shell calculations.");
   KWSCF(error_maxabs_for_diis,        VAR_FLOAT,"Do not use DIIS if error measure is larger than this value.");
   KWSCF(purification_subspace_err_limit,        VAR_FLOAT,"Requested accuracy in the occupied invariant subspace of the density matrix resulting from purification as measured by the sinus of the largest canonical angle between the exact and approximate subspaces.");
-  KWSCF(purification_eigvalue_err_limit,        VAR_FLOAT,"Requested accuracy in eigenvalues of the density matrix resulting from purification.");
   KWSCF(purification_with_acceleration, VAR_INT,"Use acceleration when doing purification.");
-  KWSCF(use_new_stopping_criterion,    VAR_INT,"Use new parameterless stopping criterion.");
   KWSCF(puri_eig_acc_factor_for_guess, VAR_FLOAT,"When doing purification of starting guess density, use the normal eigvalue_err_limit multiplied by this factor.");
   KWSCF(gap_expected_lower_bound,        VAR_FLOAT,"Expected lower bound of HOMO-LUMO gap, used in early SCF iterations when accurate gap info is not yet available.");
   KWSCF(shift_using_prev_density_matrix, VAR_FLOAT, "As input to density matrix construction, Fock/KS matrix modified by subtracting previous density matrix times this factor. This is a way to artificially increase the HOMO-LUMO gap. A.k.a. level shifting.");
@@ -1641,18 +1676,23 @@ Ergo::registerInputVariables()
   KWSCF(convergence_threshold,        VAR_FLOAT,"Convergence threshold for SCF procedure; terminate SCF procedure when FDS-SDF error measure is below threshold.");
   KWSCF(break_on_energy_increase,     VAR_INT,"Break SCF procedure if energy increases.");
   KWSCF(create_basis_func_coord_file, VAR_INT,"Write text file with coordinates of basis functions.");
+  KW(ergo.scf_params, go_back_X_iter_proj_method, VAR_INT, 10, "Parameter used in the projection method for computing eigenvectors. Defines the iteration to start computation of the eigenvectors.");
+  KW(ergo.scf_params, jump_over_X_iter_proj_method, VAR_INT, 3, "Parameter used in the projection method for computing eigenvectors. Defines how many iterations will be skipped before the next attempt if some eigenvectors are not computed.");
+  KW(ergo.scf_params, number_of_occupied_eigenvectors, VAR_INT, 1, "Number of occupied eigenvectors to compute. Use projection (purify-shift-and-project) method. Input parameter scf.output_homo_and_lumo_eigenvectors should be set to 1 and scf.eigenvectors_method should be set to \"projection\"");
+  KW(ergo.scf_params, number_of_unoccupied_eigenvectors, VAR_INT, 1, "Number of unoccupied eigenvectors to compute. Use projection (purify-shift-and-project) method. Input parameter scf.output_homo_and_lumo_eigenvectors should be set to 1 and scf.eigenvectors_method should be set to \"projection\" ");
+  KW(ergo.scf_params, eigenvectors_method, VAR_STRING, "square", "Method for computation of HOMO and LUMO molecular orbital coefficients. Possible values are square (for purify-shift-and-square) and projection (for purify-shift-and-project)");
+  KWSCF(use_prev_vector_as_initial_guess,    VAR_INT,"Use eigenvector computed in previous SCF cycle as an initial guess for the next SCF cycle if possible.");
   KWSCF(output_homo_and_lumo_eigenvectors, VAR_INT,"Output HOMO and LUMO molecular orbital coefficients.");
   KWSCF(eigensolver_accuracy,         VAR_FLOAT, "The accuracy for the eigenvalue problem solver.");
   KWSCF(eigensolver_maxiter,          VAR_INT, "The maximum number of iterations for the eigenvalue problem solver.");
+  KWSCF(purification_maxmul,          VAR_INT, "The maximum number of iterations in the recursive expansion.");
   KWSCF(create_mtx_file_S,            VAR_INT,"Write overlap matrix to file in matrix market format (mtx).");
   KWSCF(create_mtx_file_H_core,       VAR_INT,"Write core Hamiltonian matrix to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_F,           VAR_INT,"Write effective Hamiltonian matrices to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_D,           VAR_INT,"Write density matrices to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_dipole,      VAR_INT,"Write dipole matrices to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_S_and_quit,  VAR_INT,"Write overlap matrix to file in matrix market format (mtx) using different basis function orderings, and then quit.");
-
   KW(ergo.scf_params,calculation_identifier,   VAR_STRING, "N/A","String to identify the calculation, used for example in mtx-files.");
-
   KWSCF(create_2el_integral_m_file,   VAR_INT,"Create m-file with rank-4 tensor containing the values of all 2-electron integrals. Very large; O(N^4); only use for small cases.");
 
 #undef KWSCF
@@ -1712,6 +1752,7 @@ extern "C" void* yy_scan_string(const char *str);
 extern "C" void* yy_create_buffer(FILE *f, int sz);
 extern "C" void yy_switch_to_buffer(void *);
 extern "C" void yy_delete_buffer(void *);
+extern "C" void yylex_destroy(void);
 extern FILE *yyin; /**< file used by the lex-generated parser.*/
 static void
 ergo_parse_file(FILE *inputFile)
@@ -2016,7 +2057,10 @@ main(int argc, char *argv[])
     do_output_time(LOG_CAT_ERROR, LOG_AREA_MAIN, "Time of exception: ");
     do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "=============================================================");
   }
-  
+
+  // Call yylex_destroy() here to avoid memory leaks due to buffers that may remain after e.g. previous yy_switch_to_buffer() calls.
+  yylex_destroy();
+
   return 0;
 }
 /* ===================================================================
