@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file basis_func_extent.cc
+
+    @brief Code for determining extent of basis functions, for
+    2-electron integral evaluation.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <memory.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -74,8 +83,8 @@ compute_extent_for_all_basis_funcs_core(const BasisInfoStruct & basisInfo,
 					ergo_real M,
 					ergo_real maxAbsDensMatElement)
 {
-  ergo_real twotopow1o4 = std::pow(2, 0.25);
-  ergo_real pitopow5o4 = std::pow(pi, 1.25);
+  ergo_real twotopow1o4 = template_blas_pow((ergo_real)2, (ergo_real)0.25);
+  ergo_real pitopow5o4 = template_blas_pow((ergo_real)pi, (ergo_real)1.25);
   
   int n = basisInfo.noOfBasisFuncs;
   for(int i = 0; i < n; i++)
@@ -89,7 +98,7 @@ compute_extent_for_all_basis_funcs_core(const BasisInfoStruct & basisInfo,
 	{
 	  DistributionSpecStruct* prim = &basisInfo.simplePrimitiveList[start + j];
 	  ergo_real currExponent = prim->exponent;
-	  ergo_real currAbsCoeff = std::fabs(prim->coeff);
+	  ergo_real currAbsCoeff = template_blas_fabs(prim->coeff);
 	  ergo_real a = currExponent;
 	  ergo_real c_a = currAbsCoeff;
 	  // now go through all available exponents
@@ -99,8 +108,8 @@ compute_extent_for_all_basis_funcs_core(const BasisInfoStruct & basisInfo,
 	      ergo_real c_b = exponentList.list[ii].maxAbsCoeff;
 	      if(c_b > 0)
 		{
-		  ergo_real A = M * twotopow1o4 * pitopow5o4 * c_a * c_b * std::pow(a+b, (ergo_real)-1.25);
-		  ergo_real R = std::sqrt( ((a + b) / (a * b)) * std::log(maxAbsDensMatElement * A / threshold));
+		  ergo_real A = M * twotopow1o4 * pitopow5o4 * c_a * c_b * template_blas_pow(a+b, (ergo_real)-1.25);
+		  ergo_real R = template_blas_sqrt( ((a + b) / (a * b)) * template_blas_log(maxAbsDensMatElement * A / threshold));
 		  if(R > largestExtentSoFar)
 		    largestExtentSoFar = R;
 		}
