@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file basis_func_extent_1el.cc
+
+    @brief Code for determining extent of basis functions, for
+    1-electron integral evaluation.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
@@ -61,7 +70,7 @@ compute_extent_for_all_basis_funcs_1el(const BasisInfoStruct & basisInfo,
 	{
 	  DistributionSpecStruct* prim = &basisInfo.simplePrimitiveList[start + j];
 	  ergo_real currExponent = prim->exponent;
-	  ergo_real currAbsCoeff = std::fabs(prim->coeff);
+	  ergo_real currAbsCoeff = template_blas_fabs(prim->coeff);
 	  ergo_real a = currExponent;
 	  ergo_real c_a = currAbsCoeff;
 	  // now go through all available exponents
@@ -72,10 +81,10 @@ compute_extent_for_all_basis_funcs_1el(const BasisInfoStruct & basisInfo,
 	      if(c_b > 0)
 		{
 		  // This extent definition was derived from the estimate of the largest possible contribution to V given by (2 * pi * coeff / exponent).
-		  ergo_real R2 = -1 * ((a+b)/(a*b)) * std::log(threshold*(a+b)/(2*pi*c_a*c_b*maxCharge));
+		  ergo_real R2 = -1 * ((a+b)/(a*b)) * template_blas_log(threshold*(a+b)/(2*pi*c_a*c_b*maxCharge));
 		  if(R2 < 0)
 		    continue;
-		  ergo_real R = std::sqrt(R2);
+		  ergo_real R = template_blas_sqrt(R2);
 		  if(R > largestExtentSoFar)
 		    largestExtentSoFar = R;
 		}
