@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,20 +17,28 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file basis_func_pair_list.cc
+
+    @brief Functions for setting up lists of non-negligible basis
+    function pairs, for 2-electron integrals.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <cstdio>
 #include <cstdlib>
 #include <cmath>
 
-
 #include "basis_func_pair_list.h"
 #include "basis_func_extent.h"
 #include "output.h"
@@ -43,7 +51,6 @@
 #include "box_system.h"
 
 
-
 static int
 get_maxLimitingFactor(const BasisInfoStruct & basisInfo,
 		      const IntegralInfo & integralInfo,
@@ -78,7 +85,7 @@ get_maxLimitingFactor(const BasisInfoStruct & basisInfo,
       ergo_real dx = basisInfo.basisFuncList[i].centerCoords[0] - basisInfo.basisFuncList[j].centerCoords[0];
       ergo_real dy = basisInfo.basisFuncList[i].centerCoords[1] - basisInfo.basisFuncList[j].centerCoords[1];
       ergo_real dz = basisInfo.basisFuncList[i].centerCoords[2] - basisInfo.basisFuncList[j].centerCoords[2];
-      ergo_real distance = std::sqrt(dx*dx + dy*dy + dz*dz);
+      ergo_real distance = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
       if(distance > basisFuncExtentList[i] + basisFuncExtentList[j])
 	continue;
       // There may be some overlap between these two basis functions.
@@ -96,7 +103,7 @@ get_maxLimitingFactor(const BasisInfoStruct & basisInfo,
 	return -1;
       }
       for(int k = 0; k < n_psi; k++) {
-	ergo_real limitingFactor = std::sqrt(integrator.do_2e_integral(&psi_list[k]));
+	ergo_real limitingFactor = template_blas_sqrt(integrator.do_2e_integral(&psi_list[k]));
 	if(limitingFactor > maxLimitingFactor)
 	  maxLimitingFactor = limitingFactor;
       } // END FOR k
@@ -186,7 +193,7 @@ get_basis_func_pair_list_2el(const BasisInfoStruct & basisInfo,
 	ergo_real dx = basisInfo.basisFuncList[i].centerCoords[0] - basisInfo.basisFuncList[j].centerCoords[0];
 	ergo_real dy = basisInfo.basisFuncList[i].centerCoords[1] - basisInfo.basisFuncList[j].centerCoords[1];
 	ergo_real dz = basisInfo.basisFuncList[i].centerCoords[2] - basisInfo.basisFuncList[j].centerCoords[2];
-	ergo_real distance = std::sqrt(dx*dx + dy*dy + dz*dz);
+	ergo_real distance = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
 	if(distance > basisFuncExtentList[i] + basisFuncExtentList[j])
 	  continue;
 	// There may be some overlap between these two basis functions.
@@ -206,7 +213,7 @@ get_basis_func_pair_list_2el(const BasisInfoStruct & basisInfo,
 	  return -1;
 	}
 	for(int k = 0; k < n_psi; k++) {
-	  ergo_real limitingFactor = std::sqrt(integrator.do_2e_integral(&psi_list[k]));
+	  ergo_real limitingFactor = template_blas_sqrt(integrator.do_2e_integral(&psi_list[k]));
 	  if(limitingFactor*maxLimitingFactor*maxDensityMatrixElement > threshold) {
 	    // This product distr is large enough.
 	    currProductLargeEnough = 1;
