@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file basis_func_pair_list_1el.h
+
+    @brief Functions for setting up lists of non-negligible basis
+    function pairs, for 1-electron integrals.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef BASIS_FUNC_PAIR_LIST_1EL_HEADER
 #define BASIS_FUNC_PAIR_LIST_1EL_HEADER
 
@@ -42,11 +51,19 @@ typedef struct
 } basis_func_index_pair_struct_1el;
 
 int
-get_basis_func_pair_list_1el(const BasisInfoStruct & basisInfo,
-			     ergo_real threshold,
-			     ergo_real maxCharge,
-			     basis_func_index_pair_struct_1el* result_basisFuncPairList,
-			     int resultMaxCount);
+get_basis_func_pair_list_1el_for_V(const BasisInfoStruct & basisInfo,
+				   ergo_real threshold,
+				   ergo_real boxSize,
+				   ergo_real maxCharge,
+				   basis_func_index_pair_struct_1el* result_basisFuncPairList,
+				   int resultMaxCount);
+
+int
+get_basis_func_pair_list_simple(const BasisInfoStruct & basisInfo,
+				ergo_real threshold,
+				ergo_real boxSize,
+				basis_func_index_pair_struct_1el* result_basisFuncPairList,
+				int resultMaxCount);
 
 
 #endif
