@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,30 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file box_system.cc
+
+    @brief BoxSystem class representing a hierarchical data structure
+    of boxes in 3D space (an oct-tree).
+
+    The idea is that you have a list of items at different points in space,
+    and you want a hierarchical system of boxes containing those items.
+
+    You give a list of items, and the function create_box_system will
+    create a system of boxes for you.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <cmath>
 #include <stdlib.h>
 #include <vector>
@@ -33,16 +48,7 @@
 #include "output.h"
 #include "memorymanag.h"
 #include "utilities.h"
-
-/** @file box_system.cc
-
-The idea is that you have a list of items at different points in space,
-and you want a hierarchical system of boxes containing those items.
-
-You give a list of items, and the function create_box_system will
-create a system of boxes for you.
-*/
-
+#include "mat_gblas.h"
 
 
 BoxSystem::BoxSystem()
@@ -323,7 +329,7 @@ BoxSystem::create_box_system(box_item_struct* itemList,
 
 
 
-static ergo_real 
+ergo_real
 get_min_distance_from_point_to_box(const ergo_real* boxCenterCoords,
 				   ergo_real halfwidth,
 				   const ergo_real* point)
@@ -331,7 +337,7 @@ get_min_distance_from_point_to_box(const ergo_real* boxCenterCoords,
   ergo_real dxList[3];
   for(int k = 0; k < 3; k++)
     {
-      ergo_real dx = std::fabs(boxCenterCoords[k] - point[k]);
+      ergo_real dx = template_blas_fabs(boxCenterCoords[k] - point[k]);
       if(dx > halfwidth)
 	dxList[k] = dx - halfwidth;
       else
@@ -340,7 +346,7 @@ get_min_distance_from_point_to_box(const ergo_real* boxCenterCoords,
   ergo_real sum = 0;
   for(int k = 0; k < 3; k++)
     sum += dxList[k] * dxList[k];
-  return std::sqrt(sum);
+  return template_blas_sqrt(sum);
 }
 
 int BoxSystem::get_items_near_point_recursive(const box_item_struct* itemList,
@@ -370,7 +376,7 @@ int BoxSystem::get_items_near_point_recursive(const box_item_struct* itemList,
 	      ergo_real d = coords[coordNo] - currItem->centerCoords[coordNo];
 	      sum += d*d;
 	    }
-	  ergo_real currDist = std::sqrt(sum);
+	  ergo_real currDist = template_blas_sqrt(sum);
 	  if(currDist < distance)
 	    {
 	      // Add to result.
