@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file exponent_list.cc
+
+    @brief ExponentList class for keeping track of a set of unique
+    exponents in Gaussian functions, together with a max coefficient
+    value for each exponent.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include "exponent_list.h"
 #include "output.h"
 
@@ -44,12 +54,12 @@ int ExponentList::get_list_of_available_exponents(const BasisInfoStruct & basisI
 	{
 	  DistributionSpecStruct* prim = &basisInfo.simplePrimitiveList[start + j];
 	  ergo_real currExponent = prim->exponent;
-	  ergo_real currAbsCoeff = std::fabs(prim->coeff);
+	  ergo_real currAbsCoeff = template_blas_fabs(prim->coeff);
 	  // now go through list to check if we already have this exponent.
 	  int foundIndex = -1;
 	  for(int k = 0; k < count; k++)
 	    {
-	      ergo_real absDiff = std::fabs(list[k].exponent - currExponent);
+	      ergo_real absDiff = template_blas_fabs(list[k].exponent - currExponent);
 	      if(absDiff < CONST_EXPONENT_DIFF_TOLERANCE)
 		{
 		  foundIndex = k;
