@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file hermite_conversion_prep.cc
+
+    @brief Code for conversion between integrals computed for Hermite
+    Gaussians and Cartesian Gaussians, using saved information from a
+    preparatory step that can be done once and re-used many times.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include "hermite_conversion_prep.h"
 #include <cmath>
 #include <memory.h>
@@ -34,6 +44,7 @@
 #include "hermite_conversion_symb.h"
 #include "monomial_info.h"
 #include "utilities.h"
+#include "mat_gblas.h"
 
 #if BASIS_FUNC_POLY_MAX_DEGREE<6
 const int MAX_NO_OF_CONTRIBS = 1000000;
@@ -63,7 +74,7 @@ void hermite_conversion_info_struct::init(const monomial_info_struct & monomial_
       for(int j = 0; j < nMon1; j++) {
 	for(int k = 0; k < nMon1; k++) {
 	  int idx = j*nMon1+k;
-	  if(std::fabs(list[idx].coeff) > 1e-5) {
+	  if(template_blas_fabs(list[idx].coeff) > 1e-5) {
 	    for(int i = 0; i < nMon2; i++) {
 	      assert(count < MAX_NO_OF_CONTRIBS);
 	      currlist[count].destIndex = j*nMon2+i;
@@ -92,7 +103,7 @@ void hermite_conversion_info_struct::init(const monomial_info_struct & monomial_
       for(int i = 0; i < nMon2; i++) {
 	for(int k = 0; k < nMon2; k++) {
 	  int idx = i*nMon2+k;
-	  if(std::fabs(list[idx].coeff) > 1e-5) {
+	  if(template_blas_fabs(list[idx].coeff) > 1e-5) {
 	    for(int j = 0; j < nMon1; j++) {
 	      assert(count < MAX_NO_OF_CONTRIBS);
 	      currlist[count].destIndex = j*nMon2+i;
@@ -119,7 +130,7 @@ void hermite_conversion_info_struct::init(const monomial_info_struct & monomial_
     for(int j = 0; j < nMon1; j++) {
       for(int k = 0; k < nMon1; k++) {
 	int idx = j*nMon1+k;
-	if(std::fabs(list[idx].coeff) > 1e-5) {
+	if(template_blas_fabs(list[idx].coeff) > 1e-5) {
 	  assert(count < MAX_NO_OF_CONTRIBS);
 	  currlist_elements[count].idx_j = j;
 	  currlist_elements[count].idx_k = k;
@@ -143,7 +154,7 @@ void hermite_conversion_info_struct::init(const monomial_info_struct & monomial_
     for(int j = 0; j < nMon1; j++) {
       for(int k = 0; k < nMon1; k++) {
 	int idx = k*nMon1+j; // reverse meaning of j k here compared to "right" case
-	if(std::fabs(list[idx].coeff) > 1e-5) {
+	if(template_blas_fabs(list[idx].coeff) > 1e-5) {
 	  assert(count < MAX_NO_OF_CONTRIBS);
 	  currlist_elements[count].idx_j = j;
 	  currlist_elements[count].idx_k = k;
@@ -207,6 +218,7 @@ int hermite_conversion_info_struct::multiply_by_hermite_conversion_matrix_from_r
 {
   int noOfContribs = counters_right[n1max][n2max];
   hermite_conversion_contrib_struct* list = list_right[n1max][n2max];
+  assert(list != NULL);
   
   int nMon1 = monomial_info.no_of_monomials_list[n1max];
   int nMon2 = monomial_info.no_of_monomials_list[n2max];
@@ -236,6 +248,7 @@ int hermite_conversion_info_struct::multiply_by_hermite_conversion_matrix_from_l
 {
   int noOfContribs = counters_left[n1max][n2max];
   hermite_conversion_contrib_struct* list = list_left[n1max][n2max];
+  assert(list != NULL);
   
   int nMon1 = monomial_info.no_of_monomials_list[n1max];
   int nMon2 = monomial_info.no_of_monomials_list[n2max];
@@ -262,6 +275,7 @@ int hermite_conversion_info_struct::get_hermite_conversion_matrix_right(const mo
 									ergo_real* result) const {
   int noOfContribs = counters_right_simple[nmax];
   hermite_conversion_element_struct* list = list_right_simple[nmax];  
+  assert(list != NULL);
   int nMon1 = monomial_info.no_of_monomials_list[nmax];
   int Ntot = 2 * nmax;
   ergo_real apowlist[Ntot+1];
@@ -284,6 +298,7 @@ int hermite_conversion_info_struct::get_hermite_conversion_matrix_right_sparse(c
 									       i_j_val_struct* result) const {
   int noOfContribs = counters_right_simple[nmax];
   hermite_conversion_element_struct* list = list_right_simple[nmax];  
+  assert(list != NULL);
   int Ntot = 2 * nmax;
   ergo_real apowlist[Ntot+1];
   apowlist[0] = 1;
@@ -308,6 +323,7 @@ int hermite_conversion_info_struct::get_hermite_conversion_matrix_left(const mon
 								       ergo_real* result) const {
   int noOfContribs = counters_left_simple[nmax];
   hermite_conversion_element_struct* list = list_left_simple[nmax];  
+  assert(list != NULL);
   int nMon1 = monomial_info.no_of_monomials_list[nmax];
   int Ntot = 2 * nmax;
   ergo_real apowlist[Ntot+1];
@@ -329,7 +345,7 @@ int hermite_conversion_info_struct::get_hermite_conversion_matrix_left(const mon
 hermite_conversion_info_struct::hermite_conversion_info_struct(const hermite_conversion_info_struct & other) {
   // ELIAS NOTE 2015-06-17: we start by calling clear_lists to make sure there are no pointers to garbage etc.
   // There was a segfault problem when object was deleted because the list_left_simple pointer was not allocated and not null,
-  // calling clear_lists() here should avoid that, although there is probably still something wrong, list_left_simple etc should probably be set.
+  // calling clear_lists() here should avoid that.
   clear_lists();
   memcpy(counters_right, other.counters_right, sizeof(counters_right));
   memcpy(counters_left, other.counters_left, sizeof(counters_left));
@@ -341,13 +357,22 @@ hermite_conversion_info_struct::hermite_conversion_info_struct(const hermite_con
       list_left[n1][n2] = new hermite_conversion_contrib_struct[counters_left[n1][n2]];
       memcpy(list_left[n1][n2], other.list_left[n1][n2], counters_left[n1][n2]*sizeof(hermite_conversion_contrib_struct));
     }
+  // Take care of list_right_simple and list_left_simple
+  memcpy(counters_right_simple, other.counters_right_simple, sizeof(counters_right_simple));
+  memcpy(counters_left_simple, other.counters_left_simple, sizeof(counters_left_simple));
+  for(int n1 = 0; n1 <= nmax; n1++) {
+    list_right_simple[n1] = new hermite_conversion_element_struct[counters_right_simple[n1]];
+    memcpy(list_right_simple[n1], other.list_right_simple[n1], counters_right_simple[n1]*sizeof(hermite_conversion_element_struct));
+    list_left_simple[n1] = new hermite_conversion_element_struct[counters_left_simple[n1]];
+    memcpy(list_left_simple[n1], other.list_left_simple[n1], counters_left_simple[n1]*sizeof(hermite_conversion_element_struct));
+  }
 }
 
 /** Function needed for Chunks&Tasks usage. */
 void hermite_conversion_info_struct::write_to_buffer ( char * dataBuffer, size_t const bufferSize ) const {
   char* p = dataBuffer;
   if(bufferSize < get_size())
-    throw std::runtime_error("Error: bufferSize too small.");
+    throw std::runtime_error("Error in hermite_conversion_info_struct::write_to_buffer: bufferSize too small.");
   // counters_right
   memcpy(p, counters_right, sizeof(counters_right));
   p += sizeof(counters_right);
@@ -363,6 +388,20 @@ void hermite_conversion_info_struct::write_to_buffer ( char * dataBuffer, size_t
       memcpy(p, list_left [n1][n2], counters_left [n1][n2]*sizeof(hermite_conversion_contrib_struct));
       p += counters_left  [n1][n2]*sizeof(hermite_conversion_contrib_struct);
     }
+  // counters_right_simple
+  memcpy(p, counters_right_simple, sizeof(counters_right_simple));
+  p += sizeof(counters_right_simple);
+  // counters_left_simple
+  memcpy(p, counters_left_simple, sizeof(counters_left_simple));
+  p += sizeof(counters_left_simple);
+  // list_right_simple and list_left_simple
+  for(int n1 = 0; n1 <= nmax; n1++) {
+    memcpy(p, list_right_simple[n1], counters_right_simple[n1]*sizeof(hermite_conversion_element_struct));
+    p += counters_right_simple[n1]*sizeof(hermite_conversion_element_struct);
+    memcpy(p, list_left_simple[n1], counters_left_simple[n1]*sizeof(hermite_conversion_element_struct));
+    p += counters_left_simple[n1]*sizeof(hermite_conversion_element_struct);
+  }
+  assert((size_t)(p - dataBuffer) == get_size());
 }
 
 /** Function needed for Chunks&Tasks usage. */
@@ -374,7 +413,15 @@ size_t hermite_conversion_info_struct::get_size() const {
       size += counters_right[n1][n2]*sizeof(hermite_conversion_contrib_struct);
       size += counters_left [n1][n2]*sizeof(hermite_conversion_contrib_struct);
     }
-  return size + sizeof(counters_right) + sizeof(counters_left);
+  for(int n1 = 0; n1 <= nmax; n1++) {
+    size += counters_right_simple[n1]*sizeof(hermite_conversion_contrib_struct);
+    size += counters_left_simple[n1]*sizeof(hermite_conversion_contrib_struct);
+  }
+  size += sizeof(counters_right);
+  size += sizeof(counters_left);
+  size += sizeof(counters_right_simple);
+  size += sizeof(counters_left_simple);
+  return size;
 }
 
 /** Function needed for Chunks&Tasks usage. */
@@ -397,6 +444,22 @@ void hermite_conversion_info_struct::assign_from_buffer ( char const * dataBuffe
       memcpy(list_left[n1][n2], p, counters_left[n1][n2]*sizeof(hermite_conversion_contrib_struct));
       p += counters_left  [n1][n2]*sizeof(hermite_conversion_contrib_struct);
     }
+  // counters_right_simple
+  memcpy(counters_right_simple, p, sizeof(counters_right_simple));
+  p += sizeof(counters_right_simple);
+  // counters_left_simple
+  memcpy(counters_left_simple, p, sizeof(counters_left_simple));
+  p += sizeof(counters_left_simple);
+  // list_right_simple and list_left_simple
+  for(int n1 = 0; n1 <= nmax; n1++) {
+    list_right_simple[n1] = new hermite_conversion_element_struct[counters_right_simple[n1]];
+    memcpy(list_right_simple[n1], p, counters_right_simple[n1]*sizeof(hermite_conversion_element_struct));
+    p += counters_right_simple[n1]*sizeof(hermite_conversion_element_struct);
+    list_left_simple[n1] = new hermite_conversion_element_struct[counters_left_simple[n1]];
+    memcpy(list_left_simple[n1], p, counters_left_simple[n1]*sizeof(hermite_conversion_element_struct));
+    p += counters_left_simple[n1]*sizeof(hermite_conversion_element_struct);
+  }
+  assert((size_t)(p - dataBuffer) == get_size());
   // DONE!
   if(static_cast<size_t>(p-dataBuffer) > bufferSize)
     throw std::runtime_error("Error: (p > bufferSize).");  
