@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,21 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
-/** @file integral_info.cc defines IntegralInfo object.
-    IntegralInfo object provides the coefficients needed for integral
-    evaluation.
+/** @file integral_info.cc
 
-    @author: Elias Rudberg <em>responsible</em>. 
+    @brief Defines IntegralInfo object, providing the coefficients
+    needed for integral evaluation.
+
+    @author: Elias Rudberg <em>responsible</em>
 */
+
 #include <stdlib.h>
 #include <cmath>
 #include <stdio.h>
@@ -43,6 +46,7 @@
 #include "boysfunction.h"
 #include "output.h"
 #include "memorymanag.h"
+#include "mat_gblas.h"
 
 
 
@@ -89,8 +93,8 @@ get_real_solid_harmonic_poly(int l, int m,
     denominator = 2;
   else
     denominator = 1;
-  ergo_real NSlm = ((ergo_real)1 / (std::pow((ergo_real)2, abs(m))*factorial(l))) 
-    * std::sqrt(2*factorial(l+abs(m))*factorial(l-abs(m)) / denominator);  
+  ergo_real NSlm = ((ergo_real)1 / (template_blas_pow((ergo_real)2, (ergo_real)abs(m))*factorial(l))) 
+    * template_blas_sqrt(2*factorial(l+abs(m))*factorial(l-abs(m)) / denominator);  
 
   const int MAX_DEGREE = 10;
   ergo_real terms[MAX_DEGREE][MAX_DEGREE][MAX_DEGREE];
@@ -138,8 +142,8 @@ get_real_solid_harmonic_poly(int l, int m,
 
 	int power_t_plus_v_minus_vm = (int)(t + v - vm);
 	ergo_real Clmtuv = 
-	  std::pow((ergo_real)-1, power_t_plus_v_minus_vm) * 
-	  std::pow((ergo_real)0.25, t) *
+			  template_blas_pow((ergo_real)-1, (ergo_real)power_t_plus_v_minus_vm) * 
+	  template_blas_pow((ergo_real)0.25, (ergo_real)t) *
 	  getBinCoeff(l, t)   * getBinCoeff(l-t, abs(m)+t)     * getBinCoeff(t, u)   * getBinCoeff(abs(m), twov);
 
 	terms[ix][iy][iz] += NSlm * Clmtuv;
@@ -182,11 +186,11 @@ setup_basis_func_polys(IntegralInfo* b)
   ergo_real scaleFactorList[MAX_L_QUANTUM_NUMBER+1];
   scaleFactorList[0] = 1;
   scaleFactorList[1] = 1;
-  scaleFactorList[2] = std::sqrt(3.0);
-  scaleFactorList[3] = std::sqrt(15.0);
+  scaleFactorList[2] = template_blas_sqrt(3.0);
+  scaleFactorList[3] = template_blas_sqrt(15.0);
   // Set all other factors to same value. FIXME: find out if/how/why this matters.
   for(int ii = 4; ii <= MAX_L_QUANTUM_NUMBER; ii++)
-    scaleFactorList[ii] = std::sqrt(15.0); 
+    scaleFactorList[ii] = template_blas_sqrt(15.0); 
   
   for(int l = 0; l <= MAX_L_QUANTUM_NUMBER; l++) {
     for(int m = -l; m <= l; m++) {
@@ -201,7 +205,7 @@ setup_basis_func_polys(IntegralInfo* b)
       if(m == 0)
 	curr->scaledSolidHarmonicPrefactor = 1;
       else
-	curr->scaledSolidHarmonicPrefactor = std::pow((ergo_real)-1, m) / std::sqrt((ergo_real)2);
+	curr->scaledSolidHarmonicPrefactor = template_blas_pow((ergo_real)-1, (ergo_real)m) / template_blas_sqrt((ergo_real)2);
 
       // use scalefactor
       for(int i = 0; i < curr->noOfTerms; i++)
@@ -242,6 +246,10 @@ ergo_real IntegralInfo::BoysFunction(int n, ergo_real x) const {
   return boysFunctionManager.BoysFunction(n, x);
 }
 
+ergo_real IntegralInfo::BoysFunction_expensive(int n, ergo_real x, int noOfIntegrationIntervals) const {
+  return boysFunctionManager.BoysFunction_expensive(n, x, noOfIntegrationIntervals);
+}
+
 int IntegralInfo::multiply_by_hermite_conversion_matrix_from_right(int n1max,        
 								   int n2max,        
 								   ergo_real a,      
@@ -281,6 +289,10 @@ void IntegralInfo::init() {
     return;
   do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "IntegralInfo::init() calling boysFunctionManager.init().");
   boysFunctionManager.init();
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "IntegralInfo::init() calling multipolePrep.init().");
+  multipolePrep.init();
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "IntegralInfo::init() calling mmLimitTable.init().");
+  mmLimitTable.inittt(multipolePrep);
   do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "IntegralInfo::init() calling monomial_info.init().");
   monomial_info.init();
   do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "IntegralInfo::init() calling hermite_conversion_info.init().");
@@ -305,6 +317,8 @@ IntegralInfo::~IntegralInfo()
 /** Function needed for Chunks&Tasks usage. */
 IntegralInfo::IntegralInfo(const IntegralInfo & ii) 
   : boysFunctionManager(ii.boysFunctionManager), 
+    multipolePrep(ii.multipolePrep),
+    mmLimitTable(ii.mmLimitTable),
     hermite_conversion_info(ii.hermite_conversion_info), 
     initialized(ii.initialized), 
     no_of_basis_func_polys(ii.no_of_basis_func_polys), 
@@ -319,10 +333,16 @@ void IntegralInfo::write_to_buffer ( char * dataBuffer, size_t const bufferSize
     throw std::runtime_error("Error: IntegralInfo::write_to_buffer called when not initialized.");
   char* p = dataBuffer;
   if(bufferSize < get_size())
-    throw std::runtime_error("Error: bufferSize too small.");
+    throw std::runtime_error("Error in IntegralInfo::write_to_buffer: bufferSize too small.");
   // boysFunctionManager
   boysFunctionManager.write_to_buffer(p, bufferSize - (p - dataBuffer));
   p += boysFunctionManager.get_size();
+  // multipolePrep
+  multipolePrep.write_to_buffer(p, bufferSize - (p - dataBuffer));
+  p += multipolePrep.get_size();
+  // mmLimitTable
+  mmLimitTable.write_to_buffer(p, bufferSize - (p - dataBuffer));
+  p += mmLimitTable.get_size();
   // hermite_conversion_info
   hermite_conversion_info.write_to_buffer(p, bufferSize - (p - dataBuffer));
   p += hermite_conversion_info.get_size();
@@ -344,6 +364,8 @@ void IntegralInfo::write_to_buffer ( char * dataBuffer, size_t const bufferSize
 /** Function needed for Chunks&Tasks usage. */
 size_t IntegralInfo::get_size() const {
   return boysFunctionManager.get_size()
+    + multipolePrep.get_size()
+    + mmLimitTable.get_size()
     + hermite_conversion_info.get_size()
     + sizeof(bool)
     + MAX_NO_OF_BASIS_FUNC_POLYS*sizeof(basis_func_poly_struct)
@@ -357,6 +379,12 @@ void IntegralInfo::assign_from_buffer ( char const * dataBuffer, size_t const bu
   // boysFunctionManager
   boysFunctionManager.assign_from_buffer(p, bufferSize - (p - dataBuffer));
   p += boysFunctionManager.get_size();
+  // multipolePrep
+  multipolePrep.assign_from_buffer(p, bufferSize - (p - dataBuffer));
+  p += multipolePrep.get_size();
+  // mmLimitTable
+  mmLimitTable.assign_from_buffer(p, bufferSize - (p - dataBuffer));
+  p += mmLimitTable.get_size();
   // hermite_conversion_info
   hermite_conversion_info.assign_from_buffer(p, bufferSize - (p - dataBuffer));
   p += hermite_conversion_info.get_size();
