@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integral_info.h
+
+    @brief Defines IntegralInfo object, providing the coefficients
+    needed for integral evaluation.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef BASISINFO_BASIC_HEADER
 #define BASISINFO_BASIC_HEADER
 
@@ -34,6 +43,8 @@
 #include "monomial_info.h"
 #include "hermite_conversion_prep.h"
 #include "boysfunction.h"
+#include "multipole_prep.h"
+#include "mm_limit_table.h"
 
 #ifndef BASIS_FUNC_POLY_MAX_DEGREE
 #error The constant BASIS_FUNC_POLY_MAX_DEGREE must be defined.
@@ -83,6 +94,8 @@ class IntegralInfo
 {
  private:
   BoysFunctionManager boysFunctionManager;
+  MultipolePrepManager multipolePrep;
+  MMLimitTable mmLimitTable;
   hermite_conversion_info_struct hermite_conversion_info;
   bool initialized;
   IntegralInfo(); // This is to make it forbidden to create it without argument.
@@ -92,6 +105,9 @@ class IntegralInfo
   monomial_info_struct monomial_info;
   void init();
   ergo_real BoysFunction(int n, ergo_real x) const;
+  ergo_real BoysFunction_expensive(int n, ergo_real x, int noOfIntegrationIntervals) const;
+  const MultipolePrepManager & GetMultipolePrep() const { return multipolePrep; }
+  const MMLimitTable & GetMMLimitTable() const { return mmLimitTable; }
   int multiply_by_hermite_conversion_matrix_from_right(int n1max,        
 						       int n2max,        
 						       ergo_real a,      
@@ -144,7 +160,7 @@ ExchWeights() :
   
 };
 
-};
+}
 
 
 int get_poly_info_from_shell_type(int* polyid_start, int* poly_count, int shellType);
