@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_1el_kinetic.cc
+
+    @brief Code for 1-electron integrals, computation of
+    kinetic-energy matrix T.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 /* Written by Elias Rudberg, KTH, Stockholm */
 #include <stdlib.h>
 #include <math.h>
@@ -47,6 +56,7 @@
 #include "multipole.h"
 #include "integrals_2el_single.h"
 #include "integrals_1el_single.h"
+#include "basis_func_pair_list_1el.h"
 
 
 /* FIXME do not use this hard-coded value! */
@@ -134,7 +144,7 @@ simplePrimTintegral(const DistributionSpecStruct& prim1,
 
       for(k = 0; k < nNewPrims; k++)
 	{
-	  DistributionSpecStruct* currDistr = &tempList[k];
+	  const DistributionSpecStruct & currDistr = tempList[k];
 	  sum += compute_integral_of_simple_prim(currDistr);
 	} /* END FOR k */
     }
@@ -143,126 +153,133 @@ simplePrimTintegral(const DistributionSpecStruct& prim1,
 }
 
 
-int 
-compute_T_matrix_sparse(const BasisInfoStruct& basisInfo,
-			ergo_real threshold,
-			int n,
-			int* nvaluesList,
-			int** colindList,
-			ergo_real** valuesList)
+int
+compute_T_matrix_sparse_linear(const BasisInfoStruct& basisInfo,
+			       ergo_real threshold,
+			       ergo_real boxSize,
+			       int* nvaluesList,
+			       int** colindList,
+			       ergo_real** valuesList)
 {
-  int mu, nbast;
   int internal_error = 0;
-  nbast = basisInfo.noOfBasisFuncs;
+  int n = basisInfo.noOfBasisFuncs;
 
-  if(n != nbast)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_T_matrix_sparse: (n != nbast)");
-      return -1;
-    }
+  int noOfBasisFuncIndexPairs = get_basis_func_pair_list_simple(basisInfo, threshold, boxSize, NULL, 2000000000);
+  if(noOfBasisFuncIndexPairs <= 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_simple, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+    return -1;
+  }
+  std::vector<basis_func_index_pair_struct_1el> basisFuncIndexPairList(noOfBasisFuncIndexPairs);
+  noOfBasisFuncIndexPairs = get_basis_func_pair_list_simple(basisInfo, threshold, boxSize, &basisFuncIndexPairList[0], noOfBasisFuncIndexPairs);
+  if(noOfBasisFuncIndexPairs <= 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_simple, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+    return -1;
+  }
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "compute_T_matrix_sparse_linear: n = %d, threshold = %g, boxSize = %f",
+	    n, (double)threshold, (double)boxSize);
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "compute_T_matrix_sparse_linear: noOfBasisFuncIndexPairs = %i ==> storing %6.2f %% of a full matrix", 
+	    noOfBasisFuncIndexPairs, (double)100*noOfBasisFuncIndexPairs/((double)n*n));
 
   // To reduce scaling we want some kind of "extent" for each basis function.
   // Start by getting largest simple integral for each of the two basis sets.
   ergo_real A = get_largest_simple_integral(basisInfo);
-  ergo_real* basisFuncExtentList = ergo_new(nbast, ergo_real);
-  get_basis_func_extent_list(basisInfo, basisFuncExtentList, MATRIX_ELEMENT_THRESHOLD_VALUE / A);
+  std::vector<ergo_real> basisFuncExtentList(n);
+  get_basis_func_extent_list(basisInfo, &basisFuncExtentList[0], MATRIX_ELEMENT_THRESHOLD_VALUE / A);
 
+  std::vector<int> offsetVec(n);
+  std::vector<int> countVec(n);
+  int currOffset = 0;
+  int countSumToVerify = 0;
+  for(int i = 0; i < n; i++) {
+    int savedOffset = currOffset;
+    while(currOffset < noOfBasisFuncIndexPairs && basisFuncIndexPairList[currOffset].index_1 == i)
+      currOffset++;
+    int count = currOffset - savedOffset;
+    offsetVec[i] = savedOffset;
+    countVec[i] = count;
+    countSumToVerify += count;
+  }
+  assert(currOffset == noOfBasisFuncIndexPairs);
+  assert(countSumToVerify == noOfBasisFuncIndexPairs);
+  
 #ifdef _OPENMP
 #pragma omp parallel
 #endif
   {
-  // Allocate vector for results for one row.
-  ergo_real* rowValueList = ergo_new(n, ergo_real);
+    // Allocate vector for results for one row.
+    std::vector<ergo_real> rowValueList(n);
+    std::vector<int> row_nu_list(n);
 #ifdef _OPENMP
 #pragma omp for schedule(guided)
 #endif
-  for(mu = 0; mu < nbast; mu++)
-    {
+    for(int mu = 0; mu < n; mu++) {
+      int no_of_nu_values = countVec[mu];
+      int startOffset = offsetVec[mu];
       int count = 0;
-      
       BasisFuncStruct* basisFunc_mu = &basisInfo.basisFuncList[mu];
       int n_mu = basisFunc_mu->noOfSimplePrimitives;
       int start_prim_mu = basisFunc_mu->simplePrimitiveIndex;
-      DistributionSpecStruct* list_mu = 
-	&basisInfo.simplePrimitiveList[start_prim_mu];
-      for(int nu = 0; nu <= mu; nu++)
-	{
-	  // Compute distance between basis function centers
-	  ergo_real dx = basisInfo.basisFuncList[mu].centerCoords[0] - basisInfo.basisFuncList[nu].centerCoords[0];
-	  ergo_real dy = basisInfo.basisFuncList[mu].centerCoords[1] - basisInfo.basisFuncList[nu].centerCoords[1];
-	  ergo_real dz = basisInfo.basisFuncList[mu].centerCoords[2] - basisInfo.basisFuncList[nu].centerCoords[2];
-	  ergo_real distance = std::sqrt(dx*dx + dy*dy + dz*dz);
-	  // We can skip if distance is greater than sum of extents.
-	  if(distance > basisFuncExtentList[mu] + basisFuncExtentList[nu]) {
-	    rowValueList[nu] = 0;
-	    continue;
-	  }
-
-	  BasisFuncStruct* basisFunc_nu = &basisInfo.basisFuncList[nu];
-	  int n_nu = basisFunc_nu->noOfSimplePrimitives;
-	  int start_prim_nu = basisFunc_nu->simplePrimitiveIndex;
-	  DistributionSpecStruct* list_nu = 
-	    &basisInfo.simplePrimitiveList[start_prim_nu];
-	  /* compute matrix element [mu,nu] */
-	  ergo_real sum = 0;
-	  int i, j, k;
-	  for(j = 0; j < n_mu; j++)
-	    {
-	      const DistributionSpecStruct& prim_mu_j = list_mu[j];
-	      for(k = 0; k < n_nu; k++)
-		{
-		  const DistributionSpecStruct& prim_nu_k = list_nu[k];
-                  ergo_real effectiveThreshold = 2.0*threshold/(n_mu*n_nu*3);
-		  /* now loop over coordinates */
-		  for(i = 0; i < 3; i++)
-		    {
-                      /* Note that this function is not strict wrt the
-                         effectiveThreshold parameter, the
-                         approximation is only proportional to its
-                         value but it can exceed it. */
-		      sum += simplePrimTintegral(prim_mu_j,
-						 prim_nu_k,
-						 i,
-						 effectiveThreshold);
-		    } /* END FOR i */
-		} /* END FOR k */
-	    } /* END FOR j */
-	  rowValueList[nu] = -0.5 * sum;
-	  if(std::fabs(rowValueList[nu]) > MATRIX_ELEMENT_THRESHOLD_VALUE)
-	    count++;
-	} /* END FOR nu */
-
+      DistributionSpecStruct* list_mu = &basisInfo.simplePrimitiveList[start_prim_mu];
+      for(int nuCounter = 0; nuCounter < no_of_nu_values; nuCounter++) {
+	int nu = basisFuncIndexPairList[startOffset+nuCounter].index_2;
+	assert(mu == basisFuncIndexPairList[startOffset+nuCounter].index_1);
+	assert(nu <= mu);
+	// Compute distance between basis function centers
+	ergo_real dx = basisInfo.basisFuncList[mu].centerCoords[0] - basisInfo.basisFuncList[nu].centerCoords[0];
+	ergo_real dy = basisInfo.basisFuncList[mu].centerCoords[1] - basisInfo.basisFuncList[nu].centerCoords[1];
+	ergo_real dz = basisInfo.basisFuncList[mu].centerCoords[2] - basisInfo.basisFuncList[nu].centerCoords[2];
+	ergo_real distance = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
+	// We can skip if distance is greater than sum of extents.
+	if(distance > basisFuncExtentList[mu] + basisFuncExtentList[nu])
+	  continue;
+	BasisFuncStruct* basisFunc_nu = &basisInfo.basisFuncList[nu];
+	int n_nu = basisFunc_nu->noOfSimplePrimitives;
+	int start_prim_nu = basisFunc_nu->simplePrimitiveIndex;
+	DistributionSpecStruct* list_nu = &basisInfo.simplePrimitiveList[start_prim_nu];
+	/* compute matrix element [mu,nu] */
+	ergo_real sum = 0;
+	int i, j, k;
+	for(j = 0; j < n_mu; j++) {
+	  const DistributionSpecStruct& prim_mu_j = list_mu[j];
+	  for(k = 0; k < n_nu; k++) {
+	    const DistributionSpecStruct& prim_nu_k = list_nu[k];
+	    ergo_real effectiveThreshold = 2.0*threshold/(n_mu*n_nu*3);
+	    /* now loop over coordinates */
+	    for(i = 0; i < 3; i++) {
+	      /* Note that this function is not strict wrt the
+		 effectiveThreshold parameter, the
+		 approximation is only proportional to its
+		 value but it can exceed it. */
+	      sum += simplePrimTintegral(prim_mu_j,
+					 prim_nu_k,
+					 i,
+					 effectiveThreshold);
+	    } /* END FOR i */
+	  } /* END FOR k */
+	} /* END FOR j */
+	rowValueList[count] = -0.5 * sum;
+	row_nu_list[count] = nu;
+	if(template_blas_fabs(rowValueList[count]) > MATRIX_ELEMENT_THRESHOLD_VALUE)
+	  count++;
+      } /* END FOR nuCounter */
       // OK, this row done.
       // Now go through results to check which elements need to be saved.
       nvaluesList[mu] = count;
       // Now allocate result vectors for this row.
       colindList[mu] = ergo_new(count, int);
       valuesList[mu] = ergo_new(count, ergo_real);
-      count = 0;
-      for(int j = 0; j <= mu; j++)
-	{
-	  ergo_real absVal = std::fabs(rowValueList[j]);
-	  if(absVal > MATRIX_ELEMENT_THRESHOLD_VALUE)
-	    {
-	      if(count >= nvaluesList[mu]) {
-		internal_error++; /* This should never happen. */
-		break;
-	      }
-	      colindList[mu][count] = j;
-	      valuesList[mu][count] = rowValueList[j];
-	      count++;
-	    }
-	}
-
+      for(int j = 0; j < count; j++) {
+	colindList[mu][j] = row_nu_list[j];
+	valuesList[mu][j] = rowValueList[j];
+      }
     } /* END FOR mu */
-  ergo_free(rowValueList);
   }
-  ergo_free(basisFuncExtentList);
-  
+
   return internal_error ? -1 : 0;
 }
 
 
+
 int
 compute_T_matrix_full(const BasisInfoStruct& basisInfo,
 		      ergo_real threshold,
@@ -273,12 +290,13 @@ compute_T_matrix_full(const BasisInfoStruct& basisInfo,
   int** colindList = ergo_new(n, int*);
   ergo_real** valuesList = ergo_new(n, ergo_real*);
 
-  if(compute_T_matrix_sparse(basisInfo,
-			     threshold,
-			     n,
-			     nvaluesList,
-			     colindList,
-			     valuesList) != 0)
+  ergo_real boxSize = 6.3;
+  if(compute_T_matrix_sparse_linear(basisInfo,
+				    threshold,
+				    boxSize,
+				    nvaluesList,
+				    colindList,
+				    valuesList) != 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_T_matrix_sparse");
       return -1;
