@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_1el_kinetic.h
+
+    @brief Code for 1-electron integrals, computation of
+    kinetic-energy matrix T.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef INTEGRALS_1EL_KINETIC
 #define INTEGRALS_1EL_KINETIC
 
@@ -38,12 +47,12 @@ ergo_real simplePrimTintegral(const DistributionSpecStruct& prim1,
 			      int coord,
 			      ergo_real threshold);
 
-int compute_T_matrix_sparse(const BasisInfoStruct& basisInfo,
-			    ergo_real threshold,
-			    int n,
-			    int* nvaluesList,
-			    int** colindList,
-			    ergo_real** valuesList);
+int compute_T_matrix_sparse_linear(const BasisInfoStruct& basisInfo,
+				   ergo_real threshold,
+				   ergo_real boxSize,
+				   int* nvaluesList,
+				   int** colindList,
+				   ergo_real** valuesList);
 
 int compute_T_matrix_full(const BasisInfoStruct& basisInfo,
 			  ergo_real threshold,
