@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_1el_potential.cc
+
+    @brief Code for 1-electron integrals, computation of
+    electron-nuclear potential energy matrix V.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 /* Written by Elias Rudberg. First version was written in 2006 or even
    earlier, at KTH. Then the code has been updated many times after
    that.  */
@@ -64,6 +73,7 @@
 
 #include "integrals_1el.h"
 #include "integrals_1el_potential.h"
+#include "integrals_1el_potential_prep.h"
 #include "memorymanag.h"
 #include "pi.h"
 #include "output.h"
@@ -100,7 +110,7 @@ get_distance_3d(const ergo_real* x, const ergo_real* y)
       ergo_real dx = x[k] - y[k];
       sum += dx * dx;
     }
-  return std::sqrt(sum);
+  return template_blas_sqrt(sum);
 }
 
 
@@ -146,7 +156,8 @@ static void init_multipole_struct_large(multipole_struct_large & boxMultipole, c
 
 
 static int
-create_nuclei_mm_tree(int nAtoms,
+create_nuclei_mm_tree(const IntegralInfo& integralInfo,
+		      int nAtoms,
 		      const Atom* atomList,
 		      ergo_real boxSize,
 		      BoxSystem & boxSystem,
@@ -244,7 +255,7 @@ create_nuclei_mm_tree(int nAtoms,
     } // END FOR i Find center-of-charge for each box at top level
 
   // Create multipole for each box at top level (smallest boxes)
-  MMTranslator translator;
+  MMTranslator translator(integralInfo.GetMultipolePrep());
   for(int i = 0; i < noOfBoxesTopLevel; i++)
     {
       atom_box_struct* currBox = &boxListTopLevel[i];
@@ -437,11 +448,12 @@ create_nuclei_mm_tree(int nAtoms,
 
 
 
-typedef struct
-{
+typedef struct {
   DistributionSpecStruct distr;
   int pairIdx;
-} DistributionSpecStructWithPairIdx;
+  int basisFuncIdx1;
+  int basisFuncIdx2;
+} DistributionSpecStructWithIndexes;
 
 
 /**
@@ -452,7 +464,7 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 			 ergo_real* V_list,
 			 int noOfBasisFuncIndexPairs,
 			 const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
-			 const DistributionSpecStructWithPairIdx* list,
+			 const DistributionSpecStructWithIndexes* list,
 			 int nDistrs,
 			 const multipole_struct_small* multipoleList,
 			 const ergo_real* maxMomentVectorNormForDistrsList,
@@ -477,17 +489,18 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 
   // check if current box is far enough away so that we can use multipole description.
   ergo_real distance = get_distance_3d(list[0].distr.centerCoords, currBox->basicBox.centerCoords);
-  ergo_real boxRadius = currBox->basicBox.width * 0.5 * std::sqrt((ergo_real)3);
+  ergo_real boxRadius = currBox->basicBox.width * 0.5 * template_blas_sqrt((ergo_real)3);
   ergo_real requiredDistance = boxRadius + distrExtent;
 
   // Note that the distance to the box multipole is different, since
   // it is not necessarily placed at the box center.
   ergo_real multipoleDistance = get_distance_3d(list[0].distr.centerCoords, boxMultipole->centerCoords);
-  int degreeNeeded = mm_limits_get_minimum_multipole_degree_needed(multipoleDistance,
-								   boxMultipole, 
-								   maxDegreeForDistrs, 
-								   maxMomentVectorNormForDistrsList, 
-								   threshold);
+
+  int degreeNeeded = integralInfo.GetMMLimitTable().get_minimum_multipole_degree_needed(multipoleDistance,
+											boxMultipole,
+											maxDegreeForDistrs,
+											maxMomentVectorNormForDistrsList,
+											threshold);
   if(degreeNeeded < 0)
     return -1;
   degreeNeeded+=2; // We need a couple of extra degrees to handle gradient computation. FIXME: IS THIS REALL NEEDED? WHY? MAYBE IT WAS BUGS IN OTHER PLACES THAT CAUSED THE PROBLEMS?
@@ -741,13 +754,207 @@ do_interaction_recursive(const IntegralInfo & integralInfo,
 
 
 
+/**
+   Take care of interaction between list of distrs and box.
+*/
+static int
+do_interaction_recursive_2(const IntegralInfo & integralInfo,
+			   csr_matrix_struct* V_CSR,
+			   int noOfBasisFuncIndexPairs,
+			   const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
+			   const DistributionSpecStructWithIndexes2* list,
+			   int nDistrs,
+			   const multipole_struct_small* multipoleList,
+			   const ergo_real* maxMomentVectorNormForDistrsList,
+			   int maxNoOfMomentsForDistrs,
+			   int maxDegreeForDistrs,
+			   ergo_real distrExtent,
+			   const Atom *atomListReordered,
+			   const int* atomPermutation, // list of int, length=nAtoms, saying how atoms have been reordered in return_atomListReordered.
+			   ergo_real threshold,
+			   const atom_box_struct* boxList,
+			   MMInteractor & interactor,
+			   int boxIndex,
+			   int currLevel,
+			   int numberOfLevels
+			   )
+{
+  const atom_box_struct* currBox = &boxList[boxIndex];
+  const multipole_struct_large* boxMultipole = &currBox->multipole;
+
+  // check if current box is far enough away so that we can use multipole description.
+  ergo_real distance = get_distance_3d(list[0].distr.centerCoords, currBox->basicBox.centerCoords);
+  ergo_real boxRadius = currBox->basicBox.width * 0.5 * template_blas_sqrt((ergo_real)3);
+  ergo_real requiredDistance = boxRadius + distrExtent;
+
+  // Note that the distance to the box multipole is different, since
+  // it is not necessarily placed at the box center.
+  ergo_real multipoleDistance = get_distance_3d(list[0].distr.centerCoords, boxMultipole->centerCoords);
+  int degreeNeeded = integralInfo.GetMMLimitTable().get_minimum_multipole_degree_needed(multipoleDistance,
+											boxMultipole,
+											maxDegreeForDistrs,
+											maxMomentVectorNormForDistrsList,
+											threshold);
+  if(degreeNeeded < 0)
+    return -1;
+  degreeNeeded+=2; // We need a couple of extra degrees to handle gradient computation. FIXME: IS THIS REALL NEEDED? WHY? MAYBE IT WAS BUGS IN OTHER PLACES THAT CAUSED THE PROBLEMS?
+
+  bool multipoleDegreeIsSafe = false;
+  // Demand at least two degrees margin compared to
+  // MAX_MULTIPOLE_DEGREE, otherwise this may fail due to alternating
+  // odd/even degrees where only one of them are significant. This has
+  // happened in some test cases.
+  if(degreeNeeded <= MAX_MULTIPOLE_DEGREE-2)
+    multipoleDegreeIsSafe = true;
+  
+  if(distance > requiredDistance && multipoleDegreeIsSafe)
+    {
+      // OK, use multipole description of atom charges.
+      int boxNeededNoOfMoments = (degreeNeeded+1)*(degreeNeeded+1);
+      // create interaction matrix
+      ergo_real T[boxNeededNoOfMoments * maxNoOfMomentsForDistrs];
+      ergo_real dx = boxMultipole->centerCoords[0] - list[0].distr.centerCoords[0];
+      ergo_real dy = boxMultipole->centerCoords[1] - list[0].distr.centerCoords[1];
+      ergo_real dz = boxMultipole->centerCoords[2] - list[0].distr.centerCoords[2];
+
+      interactor.getInteractionMatrix(dx, dy, dz, maxDegreeForDistrs, degreeNeeded, T);
+
+      ergo_real tempVector[MAX_NO_OF_MOMENTS_PER_MULTIPOLE];
+      for(int A = 0; A < maxNoOfMomentsForDistrs; A++)
+	{
+	  ergo_real sum = 0;
+	  for(int B = 0; B < boxNeededNoOfMoments; B++)
+	    {
+	      ergo_real momB = boxMultipole->momentList[B];
+	      ergo_real Telement = T[A*boxNeededNoOfMoments+B];
+	      sum += momB * Telement;
+	    }
+	  tempVector[A] = sum;
+	}
+
+      for(int i = 0; i < nDistrs; i++)
+	{
+	  ergo_real sum = 0;
+	  for(int A = 0; A < multipoleList[i].noOfMoments; A++)
+	    sum += tempVector[A] * multipoleList[i].momentList[A];
+	  int idx1 = list[i].basisFuncIdx1;
+	  int idx2 = list[i].basisFuncIdx2;
+	  if(ergo_CSR_add_to_element(V_CSR, idx1, idx2,  -1 * sum) != 0)
+	    return -1;
+	}
+
+      return 0;
+    }
+
+  // No, multipole description could not be used in this case.
+  if(currLevel == numberOfLevels-1)
+    {
+      // We are at top level, must compute explicit interactions
+
+      int Nmax = 0;
+      for(int i = 0; i < nDistrs; i++)
+	{
+	  const DistributionSpecStruct* distr = &list[i].distr;
+	  int N = distr->monomialInts[0] + distr->monomialInts[1] + distr->monomialInts[2];
+	  if(N > Nmax)
+	    Nmax = N;
+	}
+
+      const JK::ExchWeights CAM_params_not_used;
+      
+      const Atom* currAtomList = &atomListReordered[currBox->basicBox.firstItemIndex];
+      int noOfAtomsCurrBox = currBox->basicBox.noOfItems;
+      for(int ia = 0; ia < noOfAtomsCurrBox; ia++)
+	{
+	  const Atom* currAtom = &currAtomList[ia];
+
+	  const DistributionSpecStruct* distr1 = &list[0].distr;
+	  int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[Nmax];
+	  int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[0];
+	  ergo_real primitiveIntegralList_h[noOfMonomials_1*noOfMonomials_2];
+	  ergo_real primitiveIntegralList_2[noOfMonomials_1*noOfMonomials_2];
+	  // Let the distr be 1 and the pointcharge 2
+	  ergo_real alpha1 = distr1->exponent;
+	  ergo_real alpha0 = alpha1;  
+	  int n1 = Nmax;
+	  int n2 = 0;
+	  ergo_real dx0 = currAtom->coords[0] - distr1->centerCoords[0];
+	  ergo_real dx1 = currAtom->coords[1] - distr1->centerCoords[1];
+	  ergo_real dx2 = currAtom->coords[2] - distr1->centerCoords[2];
+	  ergo_real resultPreFactor = 2 * pi / alpha1;
+	  get_related_integrals_hermite(integralInfo,
+					CAM_params_not_used,
+					n1, noOfMonomials_1,
+					n2, noOfMonomials_2,
+					dx0, 
+					dx1, 
+					dx2, 
+					alpha0,
+					resultPreFactor,
+					primitiveIntegralList_h);
+	  integralInfo.multiply_by_hermite_conversion_matrix_from_right(n1,
+									n2,
+									1.0/alpha1,
+									primitiveIntegralList_h,
+									primitiveIntegralList_2);	  
+	  for(int id = 0; id < nDistrs; id++)
+	    {
+	      const DistributionSpecStruct* distr = &list[id].distr;
+	      int n1x = distr->monomialInts[0];
+	      int n1y = distr->monomialInts[1];
+	      int n1z = distr->monomialInts[2];
+	      int monomialIndex = integralInfo.monomial_info.monomial_index_list[n1x][n1y][n1z];
+	      ergo_real integralValue = currAtom->charge * distr->coeff * primitiveIntegralList_2[monomialIndex];
+	      int idx1 = list[id].basisFuncIdx1;
+	      int idx2 = list[id].basisFuncIdx2;
+	      if(ergo_CSR_add_to_element(V_CSR, idx1, idx2, -1 * integralValue) != 0)
+		return -1;
+	    }
+	  
+	} // END FOR ia
+
+      return 0;
+    }
+
+  // Go to next level
+  int noOfChildren = currBox->basicBox.noOfChildBoxes;
+  for(int i = 0; i < noOfChildren; i++)
+    {
+      int childBoxIndex = currBox->basicBox.firstChildBoxIndex + i;
+      if(do_interaction_recursive_2(integralInfo,
+				    V_CSR,
+				    noOfBasisFuncIndexPairs,
+				    basisFuncIndexPairList,
+				    list,
+				    nDistrs,
+				    multipoleList,
+				    maxMomentVectorNormForDistrsList,
+				    maxNoOfMomentsForDistrs,
+				    maxDegreeForDistrs,
+				    distrExtent,
+				    atomListReordered,
+				    atomPermutation,
+				    threshold,
+				    boxList,
+				    interactor,
+				    childBoxIndex,
+				    currLevel + 1,
+				    numberOfLevels) != 0)
+	return -1;
+    } // END FOR i
+  return 0;
+}
+
+
+
+
 static int 
 get_list_of_distrs_for_V(const BasisInfoStruct& basisInfo,
 			 const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
 			 int noOfBasisFuncIndexPairs,
 			 ergo_real threshold,
 			 ergo_real maxCharge,
-			 DistributionSpecStructWithPairIdx* resultList,
+			 DistributionSpecStructWithIndexes* resultList,
 			 int maxCountResult)
 {
   int distrCount = 0;
@@ -771,7 +978,7 @@ get_list_of_distrs_for_V(const BasisInfoStruct& basisInfo,
 	  // now take care of psi_list[k]
 	  // Here, we estimate the largest possible contribution to V from this distr as (maxCharge * 2 * pi * coeff / exponent).
 	  DistributionSpecStruct* prim = &psi_list[k];
-	  ergo_real maxContrib = std::fabs(maxCharge * 2 * pi * prim->coeff / prim->exponent);
+	  ergo_real maxContrib = template_blas_fabs(maxCharge * 2 * pi * prim->coeff / prim->exponent);
 	  if(maxContrib > threshold)
 	    {
 	      if(maxCountResult > 0 && distrCount >= maxCountResult)
@@ -783,6 +990,8 @@ get_list_of_distrs_for_V(const BasisInfoStruct& basisInfo,
 		{
 		  resultList[distrCount].distr = *prim;
 		  resultList[distrCount].pairIdx = kk;
+		  resultList[distrCount].basisFuncIdx1 = i;
+		  resultList[distrCount].basisFuncIdx2 = j;
 		}
 	      distrCount++;
 	    } // END IF above threshold
@@ -793,91 +1002,106 @@ get_list_of_distrs_for_V(const BasisInfoStruct& basisInfo,
 
 
 
-static int
-compare_distrs(const void* p1, const void* p2)
-{
-  DistributionSpecStructWithPairIdx* d1 = (DistributionSpecStructWithPairIdx*)p1;
-  DistributionSpecStructWithPairIdx* d2 = (DistributionSpecStructWithPairIdx*)p2;
-  /* FIXME: Not nice to have these two hardcoded values here.  */
-  const ergo_real tolernance_dist = 1e-10;
-  const ergo_real tolernance_exponent = 1e-11;
-  ergo_real dx = d1->distr.centerCoords[0] - d2->distr.centerCoords[0];
-  if(dx > tolernance_dist)
-    return 1;
-  if(dx < -tolernance_dist)
-    return -1;
-  ergo_real dy = d1->distr.centerCoords[1] - d2->distr.centerCoords[1];
-  if(dy > tolernance_dist)
-    return 1;
-  if(dy < -tolernance_dist)
-    return -1;
-  ergo_real dz = d1->distr.centerCoords[2] - d2->distr.centerCoords[2];
-  if(dz > tolernance_dist)
-    return 1;
-  if(dz < -tolernance_dist)
-    return -1;
-  ergo_real de = d1->distr.exponent - d2->distr.exponent;
-  if(de > tolernance_exponent)
-    return 1;
-  if(de < -tolernance_exponent)
-    return -1; 
-  return 0;
-}
-
-
-static int
-sort_distr_list(DistributionSpecStructWithPairIdx* list, int n)
-{
-  qsort(list, n, sizeof(DistributionSpecStructWithPairIdx), compare_distrs);
-  return 0;
-}
-
-typedef struct
-{
-  int startIndex;
-  int count;
-} group_struct;
-
-static ergo_real 
-getSafeMaxDistance(const BasisInfoStruct& basisInfo, const Molecule& molecule)
-{
-  ergo_real minCoords[3];
-  ergo_real maxCoords[3];
-  for(int k = 0; k < 3; k++)
-    {
-      minCoords[k] = basisInfo.basisFuncList[0].centerCoords[k];
-      maxCoords[k] = basisInfo.basisFuncList[0].centerCoords[k];
-    }
-  for(int i = 0; i < basisInfo.noOfBasisFuncs; i++)
-    {
-      for(int k = 0; k < 3; k++)
-	{
-	  ergo_real x = basisInfo.basisFuncList[i].centerCoords[k];
-	  if(x < minCoords[k])
-	    minCoords[k] = x;
-	  if(x > maxCoords[k])
-	    maxCoords[k] = x;
-	}
-    }
-  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
-    {
-      for(int k = 0; k < 3; k++)
-	{
-	  ergo_real x = molecule.getAtom(i).coords[k];
-	  if(x < minCoords[k])
-	    minCoords[k] = x;
-	  if(x > maxCoords[k])
-	    maxCoords[k] = x;
-	}
-    }
+static ergo_real
+get_nucl_repulsion_energy_using_multipoles(const Atom *atomListReordered,
+					   const int* atomPermutation, // list of int, length=nAtoms, saying how atoms have been reordered in return_atomListReordered.
+					   ergo_real threshold,
+					   const atom_box_struct* boxList,
+					   MMInteractor & interactor,
+					   int boxIndex1,
+					   int boxIndex2,
+					   int currLevel,
+					   int numberOfLevels) {
+  const atom_box_struct* currBox1 = &boxList[boxIndex1];
+  const atom_box_struct* currBox2 = &boxList[boxIndex2];
+  const multipole_struct_large* boxMultipole1 = &currBox1->multipole;
+  const multipole_struct_large* boxMultipole2 = &currBox2->multipole;
+  // Check if we are at level of smallest boxes
+  if(currLevel == numberOfLevels-1) {
+    // Do interaction explicitly
+    const Atom* currAtomList1 = &atomListReordered[currBox1->basicBox.firstItemIndex];
+    int noOfAtomsCurrBox1 = currBox1->basicBox.noOfItems;
+    const Atom* currAtomList2 = &atomListReordered[currBox2->basicBox.firstItemIndex];
+    int noOfAtomsCurrBox2 = currBox2->basicBox.noOfItems;
+    ergo_real sum = 0;
+    for(int i1 = 0; i1 < noOfAtomsCurrBox1; i1++)
+      for(int i2 = 0; i2 < noOfAtomsCurrBox2; i2++) {
+	if(boxIndex1 == boxIndex2 && i1 == i2)
+	  continue; // skip interaction of an atom with itself
+	if(boxIndex1 == boxIndex2 && i1 > i2)
+	  continue; // do not double-count interactions
+	const Atom* currAtom1 = &currAtomList1[i1];
+	const Atom* currAtom2 = &currAtomList2[i2];
+	ergo_real dx0 = currAtom1->coords[0] - currAtom2->coords[0];
+	ergo_real dx1 = currAtom1->coords[1] - currAtom2->coords[1];
+	ergo_real dx2 = currAtom1->coords[2] - currAtom2->coords[2];
+	ergo_real distance = template_blas_sqrt(dx0*dx0+dx1*dx1+dx2*dx2);
+	sum += currAtom1->charge * currAtom2->charge / distance;
+      }
+    return sum;
+  }
+  // Check if multipole representation can be used
+  // check if boxes are far enough apart so that we can consider using multipole description.
+  ergo_real distance_between_box_centers = get_distance_3d(currBox1->basicBox.centerCoords, currBox2->basicBox.centerCoords);
+  ergo_real boxRadius = currBox1->basicBox.width * 0.5 * template_blas_sqrt((ergo_real)3);
+  ergo_real requiredDistance = 2.2*boxRadius; // 2*boxRadius should be enough, use 2.2*boxRadius to have some margin
+  if(distance_between_box_centers > requiredDistance) {
+    // Try using multipole description of atom charges.
+    // Try with different multipole degree and check the difference, if nearly same result is obtained we trust it, otherwise not.
+    const int nDegreesToTry = 3;
+    ergo_real resultMin = 0;
+    ergo_real resultMax = 0;
+    const int highest_degree_to_use = MAX_MULTIPOLE_DEGREE / 2; // different choices possible here, pick something that gives good performance
+    ergo_real dx = boxMultipole2->centerCoords[0] - boxMultipole1->centerCoords[0];
+    ergo_real dy = boxMultipole2->centerCoords[1] - boxMultipole1->centerCoords[1];
+    ergo_real dz = boxMultipole2->centerCoords[2] - boxMultipole1->centerCoords[2];
+    int boxNeededNoOfMomentsMax = (highest_degree_to_use+1)*(highest_degree_to_use+1);
+    ergo_real T[boxNeededNoOfMomentsMax * boxNeededNoOfMomentsMax];
+    interactor.getInteractionMatrix(dx, dy, dz, highest_degree_to_use, highest_degree_to_use, T);
+    for(int i = 0; i < nDegreesToTry; i++) {
+      int degree = highest_degree_to_use - i;
+      if(degree < 0)
+	throw std::runtime_error("Error in get_nucl_repulsion_energy_using_multipoles: (degree < 0).");
+      int boxNeededNoOfMoments = (degree+1)*(degree+1);
+      // create interaction matrix
+      ergo_real sum = 0;
+      for(int A = 0; A < boxNeededNoOfMoments; A++)
+	for(int B = 0; B < boxNeededNoOfMoments; B++)
+	  sum += T[A*boxNeededNoOfMomentsMax+B] * boxMultipole1->momentList[A] * boxMultipole2->momentList[B];
+      if(i == 0) {
+	resultMin = sum;
+	resultMax = sum;
+      }
+      if(sum < resultMin)
+	resultMin = sum;
+      if(sum > resultMax)
+	resultMax = sum;
+    } // end for i
+    ergo_real diff = resultMax - resultMin;
+    if(diff < threshold)
+      return 0.5*(resultMin+resultMax);
+  } // end if distance large enough to consider multipoles
+  // Go to next level, smaller boxes
   ergo_real sum = 0;
-  for(int k = 0; k < 3; k++)
-    {
-      ergo_real dx = maxCoords[k] - minCoords[k];
-      sum += dx*dx;
-    }
-  ergo_real maxDistance = std::sqrt(sum);
-  return maxDistance;
+  int noOfChildren1 = currBox1->basicBox.noOfChildBoxes;
+  int noOfChildren2 = currBox2->basicBox.noOfChildBoxes;
+  for(int i1 = 0; i1 < noOfChildren1; i1++)
+    for(int i2 = 0; i2 < noOfChildren2; i2++) {
+      if(boxIndex1 == boxIndex2 && i1 > i2)
+	continue; // do not double-count interactions
+      int childBoxIndex1 = currBox1->basicBox.firstChildBoxIndex + i1;
+      int childBoxIndex2 = currBox2->basicBox.firstChildBoxIndex + i2;
+      sum += get_nucl_repulsion_energy_using_multipoles(atomListReordered,
+							atomPermutation, // list of int, length=nAtoms, saying how atoms have been reordered in return_atomListReordered.
+							threshold,
+							boxList,
+							interactor,
+							childBoxIndex1,
+							childBoxIndex2,
+							currLevel + 1,
+							numberOfLevels);
+    } // END FOR i1 i2
+  return sum;
 }
 
 
@@ -891,9 +1115,11 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
 				  int noOfBasisFuncIndexPairs,
 				  bool compute_gradient_also,
 				  const ergo_real* D_list, // List of corresponding density matrix elemets; used for compute_gradient_also case, NULL otherwise
-				  ergo_real* gradient_list // list of result gradient values; used for compute_gradient_also case, NULL otherwise
+				  ergo_real* gradient_list, // list of result gradient values; used for compute_gradient_also case, NULL otherwise
+				  ergo_real & result_nuclearRepulsionEnergy
 				  )
 {
+  result_nuclearRepulsionEnergy = 0; // to be set later
   int errorCount = 0;
   do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear start, compute_gradient_also = %d.", compute_gradient_also);
 
@@ -922,7 +1148,7 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
       return -1;
     }
   do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear nDistrs    = %9i", nDistrs);
-  std::vector<DistributionSpecStructWithPairIdx> list(nDistrs);
+  std::vector<DistributionSpecStructWithIndexes> list(nDistrs);
   if(get_list_of_distrs_for_V(basisInfo,
 			      basisFuncIndexPairList,
 			      noOfBasisFuncIndexPairs,
@@ -951,7 +1177,7 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
       ind++;
       if(ind < nDistrs)
 	{
-	  if(compare_distrs(&list[ind], &list[currGroupInd]) == 0)
+	  if(compare_distrs<DistributionSpecStructWithIndexes>(&list[ind], &list[currGroupInd]) == 0)
 	    continue;
 	}
       // define new group
@@ -965,16 +1191,6 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
     }
   do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear groupCount = %9i", groupCount);
 
-  init_multipole_code();
-  // Get maxDistance taking into account both molecule and basis set,
-  // to cover cases where basis functions are placed on other places than on the atoms.
-  ergo_real maxDistance = getSafeMaxDistance(basisInfo, molecule);
-  // Add small margin to handle cases where maxDistance=0
-  maxDistance += 0.1;
-  // Add some more margin that can be needed when multipoles end up somewhat further away when placed at box centers.
-  maxDistance += 5.0;
-  mm_limits_init(maxDistance);
-
   // Note that boxList and atomListReordered are allocated by create_nuclei_mm_tree,
   // we must remember to free them in the end.
   BoxSystem boxSystem;
@@ -982,7 +1198,8 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
   atom_box_struct* boxList = NULL;
   Atom *atomListReordered = NULL;
   int numberOfLevels = -1;
-  if(create_nuclei_mm_tree(molecule.getNoOfAtoms(), molecule.getAtomListPtr(), boxSize,
+  if(create_nuclei_mm_tree(integralInfo,
+			   molecule.getNoOfAtoms(), molecule.getAtomListPtr(), boxSize,
 			   boxSystem,
 			   &boxList, &numberOfLevels,
 			   &atomListReordered,
@@ -995,6 +1212,27 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
 
   timeMeterInitPart.print(LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear init part (including create_nuclei_mm_tree)");
 
+  // Compute nuclear repulsion energy using multipole tree.
+  Util::TimeMeter timeMeterNuclRep;
+  {
+    MMInteractor interactor(integralInfo.GetMultipolePrep());
+    int boxIndex1 = 0;
+    int boxIndex2 = 0;
+    int currLevel = 0;
+    ergo_real nuclRepEnergy = get_nucl_repulsion_energy_using_multipoles(atomListReordered,
+									 &atomPermutation[0],
+									 threshold,
+									 boxList,
+									 interactor,
+									 boxIndex1,
+									 boxIndex2,
+									 currLevel,
+									 numberOfLevels);
+    do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "get_nucl_repulsion_energy_using_multipoles() gave nuclRepEnergy = %22.11f", nuclRepEnergy);
+    result_nuclearRepulsionEnergy = nuclRepEnergy;
+  }
+  timeMeterNuclRep.print(LOG_AREA_INTEGRALS, "get_nucl_repulsion_energy_using_multipoles");
+
   Util::TimeMeter timeMeterMainPart;
 
   const JK::ExchWeights CAM_params_not_used;
@@ -1003,7 +1241,7 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
 #pragma omp parallel
 #endif
   {
-  MMInteractor interactor;
+  MMInteractor interactor(integralInfo.GetMultipolePrep());
   ergo_real *private_V_list = V_list;
   ergo_real* private_gradient_list = gradient_list;
   multipole_struct_small* multipoleList =
@@ -1024,7 +1262,7 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
 #endif
   for(int groupIndex = 0; groupIndex < groupCount; groupIndex++)
     {
-      DistributionSpecStructWithPairIdx* currList = &list[groupList[groupIndex].startIndex];
+      DistributionSpecStructWithIndexes* currList = &list[groupList[groupIndex].startIndex];
       int nDistrsCurrGroup = groupList[groupIndex].count;
 
       // Create multipoles for distrs in this group.
@@ -1055,7 +1293,7 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
 	      ergo_real sum = 0;
 	      for(int A = startIndex; A < endIndex; A++)
 		sum += distrMultipole->momentList[A]*distrMultipole->momentList[A];
-	      ergo_real subNorm = std::sqrt(sum);
+	      ergo_real subNorm = template_blas_sqrt(sum);
 	      if(subNorm > maxMomentVectorNormForDistrsList[l])
 		maxMomentVectorNormForDistrsList[l] = subNorm;
 	    }
@@ -1066,22 +1304,15 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
       ergo_real maxabscoeff = 0;
       for(int i = 0; i < nDistrsCurrGroup; i++)
         {
-          ergo_real abscoeff = std::fabs(currList[i].distr.exponent);
+          ergo_real abscoeff = template_blas_fabs(currList[i].distr.coeff);
           if(abscoeff > maxabscoeff)
             maxabscoeff = abscoeff;
         }
       ergo_real exponent = currList[0].distr.exponent; // all exponents in group are equal anyway.
-      ergo_real R2 = -1 * (1/exponent) * std::log(threshold/(maxabscoeff*maxCharge));
-      if(R2 < 0)
-        {
-          do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_and_gradient_linear, (R2 < 0) when computing extent.");
-#ifdef _OPENMP
-#pragma omp atomic
-#endif
-          errorCount++;
-	  continue;
-        }
-      ergo_real extent = std::sqrt(R2);
+      ergo_real R2 = -1 * (1/exponent) * template_blas_log(threshold/(maxabscoeff*maxCharge));
+      ergo_real extent = 0;
+      if(R2 > 0) // R2 can become negative, e.g. if maxabscoeff is very small, in such cases we let extent be zero.
+	extent = template_blas_sqrt(R2);
       // Take care of interaction of this group with MM tree
       if(do_interaction_recursive(integralInfo,
 				  private_V_list, 
@@ -1157,8 +1388,8 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
 
     // Update gradient with nuclear-nuclear energy contribution
     Util::TimeMeter timeMeterNuclearRepulsionEnergyGradientContrib;
-    molecule.getNuclearRepulsionEnergyGradientContrib(gradient_list);
-    timeMeterNuclearRepulsionEnergyGradientContrib.print(LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear getNuclearRepulsionEnergyGradientContrib");
+    molecule.getNuclearRepulsionEnergyGradientContribQuadratic(gradient_list);
+    timeMeterNuclearRepulsionEnergyGradientContrib.print(LOG_AREA_INTEGRALS, "compute_V_and_gradient_linear getNuclearRepulsionEnergyGradientContribQuadratic");
 
     // Cleanup: delete buffers that were allocated during creation of multipole tree
     for(int boxIdx = 0; boxIdx < boxSystem.totNoOfBoxes; boxIdx++) {
@@ -1180,34 +1411,157 @@ int compute_V_and_gradient_linear(const BasisInfoStruct& basisInfo,
 }
 
 
-static ergo_real 
-simplePrimVintegralSingle(DistributionSpecStruct* prim,
-                          const Atom* atom,
-                          const IntegralInfo & integralInfo)
-{
-  ergo_real result2 = do_1e_repulsion_integral_using_symb_info(prim,
-                                                               atom->charge,
-                                                               atom->coords,
-                                                               integralInfo);
-  return result2;
+
+int compute_V_hierarchical(const BasisInfoStruct& basisInfo,
+			   const IntegralInfo& integralInfo,
+			   const Molecule& molecule,
+			   ergo_real threshold,
+			   ergo_real boxSize,
+			   const basis_func_index_pair_struct_1el* basisFuncIndexPairList,
+			   int noOfBasisFuncIndexPairs,
+			   csr_matrix_struct* V_CSR,
+			   ergo_real & result_nuclearRepulsionEnergy
+			   ) {
+  result_nuclearRepulsionEnergy = 0; // computed later
+  bool compute_gradient_also = false;
+  int errorCount = 0;
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_hierarchical start.");
+
+  Util::TimeMeter timeMeterTotal;
+  Util::TimeMeter timeMeterInitPart;
+
+  // Get maxCharge
+  ergo_real maxCharge = 0;
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++) {
+    ergo_real currCharge = molecule.getAtom(i).charge;
+    if(currCharge > maxCharge)
+      maxCharge = currCharge;
+  }
+
+  // Create list of distributions
+  int nDistrs = get_list_of_distrs_for_V(basisInfo, basisFuncIndexPairList, noOfBasisFuncIndexPairs,
+					 threshold, maxCharge, NULL, 0);
+  if(nDistrs <= 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_hierarchical: (nDistrs <= 0).");
+    return -1;
+  }
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "compute_V_hierarchical nDistrs    = %9i", nDistrs);
+  std::vector<DistributionSpecStructWithIndexes> listTmp(nDistrs);
+  if(get_list_of_distrs_for_V(basisInfo, basisFuncIndexPairList, noOfBasisFuncIndexPairs,
+			      threshold, maxCharge, &listTmp[0], nDistrs) != nDistrs) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in compute_V_hierarchical, in get_list_of_distrs_for_V.");
+    return -1;
+  }
+
+  std::vector<DistributionSpecStructWithIndexes2> listTmp2(nDistrs);
+  for(int i = 0; i < nDistrs; i++) {
+    listTmp2[i].distr = listTmp[i].distr;
+    listTmp2[i].basisFuncIdx1 = listTmp[i].basisFuncIdx1;
+    listTmp2[i].basisFuncIdx2 = listTmp[i].basisFuncIdx2;
+  }
+  listTmp.clear();
+
+  SetOfDistrsForV setOfDistrsForV;
+  organize_distrs_for_V(integralInfo, setOfDistrsForV, listTmp2, threshold, maxCharge);
+  listTmp2.clear();
+
+  // Note that boxList and atomListReordered are allocated by create_nuclei_mm_tree,
+  // we must remember to free them in the end.
+  BoxSystem boxSystem;
+  std::vector<int> atomPermutation(molecule.getNoOfAtoms());
+  atom_box_struct* boxList = NULL;
+  Atom *atomListReordered = NULL;
+  int numberOfLevels = -1;
+  if(create_nuclei_mm_tree(integralInfo,
+			   molecule.getNoOfAtoms(), molecule.getAtomListPtr(), boxSize,
+			   boxSystem, &boxList, &numberOfLevels, &atomListReordered,
+			   &atomPermutation[0], compute_gradient_also) != 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "create_nuclei_mm_tree failed");
+    return -1;
+  }
+
+  timeMeterInitPart.print(LOG_AREA_INTEGRALS, "compute_V_hierarchical init part (including create_nuclei_mm_tree)");
+
+  // Compute nuclear repulsion energy using multipole tree.
+  Util::TimeMeter timeMeterNuclRep;
+  {
+    MMInteractor interactor(integralInfo.GetMultipolePrep());
+    int boxIndex1 = 0;
+    int boxIndex2 = 0;
+    int currLevel = 0;
+    ergo_real nuclRepEnergy = get_nucl_repulsion_energy_using_multipoles(atomListReordered,
+									 &atomPermutation[0],
+									 threshold,
+									 boxList,
+									 interactor,
+									 boxIndex1,
+									 boxIndex2,
+									 currLevel,
+									 numberOfLevels);
+    do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "get_nucl_repulsion_energy_using_multipoles() gave nuclRepEnergy = %22.11f", nuclRepEnergy);
+    result_nuclearRepulsionEnergy = nuclRepEnergy;
+  }
+  timeMeterNuclRep.print(LOG_AREA_INTEGRALS, "get_nucl_repulsion_energy_using_multipoles");
+
+  Util::TimeMeter timeMeterMainPart;
+
+  const JK::ExchWeights CAM_params_not_used;
+  MMInteractor interactor(integralInfo.GetMultipolePrep());
+  int groupCount = setOfDistrsForV.groupList.size();
+  for(int groupIndex = 0; groupIndex < groupCount; groupIndex++) {
+    int groupStartIdx = setOfDistrsForV.groupList[groupIndex].startIndex;
+    DistributionSpecStructWithIndexes2* currList = &setOfDistrsForV.distrList[groupStartIdx];
+    int nDistrsCurrGroup = setOfDistrsForV.groupList[groupIndex].count;
+    ergo_real extent = setOfDistrsForV.groupList[groupIndex].maxExtent;
+    const multipole_struct_small* multipoleList = &setOfDistrsForV.multipoleList[groupStartIdx];
+    const ergo_real* maxMomentVectorNormForDistrsList = setOfDistrsForV.maxMomentVectorNormList[groupIndex].maxMomentVectorNormList;
+    int maxNoOfMoments = setOfDistrsForV.groupList[groupIndex].maxNoOfMoments;
+    int maxDegree = setOfDistrsForV.groupList[groupIndex].maxDegree;
+    // Take care of interaction of this group with MM tree
+    if(do_interaction_recursive_2(integralInfo, V_CSR, noOfBasisFuncIndexPairs,
+				  basisFuncIndexPairList, currList, nDistrsCurrGroup,
+				  multipoleList, maxMomentVectorNormForDistrsList, maxNoOfMoments,
+				  maxDegree, extent, atomListReordered,
+				  &atomPermutation[0], threshold, boxList,
+				  interactor, 0, 0, numberOfLevels) != 0) {
+      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in do_interaction_recursive");
+      errorCount++;
+      continue;
+    }
+  }
+
+  timeMeterMainPart.print(LOG_AREA_INTEGRALS, "compute_V_hierarchical main part");
+
+  delete [] boxList;
+  delete [] atomListReordered;
+  
+  timeMeterTotal.print(LOG_AREA_INTEGRALS, "compute_V_hierarchical total");
+
+  return -errorCount;
 }
 
 
-ergo_real 
-simplePrimVintegral_list(DistributionSpecStruct* list,
+static ergo_real
+simplePrimVintegralSingle(const DistributionSpecStruct & prim,
+                          const Atom & atom,
+                          const IntegralInfo & integralInfo) {
+  return do_1e_repulsion_integral_using_symb_info(prim,
+						  atom.charge,
+						  atom.coords,
+						  integralInfo);
+}
+
+
+ergo_real
+simplePrimVintegral_list(const DistributionSpecStruct* list,
                          int nPrims,
-			 const Atom* atom,
-                         ergo_real threshold,
-                         const IntegralInfo & integralInfo)
-{
-  int k;
-  ergo_real sum;
-  sum = 0;
-  for(k = 0; k < nPrims; k++)
-    {
-      DistributionSpecStruct* currDistr = &list[k];
-      sum += simplePrimVintegralSingle(currDistr, atom, integralInfo);
-    } /* END FOR k */
+			 const Atom & atom,
+                         const IntegralInfo & integralInfo) {
+  ergo_real sum = 0;
+  for(int k = 0; k < nPrims; k++) {
+    const DistributionSpecStruct & currDistr = list[k];
+    sum += simplePrimVintegralSingle(currDistr, atom, integralInfo);
+  }
   return sum;
 }
 
@@ -1255,15 +1609,14 @@ compute_V_matrix_full(const BasisInfoStruct& basisInfo,
                   if(nNewPrims < 0)
                     {
 		      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in get_product_simple_prims");
-                      exit(0);
+                      exit(EXIT_FAILURE);
                     }
                   /* now loop over atoms */
                   for(A = 0; A < nAtoms; A++)
                     {
                       sum += simplePrimVintegral_list(tempList,
                                                       nNewPrims,
-                                                      &atomList[A],
-                                                      threshold,
+                                                      atomList[A],
                                                       integralInfo);
                     } /* END FOR A */
                 } /* END FOR k */
