@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_1el_single.cc
+
+    @brief Functionality for computing a single 1-electron integral,
+    for a given primitive Gaussian distribution and a given point
+    charge.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdlib.h>
 #include <math.h>
 #include <stdio.h>
@@ -40,14 +50,14 @@
 #include "integrals_hermite.h"
 
 
-static ergo_real 
-do_1e_repulsion_integral_using_symb_info_h(DistributionSpecStruct* psi,
+static ergo_real
+do_1e_repulsion_integral_using_symb_info_h(const DistributionSpecStruct & psi,
 					   ergo_real pointCharge,
 					   const ergo_real* pointChargeCoords,
 					   const IntegralInfo & integralInfo)
 {
   // Let the distr be 1 and the pointcharge 2
-  ergo_real alpha1 = psi->exponent;
+  ergo_real alpha1 = psi.exponent;
   // alpha2 is ~ infinity
   ergo_real alpha0 = alpha1;
   // alpham is ~ infinity
@@ -56,17 +66,17 @@ do_1e_repulsion_integral_using_symb_info_h(DistributionSpecStruct* psi,
   int n1 = 0;
   int n2 = 0;
   for(int i = 0; i < 3; i++)
-    n1 += psi->monomialInts[i];
-  int n1x = psi->monomialInts[0];
-  int n1y = psi->monomialInts[1];
-  int n1z = psi->monomialInts[2];
+    n1 += psi.monomialInts[i];
+  int n1x = psi.monomialInts[0];
+  int n1y = psi.monomialInts[1];
+  int n1z = psi.monomialInts[2];
 
   int noOfMonomials_1 = integralInfo.monomial_info.no_of_monomials_list[n1];
   int noOfMonomials_2 = integralInfo.monomial_info.no_of_monomials_list[n2];
 
-  ergo_real dx0 = pointChargeCoords[0] - psi->centerCoords[0];
-  ergo_real dx1 = pointChargeCoords[1] - psi->centerCoords[1];
-  ergo_real dx2 = pointChargeCoords[2] - psi->centerCoords[2];
+  ergo_real dx0 = pointChargeCoords[0] - psi.centerCoords[0];
+  ergo_real dx1 = pointChargeCoords[1] - psi.centerCoords[1];
+  ergo_real dx2 = pointChargeCoords[2] - psi.centerCoords[2];
 
   ergo_real resultPreFactor = 2 * pi / alpha1;
 
@@ -93,7 +103,7 @@ do_1e_repulsion_integral_using_symb_info_h(DistributionSpecStruct* psi,
 								primitiveIntegralList_2);
   int monomialIndex = integralInfo.monomial_info.monomial_index_list[n1x][n1y][n1z];
 
-  ergo_real result = psi->coeff * pointCharge * primitiveIntegralList_2[monomialIndex];
+  ergo_real result = psi.coeff * pointCharge * primitiveIntegralList_2[monomialIndex];
 
   return result;
 }
@@ -176,7 +186,7 @@ std::vector<ergo_real> do_1e_repulsion_integral_derivatives_using_symb_info(cons
 
 
 ergo_real 
-do_1e_repulsion_integral_using_symb_info(DistributionSpecStruct* psi,
+do_1e_repulsion_integral_using_symb_info(const DistributionSpecStruct & psi,
 					 ergo_real pointCharge,
 					 const ergo_real* pointChargeCoords,
 					 const IntegralInfo & integralInfo)
