@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,30 +17,36 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_2el.h
+
+    @brief Parameters related to integral evaluation.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef INTEGRALS_2EL_HEADER
 #define INTEGRALS_2EL_HEADER
 
 #include "basisinfo.h"
 
-
 namespace JK {
 
 struct Params
 {
-  int use_naive_fockmatrix_construction;
+  int use_naive_fockmat_constr;
   ergo_real threshold_J;
   ergo_real threshold_K;
   ergo_real multipole_threshold_factor;
-  int use_densfit_for_J;
   int use_differential_density;
   int use_fmm;
   ergo_real fmm_box_size;
@@ -54,11 +60,10 @@ struct Params
   int noOfThreads_J;
   int noOfThreads_K;
 
-  Params() : use_naive_fockmatrix_construction(0),
+  Params() : use_naive_fockmat_constr(0),
        threshold_J(1e-12),
        threshold_K(1e-12),
        multipole_threshold_factor(1),
-       use_densfit_for_J(0),
        use_differential_density(0),
        use_fmm(1),
        fmm_box_size(5.0),
@@ -75,41 +80,6 @@ struct Params
     
 };
 
- 
-
 }
 
-
-#if 0
-
-typedef struct
-{
-  int shell_idx_1;
-  int shell_idx_2;
-  int noOfExponentPairs;
-  int exponentPairIndex;
-} shell_pair_struct;
-
-
-typedef struct
-{
-  int idx_1;
-  int idx_2;
-  int productOrgIdx;
-  ergo_real exponent_1;
-  ergo_real exponent_2;
-  ergo_real coeff_1;
-  ergo_real coeff_2;
-  ergo_real coeff_12, exponent_12;
-  ergo_real sizeOfProduct;
-  ergo_real centerCoords_1[3];
-  ergo_real centerCoords_2[3];
-  ergo_real centerCoords_12[3];
-  int n1max;
-  int n2max;
-} exponent_pair_struct;
-
-#endif
-
-
 #endif
