@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_2el_boxed.cc
+
+    @brief Code for 2-electron integrals, computation of Coulomb (J)
+    and HF exchange (K) matrices using a single box.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <string.h>
 
 #include "integrals_2el_boxed.h"
@@ -71,10 +80,10 @@ get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 					 int nClusters_1,
 					 const cluster_struct* clusterList_2,
 					 int nClusters_2,
-					 const chunk_struct* chunkList_1,
-					 int nChunks_1,
-					 const chunk_struct* chunkList_2,
-					 int nChunks_2,
+					 const batch_struct* batchList_1,
+					 int nBatchs_1,
+					 const batch_struct* batchList_2,
+					 int nBatchs_2,
 					 const basis_func_pair_struct* basisFuncPairList_1,
 					 const basis_func_pair_struct* basisFuncPairList_2,
 					 int interactionWithSelf,
@@ -90,15 +99,15 @@ get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
   ergo_real* primitiveIntegralList = bufferStructPtr->primitiveIntegralList;
   ergo_real* primitiveIntegralList_work = bufferStructPtr->primitiveIntegralList_work;
 
-  for(int chunk_i = 0; chunk_i < nChunks_1; chunk_i++)
+  for(int batch_i = 0; batch_i < nBatchs_1; batch_i++)
     {
-      int chunk_j_start = 0;
+      int batch_j_start = 0;
       if(interactionWithSelf == 1)
-	chunk_j_start = chunk_i;
-      for(int chunk_j = chunk_j_start; chunk_j < nChunks_2; chunk_j++)
+	batch_j_start = batch_i;
+      for(int batch_j = batch_j_start; batch_j < nBatchs_2; batch_j++)
 	{
-	  int noOfBasisFuncPairs_1 = chunkList_1[chunk_i].noOfBasisFuncPairs;
-	  int noOfBasisFuncPairs_2 = chunkList_2[chunk_j].noOfBasisFuncPairs;
+	  int noOfBasisFuncPairs_1 = batchList_1[batch_i].noOfBasisFuncPairs;
+	  int noOfBasisFuncPairs_2 = batchList_2[batch_j].noOfBasisFuncPairs;
 	  // set integral list to zero
 	  memset(summedIntegralList, 0, noOfBasisFuncPairs_1*noOfBasisFuncPairs_2*sizeof(ergo_real));
 
@@ -107,45 +116,45 @@ get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 	  for(int i = 0; i < noOfBasisFuncPairs_1; i++)
 	    for(int j = 0; j < noOfBasisFuncPairs_2; j++)
 	      {
-		int a = basisFuncPairList_1[chunkList_1[chunk_i].basisFuncPairListIndex+i].index_1;
-		int b = basisFuncPairList_1[chunkList_1[chunk_i].basisFuncPairListIndex+i].index_2;
-		int c = basisFuncPairList_2[chunkList_2[chunk_j].basisFuncPairListIndex+j].index_1;
-		int d = basisFuncPairList_2[chunkList_2[chunk_j].basisFuncPairListIndex+j].index_2;
+		int a = basisFuncPairList_1[batchList_1[batch_i].basisFuncPairListIndex+i].index_1;
+		int b = basisFuncPairList_1[batchList_1[batch_i].basisFuncPairListIndex+i].index_2;
+		int c = basisFuncPairList_2[batchList_2[batch_j].basisFuncPairListIndex+j].index_1;
+		int d = basisFuncPairList_2[batchList_2[batch_j].basisFuncPairListIndex+j].index_2;
 		ergo_real absval;
 		if(J != NULL)
 		  {
-		    absval = std::fabs(dens[a*n+b]);
+		    absval = template_blas_fabs(dens[a*n+b]);
 		    if(absval > maxabsdmatelement)
 		      maxabsdmatelement = absval;
-		    absval = std::fabs(dens[c*n+d]);
+		    absval = template_blas_fabs(dens[c*n+d]);
 		    if(absval > maxabsdmatelement)
 		      maxabsdmatelement = absval;
 		  }
 		if(K != NULL)
 		  {
-		    absval = std::fabs(dens[a*n+c]);
+		    absval = template_blas_fabs(dens[a*n+c]);
 		    if(absval > maxabsdmatelement)
 		      maxabsdmatelement = absval;
-		    absval = std::fabs(dens[a*n+d]);
+		    absval = template_blas_fabs(dens[a*n+d]);
 		    if(absval > maxabsdmatelement)
 		      maxabsdmatelement = absval;
-		    absval = std::fabs(dens[b*n+c]);
+		    absval = template_blas_fabs(dens[b*n+c]);
 		    if(absval > maxabsdmatelement)
 		      maxabsdmatelement = absval;
-		    absval = std::fabs(dens[b*n+d]);
+		    absval = template_blas_fabs(dens[b*n+d]);
 		    if(absval > maxabsdmatelement)
 		      maxabsdmatelement = absval;
 		  }
 	      } // END FOR i j get largest dmat element
 
-	  int cluster_i_start = chunkList_1[chunk_i].clusterStartIndex;
-	  int clusterCount1 = chunkList_1[chunk_i].noOfClusters;
+	  int cluster_i_start = batchList_1[batch_i].clusterStartIndex;
+	  int clusterCount1 = batchList_1[batch_i].noOfClusters;
 	  for(int cluster_i = cluster_i_start; cluster_i < cluster_i_start + clusterCount1; cluster_i++)
 	    {
-	      int cluster_j_start = chunkList_2[chunk_j].clusterStartIndex;
-	      int clusterCount2 = chunkList_2[chunk_j].noOfClusters;
+	      int cluster_j_start = batchList_2[batch_j].clusterStartIndex;
+	      int clusterCount2 = batchList_2[batch_j].noOfClusters;
 	      int cluterIndexEnd2 = cluster_j_start + clusterCount2;
-	      if(interactionWithSelf == 1 && chunk_i == chunk_j)
+	      if(interactionWithSelf == 1 && batch_i == batch_j)
 		cluster_j_start = cluster_i;
 	      for(int cluster_j = cluster_j_start; cluster_j < cluterIndexEnd2; cluster_j++)
 		{
@@ -168,11 +177,11 @@ get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 		  ergo_real alphaproduct = alpha_1 * alpha_2;
 		  ergo_real alpha_0 = alphaproduct / alphasum;
 
-		  ergo_real resultPreFactor = twoTimesPiToPow5half / (alphaproduct*std::sqrt(alphasum));
+		  ergo_real resultPreFactor = twoTimesPiToPow5half / (alphaproduct*template_blas_sqrt(alphasum));
 
 		  for(int group_i = group_i_start; group_i < group_i_end; group_i++)
 		    {
-		      if(interactionWithSelf == 1 && chunk_i == chunk_j && cluster_i == cluster_j)
+		      if(interactionWithSelf == 1 && batch_i == batch_j && cluster_i == cluster_j)
 			group_j_start = group_i;
 		      for(int group_j = group_j_start; group_j < group_j_end; group_j++)
 			{
@@ -222,7 +231,7 @@ get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 			      int monomialIndex_1 = minimalDistrList_1[i].monomialIndex;
 			      int j_start = groupList_2[group_j].startIndex;
 			      int j_end = j_start + groupList_2[group_j].distrCount;
-			      if(interactionWithSelf == 1 && group_j == group_i && chunk_i == chunk_j && cluster_i == cluster_j)
+			      if(interactionWithSelf == 1 && group_j == group_i && batch_i == batch_j && cluster_i == cluster_j)
 				{
 				  // take care of case i = j separately
 				  ergo_real integralValue = primitiveIntegralList[monomialIndex_1*noOfMonomials_2+monomialIndex_1];
@@ -249,16 +258,16 @@ get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 	  for(int idx_1 = 0; idx_1 < noOfBasisFuncPairs_1; idx_1++)
 	    for(int idx_2 = 0; idx_2 < noOfBasisFuncPairs_2; idx_2++)
 	      {
-		int a = basisFuncPairList_1[chunkList_1[chunk_i].basisFuncPairListIndex+idx_1].index_1;
-		int b = basisFuncPairList_1[chunkList_1[chunk_i].basisFuncPairListIndex+idx_1].index_2;
-		int c = basisFuncPairList_2[chunkList_2[chunk_j].basisFuncPairListIndex+idx_2].index_1;
-		int d = basisFuncPairList_2[chunkList_2[chunk_j].basisFuncPairListIndex+idx_2].index_2;
+		int a = basisFuncPairList_1[batchList_1[batch_i].basisFuncPairListIndex+idx_1].index_1;
+		int b = basisFuncPairList_1[batchList_1[batch_i].basisFuncPairListIndex+idx_1].index_2;
+		int c = basisFuncPairList_2[batchList_2[batch_j].basisFuncPairListIndex+idx_2].index_1;
+		int d = basisFuncPairList_2[batchList_2[batch_j].basisFuncPairListIndex+idx_2].index_2;
 		ergo_real integralValueCurr = summedIntegralList[idx_1*noOfBasisFuncPairs_2 + idx_2];
 		
 		if(a == c && b == d)
 		  integralValueCurr *= 2;
 
-		if(std::fabs(integralValueCurr)*maxabsdmatelement < threshold)
+		if(template_blas_fabs(integralValueCurr)*maxabsdmatelement < threshold)
 		  continue;
 
 		if(a != b && c != d && a != c && a != d && b != c && b != d)
@@ -340,8 +349,8 @@ get_JK_contribs_from_2_interacting_boxes(const BasisInfoStruct & basisInfo,
 		  }
 
 	      } // END FOR idx_1 idx_2
-	} // END FOR chunk_j
-    } // END FOR chunk_i
+	} // END FOR batch_j
+    } // END FOR batch_i
 
   return 0;
 }
@@ -543,24 +552,24 @@ compute_JK_single_box(const BasisInfoStruct & basisInfo,
 						    dens,
 
 						    &distr_list_description_list[i].org.minimalDistrList[0],
-						    distr_list_description_list[i].org.groupCount,
+						    distr_list_description_list[i].org.groupList.size(),
 						    &distr_list_description_list[i].org.groupList[0],
 
 						    &distr_list_description_list[j].org.minimalDistrList[0],
-						    distr_list_description_list[j].org.groupCount,
+						    distr_list_description_list[j].org.groupList.size(),
 						    &distr_list_description_list[j].org.groupList[0],
 
 						    &distr_list_description_list[i].org.clusterList[0],
-						    distr_list_description_list[i].org.clusterCount,
+						    distr_list_description_list[i].org.clusterList.size(),
 
 						    &distr_list_description_list[j].org.clusterList[0],
-						    distr_list_description_list[j].org.clusterCount,
+						    distr_list_description_list[j].org.clusterList.size(),
 
-						    &distr_list_description_list[i].org.chunkList[0],
-						    distr_list_description_list[i].org.chunkCount,
+						    &distr_list_description_list[i].org.batchList[0],
+						    distr_list_description_list[i].org.batchList.size(),
 
-						    &distr_list_description_list[j].org.chunkList[0],
-						    distr_list_description_list[j].org.chunkCount,
+						    &distr_list_description_list[j].org.batchList[0],
+						    distr_list_description_list[j].org.batchList.size(),
 
 						    &distr_list_description_list[i].org.basisFuncPairList[0],
 						    &distr_list_description_list[j].org.basisFuncPairList[0],
