@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_2el_boxed.h
+
+    @brief Code for 2-electron integrals, computation of Coulomb (J)
+    and HF exchange (K) matrices using a single box.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef INTEGRALS_2EL_BOXED_HEADER
 #define INTEGRALS_2EL_BOXED_HEADER
 
@@ -35,7 +44,6 @@
 #include "basis_func_pair_list.h"
 #include "csr_matrix.h"
 
-
 int
 compute_JK_single_box(const BasisInfoStruct & basisInfo,
 		      const IntegralInfo & integralInfo,
@@ -44,6 +52,4 @@ compute_JK_single_box(const BasisInfoStruct & basisInfo,
 		      const ergo_real* dens,
 		      ergo_real threshold);
 
-
-
 #endif
