@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,39 +17,30 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_2el_layer.h
+
+    @brief Functions for computing dense Coulomb and HF exchange
+    matrices.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef INTEGRALS_2EL_LAYER_HEADER
 #define INTEGRALS_2EL_LAYER_HEADER
 
 #include "integral_info.h"
 #include "basisinfo.h"
 #include "integrals_2el.h"
-#include "densityfitting.h"
-
-
-int compute_2e_matrix_list(const BasisInfoStruct & basisInfo,
-			   const IntegralInfo & integralInfo,
-			   const JK::ExchWeights & CAM_params,
-			   ergo_real** resultList,
-			   ergo_real** densList,
-			   int noOfMatrices,
-			   const JK::Params& J_K_params);
-
-int compute_2e_matrix_list_difden(const BasisInfoStruct & basisInfo,
-				  const IntegralInfo & integralInfo,
-				  const JK::ExchWeights & CAM_params,
-				  ergo_real** resultList,
-				  ergo_real** densList,
-				  int noOfMatrices,
-				  const JK::Params& J_K_params);
 
 int compute_2e_matrix_exchange(const BasisInfoStruct & basisInfo,
 			       const IntegralInfo & integralInfo,
@@ -58,22 +49,10 @@ int compute_2e_matrix_exchange(const BasisInfoStruct & basisInfo,
 			       ergo_real* dens,
 			       ergo_real threshold);
 
-int compute_2e_matrices_exchange(const BasisInfoStruct & basisInfo,
-				 const IntegralInfo & integralInfo,
-				 const JK::ExchWeights & CAM_params,
-				 int noOfMatrices,
-				 ergo_real** K_list,
-				 ergo_real** D_list,
-				 ergo_real threshold);
-
 int compute_2e_matrix_coulomb(const BasisInfoStruct & basisInfo,
-			      const BasisInfoStruct & basisInfoDensFit,
 			      const IntegralInfo & integralInfo,
 			      ergo_real* J,
 			      ergo_real* dens,
-			      const JK::Params& J_K_params,
-			      DensfitData* U_inverse);
-
-
+			      const JK::Params& J_K_params);
 
 #endif
