@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_2el_repeating.cc
+
+    @brief Functionality for keeping track of certain kinds of
+    integrals that are computed repeatedly, saving the computed values
+    instead of recomputing them.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdlib.h>
 #include <math.h>
 #include <stdio.h>
@@ -36,13 +46,10 @@
 #include <assert.h>
 #include "pi.h"
 #include "integrals_hermite.h"
-
 #include "integrals_2el_repeating.h"
-
 #include "realtype.h"
 
 
-
 IntegratorCase::IntegratorCase(int Nmax_in, 
 			       int noOfMonomials, 
 			       ergo_real exponent_in, 
@@ -110,7 +117,7 @@ IntegratorWithMemory::do_2e_integral(const DistributionSpecStruct* psi)
   ergo_real dx1 = 0;
   ergo_real dx2 = 0;
 
-  ergo_real resultPreFactor = twoTimesPiToPow5half / (alphaproduct*std::sqrt(alphasum));
+  ergo_real resultPreFactor = twoTimesPiToPow5half / (alphaproduct*template_blas_sqrt(alphasum));
 
   int monomialIndex = integralInfo->monomial_info.monomial_index_list[nx][ny][nz];  
 
@@ -129,11 +136,12 @@ IntegratorWithMemory::do_2e_integral(const DistributionSpecStruct* psi)
 	  else
 	    hi = mid;
 	} // END WHILE
-      ergo_real exponentDiff1 = std::fabs(caseList[lo]->exponent - alpha0);
-      if(exponentDiff1 < 1e-11)
+      const ergo_real exponent_diff_limit = 1e-11; // FIXME: SHOULD NOT USE HARD-CODED VALUE HERE!
+      ergo_real exponentDiff1 = template_blas_fabs(caseList[lo]->exponent - alpha0);
+      if(exponentDiff1 < exponent_diff_limit)
 	foundIndex = lo;
-      ergo_real exponentDiff2 = std::fabs(caseList[hi]->exponent - alpha0);
-      if(exponentDiff2 < 1e-11)
+      ergo_real exponentDiff2 = template_blas_fabs(caseList[hi]->exponent - alpha0);
+      if(exponentDiff2 < exponent_diff_limit)
 	foundIndex = hi;
     }
   if(foundIndex >= 0)
