@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_2el_single.cc
+
+    @brief Functionality for computing a single 2-electron integral,
+    for two given primitive Gaussian distributions.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdlib.h>
 #include <math.h>
 #include <stdio.h>
@@ -74,7 +83,7 @@ do_2e_integral_using_symb_info_h(const JK::ExchWeights & CAM_params,
   ergo_real dx1 = psi2->centerCoords[1] - psi1->centerCoords[1];
   ergo_real dx2 = psi2->centerCoords[2] - psi1->centerCoords[2];
 
-  ergo_real resultPreFactor = twoTimesPiToPow5half / (alphaproduct*std::sqrt(alphasum));
+  ergo_real resultPreFactor = twoTimesPiToPow5half / (alphaproduct*template_blas_sqrt(alphasum));
 
   ergo_real primitiveIntegralList_h[noOfMonomials_1*noOfMonomials_2];
   ergo_real primitiveIntegralList_tmp[noOfMonomials_1*noOfMonomials_2];
