@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,16 +17,26 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_2el_utils.cc
+
+    \brief Code for various utilities used by 2-electron integral
+    computation (i.e. computation of J and K matrices).
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #include <string.h>
+#include <stdexcept>
 
 #include "integrals_2el_utils.h"
 #include "integrals_hermite.h"
@@ -36,13 +46,66 @@
 #include "integrals_general.h"
 
 
+/* ----- ResultMatContrib implementation ----- */
+
+ResultMatContrib::ResultMatContrib() {
+  currVecIndex = 0;
+  currContribCount = 0;
+  indexInCurrVec = 0;
+  currVecReservedSize = 20;
+  for(int i = 0; i < nVectorsMax; i++)
+    vList[i] = 0;
+  vList[currVecIndex] = new std::vector<RowColVal>;
+  vList[currVecIndex]->reserve(currVecReservedSize);
+}
+
+ResultMatContrib::~ResultMatContrib() {
+  for(int i = 0; i <= currVecIndex; i++)
+    delete vList[i];
+}
+
+void ResultMatContrib::addContrib(int row, int col, ergo_real value) {
+  // Check if there is room in current vector
+  assert(indexInCurrVec <= currVecReservedSize);
+  if(indexInCurrVec == currVecReservedSize) {
+    // Current vector is full. Reserve space in next vector and use it.
+    currVecIndex++;
+    if(currVecIndex >= nVectorsMax)
+      throw std::runtime_error("Error in ResultMatContrib::addContrib: (currVecIndex >= nVectorsMax).");
+    indexInCurrVec = 0;
+    currVecReservedSize *= 2;
+    vList[currVecIndex] = new std::vector<RowColVal>;
+    vList[currVecIndex]->reserve(currVecReservedSize);
+  }
+  RowColVal tmp = {row, col, value};
+  vList[currVecIndex]->push_back(tmp);
+  indexInCurrVec++;
+  currContribCount++;
+}
+
+const ResultMatContrib::RowColVal & ResultMatContrib::fetchNextContrib(int & currVecIndexForFetch, int & indexInCurrVecForFetch) const {
+  assert(currVecIndexForFetch >= 0 && currVecIndexForFetch < nVectorsMax);
+  int currVecSize = vList[currVecIndexForFetch]->size();
+  assert(indexInCurrVecForFetch >= 0 && indexInCurrVecForFetch < currVecSize);
+  const RowColVal & result = (*vList[currVecIndexForFetch])[indexInCurrVecForFetch];
+  indexInCurrVecForFetch++;
+  if(indexInCurrVecForFetch == currVecSize) {
+    currVecIndexForFetch++;
+    indexInCurrVecForFetch = 0;
+  }
+  return result;
+}
+
+/* ----- End of ResultMatContrib implementation ----- */
+
+
 ergo_real
 get_max_abs_vector_element(int n, const ergo_real* vector)
 {
   ergo_real maxabs = 0;
   for(int i = 0; i < n; i++)
     {
-      ergo_real absval = std::fabs(vector[i]);
+      ergo_real absval = template_blas_fabs(vector[i]);
       if(absval > maxabs)
 	maxabs = absval;
     }
@@ -52,27 +115,10 @@ get_max_abs_vector_element(int n, const ergo_real* vector)
 
 
 
-distr_list_description_struct::distr_list_description_struct():
-  totCharge(0)
+box_struct::box_struct()
 {
-  memset(multipolePoint, 0, 3*sizeof(ergo_real));
-  multipole.degree = -1;
-  multipole.noOfMoments = 0;
-  memset(multipole.momentList, 0, MAX_NO_OF_MOMENTS_PER_MULTIPOLE*sizeof(ergo_real));
-  memset(maxMomentVectorNormForDistrsList, 0, (MAX_MULTIPOLE_DEGREE_BASIC+1)*sizeof(ergo_real));
-}
-
-
-box_struct::box_struct() : 
-  noOfBasisFuncs(0), 
-  noOfRelevantBasisFuncGroups(0), 
-  basisFuncGroupInfoList(NULL), 
-  largestCSfactor(0)
-{
-  memset(multipolePoint, 0, 3*sizeof(ergo_real));
-  memset(branchIndexList, 0, MAX_NO_OF_BRANCHES*sizeof(int));
-  memset(branchCountList, 0, MAX_NO_OF_BRANCHES*sizeof(int));
-  memset(multipoleEuclideanNormList, 0, (MAX_MULTIPOLE_DEGREE_BASIC+1)*sizeof(ergo_real));
+  memset(branchIndexListForJ, 0, MAX_NO_OF_BRANCHES*sizeof(int));
+  memset(branchCountListForJ, 0, MAX_NO_OF_BRANCHES*sizeof(int));
 }
 
 
@@ -82,7 +128,7 @@ allocate_buffers_needed_by_integral_code(const IntegralInfo & integralInfo,
 					 int basisFuncListCount_max,
 					 JK_contribs_buffer_struct* bufferStruct)
 {
-  bufferStruct->summedIntegralList = new ergo_real[MAX_NO_OF_BASIS_FUNC_PAIRS_PER_CHUNK*MAX_NO_OF_BASIS_FUNC_PAIRS_PER_CHUNK];
+  bufferStruct->summedIntegralList = new ergo_real[MAX_NO_OF_BASIS_FUNC_PAIRS_PER_BATCH*MAX_NO_OF_BASIS_FUNC_PAIRS_PER_BATCH];
   bufferStruct->primitiveIntegralList = new ergo_real[maxNoOfMonomials*maxNoOfMonomials];
   bufferStruct->primitiveIntegralList_work = new ergo_real[maxNoOfMonomials*maxNoOfMonomials];
   if(basisFuncListCount_max > 0)
@@ -178,7 +224,7 @@ erfc_inverse(ergo_real x, ergo_real requested_accuracy)
   if(template_blas_erfc(y_max) > x)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in erfc_inverse: (erfc(y_max) > x)");
-      exit(0);
+      exit(EXIT_FAILURE);
     }
   int count = 0;
   ergo_real y = 0;
@@ -193,7 +239,7 @@ erfc_inverse(ergo_real x, ergo_real requested_accuracy)
       if(count > 222)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in erfc_inverse: too many iterations.");
-	  exit(0);
+	  exit(EXIT_FAILURE);
 	}
     } // END WHILE requested accuracy not reached
   return y;
@@ -213,7 +259,7 @@ compute_extent_for_list_of_distributions(int n,
   for(int i = 0; i < n; i++)
     {
       ergo_real erfc_inverse_value = erfc_inverse(threshold / (distrList[i].limitingFactor * maxLimitingFactor * maxabsDmatelement), requested_erfcinv_accuracy);
-      distrList[i].distr.extent = erfc_inverse_value / std::sqrt(distrList[i].distr.exponent);
+      distrList[i].distr.extent = erfc_inverse_value / template_blas_sqrt(distrList[i].distr.exponent);
     }
 }
 
@@ -287,7 +333,7 @@ get_list_of_labeled_distrs_maxLimitingFactor(const BasisInfoStruct & basisInfo,
 	  ergo_real dx = basisInfo.basisFuncList[i].centerCoords[0] - basisInfo.basisFuncList[j].centerCoords[0];
 	  ergo_real dy = basisInfo.basisFuncList[i].centerCoords[1] - basisInfo.basisFuncList[j].centerCoords[1];
 	  ergo_real dz = basisInfo.basisFuncList[i].centerCoords[2] - basisInfo.basisFuncList[j].centerCoords[2];
-	  ergo_real distance = std::sqrt(dx*dx + dy*dy + dz*dz);
+	  ergo_real distance = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
 	  if(distance > basisFuncExtentList[i] + basisFuncExtentList[j])
 	    continue;
 	  
@@ -306,7 +352,7 @@ get_list_of_labeled_distrs_maxLimitingFactor(const BasisInfoStruct & basisInfo,
 	    }
 	  for(int k = 0; k < n_psi; k++)
 	    {
-	      ergo_real limitingFactor = std::sqrt(integrator.do_2e_integral(&psi_list[k]));
+	      ergo_real limitingFactor = template_blas_sqrt(integrator.do_2e_integral(&psi_list[k]));
 	      if(limitingFactor > maxLimitingFactor)
 		maxLimitingFactor = limitingFactor;
 	    } // END FOR k
@@ -392,7 +438,7 @@ get_list_of_labeled_distrs(const BasisInfoStruct & basisInfo,
 	  ergo_real dx = basisInfo.basisFuncList[i].centerCoords[0] - basisInfo.basisFuncList[j].centerCoords[0];
 	  ergo_real dy = basisInfo.basisFuncList[i].centerCoords[1] - basisInfo.basisFuncList[j].centerCoords[1];
 	  ergo_real dz = basisInfo.basisFuncList[i].centerCoords[2] - basisInfo.basisFuncList[j].centerCoords[2];
-	  ergo_real distance = std::sqrt(dx*dx + dy*dy + dz*dz);
+	  ergo_real distance = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
 	  if(distance > basisFuncExtentList[i] + basisFuncExtentList[j])
 	    continue;
 	  
@@ -416,7 +462,7 @@ get_list_of_labeled_distrs(const BasisInfoStruct & basisInfo,
 	    }
 	  for(int k = 0; k < n_psi; k++)
 	    {
-	      ergo_real limitingFactor = std::sqrt(integrator.do_2e_integral(&psi_list[k]));
+	      ergo_real limitingFactor = template_blas_sqrt(integrator.do_2e_integral(&psi_list[k]));
 	      if(limitingFactor*maxLimitingFactor*maxDensityMatrixElement > threshold)
 		{
 		  if(maxCountDistrs > 0 && distrCount >= maxCountDistrs)
