@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,63 +17,72 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_2el_utils.h
+
+    \brief Code for various utilities used by 2-electron integral
+    computation (i.e. computation of J and K matrices).
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #ifndef INTEGRALS_2EL_UTILS_HEADER
 #define INTEGRALS_2EL_UTILS_HEADER
 
 
 #include "organize_distrs.h"
+#include "organize_distrs_mm.h"
 #include "box_system.h"
 
 
-struct distr_list_description_struct {
-  distr_org_struct org;
-  ergo_real totCharge;
-  ergo_real multipolePoint[3];
-  multipole_struct_large multipole;
-  ergo_real maxMomentVectorNormForDistrsList[MAX_MULTIPOLE_DEGREE_BASIC+1];
-  distr_list_description_struct();
-};
-
-
 #define MAX_NO_OF_BRANCHES 10
 
 
-typedef struct
-{
-  int basisFuncGroupIndex;
-  ergo_real max_CS_factor;
-  ergo_real maxMomentVectorNormList[MAX_MULTIPOLE_DEGREE_BASIC+1];
-  int maxMultipoleDegree;
-} basis_func_group_info_for_box;
+/* Struct used for storing result matrix contributions when e.g. the J
+   kernel routine is used with Chunks and Tasks. */
+struct ResultMatContrib {
+  struct RowColVal {
+    int row;
+    int col;
+    ergo_real value;
+  };
+  static const int nVectorsMax = 40;
+  int currVecIndex;
+  int currContribCount;
+  int indexInCurrVec;
+  int currVecReservedSize;
+  std::vector<RowColVal> * vList[nVectorsMax];
+  ResultMatContrib();
+  ~ResultMatContrib();
+  void addContrib(int row, int col, ergo_real value);
+  const RowColVal & fetchNextContrib(int & currVecIndexForFetch, int & indexInCurrVecForFetch) const;
+};
 
 
 struct box_struct {
+  // General stuff
   box_struct_basic basicBox;
+  // Stuff related to J
+  distr_list_description_struct branchListForJ[MAX_NO_OF_BRANCHES];
+  int branchIndexListForJ[MAX_NO_OF_BRANCHES];
+  int branchCountListForJ[MAX_NO_OF_BRANCHES];
+  // Stuff related to K
   distr_list_description_struct distrListForK;
-  distr_list_description_struct branchList[MAX_NO_OF_BRANCHES];
-  ergo_real multipolePoint[3];
-  int branchIndexList[MAX_NO_OF_BRANCHES];
-  int branchCountList[MAX_NO_OF_BRANCHES];
-  int noOfBasisFuncs;
-  int noOfRelevantBasisFuncGroups;
-  basis_func_group_info_for_box* basisFuncGroupInfoList;
-  ergo_real largestCSfactor; // Largest Cauchy-Schwartz factor in box
-  ergo_real multipoleEuclideanNormList[MAX_MULTIPOLE_DEGREE_BASIC+1];
+  // Constructor
   box_struct();
 };
 
 
-typedef struct
-{
+struct JK_contribs_buffer_struct {
   ergo_real* summedIntegralList;
   ergo_real* primitiveIntegralList;
   ergo_real* primitiveIntegralList_work;
@@ -81,9 +90,7 @@ typedef struct
   ergo_real* partial_dmat_2; // used in non-symmetric case
   ergo_real* partial_K_1;
   ergo_real* partial_K_2; // used in non-symmetric case
-} JK_contribs_buffer_struct;
-
-
+};
 
 
 ergo_real get_max_abs_vector_element(int n, const ergo_real* vector);
