@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,23 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integrals_general.cc
+
+    @brief General functionality related to computation of integrals
+    involving Gaussian basis functions.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
 
 /* Written by Elias Rudberg, KTH, Stockholm */
 #include <stdlib.h>
@@ -120,11 +128,11 @@ get_product_simple_prims(const DistributionSpecStruct& primA_in,
       ergo_real temp = primA->centerCoords[k] - primB->centerCoords[k];
       sum += temp * temp;
     } /* END FOR k */
-  CxCyCz = std::exp(-primA->exponent * primB->exponent * 
+  CxCyCz = template_blas_exp(-primA->exponent * primB->exponent * 
 	       sum / (primA->exponent + primB->exponent));
 
   // FIXME: do this screening properly!
-  if(std::fabs(CxCyCz) < threshold)
+  if(template_blas_fabs(CxCyCz) < threshold)
     return 0;
 
   AiAj = primA->coeff * primB->coeff;
@@ -216,7 +224,7 @@ get_product_simple_prims(const DistributionSpecStruct& primA_in,
 	    {
 	      ergo_real newCoeff = AiAj * CxCyCz * poly0[k] * poly1[l] * poly2[m];
 
-	      ergo_real sqrtValue = std::sqrt(pi / alphaNew);
+	      ergo_real sqrtValue = template_blas_sqrt(pi / alphaNew);
 	      ergo_real absvalue = newCoeff * sqrtValue * sqrtValue * sqrtValue;
 	      if(absvalue < 0) absvalue *= -1;
 
@@ -306,26 +314,23 @@ get_product_simple_primitives(const BasisInfoStruct & basisInfoA, int iA,
 }
 
 
-ergo_real 
-compute_integral_of_simple_prim(DistributionSpecStruct* distr)
-{
+ergo_real
+compute_integral_of_simple_prim(const DistributionSpecStruct & distr) {
   ergo_real result, alpha;
   int i, n;
-  if( ((distr->monomialInts[0]|
-	distr->monomialInts[1]|
-	distr->monomialInts[2]) & 1) == 1) /* odd integrals disappear */
+  if( ((distr.monomialInts[0]|
+	distr.monomialInts[1]|
+	distr.monomialInts[2]) & 1) == 1) /* odd integrals disappear */
     return 0;
-
-  alpha = distr->exponent;
-  result = distr->coeff * std::pow((ergo_real)pi/alpha, (ergo_real)1.5);
-  ergo_real twoA = 2*alpha;  
-  for(i = 0; i < 3; i++)
-    {
-      n = distr->monomialInts[i];
-      for(int j=0; j<n; j+=2)
-	result *= (j+1)/twoA;
-    } /* END FOR i */
-  return result;  
+  alpha = distr.exponent;
+  result = distr.coeff * template_blas_pow((ergo_real)pi/alpha, (ergo_real)1.5);
+  ergo_real twoA = 2*alpha;
+  for(i = 0; i < 3; i++) {
+    n = distr.monomialInts[i];
+    for(int j=0; j<n; j+=2)
+      result *= (j+1)/twoA;
+  } /* END FOR i */
+  return result;
 }
 
 
@@ -355,7 +360,7 @@ get_largest_simple_integral(const BasisInfoStruct & basisInfo)
 	  distr.monomialInts[0] = 0;
 	  distr.monomialInts[1] = 0;
 	  distr.monomialInts[2] = 0;
-	  ergo_real a = compute_integral_of_simple_prim(&distr);
+	  ergo_real a = compute_integral_of_simple_prim(distr);
 	  if(a > A)
 	    A = a;
 	} // END FOR j
@@ -379,7 +384,7 @@ ergo_real get_max_basis_func_abs_value(const BasisInfoStruct & basisInfo) {
     int start = basisFunc->simplePrimitiveIndex;
     for(int j = 0; j < nPrims; j++) {
       DistributionSpecStruct* prim = &basisInfo.simplePrimitiveList[start + j];
-      ergo_real valueAtCenter = std::fabs(prim->coeff); // exp(0) = 1
+      ergo_real valueAtCenter = template_blas_fabs(prim->coeff); // exp(0) = 1
       if(valueAtCenter > maxValue)
 	maxValue = valueAtCenter;
     } // END FOR j
@@ -406,7 +411,7 @@ get_basis_func_extent_list(const BasisInfoStruct & basisInfo, ergo_real* basisFu
       for(int j = 0; j < nPrims; j++)
 	{
 	  DistributionSpecStruct* prim = &basisInfo.simplePrimitiveList[start + j];
-	  ergo_real currExtent = std::sqrt((1.0 / prim->exponent) * std::log(std::fabs(prim->coeff) / maxAbsValue));
+	  ergo_real currExtent = template_blas_sqrt((1.0 / prim->exponent) * template_blas_log(template_blas_fabs(prim->coeff) / maxAbsValue));
 	  if(currExtent > maxExtent)
 	    maxExtent = currExtent;
 	} // END FOR j
