@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,27 +17,29 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /** @file integrals_hermite.cc
 
-    \brief Code for computation of Coulomb integrals of Hermite
+    @brief Code for computation of Coulomb integrals of Hermite
     Gaussians, using the the McMurchie-Davidson scheme as described in
     the book "Molecular electronic-structure theory" by Trygve
     Helgaker, Poul Jorgensen, and Jeppe Olsen.
 
-    @author: Elias Rudberg <em>responsible</em>. 
+    @author: Elias Rudberg <em>responsible</em>
 */
 
 #include "integrals_hermite.h"
 #include "boysfunction.h"
+#include "mat_gblas.h"
 #include <cmath>
 #include <stdio.h>
 
@@ -77,24 +79,24 @@ get_related_integrals_hermite(const IntegralInfo & integralInfo,
       ergo_real BoysFunctionList_mod[Nmax+1];
       ergo_real mu = paramsCAM.mu;
       ergo_real v1_squared = mu * mu / (mu * mu + alpha0);
-      ergo_real v1 = std::sqrt(v1_squared);
+      ergo_real v1 = template_blas_sqrt(v1_squared);
       /* Prepare BoysFunctionList_std */
       /* Use downward recursion to get Boys function values */
       ergo_real arg1 =  alpha0 * R_12_squared;
-      ergo_real expMinusArg1 = std::exp(-arg1);
+      ergo_real expMinusArg1 = template_blas_exp(-arg1);
       BoysFunctionList_std[Nmax] = integralInfo.BoysFunction(Nmax, arg1);
       for(int i = Nmax-1; i >= 0; i--)
 	BoysFunctionList_std[i] = (2*arg1*BoysFunctionList_std[i+1] + expMinusArg1) / (2*i+1);
       /* Prepare BoysFunctionList_mod */
       /* Use downward recursion to get Boys function values */
       ergo_real arg2 =  alpha0 * R_12_squared * v1_squared;
-      ergo_real expMinusArg2 = std::exp(-arg2);
+      ergo_real expMinusArg2 = template_blas_exp(-arg2);
       BoysFunctionList_mod[Nmax] = integralInfo.BoysFunction(Nmax, arg2);
       for(int i = Nmax-1; i >= 0; i--)
 	BoysFunctionList_mod[i] = (2*arg2*BoysFunctionList_mod[i+1] + expMinusArg2) / (2*i+1);
       // rescale
       for(int i = 0; i <= Nmax; i++)
-	BoysFunctionList_mod[i] *= v1 * std::pow(v1_squared, i);  /* TODO: avoid using pow() here! */
+	BoysFunctionList_mod[i] *= v1 * template_blas_pow(v1_squared, (ergo_real)i);  /* TODO: avoid using pow() here! */
       // add BoysFunctionList_std and BoysFunctionList_mod using weights given by cam_param_alpha and cam_param_beta
       for(int i = 0; i <= Nmax; i++)
 	BoysList[i] = paramsCAM.alpha * BoysFunctionList_std[i] + paramsCAM.beta * BoysFunctionList_mod[i];
@@ -107,7 +109,7 @@ get_related_integrals_hermite(const IntegralInfo & integralInfo,
       BoysList[Nmax] = integralInfo.BoysFunction(Nmax, arg);
       if(Nmax > 0)
 	{
-	  ergo_real expMinusArg = std::exp(-arg); 
+	  ergo_real expMinusArg = template_blas_exp(-arg); 
 	  for(int i = Nmax-1; i >= 0; i--)
 	    BoysList[i] = (2*arg*BoysList[i+1] + expMinusArg) / (2*i+1);
 	}
