@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,71 +17,40 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file mm_limit_table.cc
+
+    @brief MMLimitTable class used to predict the magnitude of
+    contributions when using truncated multipole expansions.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdio.h>
 #include <vector>
 #include <limits>
+#include <stdexcept>
 
 #include "mm_limit_table.h"
 
 #include "matrix_norm.h"
 #include "template_blas_basicmath.h"
+#include "multipole.h"
 
 
-const ergo_real HUGE_REAL_NUMBER = template_blas_sqrt(std::numeric_limits<ergo_real>::max()) / 100;
-const int NO_OF_STEPS_PER_RANGE = 5;
+const ergo_real HUGE_REAL_NUMBER = template_blas_sqrt(template_blas_get_num_limit_max<ergo_real>()) / 100;
 const ergo_real INITIAL_STEP = 0.5;
-const ergo_real RANGE_STEP_DIFF_FACTOR = 0.15;
-const int NO_OF_RANGES = 40;
-
-
-
-typedef struct
-{
-  ergo_real x[MAX_MULTIPOLE_DEGREE+1][MAX_MULTIPOLE_DEGREE_BASIC+1];
-} interaction_matrix_limit_struct;
-
-typedef struct
-{
-  ergo_real startDistance;
-  ergo_real maxDistance;
-  ergo_real step;
-  std::vector<interaction_matrix_limit_struct> list;
-} interaction_matrix_limit_range_struct;
-
-
-class MMLimitTable {
-  const interaction_matrix_limit_struct & get_x_from_distance(ergo_real distance) const;
- public:
-  MMLimitTable();
-  ~MMLimitTable();
-  void init(ergo_real maxDistance);
-  ergo_real get_max_abs_mm_contrib(int degree1,
-				   const ergo_real* maxMomentVectorNormList1,
-				   int degree2,
-				   const ergo_real* maxMomentVectorNormList2,
-				   ergo_real distance) const;
-  int get_minimum_multipole_degree_needed(ergo_real distance,
-					  const multipole_struct_large* boxMultipole, 
-					  int maxDegreeForDistrs, 
-					  const ergo_real* maxMomentVectorNormForDistrsList, 
-					  ergo_real threshold) const;
-  int noOfRangesUsed;
-  interaction_matrix_limit_range_struct rangeList[NO_OF_RANGES];
-};
-
-
-static MMLimitTable global_mmLimitTable;
-
-
+const ergo_real RANGE_STEP_DIFF_FACTOR = 0.25;
+const ergo_real MAX_ALLOWED_DISTANCE = 1e5;
 
 
 MMLimitTable::MMLimitTable()
@@ -93,10 +62,10 @@ MMLimitTable::~MMLimitTable()
 {
 }
 
-void MMLimitTable::init(ergo_real maxDistance_input)
+void MMLimitTable::inittt(const MultipolePrepManager & multipolePrep)
 {
+  ergo_real maxDistance_input = MAX_ALLOWED_DISTANCE;
   ergo_real maxDistance = maxDistance_input + 0.1; // ELIAS NOTE 2013-11-29: add small value here to avoid problems at exactly the maxDistance_input distance.
-  init_multipole_code();
   ergo_real r = 0;
   ergo_real currStep = INITIAL_STEP;
   int rangeCount = 0;
@@ -124,14 +93,13 @@ void MMLimitTable::init(ergo_real maxDistance_input)
   dxlist[6][0] = 1;
   dxlist[6][1] = 1;
   dxlist[6][2] = 1;
-  MMInteractor interactor;
+  MMInteractor interactor(multipolePrep);
 
   while(r < maxDistance)
     {
       interaction_matrix_limit_range_struct & range = rangeList[rangeCount];
       range.startDistance = r;
       range.step = currStep;
-      range.list.resize(NO_OF_STEPS_PER_RANGE);
       for(int i = 0; i < NO_OF_STEPS_PER_RANGE; i++)
 	{
 	  r = range.startDistance + i*range.step;
@@ -146,7 +114,7 @@ void MMLimitTable::init(ergo_real maxDistance_input)
 	      ergo_real dx = dxlist[randloop][0];
 	      ergo_real dy = dxlist[randloop][1];
 	      ergo_real dz = dxlist[randloop][2];
-	      ergo_real norm = std::sqrt(dx*dx+dy*dy+dz*dz);
+	      ergo_real norm = template_blas_sqrt(dx*dx+dy*dy+dz*dz);
 	      dx *= r / norm;
 	      dy *= r / norm;
 	      dz *= r / norm;
@@ -200,8 +168,10 @@ void MMLimitTable::init(ergo_real maxDistance_input)
 }
 
 
-const interaction_matrix_limit_struct & MMLimitTable::get_x_from_distance(ergo_real distance) const
+const MMLimitTable::interaction_matrix_limit_struct & MMLimitTable::get_x_from_distance(ergo_real distance) const
 {
+  if(distance < 0)
+    throw "Error in MMLimitTable::get_x_from_distance(): (distance < 0).";
   int rangeIndex = 0;
   while(rangeList[rangeIndex].maxDistance <= distance) // ELIAS NOTE 2013-11-29: changed from "<" to "<=" here to fix problem found by Anastasia, for case when (distanceLeft / range.step) divides evenly.
     {
@@ -215,7 +185,7 @@ const interaction_matrix_limit_struct & MMLimitTable::get_x_from_distance(ergo_r
   if(i < 0 || i >= NO_OF_STEPS_PER_RANGE)
     {
       if(i < 0)
-	throw "error in MMLimitTable::get_x_from_distance: i <= 0";
+	throw "error in MMLimitTable::get_x_from_distance: i < 0";
       throw "error in MMLimitTable::get_x_from_distance: i >= NO_OF_STEPS_PER_RANGE";
     }
   const interaction_matrix_limit_struct & x = range.list[i];
@@ -275,50 +245,21 @@ int MMLimitTable::get_minimum_multipole_degree_needed(ergo_real distance,
   return degreeNeeded;
 }
 
-
-
-
-
-
-void 
-mm_limits_init(ergo_real maxDistance)
-{
-  if(global_mmLimitTable.noOfRangesUsed > 0)
-    {
-      // We have initialized before, we can skip doing it again unless distance is too large.
-      if(global_mmLimitTable.rangeList[global_mmLimitTable.noOfRangesUsed-1].maxDistance >= maxDistance)
-	return;
-    }
-  global_mmLimitTable.init(maxDistance);
+void MMLimitTable::write_to_buffer ( char * dataBuffer, size_t const bufferSize ) const {
+  char* p = dataBuffer;
+  if(bufferSize < get_size())
+    throw std::runtime_error("Error in MMLimitTable::write_to_buffer: bufferSize too small.");
+  memcpy(p, this, sizeof(MMLimitTable));
 }
 
-
-ergo_real 
-mm_limits_get_max_abs_mm_contrib(int degree1,
-				 const ergo_real* maxMomentVectorNormList1,
-				 int degree2,
-				 const ergo_real* maxMomentVectorNormList2,
-				 ergo_real distance)
-{
-  return global_mmLimitTable.get_max_abs_mm_contrib(degree1,
-						    maxMomentVectorNormList1,
-						    degree2,
-						    maxMomentVectorNormList2,
-						    distance);
+size_t MMLimitTable::get_size() const {
+  return sizeof(MMLimitTable);
 }
 
-
-int 
-mm_limits_get_minimum_multipole_degree_needed(ergo_real distance,
-					      const multipole_struct_large* boxMultipole, 
-					      int maxDegreeForDistrs, 
-					      const ergo_real* maxMomentVectorNormForDistrsList, 
-					      ergo_real threshold)
-{
-  return global_mmLimitTable.get_minimum_multipole_degree_needed(distance,
-								 boxMultipole, 
-								 maxDegreeForDistrs, 
-								 maxMomentVectorNormForDistrsList, 
-								 threshold);
+void MMLimitTable::assign_from_buffer ( char const * dataBuffer, size_t const bufferSize) {
+  const char* p = dataBuffer;
+  if(bufferSize < sizeof(MMLimitTable))
+    throw std::runtime_error("Error in MMLimitTable::assign_from_buffer: bufferSize too small.");
+  memcpy(this, p, sizeof(MMLimitTable));
 }
 
