@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file monomial_info.cc
+
+    @brief Code used to organize monomials; contributions like x, y,
+    z, xy, xz etc. that can be used to describe e.g. basis functions
+    and basis function products.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include "monomial_info.h"
 #include <assert.h>
 #include <stdexcept>
@@ -100,7 +110,7 @@ monomial_info_struct::monomial_info_struct(const monomial_info_struct & other)
 void monomial_info_struct::write_to_buffer ( char * dataBuffer, size_t const bufferSize ) const {
   char* p = dataBuffer;
   if(bufferSize < get_size())
-    throw std::runtime_error("Error: bufferSize too small.");
+    throw std::runtime_error("Error in monomial_info_struct::write_to_buffer: bufferSize too small.");
   // noOfMonomialsTot
   memcpy(p, &noOfMonomialsTot, sizeof(int));
   p += sizeof(int);
