@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,21 +17,22 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /** @file multipole.cc
 
-    \brief Code for computing multipole moments, and multipole
+    @brief Code for computing multipole moments, and multipole
     interaction and translation matrices.
 
-    @author: Elias Rudberg <em>responsible</em>. 
+    @author: Elias Rudberg <em>responsible</em>
 */
 
 #include <memory.h>
@@ -44,64 +45,6 @@
 #include "integrals_general.h"
 
 
-typedef struct {
-  int l;
-  int m;
-} l_m_struct;
-
-static int global_initialized_flag = 0;
-static ergo_real global_lm_factor_list[MAX_MULTIPOLE_DEGREE+1][MAX_MULTIPOLE_DEGREE+1];
-static l_m_struct global_l_m_list[MAX_NO_OF_MOMENTS_PER_MULTIPOLE];
-
-static ergo_real
-slow_factorial(int n) {
-  if(n == 0)
-    return 1;
-  return n * slow_factorial(n-1);
-}
-
-static ergo_real
-get_lm_factor_slow(int l, int m) {
-  return 1.0 / std::sqrt(slow_factorial(l-m)*slow_factorial(l+m));
-}
-
-static ergo_real
-get_lm_factor(int l, int m) {
-  if(m >= 0)
-    return global_lm_factor_list[l][m];
-  else
-    return global_lm_factor_list[l][-m];
-}
-
-static void
-get_l_m_from_index(int index, int* result_l, int* result_m) {
-  int l = 0;
-  int count = 0;
-  while(count < index) {
-    l++;
-    count += 2 * l + 1;
-  }
-  int m = l + index - count;
-  *result_l = l;
-  *result_m = m;
-}
-
-
-int 
-init_multipole_code() {
-  if(global_initialized_flag)
-    return 0;
-  for(int l = 0; l <= MAX_MULTIPOLE_DEGREE; l++)
-    for(int m = 0; m <= l; m++)
-      global_lm_factor_list[l][m] = get_lm_factor_slow(l, m);
-  global_initialized_flag = 1;
-  // initialize global_l_m_list
-  for(int A = 0; A < MAX_NO_OF_MOMENTS_PER_MULTIPOLE; A++)
-    get_l_m_from_index(A, &global_l_m_list[A].l, &global_l_m_list[A].m);
-  return 0;
-}
-
-
 int
 compute_multipole_moments(const IntegralInfo& integralInfo, 
 			  const DistributionSpecStruct* distr,
@@ -138,64 +81,57 @@ compute_multipole_moments(const IntegralInfo& integralInfo,
       if(j > 0) {
 	if(termDegree != savedDegree)
 	  throw "Error in compute_multipole_moments: (termDegree != savedDegree).";
-    }
-    savedDegree = termDegree;
-    prim.coeff *= curr->termList[j].coeff;
-    sum += compute_integral_of_simple_prim(&prim);
-  } // END FOR j
-
-  if(savedDegree <= distrDegree) {
-    result->noOfMoments++;
-    result->momentList[i] = curr->scaledSolidHarmonicPrefactor * sum;
-  }
-  else
-    {
-      // now the computed moment should be zero
-      if(std::fabs(sum) > 1e-3) {
-	do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error: computed moment not zero when (savedDegree > distrDegree)");
-	do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "computed moment : %22.11f", (double)sum);
-	exit(0);
       }
-      result->momentList[i] = 0;
-    }
-      
-  funcCountCurr_l++;
-  if(funcCountCurr_l == 2*l+1) {
-    l++;
-    funcCountCurr_l = 0;
-  }
-} // END FOR i
+      savedDegree = termDegree;
+      prim.coeff *= curr->termList[j].coeff;
+      sum += compute_integral_of_simple_prim(prim);
+    } // END FOR j
 
-if(distrDegree > MAX_MULTIPOLE_DEGREE_BASIC)
-  {
+    if(savedDegree <= distrDegree) {
+      result->noOfMoments++;
+      result->momentList[i] = curr->scaledSolidHarmonicPrefactor * sum;
+    }
+    else
+      {
+	// now the computed moment should be zero
+	if(template_blas_fabs(sum) > 1e-3) {
+	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error: computed moment not zero when (savedDegree > distrDegree)");
+	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "computed moment : %22.11f", (double)sum);
+	  exit(EXIT_FAILURE);
+	}
+	result->momentList[i] = 0;
+      }
+
+    funcCountCurr_l++;
+    if(funcCountCurr_l == 2*l+1) {
+      l++;
+      funcCountCurr_l = 0;
+    }
+  } // END FOR i
+
+  if(distrDegree > MAX_MULTIPOLE_DEGREE_BASIC) {
     // This should not happen, really, but for testing purposes it is nice to be able to set
     // MAX_MULTIPOLE_DEGREE_BASIC to be lower than needed to describe product distributions correctly.
     // The accuracy will then be bad, but the program should still work.
     result->degree = MAX_MULTIPOLE_DEGREE_BASIC;
   }
- else
-   result->degree = distrDegree;
+  else
+    result->degree = distrDegree;
 
-for(int k = 0; k < 3; k++)
-  result->centerCoords[k] = distr->centerCoords[k];
+  for(int k = 0; k < 3; k++)
+    result->centerCoords[k] = distr->centerCoords[k];
   
-return 0;
+  return 0;
 }
 
 
-MMTranslator::MMTranslator()
+MMTranslator::MMTranslator(const MultipolePrepManager & multipolePrepManager)
+  : multipolePrep(multipolePrepManager)
 {
   buffer_W_cc = new ergo_real[MMDP1*MMDP1*MMDP1*MMDP1];
   buffer_W_cs = new ergo_real[MMDP1*MMDP1*MMDP1*MMDP1];
   buffer_W_sc = new ergo_real[MMDP1*MMDP1*MMDP1*MMDP1];
   buffer_W_ss = new ergo_real[MMDP1*MMDP1*MMDP1*MMDP1];
-
-  if(global_initialized_flag == 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS,
-                "error in get_translation_matrix: (global_initialized_flag == 0)");
-      exit(0);
-    }
 }
 
 MMTranslator::~MMTranslator()
@@ -214,8 +150,11 @@ MMTranslator::getTranslationMatrix(ergo_real dx,
                                    int l_2,
                                    ergo_real* result_W) const
 {
+  assert(multipolePrep.is_initialized());
   ergo_real r2 = dx*dx + dy*dy + dz*dz;
 
+  const MultipolePrepManager::l_m_struct* l_m_list = multipolePrep.get_l_m_list_ptr();
+
   // generate values of all needed "scaled regular solid harmonics"
   int largest_l_needed = MAX_MULTIPOLE_DEGREE;
 
@@ -310,10 +249,10 @@ MMTranslator::getTranslationMatrix(ergo_real dx,
   for(int A = 0; A < noOfMoments_1; A++)
     for(int B = 0; B < noOfMoments_2; B++)
       {
-	int l = global_l_m_list[A].l;
-	int m = global_l_m_list[A].m;
-	int j = global_l_m_list[B].l;
-	int k = global_l_m_list[B].m;
+	int l = l_m_list[A].l;
+	int m = l_m_list[A].m;
+	int j = l_m_list[B].l;
+	int k = l_m_list[B].m;
 
 	result_W[A*noOfMoments_2+B] = 0;
 
@@ -326,21 +265,20 @@ MMTranslator::getTranslationMatrix(ergo_real dx,
 	if(m < 0 && k < 0)
 	  result_W[A*noOfMoments_2+B] = W_ss[l][-m][j][-k];
 
-	result_W[A*noOfMoments_2+B] *= get_lm_factor(j, k) / get_lm_factor(l, m);
+	result_W[A*noOfMoments_2+B] *= multipolePrep.get_lm_factor(j, k) / multipolePrep.get_lm_factor(l, m);
       }
 
   return 0;
 }
 
 
-MMInteractor::MMInteractor()
+MMInteractor::MMInteractor(const MultipolePrepManager & multipolePrepManager)
+  : multipolePrep(multipolePrepManager)
 {
   buffer_T_cc = new ergo_real[MMDP1*MMDP1*MMDP1*MMDP1];
   buffer_T_cs = new ergo_real[MMDP1*MMDP1*MMDP1*MMDP1];
   buffer_T_sc = new ergo_real[MMDP1*MMDP1*MMDP1*MMDP1];
   buffer_T_ss = new ergo_real[MMDP1*MMDP1*MMDP1*MMDP1];
-  if(global_initialized_flag == 0)
-    throw "Error in MMInteractor::MMInteractor: (global_initialized_flag == 0).";
 }
 
 MMInteractor::~MMInteractor()
@@ -359,9 +297,12 @@ MMInteractor::getInteractionMatrix(ergo_real dx,
                                    int l_2,
                                    ergo_real* result_T)
 {
+  assert(multipolePrep.is_initialized());
   ergo_real r2 = dx*dx + dy*dy + dz*dz;
   ergo_real oneOverR2 = (ergo_real)1 / r2;
-  ergo_real oneOverR  = std::sqrt(oneOverR2);
+  ergo_real oneOverR  = template_blas_sqrt(oneOverR2);
+
+  const MultipolePrepManager::l_m_struct* l_m_list = multipolePrep.get_l_m_list_ptr();
 
   // generate values of all needed "scaled irregular solid harmonics"
   //int largest_l_needed = distrMultipole->degree + otherMultipole->degree;
@@ -450,10 +391,10 @@ MMInteractor::getInteractionMatrix(ergo_real dx,
 
   for(int A = 0; A < noOfMoments_1; A++)
     for(int B = 0; B < noOfMoments_2; B++) {
-      int l = global_l_m_list[A].l;
-      int m = global_l_m_list[A].m;
-      int j = global_l_m_list[B].l;
-      int k = global_l_m_list[B].m;
+      int l = l_m_list[A].l;
+      int m = l_m_list[A].m;
+      int j = l_m_list[B].l;
+      int k = l_m_list[B].m;
 
       result_T[A*noOfMoments_2+B] = 0;
 
@@ -466,7 +407,7 @@ MMInteractor::getInteractionMatrix(ergo_real dx,
       if(m < 0 && k < 0)
 	result_T[A*noOfMoments_2+B] = T_ss[l][-m][j][-k];
 
-      result_T[A*noOfMoments_2+B] *= get_lm_factor(l, m) * get_lm_factor(j, k);
+      result_T[A*noOfMoments_2+B] *= multipolePrep.get_lm_factor(l, m) * multipolePrep.get_lm_factor(j, k);
     }
 
   return 0;
@@ -481,13 +422,13 @@ setup_multipole_maxAbsMomentList(multipole_struct_large* multipole) {
     int endIndex = (degree+1)*(degree+1);
     ergo_real sumOfSquares = 0;
     for(int i = startIndex; i < endIndex; i++) {
-      ergo_real absMoment = std::fabs(multipole->momentList[i]);
+      ergo_real absMoment = template_blas_fabs(multipole->momentList[i]);
       if(absMoment > largestAbsMomentSoFar)
 	largestAbsMomentSoFar = absMoment;
       sumOfSquares += absMoment * absMoment;
     }
     multipole->maxAbsMomentList[degree] = largestAbsMomentSoFar;
-    multipole->euclideanNormList[degree] = std::sqrt(sumOfSquares);
+    multipole->euclideanNormList[degree] = template_blas_sqrt(sumOfSquares);
   } // END FOR degree
   return 0;
 }
