@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,50 +17,31 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file multipole.h
+
+    @brief Code for computing multipole moments, and multipole
+    interaction and translation matrices.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef MULTIPOLE_HEADER
 #define MULTIPOLE_HEADER
 
-
 #include "realtype.h"
 #include "integral_info.h"
 #include "basisinfo.h"
-
-#define MAX_MULTIPOLE_DEGREE 15
-#define MAX_NO_OF_MOMENTS_PER_MULTIPOLE ((MAX_MULTIPOLE_DEGREE+1)*(MAX_MULTIPOLE_DEGREE+1))
-
-#define MAX_MULTIPOLE_DEGREE_BASIC BASIS_FUNC_POLY_MAX_DEGREE
-#define MAX_NO_OF_MOMENTS_PER_MULTIPOLE_BASIC ((MAX_MULTIPOLE_DEGREE_BASIC+1)*(MAX_MULTIPOLE_DEGREE_BASIC+1))
-
-
-typedef struct
-{
-  ergo_real centerCoords[3];
-  int degree;
-  int noOfMoments;
-  ergo_real momentList[MAX_NO_OF_MOMENTS_PER_MULTIPOLE];
-  ergo_real maxAbsMomentList[MAX_MULTIPOLE_DEGREE+1];
-  ergo_real euclideanNormList[MAX_MULTIPOLE_DEGREE+1];
-} multipole_struct_large;
-
-typedef struct
-{
-  ergo_real centerCoords[3];
-  int degree;
-  int noOfMoments;
-  ergo_real momentList[MAX_NO_OF_MOMENTS_PER_MULTIPOLE_BASIC];
-} multipole_struct_small;
-
-
-int init_multipole_code();
+#include "multipole_prep.h"
 
 int
 compute_multipole_moments(const IntegralInfo& integralInfo,
@@ -73,8 +54,9 @@ class MMTranslator {
   ergo_real *buffer_W_cs;
   ergo_real *buffer_W_sc;
   ergo_real *buffer_W_ss;
+  const MultipolePrepManager & multipolePrep;
  public:
-  MMTranslator();
+  MMTranslator(const MultipolePrepManager & multipolePrepManager);
   ~MMTranslator();
   int getTranslationMatrix(ergo_real dx,
                            ergo_real dy,
@@ -90,8 +72,9 @@ class MMInteractor {
   ergo_real *buffer_T_cs;
   ergo_real *buffer_T_sc;
   ergo_real *buffer_T_ss;
+  const MultipolePrepManager & multipolePrep;
  public:
-  MMInteractor();
+  MMInteractor(const MultipolePrepManager & multipolePrepManager);
   ~MMInteractor();
   int getInteractionMatrix(ergo_real dx,
                            ergo_real dy,
@@ -101,7 +84,6 @@ class MMInteractor {
                            ergo_real* result_T);
 };
 
-
 int
 setup_multipole_maxAbsMomentList(multipole_struct_large* multipole);
 
