@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file operator_matrix.cc
+
+    @brief Functions for computing the matrix of a
+    dipole/quadrupole/etc operator. Full and sparse versions.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdlib.h>
 #include <math.h>
 #include <stdio.h>
@@ -43,14 +52,8 @@
 #include "box_system.h"
 #include "operator_matrix.h"
 
-/** @file operator_matrix.cc
 
-Functions for computing the matrix of a dipole/quadrupole/etc operator.
-Full and sparse versions.
-*/
-
-
-static const ergo_real MATRIX_ELEMENT_THRESHOLD_VALUE = 1e-12;
+static const ergo_real MATRIX_ELEMENT_THRESHOLD_VALUE = 1e-12; // FIXME: SHOULD NOT USE HARD-CODED VALUE HERE!
 
 
 int
@@ -98,16 +101,16 @@ compute_operator_matrix_full(const BasisInfoStruct & basisInfoA,
 
 
 /** computes the matrix of a dipole/quadrupole/etc operator. The
-    columns and rows enumerate @param basisInfoA and @param
+    columns and rows enumerate basisInfoA and
     basisInfoB respectively. The operator is in the form: X =
     (x^pow_x*y^pow_y*z^pow_z). The resulting matrix (possibly
     rectangular) is returned in nvaluesList, colindList, valuesList. 
     Overlap matrix is
     associated with triple (0,0,0), X component of the dipole moment
     with (1,0,0), etc.
-    The parameters @param pow_x @param pow_y @param pow_z determine the operator.
-    The parameters @param n_A @param n_B give the number of basis functions in each of the two basis sets.
-    The result is stored using the lists @param nvaluesList @param colindList @param valuesList each having length n_A.
+    The parameters pow_x pow_y pow_z determine the operator.
+    The parameters n_A n_B give the number of basis functions in each of the two basis sets.
+    The result is stored using the lists nvaluesList colindList valuesList each having length n_A.
 */
 int 
 compute_operator_matrix_sparse(const BasisInfoStruct & basisInfoA, 
@@ -206,7 +209,7 @@ compute_operator_matrix_sparse(const BasisInfoStruct & basisInfoA,
 	  ergo_real dx = basisInfoA.basisFuncList[i].centerCoords[0] - basisInfoB.basisFuncList[j].centerCoords[0];
 	  ergo_real dy = basisInfoA.basisFuncList[i].centerCoords[1] - basisInfoB.basisFuncList[j].centerCoords[1];
 	  ergo_real dz = basisInfoA.basisFuncList[i].centerCoords[2] - basisInfoB.basisFuncList[j].centerCoords[2];
-	  ergo_real distance = std::sqrt(dx*dx + dy*dy + dz*dz);
+	  ergo_real distance = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
 	  // We can skip if distance is greater than sum of extents.
 	  if(distance > basisFuncExtentList_A[i] + basisFuncExtentList_B[j]) {
             rowValueList[jRelevant] = 0.0;
@@ -277,10 +280,10 @@ compute_operator_matrix_sparse(const BasisInfoStruct & basisInfoA,
 		} // END FOR coordNo
 	      // now tempList2 contains all the final distributions
 	      for(int ii = 0; ii < tempList2_count; ii++)
-		sum += compute_integral_of_simple_prim(&tempList2[ii]);
+		sum += compute_integral_of_simple_prim(tempList2[ii]);
 	    } /* END FOR k */
 	  rowValueList[jRelevant] = sum;
-	  if(std::fabs(sum) > MATRIX_ELEMENT_THRESHOLD_VALUE)
+	  if(template_blas_fabs(sum) > MATRIX_ELEMENT_THRESHOLD_VALUE)
 	    count++;
 	} /* END FOR jRelevant */
       // OK, this row done.
@@ -293,7 +296,7 @@ compute_operator_matrix_sparse(const BasisInfoStruct & basisInfoA,
       for(int jRelevant = 0; jRelevant < nRelevant; jRelevant++)
 	{
 	  int j = orgIndexList[jRelevant];
-	  ergo_real absVal = std::fabs(rowValueList[jRelevant]);
+	  ergo_real absVal = template_blas_fabs(rowValueList[jRelevant]);
 	  if(absVal > MATRIX_ELEMENT_THRESHOLD_VALUE)
 	    {
 	      if(count >= nvaluesList[i]) {
