@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,26 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file organize_distrs.h
+
+    @brief Code for organizing a given set of primitive Gaussian
+    distributions (typically coming from basis function products); the
+    distributions are grouped according to their location in space,
+    their exponents, etc.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef ORGANIZE_DISTRS_HEADER
 #define ORGANIZE_DISTRS_HEADER
 
@@ -47,8 +58,7 @@ typedef struct
   ergo_real maxExtentGroup;
   ergo_real maxLimitingFactorGroup;
   ergo_real maxAbsDmatElementGroup;
-  multipole_struct_small* multipolePtr;
-  ergo_real multipoleEuclideanNormList[MAX_MULTIPOLE_DEGREE_BASIC+1];
+  ergo_real multipoleEuclNormListForK[MAX_MULTIPOLE_DEGREE_BASIC+1];
 } distr_group_struct;
 
 typedef struct
@@ -65,7 +75,7 @@ typedef struct
   int groupStartIndex;
   int noOfGroups;
   ergo_real maxLimitingFactorForCluster;
-  ergo_real multipoleEuclideanNormList[MAX_MULTIPOLE_DEGREE_BASIC+1];
+  ergo_real multipoleEuclNormListForK[MAX_MULTIPOLE_DEGREE_BASIC+1];
 } cluster_struct;
 
 typedef struct
@@ -84,9 +94,9 @@ typedef struct
 #error The constant BASIS_FUNC_POLY_MAX_DEGREE must be defined.
 #endif
 #if BASIS_FUNC_POLY_MAX_DEGREE<6
-#define MAX_NO_OF_BASIS_FUNC_PAIRS_PER_CHUNK 1000
+#define MAX_NO_OF_BASIS_FUNC_PAIRS_PER_BATCH 1000
 #else
-#define MAX_NO_OF_BASIS_FUNC_PAIRS_PER_CHUNK 10000
+#define MAX_NO_OF_BASIS_FUNC_PAIRS_PER_BATCH 10000
 #endif
 
 typedef struct
@@ -95,49 +105,46 @@ typedef struct
   int noOfClusters;
   int noOfBasisFuncPairs;
   int basisFuncPairListIndex;
-  int basisFuncForChunksIndex;
-  int basisFuncForChunkCount;
+  int basisFuncForBatchsIndex;
+  int basisFuncForBatchCount;
   int global_debug_id;
-} chunk_struct;
+} batch_struct;
+
+struct basis_func_group_info_for_box {
+  int basisFuncGroupIndex;
+  ergo_real max_CS_factor;
+  ergo_real maxMomentVectorNormList[MAX_MULTIPOLE_DEGREE_BASIC+1];
+  int maxMultipoleDegree;
+};
 
 
 struct distr_org_struct {
   std::vector<minimal_distr_struct> minimalDistrList;
   std::vector<distr_group_struct> groupList;
   std::vector<cluster_struct> clusterList;
-  std::vector<chunk_struct> chunkList;
+  std::vector<batch_struct> batchList;
   std::vector<basis_func_pair_struct> basisFuncPairList;
-  std::vector<int> basisFuncListForChunks;
-  std::vector<int> basisFuncListForChunks_map;
+  std::vector<int> basisFuncListForBatchs;
+  std::vector<int> basisFuncListForBatchs_map;
   std::vector<int> basisFuncList;
   std::vector<i_j_val_struct> spMatElementList;
   std::vector<int> spMatCountList;
   std::vector<int> spMatIdxList;
-  int minimalDistrCount;
-  int groupCount;
-  int clusterCount;
-  int chunkCount;
-  int basisFuncPairCount;
-  int basisFuncForChunksCount;
-  int basisFuncListCount;
-  ergo_real maxExtent;
-  ergo_real maxDistanceOutsideBox;
-  distr_org_struct():
-    minimalDistrCount(0), 
-    groupCount(0), 
-    clusterCount(0), 
-    chunkCount(0), 
-    basisFuncPairCount(0), 
-    basisFuncForChunksCount(0), 
-    basisFuncListCount(0), 
-    maxExtent(0), 
-    maxDistanceOutsideBox(0)
-  {}
+  std::vector<basis_func_group_info_for_box> basisFuncGroupInfoListForK;
+  struct Data {
+    ergo_real maxExtent;
+    ergo_real maxDistanceOutsideBox;
+    int maxNoOfMonomials;
+    Data();
+  };
+  Data data;
+  // Functions needed for CHT usage
+  void writeToBuffer(char* dataBuffer, size_t const bufferSize) const;
+  size_t getSize() const;
+  void assignFromBuffer(char const * dataBuffer, size_t const bufferSize);
 };
 
 
-
-
 int
 organize_distributions(const IntegralInfo & integralInfo,
 		       DistributionSpecStructLabeled* distrList_in, 
