@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -177,18 +178,24 @@ namespace mat {
 	 throw std::runtime_error("FileWritable copyFromFile error: unable to open file for reading.");
        // Open destination file for writing.
        std::ofstream outFile;
-       outFile.exceptions( std::ifstream::eofbit  | 
-			   std::ifstream::failbit | 
-			   std::ifstream::badbit );
-       outFile.open(fileName, std::ios::out|std::ios::binary);
-       if (!outFile.is_open())
-	 throw std::runtime_error("FileWritable copyFromFile error: unable to open file for writing.");
-       // Copy data.
-       try {
-	 outFile << inFile.rdbuf();
-       }
+         outFile.exceptions( std::ifstream::eofbit  | 
+  			   std::ifstream::failbit | 
+  			   std::ifstream::badbit );
+          try {
+            outFile.open(fileName, std::ios::out|std::ios::binary);
+          }
+          catch(std::ofstream::failure& e){
+              throw std::runtime_error("FileWritable copyFromFile error: an error "
+                                 "has occurred on the associated stream. "
+                                 "Did you set a wrong path in setPath()?");}
+         if (!outFile.is_open())
+  	         throw std::runtime_error("FileWritable copyFromFile error: unable to open file for writing.");
+         // Copy data.
+         try {
+	         outFile << inFile.rdbuf();
+        }
        catch (std::ofstream::failure& e) {
-	 throw std::runtime_error("FileWritable copyFromFile error: failed to copy data. Out of disk space?");
+	         throw std::runtime_error("FileWritable copyFromFile error: failed to copy data. Out of disk space?");
        }
        inFile.close();
        outFile.close();
@@ -213,10 +220,10 @@ namespace mat {
 
        this->readFromFileProt(file);
        file.close();
-
-       writeToFile(); // since active is false, it will just change a flag objectIsOnFile to true
      }
 
+     this->inMemorySet(false);
+     objectIsOnFile = true;
 
  }
 
