@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -48,7 +49,7 @@ namespace mat { /* Matrix namespace */
       LanczosLargestMagnitudeEig(Tmatrix const & AA, Tvector const & startVec, 
 				 int maxIter = 100, int cap = 100) 
 	: Lanczos<Treal, Tmatrix, Tvector>(AA, startVec, maxIter, cap), 
-	eVal(0), acc(0), eigVectorTri(0), absTol( std::numeric_limits<Treal>::max() ),    
+	eVal(0), acc(0), eigVectorTri(0), absTol( template_blas_get_num_limit_max<Treal>() ),
 	relTol(template_blas_sqrt(template_blas_sqrt(getRelPrecision<Treal>()))), 
 	eValTmp(0), accTmp(0) {}
       void setRelTol(Treal const newTol) { relTol = newTol; }
@@ -101,7 +102,8 @@ namespace mat { /* Matrix namespace */
 	    computeEigVec();
 	  }
 	  // Guard against unrealistically small accuracies
-	  acc = acc / template_blas_fabs(eVal) > 2 * std::numeric_limits<Treal>::epsilon() ? acc : 2 * template_blas_fabs(eVal) * std::numeric_limits<Treal>::epsilon();
+	  Treal machine_epsilon = mat::getMachineEpsilon<Treal>();
+	  acc = acc / template_blas_fabs(eVal) > 2 * machine_epsilon ? acc : 2 * template_blas_fabs(eVal) * machine_epsilon;
 #endif
 	}
 
@@ -259,7 +261,8 @@ namespace mat { /* Matrix namespace */
 				Treal const requestedAbsAccuracy,
 				Treal const requestedRelAccuracy,
 				Treal const maxAbsVal,
-				typename Tmatrix::VectorType * const eVecPtr = 0) {
+				typename Tmatrix::VectorType * const eVecPtr = 0,
+				int maxIter = -1) {
     assert(requestedAbsAccuracy >= 0);
     //    Treal frobTmp;
     /* Check if norm is really small, or can easily be determined to be 'large', in those cases quick return */
@@ -271,8 +274,9 @@ namespace mat { /* Matrix namespace */
       return Interval<Treal>( euclLowerBound, euclUpperBound );
     if ( euclLowerBound > maxAbsVal )
       // Norm is not small, quick return
-      return Interval<Treal>( euclLowerBound, euclUpperBound );    
-    int maxIter = M.get_nrows() * 100;
+      return Interval<Treal>( euclLowerBound, euclUpperBound );
+    if(maxIter == -1)
+      maxIter = M.get_nrows() * 100;
     typename Tmatrix::VectorType guess;
     SizesAndBlocks cols;
     M.getCols(cols);
