@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -71,8 +72,14 @@ namespace mat {
 
     MatrixSymmetric()
       :MatrixBase<Treal, Tmatrix>() {} /**< Default constructor  */
+      /* In the code we are using std::vector<MatrixSymmetric> which in the c++ standard before c++11 requires move operation like T x_new = x which calls implicitly the copy constructor. To make it work with g++ versions without c++11 support we remove the keyword explicit. */
+#if __cplusplus >= 201103L      
     explicit MatrixSymmetric(const MatrixSymmetric<Treal, Tmatrix>& symm)
       :MatrixBase<Treal, Tmatrix>(symm) {} /**< Copy constructor  */
+#else 
+    MatrixSymmetric(const MatrixSymmetric<Treal, Tmatrix>& symm)
+      :MatrixBase<Treal, Tmatrix>(symm) {} /**< Copy constructor  */      
+#endif
     explicit MatrixSymmetric(const XY<Treal, MatrixSymmetric<Treal, Tmatrix> >& sm)
       :MatrixBase<Treal, Tmatrix>() { *this = sm.A * sm.B; }
     explicit MatrixSymmetric(const MatrixGeneral<Treal, Tmatrix>& matr)
@@ -404,7 +411,8 @@ namespace mat {
     static Treal eucl_diff
       (const MatrixSymmetric<Treal, Tmatrix>& A,
        const MatrixSymmetric<Treal, Tmatrix>& B,
-       Treal const requestedAccuracy);
+       Treal const requestedAccuracy,
+       int maxIter = -1);
     
     /** Returns the 'mixed' norm of A - B 
      *  ( || A - B ||_mixed )
@@ -760,15 +768,17 @@ namespace mat {
     Treal MatrixSymmetric<Treal, Tmatrix>::eucl_diff
     (const MatrixSymmetric<Treal, Tmatrix>& A,
      const MatrixSymmetric<Treal, Tmatrix>& B,
-     Treal const requestedAccuracy) {
+     Treal const requestedAccuracy,
+     int maxIter) {
     // DiffMatrix is a lightweight proxy object:
     mat::DiffMatrix< MatrixSymmetric<Treal, Tmatrix>, Treal> Diff(A, B);
     Treal maxAbsVal = 2 * frob_diff(A,B); 
     // Now, maxAbsVal should be larger than the Eucl norm
     // Note that mat::euclIfSmall lies outside this class 
-    Treal relTol = sqrt(sqrt(std::numeric_limits<Treal>::epsilon()));
+    Treal relTol = template_blas_sqrt(template_blas_sqrt(mat::getMachineEpsilon<Treal>()));
+    VectorType * const eVecPtrNotUsed = 0;
     Interval<Treal> euclInt = 
-      mat::euclIfSmall(Diff, requestedAccuracy, relTol, maxAbsVal); 
+      mat::euclIfSmall(Diff, requestedAccuracy, relTol, maxAbsVal, eVecPtrNotUsed, maxIter);
     return euclInt.midPoint();
   }
 
@@ -799,7 +809,7 @@ namespace mat {
     // DiffMatrix is a lightweight proxy object:
     mat::DiffMatrix< MatrixSymmetric<Treal, Tmatrix>, Treal> Diff(A, B);
     // Note that this function lies outside this class 
-    Treal relTol = sqrt(sqrt(std::numeric_limits<Treal>::epsilon()));
+    Treal relTol = template_blas_sqrt(template_blas_sqrt(mat::getMachineEpsilon<Treal>()));
     Interval<Treal> tmpInterval = mat::euclIfSmall(Diff, requestedAccuracy, relTol, maxAbsVal, eVecPtr);
     // Emanuel note: Ugly fix to make certain tests pass, we expand
     // the interval up to the requested accuracy. Note that larger
@@ -928,6 +938,12 @@ namespace mat {
     inline MatrixSymmetric<Treal, Tmatrix>& 
     MatrixSymmetric<Treal, Tmatrix>::operator= 
     (XY<Treal, MatrixSymmetric<Treal, Tmatrix> > const & sm) {
+  
+  if(this == &sm.B) // A = B
+      {
+        *this *= sm.A; // B *= alpha
+        return *this;
+      }
     assert(!sm.tB);
     /* Data structure set by assign - therefore set haveDataStructure to true */
     this->matrixPtr.haveDataStructureSet(true); 
@@ -1164,6 +1180,8 @@ namespace mat {
 				beta, *C.matrixPtr);
   }
 
+  
+
 
 
   /* Addition */  
