@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -58,6 +59,7 @@ namespace mat {
     class MatrixTriangular : public MatrixBase<Treal, Tmatrix> {
   public:
     typedef VectorGeneral<Treal, typename Tmatrix::VectorType> VectorType;
+    typedef Treal real;
 
     MatrixTriangular()
       :MatrixBase<Treal, Tmatrix>() {} /**< Default constructor  */
@@ -265,7 +267,7 @@ namespace mat {
     arn::LanczosLargestMagnitudeEig
       <Treal, ATAMatrix<MatrixTriangular<Treal, Tmatrix>, Treal>, VectorType>
       lan(ztz, guess, maxIter);
-    lan.setRelTol( 100 * std::numeric_limits<Treal>::epsilon() );
+    lan.setRelTol( 100 * mat::getMachineEpsilon<Treal>() );
     lan.run();
     Treal eVal = 0;
     Treal acc = 0;
@@ -275,8 +277,8 @@ namespace mat {
     if ( euclInt.low() < 0 )
       euclInt = Interval<Treal>( 0, template_blas_sqrt(eVal+acc) );
     if ( euclInt.length() / 2.0 > requestedAccuracy ) {
-      std::cout << "req: " << requestedAccuracy
-		<< "  obt: " << euclInt.length() / 2.0 << std::endl;
+      std::cout << "req: " << (double)requestedAccuracy
+		<< "  obt: " << (double)(euclInt.length() / 2.0) << std::endl;
       throw std::runtime_error("Desired accuracy not obtained in Lanczos.");
     }
     return euclInt.midPoint();
