@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,22 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file blastime.cc
+
+    \brief Benchmark code for matrix-matrix multiplication (gemm)
+    operation, measuring timings for different matrix sizes.
+*/
+
 #include <iostream>
 #include <fstream>
 #include <iomanip> /* For setprecision in fstream */
@@ -33,12 +40,21 @@
 #include <cmath>
 #include <cstdlib>
 #include <string.h>
+#include <sys/time.h>
+#include <vector>
 
 #include "mat_gblas.h"
 
 static const int MIN_TIME_PER_STEP = 5;
 static const int SIZE_INCREMENT = 2;
 
+static double get_wall_seconds() {
+  struct timeval tv;
+  gettimeofday(&tv, NULL);
+  double seconds = tv.tv_sec + (double)tv.tv_usec / 1000000;
+  return seconds;
+}
+
 template<class T>
 static void tomatlabfile(char* name,T* values,int s,std::ofstream& output);
 
@@ -48,7 +64,7 @@ int mainFun(int maxDim, double* timev, double* gflops, bool writeTomFile) {
     //    bool CPUtime = false;
     const real ONE=1.0;
     const real ZERO=0.0;
-    clock_t start, end;
+    double start, end;
     int i;
     int steps = 500000;//50000000;
 
@@ -66,12 +82,12 @@ int mainFun(int maxDim, double* timev, double* gflops, bool writeTomFile) {
       int m = testSize;
       int n = testSize;
       int k = testSize;
-      start = clock();
+      start = get_wall_seconds();
       for(int j=0; j<steps; j++) {
-	mat::gemm("N","N",&m,&n,&k,&ONE,A,&m,B,&k,&ZERO,C,&m);
+	       mat::gemm("N","N",&m,&n,&k,&ONE,A,&m,B,&k,&ZERO,C,&m);
       }
-      end=clock();
-      secondsTaken = ((double)(end-start))/((double)CLOCKS_PER_SEC);
+      end = get_wall_seconds();
+      secondsTaken = end-start;
     }
     printf("%d tests took %6.2f seconds.\n", steps, secondsTaken);
     
@@ -89,14 +105,12 @@ int mainFun(int maxDim, double* timev, double* gflops, bool writeTomFile) {
       int n = size;
       int k = size;
       
-      start = clock();
+      start = get_wall_seconds();
       for(int j=0; j<steps; j++) {
-	mat::gemm("N","N",&m,&n,&k,&ONE,A,&m,B,&k,&ZERO,C,&m);
+	       mat::gemm("N","N",&m,&n,&k,&ONE,A,&m,B,&k,&ZERO,C,&m);
       }
-      end=clock();
+      end = get_wall_seconds();
       secondsTaken = end-start;
-
-      secondsTaken = ((double)(end-start))/((double)CLOCKS_PER_SEC);
       timev[step-1] = secondsTaken;
       gflops[step-1]=(2*pow(double(size),3)+4*pow(double(size),2))/(timev[step-1]*1e9)*steps;
 
@@ -113,7 +127,7 @@ int mainFun(int maxDim, double* timev, double* gflops, bool writeTomFile) {
 	/* This prediction does not really work for large relative
 	   matrix size increments. But it does its job sufficiently well. */
 	int newSteps = 
-	  int(double(steps)*MIN_TIME_PER_STEP*1.5/double(timev[step-1]));
+	  ceil(double(steps)*MIN_TIME_PER_STEP*1.5/double(timev[step-1]));
 	std::cout << "Recomputing new steps "<< newSteps <<" = "
 		  << steps << " * " << MIN_TIME_PER_STEP << " / "
 		  << timev[step-1] << std::endl;
@@ -121,7 +135,7 @@ int mainFun(int maxDim, double* timev, double* gflops, bool writeTomFile) {
       }
     }
   }
-  catch (std::exception e) {
+  catch (std::exception & e) {
     std::cout << "Exception caught: "<<e.what() << std::endl;
     std::exit(1);
   }
@@ -157,21 +171,21 @@ int main(int argc,char* argv[]) {
     strcpy(path, argv[2]);  /* Matlab filename.      */
     break;
   default:
-    std::cerr<<"Wrong number of input arguments"<<std::endl;
+    std::cerr<<"Wrong number of input arguments. Give at least one argument (max matrix dimension) and optionally as a second argument the name of the m-file to create." << std::endl;
     std::exit(1);
   }
   int maxSize = maxDim/SIZE_INCREMENT;
-  double timevDouble[maxSize];
-  double gflopsDouble[maxSize];
-  double timevSingle[maxSize];
-  double gflopsSingle[maxSize];
+  std::vector<double> timevDouble(maxSize);
+  std::vector<double> gflopsDouble(maxSize);
+  std::vector<double> timevSingle(maxSize);
+  std::vector<double> gflopsSingle(maxSize);
   maxDim = maxSize*SIZE_INCREMENT;
 
   time_t startTime;
   time_t endTime;
   std::cout<<"Starting gemm benchmark, double precision"<<std::endl;
   time(&startTime);
-  if (!mainFun<double>(maxDim, timevDouble, gflopsDouble, writeTomFile)) {
+  if (!mainFun<double>(maxDim, &timevDouble[0], &gflopsDouble[0], writeTomFile)) {
     time(&endTime);
     std::cout<<"Ended gemm benchmark, double precision, wall time: "
 	     <<endTime-startTime
@@ -179,7 +193,7 @@ int main(int argc,char* argv[]) {
   }
   std::cout<<"Starting gemm benchmark, single precision"<<std::endl;
   time(&startTime);
-  if (!mainFun<float>(maxDim, timevSingle, gflopsSingle, writeTomFile)) {
+  if (!mainFun<float>(maxDim, &timevSingle[0], &gflopsSingle[0], writeTomFile)) {
     time(&endTime);
     std::cout<<"Ended gemm benchmark, single precision, wall time: "
 	     <<endTime-startTime
@@ -193,10 +207,10 @@ int main(int argc,char* argv[]) {
       std::exit(1);
     }
     output<<"nv=1:" << SIZE_INCREMENT <<":"<<maxDim<<";"<<std::endl;
-    tomatlabfile<double>("timeDouble",timevDouble,maxSize,output);
-    tomatlabfile<double>("GflopsDouble",gflopsDouble,maxSize,output);
-    tomatlabfile<double>("timeSingle",timevSingle,maxSize,output);
-    tomatlabfile<double>("GflopsSingle",gflopsSingle,maxSize,output);
+    tomatlabfile<double>("timeDouble"  , &timevDouble [0], maxSize, output);
+    tomatlabfile<double>("GflopsDouble", &gflopsDouble[0], maxSize, output);
+    tomatlabfile<double>("timeSingle"  , &timevSingle [0], maxSize, output);
+    tomatlabfile<double>("GflopsSingle", &gflopsSingle[0], maxSize, output);
     output<<"minX = 0;\n"
 	  <<"maxX = max(nv);\n"
 	  <<"minY = 0;\n"
@@ -226,4 +240,4 @@ int main(int argc,char* argv[]) {
   
   
   std::exit(0);
-};
+}
