@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -49,9 +50,11 @@
 #include <omp.h>
 #endif
 
-/* We need to include config.h to get the USE_SSE_INTRINSICS flag. */
+/* We need to include config.h to get the PRECISION_QUAD_FLT128 and USE_SSE_INTRINSICS flags. */
 #include "config.h"
 
+#include "template_blas_num_limits.h"
+
 #include "Failure.h"
 #include "DebugPolicies.h"
 #include "SizesAndBlocks.h"
@@ -136,24 +139,18 @@ namespace mat{
   enum normType {frobNorm, euclNorm, mixedNorm};
   normType getNormType(const char* normStr);
   std::string getNormTypeString(normType nType);
-  
-  
+
+  /* getMachineEpsilon(): function for getting the machine epsilon (the
+     difference between 1 and the least value greater than 1 that is
+     representable) for the given floating-point type.  */
+  template<typename Treal>
+    inline static Treal getMachineEpsilon() {
+    return template_blas_get_machine_epsilon<Treal>();
+  }
+
   template<typename Treal>
     inline static Treal getRelPrecision() {
-    throw Failure("getPrecision() : The used type is not supported by"
-		  " getPrecision() ");
-  }
-  template<>
-    inline long double getRelPrecision<long double>() {
-    return std::numeric_limits<long double>::epsilon();
-  }
-  template<>
-    inline double getRelPrecision<double>() {
-    return std::numeric_limits<double>::epsilon();
-  }
-  template<>
-    inline float getRelPrecision<float>() {
-    return std::numeric_limits<float>::epsilon();
+    return getMachineEpsilon<Treal>();
   }
 
   class Time {
