@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -36,7 +37,7 @@
 #ifndef TEMPLATE_BLAS_BASICMATH_HEADER
 #define TEMPLATE_BLAS_BASICMATH_HEADER
 
-#include <limits>
+#include "template_blas_num_limits.h"
 
 template<class Treal>
 Treal template_blas_fabs(Treal x);
@@ -50,12 +51,24 @@ Treal template_blas_exp(Treal x);
 template<class Treal>
 Treal template_blas_log(Treal x);
 
+template<class Treal>
+Treal template_blas_log10(Treal x);
+
 template<class Treal>
 Treal template_blas_erf(Treal x);
 
 template<class Treal>
 Treal template_blas_erfc(Treal x);
 
+template<class Treal>
+Treal template_blas_sin(Treal x);
+
+template<class Treal>
+Treal template_blas_cos(Treal x);
+
+template<class Treal>
+Treal template_blas_pow(Treal x, Treal y);
+
 
 /* template_blas_compute_pi_BBP
    This routine computes the number pi up to the precision of Treal
@@ -63,7 +76,7 @@ Treal template_blas_erfc(Treal x);
 template<class Treal>
 Treal template_blas_compute_pi_BBP(Treal dummy)
 {
-  Treal epsilon = std::numeric_limits<Treal>::epsilon();
+  Treal epsilon = template_blas_get_machine_epsilon<Treal>();
   Treal one_over_16 = (Treal)1 / (Treal)16;
   Treal one_over_16_to_pow_k = 1;
   Treal sum = 0;
