@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -53,12 +54,12 @@ int dummyroutine()
 /* assign strings:  a = b */
 
 #ifdef KR_headers
-VOID s_copy(a, b, la, lb) register char *a, *b; ftnlen la, lb;
+VOID s_copy(a, b, la, lb) char *a, *b; ftnlen la, lb;
 #else
-void s_copy(register char *a, const register char *b, ftnlen la, ftnlen lb)
+void s_copy(char *a, const char *b, ftnlen la, ftnlen lb)
 #endif
 {
-const register char *aend, *bend;
+const char *aend, *bend;
 
 aend = a + la;
 
@@ -80,9 +81,9 @@ else
 
 /* compare two strings */
 
-integer s_cmp(const register char *a, const register char *b, ftnlen la, ftnlen lb)
+integer s_cmp(const char *a, const char *b, ftnlen la, ftnlen lb)
 {
-const register char *aend, *bend;
+const char *aend, *bend;
 aend = a + la;
 bend = b + lb;
 
