@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -53,6 +54,7 @@ Treal template_lapack_d_sign(const Treal *a, const Treal *b)
 
 
 
+// FIXME: ON THE NEXT LINE IS A HARD-CODED CONSTANT THAT NEEDS MORE DIGITS FOR QUAD PRECISION
 #define log10e 0.43429448190325182765
 template<class Treal>
 Treal template_blas_lg10(Treal *x)
@@ -248,15 +250,15 @@ Treal template_lapack_lamch(const char *cmach, Treal dummyReal)
    /* Initialization added by Elias to get rid of compiler warnings. */
    rmach = 0;
   if (template_blas_lsame(cmach, "E")) { /* Epsilon */
-    rmach = std::numeric_limits<Treal>::epsilon();
+    rmach = template_blas_get_machine_epsilon<Treal>();
   } else if (template_blas_lsame(cmach, "S")) { /* Safe minimum */
-    rmach = std::numeric_limits<Treal>::min();
+    rmach = template_blas_get_num_limit_min<Treal>();
   } else if (template_blas_lsame(cmach, "B")) { /* Base */
     /* Assume "base" is 2 */
     rmach = 2.0;
   } else if (template_blas_lsame(cmach, "P")) { /* Precision */
     /* Assume "base" is 2 */
-    rmach = 2.0 * std::numeric_limits<Treal>::epsilon();
+    rmach = 2.0 * template_blas_get_machine_epsilon<Treal>();
   } else if (template_blas_lsame(cmach, "N")) {
     std::cout << "ERROR in template_lapack_lamch: case N not implemented." << std::endl;
     throw "ERROR in template_lapack_lamch: case N not implemented.";
