@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -36,11 +37,10 @@
 #include <iostream>
 #include <math.h>
 #include <string.h>
+#include <vector>
 
 #include "template_lapack_common.h"
 
-#include <string.h>
-
 typedef double realtype;
 
 #if 0
@@ -172,7 +172,7 @@ int main()
   {
     integer n = 3;
 
-    double A[n*n];
+    std::vector<double> A(n*n);
 
     A[0] = 1;
     A[1] = 2;
@@ -186,17 +186,17 @@ int main()
     A[7] = 1;
     A[8] = 0;
 
-    double RHS[n];
+    std::vector<double> RHS(n);
     RHS[0] = 1;
     RHS[1] = 0;
     RHS[2] = 1;
 
-    integer IPIV[n];
+    std::vector<integer> IPIV(n);
     integer NRHS = 1;
 
     integer info = -1;
 
-    template_lapack_gesv(&n, &NRHS, A, &n, IPIV, RHS, &n, &info);
+    template_lapack_gesv(&n, &NRHS, &A[0], &n, &IPIV[0], &RHS[0], &n, &info);
     if(info != 0)
       {
 	std::cout << "ERROR!" << std::endl;
