@@ -33,12 +33,23 @@ echo Testing matrix library
 "$top_builddir"/source/matrix/mattest "$top_srcdir"/source/matrix
 if [ $? -eq 0 ]
 then
-echo All matrix library tests OK
+echo Matrix library tests OK
 else
 echo ERROR in matrix library test
 exit 1
 fi
 
+echo
+echo Testing matrix library LanczosSeveralLargestEig functionality
+"$top_builddir"/source/matrix/mattest_lan
+if [ $? -eq 0 ]
+then
+echo Matrix library LanczosSeveralLargestEig test OK
+else
+echo ERROR in matrix library LanczosSeveralLargestEig test
+exit 1
+fi
+
 if [ "$RUN_BENCHMARK" = "1" ]
     then
     echo
