@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -184,7 +185,7 @@ namespace mat { /* Matrix namespace */
   template<typename Tmatrix, typename Treal>
     Interval<Treal> EuclTruncationSymm<Tmatrix, Treal>::euclIfSmall( Treal const absTol,
 								     Treal const threshold ) {
-    Treal relTol = std::sqrt(std::sqrt(std::numeric_limits<Treal>::epsilon()));
+    Treal relTol = template_blas_sqrt(template_blas_sqrt(mat::getMachineEpsilon<Treal>()));
     Interval<Treal> tmpInterval =  mat::euclIfSmall(this->E, absTol, relTol, threshold); 
     if ( tmpInterval.length() < 2*absTol )
       return Interval<Treal>( tmpInterval.midPoint()-absTol, 
@@ -220,13 +221,13 @@ namespace mat { /* Matrix namespace */
     Treal thresholdTakingZIntoAccount = threshold / (Zfrob * Zfrob);
     /* Divide by 2 because of symmetry */
     lowTrunc  = thresholdTakingZIntoAccount * thresholdTakingZIntoAccount / 2.0;
-    highTrunc = std::numeric_limits<Treal>::max();
+    highTrunc = template_blas_get_num_limit_max<Treal>();
   }
   
   template<typename Tmatrix, typename TmatrixZ, typename Treal>
     Interval<Treal> EuclTruncationSymmWithZ<Tmatrix, TmatrixZ, Treal>::euclIfSmall( Treal const absTol,
 										    Treal const threshold ) {
-    Treal relTol = std::sqrt(std::sqrt(std::numeric_limits<Treal>::epsilon()));
+    Treal relTol = template_blas_sqrt(template_blas_sqrt(mat::getMachineEpsilon<Treal>()));
     mat::TripleMatrix<Tmatrix, TmatrixZ, Treal> ErrMatTriple( this->E, Z);
     Interval<Treal> tmpInterval = mat::euclIfSmall(ErrMatTriple, absTol, relTol, threshold);      
     if ( tmpInterval.length() < 2*absTol )
@@ -314,10 +315,10 @@ namespace mat { /* Matrix namespace */
     // some relative precision is always requested instead of the input
     // absTol which in the current case is not used(!)
     mat::ATAMatrix<Tmatrix, Treal> EtE(this->E);    
-    Treal absTolDummy = std::numeric_limits<Treal>::max(); // Treal(threshold * 1e-2)
-    Treal relTol = 100 * std::numeric_limits<Treal>::epsilon();	
+    Treal absTolDummy = template_blas_get_num_limit_max<Treal>(); // Treal(threshold * 1e-2)
+    Treal relTol = 100 * mat::getMachineEpsilon<Treal>();
     Interval<Treal> tmpInterval = mat::euclIfSmall(EtE, absTolDummy, relTol, threshold);
-    tmpInterval = Interval<Treal>( std::sqrt(tmpInterval.low()), std::sqrt(tmpInterval.upp()) );
+    tmpInterval = Interval<Treal>( template_blas_sqrt(tmpInterval.low()), template_blas_sqrt(tmpInterval.upp()) );
     if ( tmpInterval.length() < 2*absTol )
       return Interval<Treal>( tmpInterval.midPoint()-absTol, 
 			      tmpInterval.midPoint()+absTol );
@@ -354,15 +355,15 @@ namespace mat { /* Matrix namespace */
 											Treal const threshold ) {
     Treal Afrob = this->A.frob();
     Treal Bfrob = B.frob();
-    Treal tmp = -Afrob + std::sqrt( Afrob*Afrob + threshold / Bfrob );
+    Treal tmp = -Afrob + template_blas_sqrt( Afrob*Afrob + threshold / Bfrob );
     lowTrunc = tmp*tmp; 
-    highTrunc   = std::numeric_limits<Treal>::max();
+    highTrunc   = template_blas_get_num_limit_max<Treal>();
   }
   
   template<typename Tmatrix, typename TmatrixB, typename Treal>
     Interval<Treal> EuclTruncationCongrTransMeasure<Tmatrix, TmatrixB, Treal>::euclIfSmall( Treal const absTol,
 											    Treal const threshold ) {
-    Treal relTol = std::sqrt(std::sqrt(std::numeric_limits<Treal>::epsilon()));
+    Treal relTol = template_blas_sqrt(template_blas_sqrt(mat::getMachineEpsilon<Treal>()));
     mat::CongrTransErrorMatrix<TmatrixB, Tmatrix, Treal> ErrMatTriple( B, this->A, this->E );
     Interval<Treal> tmpInterval = mat::euclIfSmall(ErrMatTriple, absTol, relTol, threshold);      
     if ( tmpInterval.length() < 2*absTol ) {
