@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file SCF_general.cc
+
+    @brief Class for self-consistent field (SCF) procedure; base class
+    that can be used for both restricted and unrestricted cases.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #include <sstream>
 #include <sys/types.h>
 #include <unistd.h>
@@ -41,10 +50,37 @@
 #include "AllocatorManager.h"
 
 
+static ergo_real get_eucl_norm_try_different_acc(const symmMatrix & A, ergo_real & acc) {
+  // First try with acc = sqrt(epsilon) and if that is too difficult try with lower accuracy.
+  acc = template_blas_sqrt(mat::getMachineEpsilon<ergo_real>());
+  int maxIter = 1000;
+  ergo_real result_eucl_norm = -1;
+  try {
+    result_eucl_norm = A.eucl(acc, maxIter);
+    return result_eucl_norm;
+  }
+  catch(...) {
+    ergo_real accNew = template_blas_sqrt(acc);
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_eucl_norm_try_different_acc(): first attempt failed, now reducing accuracy from %g to %g", acc, accNew);
+    acc = accNew;
+  }
+  try {
+    result_eucl_norm = A.eucl(acc, maxIter);
+    return result_eucl_norm;
+  }
+  catch(...) {
+    ergo_real accNew = template_blas_sqrt(acc);
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_eucl_norm_try_different_acc(): second attempt failed, now reducing accuracy from %g to %g", acc, accNew);
+    acc = accNew;
+  }
+  // Now call eucl without maxIter, let it use as many iterations as it needs to reach the reduced accuracy acc.
+  return A.eucl(acc);
+}
+
+
 SCF_general::SCF_general(const Molecule& molecule_,
 			 const Molecule& extraCharges_,
 			 const BasisInfoStruct & basisInfo_, 
-			 const BasisInfoStruct & basisInfoDensFit_,
 			 const IntegralInfo & integralInfo_,
 			 const char* guessDmatFileName_,
 			 const JK::Params& J_K_params_,
@@ -56,7 +92,6 @@ SCF_general::SCF_general(const Molecule& molecule_,
   molecule(molecule_),
   extraCharges(extraCharges_),
   basisInfo(basisInfo_),
-  basisInfoDensFit(basisInfoDensFit_),
   integralInfo(integralInfo_), 
   guessDmatFileName(guessDmatFileName_), // FIXME: copy this object properly
   J_K_params(J_K_params_), 
@@ -64,7 +99,6 @@ SCF_general::SCF_general(const Molecule& molecule_,
   scfopts(scfoptsPtr), 
   matOpts(matOpts_),
   threshold_integrals_1el(threshold_integrals_1el_input),
-  densfit_data(NULL),
   DIIS(NULL),
   curr_cycle_stats(NULL)
 {
@@ -91,7 +125,7 @@ SCF_general::SCF_general(const Molecule& molecule_,
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Process ID (PID): %10i", getpid());
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "************************************************************");
   ergo_real minDist, maxDist;
-  molecule.getExtremeInternuclearDistances(minDist, maxDist);
+  molecule.getExtremeInternuclearDistancesQuadratic(minDist, maxDist);
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Min internuclear distance: %12.5f a.u.  = %12.5f Angstrom", (double)minDist, (double)(minDist/UNIT_one_Angstrom));
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Max internuclear distance: %12.5f a.u.  = %12.5f Angstrom", (double)maxDist, (double)(maxDist/UNIT_one_Angstrom));
 
@@ -104,14 +138,6 @@ SCF_general::SCF_general(const Molecule& molecule_,
       throw "error in compute_overlap_matrix_sparse";
     }
 
-  if ( scfopts.do_sparsity_investigation == 1 ) {
-    output_distance_vs_magnitude( basisInfo, S_symm, 
-				  matOpts.inversePermutationHML,
-				  "dist_vs_mag_S",
-				  scfopts.sparsity_plots_resolution_r,  
-				  scfopts.sparsity_plots_resolution_m);
-  }
-
   output_sparsity_symm(n, S_symm, "S_symm before trunc");  
   output_current_memory_usage(LOG_AREA_SCF, "after getting overlap matrix");
   {
@@ -133,6 +159,9 @@ SCF_general::SCF_general(const Molecule& molecule_,
     write_matrix_in_matrix_market_format( S_symm, matOpts.inversePermutationHML, "S_matrix", 
 					  ss_id.str(), scfopts.method_and_basis_set );
   }
+  if ( scfopts.create_basis_func_coord_file == 1 ) {
+    write_basis_func_coord_file(basisInfo);
+  }
   if( scfopts.create_mtx_files_S_and_quit == 1 ) {
     // Write overlap matrix in matrix market format using different basis function orderings, and then quit.
     create_mtx_files_with_different_orderings(S_symm,
@@ -142,18 +171,18 @@ SCF_general::SCF_general(const Molecule& molecule_,
 					      basisInfo);
     throw "Breaking because create_mtx_files_S_and_quit was set.";
   }
-  if ( scfopts.create_basis_func_coord_file == 1 ) {
-    write_basis_func_coord_file(basisInfo);
-  }
   if ( scfopts.create_2el_integral_m_file == 1 ) 
     write_2el_integral_m_file(basisInfo, integralInfo);
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Computing Euclidean norm of overlap matrix (just for fun)...");  
-  Util::TimeMeter timeMeterEuclS;
-  ergo_real S_symm_euclnorm = S_symm.eucl( std::sqrt(std::numeric_limits<ergo_real>::epsilon()) );
-  timeMeterEuclS.print(LOG_AREA_SCF, "S_symm.eucl()");
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Euclidean norm of overlap matrix = %22.11f", 
-	    (double)S_symm_euclnorm);
+  {
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Computing Euclidean norm of overlap matrix (just for fun)...");
+    Util::TimeMeter timeMeterEuclS;
+    ergo_real acc = -1; // value set by call below
+    ergo_real S_symm_euclnorm = get_eucl_norm_try_different_acc(S_symm, acc);
+    timeMeterEuclS.print(LOG_AREA_SCF, "S_symm.eucl()");
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Euclidean norm of overlap matrix = %22.11f  (acc %7.3g)",
+	      (double)S_symm_euclnorm, (double)acc);
+  }
 
   invCholFactor.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
@@ -163,10 +192,6 @@ SCF_general::SCF_general(const Molecule& molecule_,
   invCholFactor.inch(S_symm, matOpts.threshold_inch, mat::right);
   timeMeterInch.print(LOG_AREA_SCF, "invCholFactor.inch");
 
-  if ( scfopts.do_sparsity_investigation == 1 ) {
-    output_magnitude_histogram( invCholFactor, "mag_histogram_invChol", scfopts.sparsity_plots_resolution_m);
-  }
-
   output_sparsity_triang(n, invCholFactor, "invCholFactor before truncation");
   {
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
@@ -184,7 +209,7 @@ SCF_general::SCF_general(const Molecule& molecule_,
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Frobenius norm of invCholFactor after truncation = %22.11f", 
 	      (double)invCholFactor_frobnorm);
     Util::TimeMeter timeMeterEucl;
-    invCholFactor_euclnorm = invCholFactor.eucl( std::sqrt(std::numeric_limits<ergo_real>::epsilon()) );
+    invCholFactor_euclnorm = invCholFactor.eucl( template_blas_sqrt(mat::getMachineEpsilon<ergo_real>()) );
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Euclidean norm of invCholFactor after truncation = %22.11f", 
 	      (double)invCholFactor_euclnorm);
     timeMeterEucl.print(LOG_AREA_SCF, "invCholFactor.eucl()");
@@ -231,6 +256,7 @@ SCF_general::SCF_general(const Molecule& molecule_,
   output_current_memory_usage(LOG_AREA_SCF, "after writing invCholFactor and S_symm to file");
 
   H_core_Matrix.resetSizesAndBlocks(matOpts.size_block_info, matOpts.size_block_info);
+  ergo_real nuclearRepulsionEnergyTmp = 0;
   if(scfopts.skip_H_core == 1)
     {
       do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "NOTE: skip_H_core parameter set, will skip construction of H_core matrix!");
@@ -254,7 +280,8 @@ SCF_general::SCF_general(const Molecule& molecule_,
 					      threshold_integrals_1el,
 					      scfopts.no_of_threads_for_V,
 					      matOpts.size_block_info,
-					      matOpts.permutationHML) != 0)
+					      matOpts.permutationHML,
+					      nuclearRepulsionEnergyTmp) != 0)
 	  {
 	    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_h_core_matrix_simple_dense");
 	    throw "error in compute_h_core_matrix_simple_dense";
@@ -271,6 +298,8 @@ SCF_general::SCF_general(const Molecule& molecule_,
 					H_core_Matrix,
 					threshold_integrals_1el,
 					scfopts.no_of_threads_for_V,
+					scfopts.box_size_for_V_and_T,
+					nuclearRepulsionEnergyTmp,
 					matOpts.size_block_info,
 					matOpts.permutationHML,
 					scfopts.create_mtx_files_dipole,
@@ -301,6 +330,9 @@ SCF_general::SCF_general(const Molecule& molecule_,
   
   output_current_memory_usage(LOG_AREA_SCF, "after getting H_core_Matrix");
 
+  ergo_real nuclearElectricFieldEnergyTmp = molecule.getNuclearElectricFieldEnergy(scfopts.electric_field);
+  nuclearEnergy = nuclearRepulsionEnergyTmp + nuclearElectricFieldEnergyTmp;
+
   if ( scfopts.create_mtx_file_H_core == 1 ) {
     // Write H_core matrix in matrix market format
     std::stringstream ss_id;
@@ -311,18 +343,7 @@ SCF_general::SCF_general(const Molecule& molecule_,
 
   H_core_Matrix.writeToFile();
   output_current_memory_usage(LOG_AREA_SCF, "after writing H_core_Matrix to file");
-  
 
-  if(J_K_params.use_densfit_for_J == 1)
-    {
-      densfit_data = densfit_init(&integralInfo, basisInfoDensFit);
-      if (densfit_data == NULL)
-        {
-	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in densfit_init");
-	  throw "error in densfit_init";
-        }
-    }
-  
   noOfElectrons = molecule.getNumberOfElectrons();  
 
   get_hf_weight_and_cam_params(scfopts.use_dft, &CAM_params.alpha,
@@ -357,46 +378,29 @@ SCF_general::SCF_general(const Molecule& molecule_,
   DensFromFock.set_stopCriterionNormPurification(scfopts.purification_stop_crit_norm);
   DensFromFock.set_invCholFactor(invCholFactor, invCholFactor_euclnorm);
   DensFromFock.set_gap_expected_lower_bound(scfopts.gap_expected_lower_bound);
-  DensFromFock.set_sparsity_plots_resolution_m(scfopts.sparsity_plots_resolution_m);
   DensFromFock.set_purification_maxmul(scfopts.purification_maxmul);
 
 
-  int set = 1;
-
-  if( scfopts.do_sparsity_investigation == set)
-    DensFromFock.set_do_sparsity_investigation();
-  else
-    DensFromFock.unset_do_sparsity_investigation();
-
-   
-
-  if( scfopts.do_comparison_to_simple_purification == set )
-    DensFromFock.set_do_comparison_to_simple_purification();
-  else
-    DensFromFock.unset_do_comparison_to_simple_purification();
-
-
-  if( scfopts.do_puri_mmul_tests == set )
-    DensFromFock.set_do_puri_mmul_tests();
-  else
-    DensFromFock.unset_do_puri_mmul_tests();
-
-
-
-  if( scfopts.purification_create_m_files == set )
+  if( scfopts.purification_create_m_files > 0 )
     DensFromFock.set_purification_create_m_files();
   else
     DensFromFock.unset_purification_create_m_files();
 
 
 
-  if( scfopts.output_homo_and_lumo_eigenvectors == set )
+  if( scfopts.output_homo_and_lumo_eigenvectors > 0 )
     DensFromFock.set_output_homo_and_lumo_eigenvectors();
   else
     DensFromFock.unset_output_homo_and_lumo_eigenvectors();
 
 
-
+  DensFromFock.set_number_of_eigenvectors_to_compute(
+    scfopts.number_of_occupied_eigenvectors, scfopts.number_of_unoccupied_eigenvectors);
+  
+  DensFromFock.set_projection_method_params(
+    scfopts.go_back_X_iter_proj_method, 
+    scfopts.jump_over_X_iter_proj_method);
+  
   /*
     do not plot figures from the purification by default
     set flag before calling function for purification
@@ -406,50 +410,50 @@ SCF_general::SCF_general(const Molecule& molecule_,
   DensFromFock.unset_generate_figures();
 
 
-  if( scfopts.purification_ignore_failure == set )
+  if( scfopts.purification_ignore_failure > 0 )
     DensFromFock.set_purification_ignore_failure();
   else
     DensFromFock.unset_purification_ignore_failure();
 
 
 
-  if( scfopts.purification_use_rand_perturbation_for_alleigsint == set )
+  if( scfopts.purification_use_rand_perturbation_for_alleigsint > 0 )
     DensFromFock.set_purification_use_rand_perturbation_for_alleigsint();
   else
     DensFromFock.unset_purification_use_rand_perturbation_for_alleigsint();
 
 
   // purification or something else?
-  if(scfopts.use_stochastic_orbs != set && scfopts.use_diagonalization != set) // then use purification
+  if(scfopts.use_diagonalization == 0) // then use purification
     DensFromFock.set_use_purification();
   else
     DensFromFock.unset_use_purification();
 
   // purification settings
-  if(DensFromFock.get_use_purification() == set)
+  if(DensFromFock.get_use_purification() > 0)
     {
       DensFromFock.set_purification_limits(scfopts.purification_subspace_err_limit,
 					   scfopts.purification_eigvalue_err_limit, 
 					   scfopts.puri_eig_acc_factor_for_guess);
 
-      if(scfopts.purification_with_acceleration == set)
+      if(scfopts.purification_with_acceleration > 0)
 	DensFromFock.set_use_acceleration();
       else
 	DensFromFock.unset_use_acceleration();
 
-      if(scfopts.use_new_stopping_criterion == set)
+      if(scfopts.use_new_stopping_criterion > 0)
 	DensFromFock.set_use_new_stopping_criterion();
       else
 	DensFromFock.unset_use_new_stopping_criterion();
     }
 
 
-  if(scfopts.use_diag_on_error == set)
+  if(scfopts.use_diag_on_error > 0)
     DensFromFock.set_use_diag_on_error();
   else
     DensFromFock.unset_use_diag_on_error();
 
-  if(scfopts.use_diag_on_error_guess == set)
+  if(scfopts.use_diag_on_error_guess > 0)
     DensFromFock.set_use_diag_on_error_guess();
   else
     DensFromFock.unset_use_diag_on_error_guess();
@@ -458,7 +462,7 @@ SCF_general::SCF_general(const Molecule& molecule_,
 
 
   // diagonalization settings
-  if(scfopts.use_diagonalization == set)
+  if(scfopts.use_diagonalization > 0)
     DensFromFock.set_use_diagonalization();
   else
     DensFromFock.unset_use_diagonalization();
@@ -467,18 +471,16 @@ SCF_general::SCF_general(const Molecule& molecule_,
   DensFromFock.set_diagonalization_params(scfopts.electronic_temperature,
 					  S_symm);
 
-  if(scfopts.use_diagonalization == set && scfopts.store_all_eigenvalues_to_file == set)
+  if(scfopts.use_diagonalization > 0 && scfopts.store_all_eigenvalues_to_file > 0)
     DensFromFock.set_store_all_eigenvalues_to_file();
   else
     DensFromFock.unset_store_all_eigenvalues_to_file();
-  
 
-  // stochastic orbitals settings
-  if(scfopts.use_stochastic_orbs == set)
-    DensFromFock.set_use_stochastic_orbitals();
+
+  if(scfopts.save_permuted_F_matrix_in_bin > 0)
+    DensFromFock.set_save_permuted_F_matrix_in_bin();
   else
-    DensFromFock.unset_use_stochastic_orbitals();
- 
+    DensFromFock.unset_save_permuted_F_matrix_in_bin();
 }
 
 
@@ -490,7 +492,7 @@ SCF_general::~SCF_general()
 
 ergo_real SCF_general::GetEuclideanNormOfMatrix(const symmMatrix & A)
 {
-  ergo_real acc = std::sqrt(std::numeric_limits<ergo_real>::epsilon());
+  ergo_real acc = template_blas_sqrt(mat::getMachineEpsilon<ergo_real>());
   return A.eucl(acc);
 }
 
@@ -537,11 +539,6 @@ void SCF_general::do_SCF_iterations()
   if(DIIS->Initialize(scfopts.max_no_of_diis_matrices) != 0)
     throw "Error in DIIS->Initialize";
 
-  ergo_real nuclearRepulsionEnergyTmp = molecule.getNuclearRepulsionEnergy();
-  ergo_real nuclearElectricFieldEnergyTmp =
-    molecule.getNuclearElectricFieldEnergy(scfopts.electric_field);
-  nuclearEnergy = nuclearRepulsionEnergyTmp + nuclearElectricFieldEnergyTmp;
-
   initialize_matrices();
 
   if(J_K_params.threshold_J <= 0 || J_K_params.threshold_K <= 0)
@@ -572,11 +569,6 @@ void SCF_general::do_SCF_iterations()
   }
 
 
-  if(scfopts.use_stochastic_orbs == 1)
-    prepare_stochastic_orbitals();
-
-
-
   // Use at least two iterations if disimple_starting_guess used.
   int min_number_of_iterations = scfopts.min_number_of_iterations;
   if(scfopts.use_simple_starting_guess && min_number_of_iterations < 2)
@@ -893,9 +885,9 @@ void SCF_general::do_SCF_iterations()
 	create_mtx_files_D( step );
 
       if ( scfopts.output_homo_and_lumo_eigenvectors ) {
-	// Write homo and lumo eigenvectors to file
-	create_eigenvectors_files();
-	create_gabedit_file();
+      	// Write homo and lumo eigenvectors to file
+      	create_eigenvectors_files();
+      	create_gabedit_file();
       }
             
       do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF cycle %3i finished.", step);
@@ -938,14 +930,14 @@ void SCF_general::do_SCF_iterations()
   if(scfopts.save_final_potential == 1)
     save_final_potential();
   
+  if(scfopts.output_expected_values_pos_operator == 1)
+    output_expected_values_pos_operator();
+
   if(scfopts.output_density_images == 1)
     output_density_images();
   
   if(scfopts.write_diag_dens_to_file == 1)
     write_diag_dens_to_file();
-  
-  if(scfopts.output_csr_matrices_for_gao == 1)
-    output_csr_matrices_for_gao();
 
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_general::do_SCF_iterations finished.");
   timeMeterTot.print(LOG_AREA_SCF, "SCF_general::do_SCF_iterations");
