@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file SCF_general.h
+
+    @brief Class for self-consistent field (SCF) procedure; base class
+    that can be used for both restricted and unrestricted cases.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #ifndef SCF_GENERAL_HEADER
 #define SCF_GENERAL_HEADER
 
@@ -34,7 +43,6 @@
 #include "integrals_2el.h"
 #include "grid_stream.h"
 #include "scf.h"
-#include "densityfitting.h"
 #include "diis_general.h"
 #include "SCF_statistics.h"
 #include "GetDensFromFock.h"
@@ -57,7 +65,6 @@ class SCF_general
   SCF_general(const Molecule& molecule_, 
 	      const Molecule& extraCharges_,
 	      const BasisInfoStruct & basisInfo_, 
-	      const BasisInfoStruct & basisInfoDensFit_,
 	      const IntegralInfo & integralInfo_,
 	      const char* guessDmatFileName_,
 	      const JK::Params& J_K_params_,
@@ -72,7 +79,6 @@ class SCF_general
   const Molecule& molecule;
   const Molecule& extraCharges;
   const BasisInfoStruct & basisInfo;
-  const BasisInfoStruct & basisInfoDensFit;
   const IntegralInfo& integralInfo;
   const char* guessDmatFileName;
   const JK::Params& J_K_params;
@@ -80,7 +86,6 @@ class SCF_general
   const SCF::Options& scfopts;
   const SCF::MatOptions& matOpts;
   ergo_real threshold_integrals_1el;
-  DensfitData* densfit_data;
 
   //integral_prep_struct* integralPrep;
 
@@ -142,9 +147,8 @@ class SCF_general
   virtual void write_density_to_file() = 0;
   virtual void save_final_potential() = 0;
   virtual void add_random_disturbance_to_starting_guess() = 0;
+  virtual void output_expected_values_pos_operator() = 0;
   virtual void output_density_images() = 0;
-  virtual void prepare_stochastic_orbitals() = 0;
-  virtual void output_csr_matrices_for_gao() = 0;
   virtual void write_diag_dens_to_file() = 0;
   virtual void report_final_results() = 0;
   virtual void save_density_as_prevdens() = 0;
@@ -157,8 +161,6 @@ class SCF_general
   virtual void report_density_difference() = 0;
   virtual void create_mtx_files_F(int const scfIter) = 0;
   virtual void create_mtx_files_D(int const scfIter) = 0;
-  /* virtual void create_homo_eigvec_file() const = 0; */
-  /* virtual void create_lumo_eigvec_file() const = 0; */
   virtual void create_eigenvectors_files() const = 0;
   virtual void create_gabedit_file() const = 0;
   virtual void compute_dipole_moment() = 0;
