@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file SCF_restricted.h
+
+    @brief Class for self-consistent field (SCF) procedure;
+    spin-restricted case.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #ifndef SCF_RESTRICTED_HEADER
 #define SCF_RESTRICTED_HEADER
 
@@ -42,9 +51,8 @@ class SCF_restricted : public SCF_general
   SCF_restricted(const Molecule& molecule_,
 		 const Molecule& extraCharges_,
 		 const BasisInfoStruct & basisInfo_, 
-		 const BasisInfoStruct & basisInfoDensFit_,
 		 const IntegralInfo& integralInfo_,
-		 const char* guessDmatFileNamePtr,
+		 const char* guessDmatFileNamePtr, // FIXME ELIAS: use std::string for this instead?
 		 const JK::Params& J_K_paramsPtr,
 		 const Dft::GridParams& gridParams_,
 		 const SCF::Options& scfopts,
@@ -79,9 +87,10 @@ class SCF_restricted : public SCF_general
   void write_density_to_file();
   void save_final_potential();
   void add_random_disturbance_to_starting_guess();
+  void output_expected_values_pos_operator();
+  void get_expected_values_pos_operator(generalVector &eigVec, const char *vector_name);
   void output_density_images();
-  void prepare_stochastic_orbitals();
-  void output_csr_matrices_for_gao();
+  void output_density_images_orbital(generalVector &eigVec, const std::string &filename_id);
   void write_diag_dens_to_file();
   void report_final_results();
   void save_density_as_prevdens();
@@ -97,10 +106,12 @@ class SCF_restricted : public SCF_general
   void create_homo_eigvec_file() const;
   void create_lumo_eigvec_file() const;
   void create_eigenvectors_files() const;
+  void create_eigenvalues_files() const;
   void create_eigvec_file(const generalVector &eigVec, 
 			  const char *vector_name, 
 			  const char *filename_id) const;
   void create_gabedit_file() const;
+  void create_gabedit_file_2() const;
   void compute_dipole_moment();
   void do_mulliken_pop_stuff();
   void compute_gradient_fixeddens();
@@ -131,21 +142,16 @@ class SCF_restricted : public SCF_general
   symmMatrix K_matrix;
   symmMatrix Fxc_matrix;
 
-  generalVector eigVecLUMO;
-  generalVector eigVecHOMO;
-  generalVector eigVecLUMO2;
-  generalVector eigVecHOMO2;
+  std::vector<generalVector> eigVecOCC;
+  std::vector<generalVector> eigVecUNOCC;
+  std::vector<ergo_real> eigValOCC; 
+  std::vector<ergo_real> eigValUNOCC;
 
   intervalType homoInterval_F_ort_prev;
   intervalType lumoInterval_F_ort_prev;
-  intervalType homo2Interval_F_ort_prev;
-  intervalType lumo2Interval_F_ort_prev;
   intervalType homoInterval_Fprev;
   intervalType lumoInterval_Fprev;
 
-  std::vector< std::vector<ergo_real> > stochastic_orbitals;
-
-
 };
 
 
