@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file SCF_unrestricted.cc
+
+    @brief Class for self-consistent field (SCF) procedure;
+    spin-unrestricted case.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #include <sstream>
 #include "SCF_unrestricted.h"
 #include "output.h"
@@ -45,7 +54,6 @@
 SCF_unrestricted::SCF_unrestricted(const Molecule& molecule_,
 				   const Molecule& extraCharges_,
 				   const BasisInfoStruct & basisInfo_, 
-				   const BasisInfoStruct & basisInfoDensFit_,
 				   const IntegralInfo& integralInfo_,
 				   const char* guessDmatFileName_,
 				   const JK::Params& J_K_params_,
@@ -57,7 +65,6 @@ SCF_unrestricted::SCF_unrestricted(const Molecule& molecule_,
   :   SCF_general(molecule_,
 		  extraCharges_,
 		  basisInfo_, 
-		  basisInfoDensFit_,
 		  integralInfo_,
 		  guessDmatFileName_,
 		  J_K_params_,
@@ -156,6 +163,7 @@ void SCF_unrestricted::get_starting_guess_density()
   // set up starting guess
 
   int n = basisInfo.noOfBasisFuncs;
+  DensFromFock.set_SCF_step(SCF_step);
 
   if(guessDmatFileName != NULL)
     {
@@ -214,9 +222,6 @@ void SCF_unrestricted::get_starting_guess_density()
 
 
 	  // save flags
-	  int use_stochastic_orbitals = DensFromFock.get_use_stochastic_orbitals();
-	  DensFromFock.unset_use_stochastic_orbitals();
-
 	  int use_diag_on_error = DensFromFock.get_use_diag_on_error();
 
 	  if(DensFromFock.get_use_diag_on_error_guess() == 1)
@@ -233,8 +238,6 @@ void SCF_unrestricted::get_starting_guess_density()
 	  DensFromFock.set_no_occupied_orbs(noOfElectrons_alpha);
 	  DensFromFock.clean_eigs_intervals();
 
-	  DensFromFock.set_generate_figures(0, "_alpha");
-
   	  if(DensFromFock.get_dens_from_fock(H_core_Matrix,
 					     densityMatrix_alpha,
 					     F_ort_prev_dummy_alpha) != 0)
@@ -253,8 +256,6 @@ void SCF_unrestricted::get_starting_guess_density()
 	  DensFromFock.set_no_occupied_orbs(noOfElectrons_beta);
 	  DensFromFock.clean_eigs_intervals();
 
-	  DensFromFock.set_generate_figures(0, "_beta");
-
   	  if(DensFromFock.get_dens_from_fock(H_core_Matrix,
 					     densityMatrix_beta,
 					     F_ort_prev_dummy_beta) != 0)
@@ -263,9 +264,6 @@ void SCF_unrestricted::get_starting_guess_density()
 	    }
 
 	  // return values of flags
-	  if( use_stochastic_orbitals == 1 )
-	    DensFromFock.set_use_stochastic_orbitals();
-
 	  if( use_diag_on_error != 1 )
 	    DensFromFock.unset_use_diag_on_error();
 	    
@@ -377,7 +375,6 @@ void SCF_unrestricted::get_2e_part_and_energy()
     densityMatrix_alpha.readFromFile();
     densityMatrix_beta.readFromFile();
     if(get_2e_matrices_and_energy_sparse_unrestricted(basisInfo,
-						      basisInfoDensFit, 
 						      molecule,
 						      integralInfo, 
 						      CAM_params,
@@ -390,7 +387,6 @@ void SCF_unrestricted::get_2e_part_and_energy()
 						      scfopts.use_dft,
 						      &energy_2el,
 						      noOfElectrons,
-						      densfit_data,
 						      matOpts.size_block_info,
 						      matOpts.permutationHML,
 						      matOpts.inversePermutationHML) != 0)
@@ -417,7 +413,6 @@ void SCF_unrestricted::get_2e_part_and_energy()
     densityMatrix_alpha.readFromFile();
     densityMatrix_beta.readFromFile();
     if(get_2e_matrices_and_energy_restricted_open(basisInfo,
-						  basisInfoDensFit, 
 						  molecule,
 						  integralInfo, 
 						  CAM_params,
@@ -430,7 +425,6 @@ void SCF_unrestricted::get_2e_part_and_energy()
 						  scfopts.use_dft,
 						  &energy_2el,
 						  noOfElectrons,
-						  densfit_data,
 						  matOpts.size_block_info,
 						  matOpts.permutationHML,
 						  matOpts.inversePermutationHML) != 0)
@@ -765,49 +759,49 @@ void SCF_unrestricted::get_new_density_matrix()
 	    "SCF_unrestricted::get_new_density_matrix, noOfElectrons_alpha noOfElectrons_beta = %5i %5i",
 	    noOfElectrons_alpha, noOfElectrons_beta);
 
+  DensFromFock.set_SCF_step(SCF_step);
+
   // ALPHA
   DensFromFock.set_no_occupied_orbs( noOfElectrons_alpha);
   DensFromFock.set_eigs_F_ort_prev(homoInterval_F_ort_prev_alpha, lumoInterval_F_ort_prev_alpha);
-  DensFromFock.set_eigs2_F_ort_prev(homo2Interval_F_ort_prev_alpha, lumo2Interval_F_ort_prev_alpha);
-  DensFromFock.set_generate_figures(SCF_step, "_alpha");
+  DensFromFock.set_generate_figures("_alpha");
 
  // if eigenvectors are needed, set params
   int use_init_guess = 0;
   if(scfopts.use_prev_vector_as_initial_guess == 1 && 
-     scfopts.min_number_of_iterations <= SCF_step &&
+     scfopts.min_number_of_iterations-1 <= SCF_step &&
      SCF_step > 1 &&
-     !eigVecLUMO_alpha.is_empty() && !eigVecHOMO_alpha.is_empty()) // ensure that we computed vectors in previous cycle
+     !eigVecUNOCC_alpha.empty() && !eigVecOCC_alpha.empty()) // ensure that we computed vectors in previous cycle
     use_init_guess = 1;
 
 // if we use new purification
   if(SCF_step > 1 && 
+		 (scfopts.min_number_of_iterations-1 <= SCF_step) &&
      DensFromFock.get_use_purification() == 1 &&
      DensFromFock.get_output_homo_and_lumo_eigenvectors() == 1)
     {   
       DensFromFock.compute_eigenvectors(scfopts.eigenvectors_method, 
 					scfopts.eigenvectors_iterative_method, 
-					scfopts.number_of_eigenpairs, 
 					scfopts.eigensolver_accuracy, 
 					scfopts.eigensolver_maxiter,
 					use_init_guess,
 					scfopts.try_eigv_on_next_iteration_if_fail);
+
+     DensFromFock.compute_eigenvectors_extra(scfopts.puri_compute_eigv_in_each_iteration, scfopts.run_shift_and_square_method_on_F);
+	 }
       
-      // if we need more eigenpairs, initialize pointers in the DensFromFock class
-      if(scfopts.number_of_eigenpairs == 2)
-	DensFromFock.set_second_eigenvectors_pointers(&eigVecLUMO2_alpha, &eigVecHOMO2_alpha);
-    }
 
   if(DensFromFock.get_dens_from_fock(FockMatrix_alpha,
 				     densityMatrix_alpha,
-				     F_ort_prev_alpha,
-				     &eigVecLUMO_alpha,
-			             &eigVecHOMO_alpha) != 0)
+				     F_ort_prev_alpha) != 0)
     {
       throw "SCF_unrestricted::get_new_density_matrix: Error in get_dens_from_fock for alpha";
     }
+		
+	// return empty vectors in case nothing is computed
+	DensFromFock.get_computed_eigenpairs(eigVecUNOCC_alpha, eigVecOCC_alpha, eigValUNOCC_alpha, eigValOCC_alpha);
 	
   DensFromFock.get_eigs_F_ort_prev(homoInterval_F_ort_prev_alpha, lumoInterval_F_ort_prev_alpha);
-  DensFromFock.get_eigs2_F_ort_prev(homo2Interval_F_ort_prev_alpha, lumo2Interval_F_ort_prev_alpha);
   DensFromFock.get_eigs_Fprev(homoInterval_Fprev_alpha, lumoInterval_Fprev_alpha);
 
   DensFromFock.unset_generate_figures();
@@ -817,47 +811,43 @@ void SCF_unrestricted::get_new_density_matrix()
   // BETA
   DensFromFock.set_no_occupied_orbs( noOfElectrons_beta);
   DensFromFock.set_eigs_F_ort_prev(homoInterval_F_ort_prev_beta, lumoInterval_F_ort_prev_beta);
-  DensFromFock.set_eigs2_F_ort_prev(homo2Interval_F_ort_prev_beta, lumo2Interval_F_ort_prev_beta);
-  DensFromFock.set_generate_figures(SCF_step, "_beta");
+  DensFromFock.set_generate_figures("_beta");
 
  // if eigenvectors are needed, set params
   use_init_guess = 0;
   if(scfopts.use_prev_vector_as_initial_guess == 1 && 
-     scfopts.min_number_of_iterations <= SCF_step &&
+     scfopts.min_number_of_iterations-1 <= SCF_step &&
      SCF_step > 1 &&
-     !eigVecLUMO_beta.is_empty() && !eigVecHOMO_beta.is_empty()) // ensure that we computed vectors in previous cycle
+     !eigVecUNOCC_beta.empty() && !eigVecOCC_beta.empty()) // ensure that we computed vectors in previous cycle
     use_init_guess = 1;
 
 // if we use new purification
   if(SCF_step > 1 && 
+		 (scfopts.min_number_of_iterations-1 <= SCF_step) &&
      DensFromFock.get_use_purification() == 1  &&
      DensFromFock.get_output_homo_and_lumo_eigenvectors() == 1)
     {   
       DensFromFock.compute_eigenvectors(scfopts.eigenvectors_method, 
 					scfopts.eigenvectors_iterative_method, 
-					scfopts.number_of_eigenpairs, 
 					scfopts.eigensolver_accuracy, 
 					scfopts.eigensolver_maxiter,
 					use_init_guess,
 					scfopts.try_eigv_on_next_iteration_if_fail);
 
-      // if we need more eigenpairs, initialize pointers in the DensFromFock class
-      if(scfopts.number_of_eigenpairs == 2)
-	DensFromFock.set_second_eigenvectors_pointers(&eigVecLUMO2_beta, &eigVecHOMO2_beta);
-    }
+      DensFromFock.compute_eigenvectors_extra(scfopts.puri_compute_eigv_in_each_iteration, scfopts.run_shift_and_square_method_on_F);
+		}
 
 
   if(DensFromFock.get_dens_from_fock(FockMatrix_beta,
 				     densityMatrix_beta,
-				     F_ort_prev_beta,
-				     &eigVecLUMO_beta,
-			             &eigVecHOMO_beta) != 0)
+				     F_ort_prev_beta) != 0)
     {
       throw "SCF_unrestricted::get_new_density_matrix: Error in get_dens_from_fock for beta";
     }
+	// return empty vectors in case nothing is computed
+	DensFromFock.get_computed_eigenpairs(eigVecUNOCC_beta, eigVecOCC_beta, eigValUNOCC_beta, eigValOCC_beta);
 	
   DensFromFock.get_eigs_F_ort_prev(homoInterval_F_ort_prev_beta, lumoInterval_F_ort_prev_beta);
-  DensFromFock.get_eigs2_F_ort_prev(homo2Interval_F_ort_prev_beta, lumo2Interval_F_ort_prev_beta);
   DensFromFock.get_eigs_Fprev(homoInterval_Fprev_beta, lumoInterval_Fprev_beta);
 
   DensFromFock.unset_generate_figures();
@@ -889,6 +879,24 @@ void SCF_unrestricted::get_new_density_matrix()
 
 
 
+static void prep_matrix_description_struct(matrix_description_struct & mat,
+					   int nvalues,
+					   std::vector<int> & rowind,
+					   std::vector<int> & colind,
+					   std::vector<ergo_real> & values) {
+  mat.nvalues = nvalues;
+  if(nvalues > 0) {
+    mat.rowind = &rowind[0];
+    mat.colind = &colind[0];
+    mat.values = &values[0];
+  }
+  else {
+    mat.rowind = NULL;
+    mat.colind = NULL;
+    mat.values = NULL;
+  }
+}
+
 void SCF_unrestricted::write_density_to_file()
 {
   matrix_description_struct matrixList[2];
@@ -907,10 +915,7 @@ void SCF_unrestricted::write_density_to_file()
 				     matOpts.inversePermutationHML,
 				     matOpts.inversePermutationHML);
   densityMatrix_alpha.writeToFile();
-  matrixList[0].nvalues = nvalues_alpha;
-  matrixList[0].rowind = &rowind_alpha[0];
-  matrixList[0].colind = &colind_alpha[0];
-  matrixList[0].values = &values_alpha[0];
+  prep_matrix_description_struct(matrixList[0], nvalues_alpha, rowind_alpha, colind_alpha, values_alpha);
   // BETA
   densityMatrix_beta.readFromFile();
   int nvalues_beta = densityMatrix_beta.nvalues();
@@ -926,10 +931,7 @@ void SCF_unrestricted::write_density_to_file()
 				    matOpts.inversePermutationHML,
 				    matOpts.inversePermutationHML);
   densityMatrix_beta.writeToFile();
-  matrixList[1].nvalues = nvalues_beta;
-  matrixList[1].rowind = &rowind_beta[0];
-  matrixList[1].colind = &colind_beta[0];
-  matrixList[1].values = &values_beta[0];
+  prep_matrix_description_struct(matrixList[1], nvalues_beta, rowind_beta, colind_beta, values_beta);
 
   if(ddf_writeShellListAndDensityMatricesToFile_sparse(basisInfo,
 						       2,
@@ -951,7 +953,20 @@ void SCF_unrestricted::save_final_potential()
 }
 
 
-void SCF_unrestricted::output_density_images_orbital(generalVector &eigVec, const std::string &filename_id){}
+void SCF_unrestricted::output_expected_values_pos_operator()
+{
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, 
+		"error: output_expected_values_pos_operator not implemented for unrestricted case.");
+      throw "error: output_expected_values_pos_operator not implemented for unrestricted case.";
+}
+
+
+void SCF_unrestricted::output_density_images_orbital(generalVector &eigVec, const std::string &filename_id)
+{
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, 
+		"error: output_density_images_orbital not implemented for unrestricted case.");
+      throw "error: output_density_images_orbital not implemented for unrestricted case.";
+}
 
 void SCF_unrestricted::output_density_images()
 {
@@ -997,11 +1012,6 @@ void SCF_unrestricted::output_density_images()
 }
 
 
-void SCF_unrestricted::prepare_stochastic_orbitals() {
-  throw "Error: SCF_unrestricted::prepare_stochastic_orbitals() not implemented.";
-}
-
-
 void SCF_unrestricted::do_spin_flip(int atomCount)
 {
   int n = basisInfo.noOfBasisFuncs;
@@ -1048,9 +1058,9 @@ void SCF_unrestricted::do_spin_flip(int atomCount)
 	    int jok = 1;
 	    for(int coord = 0; coord < 3; coord++)
 	      {
-		if(std::fabs(molecule.getAtom(k).coords[coord] - basisInfo.basisFuncList[i].centerCoords[coord]) > 1e-5)
+		if(template_blas_fabs(molecule.getAtom(k).coords[coord] - basisInfo.basisFuncList[i].centerCoords[coord]) > 1e-5)
 		  iok = 0;
-		if(std::fabs(molecule.getAtom(k).coords[coord] - basisInfo.basisFuncList[j].centerCoords[coord]) > 1e-5)
+		if(template_blas_fabs(molecule.getAtom(k).coords[coord] - basisInfo.basisFuncList[j].centerCoords[coord]) > 1e-5)
 		  jok = 0;
 	      }
 	    if(iok == 1)
@@ -1092,13 +1102,6 @@ void SCF_unrestricted::do_spin_flip(int atomCount)
 
 
 
-
-void SCF_unrestricted::output_csr_matrices_for_gao()
-{
-  throw "error: SCF_unrestricted::output_csr_matrices_for_gao not implemented.";
-}
-
-
 void SCF_unrestricted::write_diag_dens_to_file()
 {
   int n = basisInfo.noOfBasisFuncs;
@@ -1297,121 +1300,87 @@ void SCF_unrestricted::create_mtx_files_D(int const scfIter) {
   }  
 }
 
-// void SCF_unrestricted::create_homo_eigvec_file() const
-// {
-//   // ALPHA
-//   if (eigVecHOMO_alpha.is_empty()) {
-//     do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO (alpha) eigenvector to file. No HOMO (alpha) eigenvector stored.");
-//   }
-//   else{
-//     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing HOMO (alpha) eigenvector to file homo_coefficient_vec_alpha.txt.");
-//     int n = basisInfo.noOfBasisFuncs;
-//     std::vector<ergo_real> homo_vec_perm(n);
-//     eigVecHOMO_alpha.fullvector(homo_vec_perm); 
-//     // now we have the permuted vector
-//     std::vector<ergo_real> homo_vec(n);
-//     for (int ind = 0; ind < n; ind++) 
-//       homo_vec[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
-//     char ffname[888];
-//     sprintf(ffname, "homo_coefficient_vec_alpha.txt");
-//     std::ofstream ff(ffname);
-//     for (int ind = 0; ind < n; ind++) 
-//       ff << homo_vec[ind] << std::endl;
-//     ff.close();
-//   }
 
-
-//   // BETA
-//   if (eigVecHOMO_beta.is_empty()) {
-//     do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO (beta) eigenvector to file. No HOMO (beta) eigenvector stored.");
-//   }
-//   else{
-//     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing HOMO (beta) eigenvector to file homo_coefficient_vec_beta.txt.");
-//     int n = basisInfo.noOfBasisFuncs;
-//     std::vector<ergo_real> homo_vec_perm(n);
-//     eigVecHOMO_beta.fullvector(homo_vec_perm); 
-//     // now we have the permuted vector
-//     std::vector<ergo_real> homo_vec(n);
-//     for (int ind = 0; ind < n; ind++) 
-//       homo_vec[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
-//     char ffname[888];
-//     sprintf(ffname, "homo_coefficient_vec_beta.txt");
-//     std::ofstream ff(ffname);
-//     for (int ind = 0; ind < n; ind++) 
-//       ff << homo_vec[ind] << std::endl;
-//     ff.close();
-//   }
-// }
-
-
-
-// void SCF_unrestricted::create_lumo_eigvec_file() const
-// {
-//   if (eigVecLUMO_alpha.is_empty()) {
-//     do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output LUMO (alpha) eigenvector to file. No LUMO (alpha) eigenvector stored.");
-//   }
-//   else{
-//     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing LUMO (alpha) eigenvector to file lumo_coefficient_vec_alpha.txt.");
-//     int n = basisInfo.noOfBasisFuncs;
-//     std::vector<ergo_real> lumo_vec_perm(n);
-//     eigVecLUMO_alpha.fullvector(lumo_vec_perm); 
-//     // now we have the permuted vector
-//     std::vector<ergo_real> lumo_vec(n);
-//     for (int ind = 0; ind < n; ind++) 
-//       lumo_vec[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
-//     char ffname[888];
-//     sprintf(ffname, "lumo_coefficient_vec_alpha.txt");
-//     std::ofstream ff(ffname);
-//     for (int ind = 0; ind < n; ind++) 
-//       ff << lumo_vec[ind] << std::endl;
-//     ff.close();
-//   } 
-
-
-//   if (eigVecLUMO_beta.is_empty()) {
-//     do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output LUMO (beta) eigenvector to file. No LUMO (beta) eigenvector stored.");
-//   }
-//   else{
-//     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing LUMO (beta) eigenvector to file lumo_coefficient_vec_beta.txt.");
-//     int n = basisInfo.noOfBasisFuncs;
-//     std::vector<ergo_real> lumo_vec_perm(n);
-//     eigVecLUMO_beta.fullvector(lumo_vec_perm); 
-//     // now we have the permuted vector
-//     std::vector<ergo_real> lumo_vec(n);
-//     for (int ind = 0; ind < n; ind++) 
-//       lumo_vec[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
-//     char ffname[888];
-//     sprintf(ffname, "lumo_coefficient_vec_beta.txt");
-//     std::ofstream ff(ffname);
-//     for (int ind = 0; ind < n; ind++) 
-//       ff << lumo_vec[ind] << std::endl;
-//     ff.close(); 
-//   }
-
-// }
+void SCF_unrestricted::create_eigenvalues_files() const
+{
+  if(! eigValOCC_alpha.empty() )
+  {
+    std::ofstream ff("occupied_spectrum_alpha.txt");
+    for (size_t ind = 0; ind < eigValOCC_alpha.size(); ind++)
+    {
+       ff << (double)eigValOCC_alpha[ind] << std::endl;
+    }
+    ff.close();
+  }
+	
+	if(! eigValOCC_beta.empty() )
+  {
+    std::ofstream ff("occupied_spectrum_beta.txt");
+    for (size_t ind = 0; ind < eigValOCC_beta.size(); ind++)
+    {
+       ff << (double)eigValOCC_beta[ind] << std::endl;
+    }
+    ff.close();
+  }
+  
+  if(! eigValUNOCC_alpha.empty() )
+  {
+    std::ofstream ff("unoccupied_spectrum_alpha.txt");
+    for (size_t ind = 0; ind < eigValUNOCC_alpha.size(); ind++)
+    {
+       ff << (double)eigValUNOCC_alpha[ind] << std::endl;
+    }
+    ff.close();
+  }
+	if(! eigValUNOCC_beta.empty() )
+  {
+    std::ofstream ff("unoccupied_spectrum_beta.txt");
+    for (size_t ind = 0; ind < eigValUNOCC_beta.size(); ind++)
+    {
+       ff << (double)eigValUNOCC_beta[ind] << std::endl;
+    }
+    ff.close();
+  }
+}
 
 
 void SCF_unrestricted::create_eigenvectors_files() const
 {
-  create_eigvec_file(eigVecHOMO_alpha, 
-		     eigVecHOMO_beta, 
-		     "HOMO", 
-		     "homo_coefficient_vec");
+	create_eigenvalues_files();	
+	
+	if(! eigVecOCC_alpha.empty() &&  ! eigVecOCC_beta.empty() )
+	{
+		create_eigvec_file(eigVecOCC_alpha[0], eigVecOCC_beta[0],
+											"HOMO",
+											"homo_coefficient_vec");
+		assert(eigVecOCC_alpha.size() == eigVecOCC_beta.size());								
+		for (size_t i = 1; i < eigVecOCC_alpha.size(); i++) 
+		{
+			std::stringstream ss_name, ss_filename;
+			ss_name << "HOMO-" << i;
+			ss_filename << "occ_" << i << "_coefficient_vec";
+			create_eigvec_file(eigVecOCC_alpha[i], eigVecOCC_beta[i],
+												ss_name.str().c_str(),
+												ss_filename.str().c_str());
+		}
+	}
 
-  create_eigvec_file(eigVecLUMO_alpha, 
-		     eigVecLUMO_beta, 
-		     "LUMO", 
-		     "lumo_coefficient_vec");
-
-  create_eigvec_file(eigVecHOMO2_alpha, 
-  		     eigVecHOMO2_beta, 
-  		     "HOMO2", 
-  		     "homo2_coefficient_vec");
-
-  create_eigvec_file(eigVecLUMO2_alpha, 
-  		     eigVecLUMO2_beta, 
-  		     "LUMO2", 
-  		     "lumo2_coefficient_vec");
+	if(! eigVecUNOCC_alpha.empty() &&  ! eigVecUNOCC_beta.empty() )
+	{
+		create_eigvec_file(eigVecUNOCC_alpha[0], eigVecUNOCC_beta[0],
+											"LUMO",
+											"lumo_coefficient_vec");
+		assert(eigVecUNOCC_alpha.size() == eigVecUNOCC_beta.size());								
+		for (size_t i = 1; i < eigVecUNOCC_alpha.size(); i++) 
+		{
+			std::stringstream ss_name, ss_filename;
+			ss_name << "LUMO-" << i;
+			ss_filename << "unocc_" << i << "_coefficient_vec";
+			create_eigvec_file(eigVecUNOCC_alpha[i], eigVecUNOCC_beta[i],
+												ss_name.str().c_str(),
+												ss_filename.str().c_str());
+		}
+	}
 
 } 
 
@@ -1425,8 +1394,10 @@ void SCF_unrestricted::create_eigvec_file(const generalVector &eigVec_alpha,
   	      "No %s (alpha) eigenvector stored.", vector_name, vector_name);
   }
   else{
+		char ffname[888];
+		sprintf(ffname, "%s_alpha.txt", filename_id);
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing %s (alpha) eigenvector to file "
-  	      "%s.", vector_name, filename_id);
+  	      "%s.", vector_name, ffname);
     int n = basisInfo.noOfBasisFuncs;
     std::vector<ergo_real> vec_perm(n);
     eigVec_alpha.fullvector(vec_perm); 
@@ -1434,11 +1405,9 @@ void SCF_unrestricted::create_eigvec_file(const generalVector &eigVec_alpha,
     std::vector<ergo_real> vec(n);
     for (int ind = 0; ind < n; ind++) 
       vec[ind] = vec_perm[matOpts.inversePermutationHML[ind]];
-    char ffname[888];
-    sprintf(ffname, "%s_alpha.txt", filename_id);
     std::ofstream ff(ffname);
     for (int ind = 0; ind < n; ind++) 
-      ff << vec[ind] << std::endl;
+      ff << (double)vec[ind] << std::endl;
     ff.close();
   } 
 
@@ -1448,8 +1417,10 @@ void SCF_unrestricted::create_eigvec_file(const generalVector &eigVec_alpha,
 	      "No %s (beta) eigenvector stored.", vector_name, vector_name);
   }
   else{
+		char ffname[888];
+		sprintf(ffname, "%s_beta.txt", filename_id);
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing %s (beta) eigenvector to file "
-	      "%s.", vector_name, filename_id);
+	      "%s.", vector_name, ffname);
     int n = basisInfo.noOfBasisFuncs;
     std::vector<ergo_real> vec_perm(n);
     eigVec_beta.fullvector(vec_perm); 
@@ -1457,11 +1428,9 @@ void SCF_unrestricted::create_eigvec_file(const generalVector &eigVec_alpha,
     std::vector<ergo_real> vec(n);
     for (int ind = 0; ind < n; ind++) 
       vec[ind] = vec_perm[matOpts.inversePermutationHML[ind]];
-    char ffname[888];
-    sprintf(ffname, "%s_beta.txt", filename_id);
     std::ofstream ff(ffname);
     for (int ind = 0; ind < n; ind++) 
-      ff << vec[ind] << std::endl;
+      ff << (double)vec[ind] << std::endl;
     ff.close(); 
   }
 
@@ -1481,39 +1450,39 @@ output_orbital_coeffs_in_gabedit_order(const BasisInfoStruct & basisInfo,
     int startIdx = basisInfo.shellList[k].startIndexInMatrix;
     switch(basisInfo.shellList[k].shellType) {
     case 0: // s-type shell
-      ff << count+1 << "   " << orbital_vec[startIdx] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx] << std::endl; count++;
       break;
     case 1: // p-type shell
-      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+0] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+1] << std::endl; count++;
       break;
     case 2: // d-type shell
-      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+3] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+4] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+3] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+1] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+4] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+0] << std::endl; count++;
       break;
     case 3: // f-type shell
-      ff << count+1 << "   " << orbital_vec[startIdx+3] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+4] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+5] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+6] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+3] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+4] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+5] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+1] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+6] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+0] << std::endl; count++;
       break;
     case 4: // g-type shell
-      ff << count+1 << "   " << orbital_vec[startIdx+4] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+5] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+3] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+6] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+7] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+8] << std::endl; count++;
-      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+4] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+5] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+3] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+6] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+7] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+1] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+8] << std::endl; count++;
+      ff << count+1 << "   " << (double)orbital_vec[startIdx+0] << std::endl; count++;
       break;
     default: throw "error in output_orbital_coeffs_in_gabedit_order: shell types beyond g not implemented!";
     }
@@ -1526,67 +1495,19 @@ output_orbital_coeffs_in_gabedit_order(const BasisInfoStruct & basisInfo,
 
 void SCF_unrestricted::create_gabedit_file() const
 {
-  if (eigVecHOMO_alpha.is_empty() || eigVecLUMO_alpha.is_empty() ||
-      eigVecHOMO_beta.is_empty() || eigVecLUMO_beta.is_empty()) {
-    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO/LUMO eigenvectors to gabedit file; no HOMO/LUMO info available.");
+  if (eigVecOCC_alpha.empty() || eigVecUNOCC_alpha.empty() ||
+      eigVecOCC_beta.empty() || eigVecUNOCC_beta.empty()) {
+    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output eigenvectors to gabedit file; no eigenvectors info available.");
     return;
   }
   if(basisInfo.use_6_d_funcs == 1) {
-    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO/LUMO eigenvectors to gabedit file; not implemented for use_6_d_funcs case.");
+    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output eigenvectors to gabedit file; not implemented for use_6_d_funcs case.");
     return;
   }
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating Gabedit file with HOMO/LUMO eigenvector info.");
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating Gabedit file with eigenvector info.");
   int n = basisInfo.noOfBasisFuncs;
 
-  // ALPHA
-  // Get HOMO
-  // now we have the permuted vector
-  std::vector<ergo_real> homo_vec_alpha(n);
-  { 
-    std::vector<ergo_real> homo_vec_perm(n);
-    eigVecHOMO_alpha.fullvector(homo_vec_perm); 
-    for (int ind = 0; ind < n; ind++) 
-      homo_vec_alpha[ind] = homo_vec_perm[matOpts.permutationHML[ind]];
-    //homo_vec_alpha[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
-  }
-
-
-  // Get LUMO
-  // now we have the permuted vector
-  std::vector<ergo_real> lumo_vec_alpha(n);
-  {
-    std::vector<ergo_real> lumo_vec_perm(n);
-    eigVecLUMO_alpha.fullvector(lumo_vec_perm); 
-    for (int ind = 0; ind < n; ind++) 
-      lumo_vec_alpha[ind] = lumo_vec_perm[matOpts.permutationHML[ind]];
-    //lumo_vec_alpha[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
-  }
-
-  // BETA
-  // Get HOMO
-  // now we have the permuted vector
-  std::vector<ergo_real> homo_vec_beta(n);
-  { 
-    std::vector<ergo_real> homo_vec_perm(n);
-    eigVecHOMO_beta.fullvector(homo_vec_perm); 
-    for (int ind = 0; ind < n; ind++) 
-      homo_vec_beta[ind] = homo_vec_perm[matOpts.permutationHML[ind]];
-    //homo_vec_beta[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
-  }
-
-
-  // Get LUMO
-  // now we have the permuted vector
-  std::vector<ergo_real> lumo_vec_beta(n);
-  {
-    std::vector<ergo_real> lumo_vec_perm(n);
-    eigVecLUMO_beta.fullvector(lumo_vec_perm); 
-    for (int ind = 0; ind < n; ind++) 
-      lumo_vec_beta[ind] = lumo_vec_perm[matOpts.permutationHML[ind]];
-    //lumo_vec_beta[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
-  }
-
   // Create Gabedit file.
   const char fileName [] = "gabeditfile.gab";
   std::ofstream ff(fileName);
@@ -1598,10 +1519,10 @@ void SCF_unrestricted::create_gabedit_file() const
   for(int i = 0; i < molecule.getNoOfAtoms(); i++) {
     char atomLabelString[4];
     get_atom_label_from_charge_int(molecule.getAtom(i).charge, atomLabelString, 4);
-    ff << atomLabelString << " " << i+1 << " " << molecule.getAtom(i).charge
-       << "   " << molecule.getAtom(i).coords[0] / UNIT_one_Angstrom
-       << "   " << molecule.getAtom(i).coords[1] / UNIT_one_Angstrom
-       << "   " << molecule.getAtom(i).coords[2] / UNIT_one_Angstrom 
+    ff << atomLabelString << " " << i+1 << " " << (double)molecule.getAtom(i).charge
+       << "   " << (double)(molecule.getAtom(i).coords[0] / UNIT_one_Angstrom)
+       << "   " << (double)(molecule.getAtom(i).coords[1] / UNIT_one_Angstrom)
+       << "   " << (double)(molecule.getAtom(i).coords[2] / UNIT_one_Angstrom)
        << std::endl;
   }
 
@@ -1614,9 +1535,9 @@ void SCF_unrestricted::create_gabedit_file() const
     // Now output info about shells for this atom.
     for(int k = 0; k < basisInfo.noOfShells; k++) {
       // Check if this shell belongs to the current atom.
-      ergo_real absdx = std::abs(basisInfo.shellList[k].centerCoords[0] - molecule.getAtom(i).coords[0]);
-      ergo_real absdy = std::abs(basisInfo.shellList[k].centerCoords[1] - molecule.getAtom(i).coords[1]);
-      ergo_real absdz = std::abs(basisInfo.shellList[k].centerCoords[2] - molecule.getAtom(i).coords[2]);
+      ergo_real absdx = template_blas_fabs(basisInfo.shellList[k].centerCoords[0] - molecule.getAtom(i).coords[0]);
+      ergo_real absdy = template_blas_fabs(basisInfo.shellList[k].centerCoords[1] - molecule.getAtom(i).coords[1]);
+      ergo_real absdz = template_blas_fabs(basisInfo.shellList[k].centerCoords[2] - molecule.getAtom(i).coords[2]);
       ergo_real distlimit = 0.01;
       if(absdx > distlimit || absdy > distlimit || absdz > distlimit)
 	continue;
@@ -1638,7 +1559,7 @@ void SCF_unrestricted::create_gabedit_file() const
 	ergo_real exponent = basisInfo.shellList[k].exponentList[contridx];
 	ergo_real shellFactor = sfi.getShellFactor(integralInfo, exponent, shellType, use_6_d_funcs);
 	ergo_real coeff = basisInfo.shellList[k].coeffList[contridx] / shellFactor;
-	ff << exponent << "   " << coeff << std::endl;
+	ff << (double)exponent << "   " << (double)coeff << std::endl;
       }
     }
     // Blank line before shells for next atom.
@@ -1652,25 +1573,85 @@ void SCF_unrestricted::create_gabedit_file() const
   // MO section.
   ff << "[MO]" << std::endl;
 
-  // ALPHA
-  // HOMO
-  ff << "Spin=Alpha" << std::endl;
-  ff << "Occup=   1.000000" << std::endl;
-  output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, homo_vec_alpha);
-  // LUMO
-  ff << "Spin=Alpha" << std::endl;
-  ff << "Occup=   0.000000" << std::endl;
-  output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, lumo_vec_alpha);
+  // ALPHA	
+	// Occupied orbitals 
+	for (int orb = (int)eigVecOCC_alpha.size()-1; orb >= 0; orb--) // note that orb can be negative
+	{
+		std::vector<ergo_real> orb_vec_perm(n);
+		eigVecOCC_alpha[orb].fullvector(orb_vec_perm);
+		// now we have the permuted vector
+		std::vector<ergo_real> orb_vec(n);
+		for (int ind = 0; ind < n; ind++)
+		{
+			 orb_vec[matOpts.inversePermutationHML[ind]] = orb_vec_perm[ind];
+		}
+		ff << "Spin=Alpha" << std::endl;
+		if((int)eigValOCC_alpha.size() > orb) // in case we do not have eigenvalues
+		 ff << "Energy= " << (double)eigValOCC_alpha[orb] << std::endl;
+		ff << "Occup=   1.000000" << std::endl;
+		output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, orb_vec);
+	}
+	
+	// Unoccupied orbitals 
+	for (size_t orb = 0; orb < eigVecUNOCC_alpha.size(); orb++) 
+	{
+		std::vector<ergo_real> orb_vec_perm(n);
+		eigVecUNOCC_alpha[orb].fullvector(orb_vec_perm);
+		// now we have the permuted vector
+		std::vector<ergo_real> orb_vec(n);
+		for (int ind = 0; ind < n; ind++)
+		{
+			 orb_vec[matOpts.inversePermutationHML[ind]] = orb_vec_perm[ind];
+		}
+		ff << "Spin=Alpha" << std::endl;
+		if(eigValUNOCC_alpha.size() > orb) // in case we do not have eigenvalues
+			ff << "Energy= " << (double)eigValUNOCC_alpha[orb] << std::endl;
+		ff << "Occup=   0.000000" << std::endl;
+		output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, orb_vec);
+	}
+
+	
+	
+	
+	
+	
 
   // BETA
-  // HOMO
-  ff << "Spin=Beta" << std::endl;
-  ff << "Occup=   1.000000" << std::endl;
-  output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, homo_vec_beta);
-  // LUMO
-  ff << "Spin=Beta" << std::endl;
-  ff << "Occup=   0.000000" << std::endl;
-  output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, lumo_vec_beta);
+	// Occupied orbitals 
+	for (int orb = (int)eigVecOCC_beta.size()-1; orb >= 0; orb--)  // note that orb can be negative
+	{
+		std::vector<ergo_real> orb_vec_perm(n);
+		eigVecOCC_beta[orb].fullvector(orb_vec_perm);
+		// now we have the permuted vector
+		std::vector<ergo_real> orb_vec(n);
+		for (int ind = 0; ind < n; ind++)
+		{
+			 orb_vec[matOpts.inversePermutationHML[ind]] = orb_vec_perm[ind];
+		}
+		ff << "Spin=Beta" << std::endl;
+		if((int)eigValOCC_beta.size() > orb) // in case we do not have eigenvalues
+		 ff << "Energy= " << (double)eigValOCC_beta[orb] << std::endl;
+		ff << "Occup=   1.000000" << std::endl;
+		output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, orb_vec);
+	}
+	
+	// Unoccupied orbitals 
+	for (size_t orb = 0; orb < eigVecUNOCC_beta.size(); orb++) 
+	{
+		std::vector<ergo_real> orb_vec_perm(n);
+		eigVecUNOCC_beta[orb].fullvector(orb_vec_perm);
+		// now we have the permuted vector
+		std::vector<ergo_real> orb_vec(n);
+		for (int ind = 0; ind < n; ind++)
+		{
+			 orb_vec[matOpts.inversePermutationHML[ind]] = orb_vec_perm[ind];
+		}
+		ff << "Spin=Beta" << std::endl;
+		if(eigValUNOCC_beta.size() > orb) // in case we do not have eigenvalues
+			ff << "Energy= " << (double)eigValUNOCC_beta[orb] << std::endl;
+		ff << "Occup=   0.000000" << std::endl;
+		output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, orb_vec);
+	}
 
 
   // Blank line before end of file.
@@ -1678,13 +1659,9 @@ void SCF_unrestricted::create_gabedit_file() const
   // Close file.
   ff.close();
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Gabedit file '%s' with HOMO/LUMO eigenvector info created OK.", fileName);
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Gabedit file '%s' with eigenvectors info created OK.", fileName);
 }
 
-void SCF_unrestricted::create_gabedit_file_2() const
-{
-  throw "Error: SCF_unrestricted::create_gabedit_file_2() not implemented";
-}
 
 void SCF_unrestricted::update_subspace_diff()
 {
