@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file SCF_unrestricted.h
+
+    @brief Class for self-consistent field (SCF) procedure;
+    spin-unrestricted case.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #ifndef SCF_UNRESTRICTED_HEADER
 #define SCF_UNRESTRICTED_HEADER
 
@@ -40,7 +49,6 @@ class SCF_unrestricted : public SCF_general
   SCF_unrestricted(const Molecule& molecule_, 
 		   const Molecule& extraCharges_,
 		   const BasisInfoStruct & basisInfo_, 
-		   const BasisInfoStruct & basisInfoDensFit_,
 		   const IntegralInfo & integralInfo_,
 		   const char* guessDmatFileName_,
 		   const JK::Params& J_K_params_,
@@ -78,10 +86,9 @@ class SCF_unrestricted : public SCF_general
   void write_density_to_file();
   void save_final_potential();
   void add_random_disturbance_to_starting_guess();
+  void output_expected_values_pos_operator();
   void output_density_images();
   void output_density_images_orbital(generalVector &eigVec, const std::string &filename_id);
-  void prepare_stochastic_orbitals();
-  void output_csr_matrices_for_gao();
   void write_diag_dens_to_file();
   void report_final_results();
   void save_density_as_prevdens();
@@ -94,15 +101,13 @@ class SCF_unrestricted : public SCF_general
   void report_density_difference();
   void create_mtx_files_F(int const scfIter);
   void create_mtx_files_D(int const scfIter);
-  /* void create_homo_eigvec_file() const; */
-  /* void create_lumo_eigvec_file() const; */
+  void create_eigenvalues_files() const;
   void create_eigenvectors_files() const;
   void create_eigvec_file(const generalVector &eigVec_alpha, 
 			  const generalVector &eigVec_beta,
 			  const char *vector_name, 
 			  const char *filename_id) const;
   void create_gabedit_file() const;
-  void create_gabedit_file_2() const;
   void compute_dipole_moment();
   void do_mulliken_pop_stuff();
   void compute_gradient_fixeddens();
@@ -135,10 +140,6 @@ class SCF_unrestricted : public SCF_general
   intervalType lumoInterval_F_ort_prev_alpha;
   intervalType homoInterval_F_ort_prev_beta;
   intervalType lumoInterval_F_ort_prev_beta;
-  intervalType homo2Interval_F_ort_prev_alpha;
-  intervalType lumo2Interval_F_ort_prev_alpha;
-  intervalType homo2Interval_F_ort_prev_beta;
-  intervalType lumo2Interval_F_ort_prev_beta;
   intervalType homoInterval_Fprev_alpha;
   intervalType lumoInterval_Fprev_alpha;
   intervalType homoInterval_Fprev_beta;
@@ -148,14 +149,14 @@ class SCF_unrestricted : public SCF_general
   int noOfElectrons_alpha;
   int noOfElectrons_beta;
 
-  generalVector eigVecLUMO_alpha;
-  generalVector eigVecHOMO_alpha;
-  generalVector eigVecLUMO_beta;
-  generalVector eigVecHOMO_beta;
-  generalVector eigVecLUMO2_alpha;
-  generalVector eigVecHOMO2_alpha;
-  generalVector eigVecLUMO2_beta;
-  generalVector eigVecHOMO2_beta;
+  std::vector<generalVector> eigVecOCC_alpha;
+  std::vector<generalVector> eigVecUNOCC_alpha;
+  std::vector<ergo_real> eigValOCC_alpha; 
+  std::vector<ergo_real> eigValUNOCC_alpha;
+  std::vector<generalVector> eigVecOCC_beta;
+  std::vector<generalVector> eigVecUNOCC_beta;
+  std::vector<ergo_real> eigValOCC_beta; 
+  std::vector<ergo_real> eigValUNOCC_beta;
 };
 
 
