@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file diis_restricted.cc
+
+    @brief DIISManagerRestricted class implementing direct inversion
+    in the iterative subspace (DIIS) for restricted SCF calculations.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #include <string.h>
 
 #include "diis_restricted.h"
@@ -158,12 +167,6 @@ int DIISManagerRestricted::GetCombinedFockMatrix(symmMatrix & result)
       return -1;
     }
   
-#if 0
-  printf("cVector:\n");
-  for(i = 0; i < dimB; i++)
-    printf("%22.11f\n", (double)cVector[i]);
-#endif
-  
   // Create linear combination of Fock matrices using coefficients from cVector.
   F_list[0][0]->readFromFile();
   result = *F_list[0][0];
