@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file scf.cc
+
+    @brief Code for classes containing various options related to
+    self-consistent field (SCF) calculations.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #include <math.h>
 #include <memory.h>
 #include <stdio.h>
@@ -43,8 +52,11 @@
 
 void SCF::MatOptions::prepare(const BasisInfoStruct& basisInfo)
 {
-  int bufferSize = sparse_matrix_block_size * sparse_matrix_block_size;
-  mat::AllocatorManager<ergo_real>::instance().init(bufferSize, no_of_buffers_per_allocator);
+  if(use_allocator_manager)
+  {
+    int bufferSize = sparse_matrix_block_size * sparse_matrix_block_size;
+    mat::AllocatorManager<ergo_real>::instance().init(bufferSize, no_of_buffers_per_allocator);
+  }
   
   size_block_info = prepareMatrixSizesAndBlocks(basisInfo.noOfBasisFuncs,
 						sparse_matrix_block_size,
