@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,17 +17,26 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
-#ifndef SCFHEADER
-#define SCFHEADER
+/** @file scf.h
+
+    @brief Code for classes containing various options related to
+    self-consistent field (SCF) calculations.
+
+    @author: Elias Rudberg <em>responsible</em>.
+*/
+
+#ifndef SCF_HEADER
+#define SCF_HEADER
 
 #include <string.h>
 
@@ -68,6 +77,7 @@ struct Options {
   ergo_real subspace_factor_dens;
   int use_artificial_subspace_disturbances;
   int no_of_threads_for_V;
+  ergo_real box_size_for_V_and_T;
   int purification_maxmul;
   int purification_create_m_files;
   int purification_ignore_failure;
@@ -92,21 +102,14 @@ struct Options {
   int no_of_impr_req_for_diis;
   int use_diis_always;
   int do_f_thresh_verification;
-  int do_comparison_to_simple_purification;
-  int do_puri_mmul_tests;
   int output_statistics_mfiles;
-  int do_sparsity_investigation;
-  int do_sparsity_investigation_reppuri;
-  int sparsity_plots_resolution_r;
-  int sparsity_plots_resolution_m;
   int no_of_careful_first_scf_steps;
   int do_report_density_diff;
   ergo_real error_maxabs_for_diis;
   int min_number_of_iterations;
   int max_number_of_iterations;
-  int number_of_eigenpairs;
   int output_density_at_every_step;
-  int output_csr_matrices_for_gao;
+  int output_expected_values_pos_operator;
   int output_density_images;
   int output_density_images_only;
   int write_guess_density_only;
@@ -120,6 +123,9 @@ struct Options {
   int use_diag_on_error_guess;
   int store_all_eigenvalues_to_file;
   int try_eigv_on_next_iteration_if_fail;
+  int puri_compute_eigv_in_each_iteration; 
+  int run_shift_and_square_method_on_F;
+  int save_permuted_F_matrix_in_bin;
   int write_overlap_matrix;
   int save_full_matrices_for_matlab;
   int analyze_result_after_scf;
@@ -140,6 +146,10 @@ struct Options {
   int create_basis_func_coord_file;
   int use_prev_vector_as_initial_guess;
   int output_homo_and_lumo_eigenvectors;
+  int number_of_occupied_eigenvectors;
+  int number_of_unoccupied_eigenvectors;
+  int jump_over_X_iter_proj_method;
+  int go_back_X_iter_proj_method;
   std::string eigenvectors_method;	
   std::string eigenvectors_iterative_method;
   ergo_real eigensolver_accuracy;
@@ -148,11 +158,6 @@ struct Options {
   int compute_gradient_fixeddens;
   int verify_gradient_fixeddens;
 
-  int use_stochastic_orbs;
-  int stochastic_orbs_no_of_vectors;
-  int stochastic_orbs_use_unit_vectors;
-  int stochastic_orbs_rand_param;
-
   /** Initializes all the fields to sane values. */
 Options() : calculation_identifier("N/A"),
     method_and_basis_set("N/A"),
@@ -180,6 +185,7 @@ Options() : calculation_identifier("N/A"),
     subspace_factor_dens(0.1),
     use_artificial_subspace_disturbances(0),
     no_of_threads_for_V(1),
+    box_size_for_V_and_T(6.5),
     purification_maxmul(100),
     purification_create_m_files(0),
     purification_ignore_failure(0),
@@ -203,21 +209,14 @@ Options() : calculation_identifier("N/A"),
     no_of_impr_req_for_diis(4),
     use_diis_always(0),
     do_f_thresh_verification(0),
-    do_comparison_to_simple_purification(0),
-    do_puri_mmul_tests(0),
     output_statistics_mfiles(0),
-    do_sparsity_investigation(0),
-    do_sparsity_investigation_reppuri(0),
-    sparsity_plots_resolution_r(100),
-    sparsity_plots_resolution_m(100),
     no_of_careful_first_scf_steps(0),
     do_report_density_diff(1),
     error_maxabs_for_diis(0.5),
     min_number_of_iterations(),
     max_number_of_iterations(),
-    number_of_eigenpairs(1),
     output_density_at_every_step(1),
-    output_csr_matrices_for_gao(0),
+    output_expected_values_pos_operator(0),
     output_density_images(0),
     output_density_images_only(0),
     write_guess_density_only(0),
@@ -231,6 +230,9 @@ Options() : calculation_identifier("N/A"),
     use_diag_on_error_guess(1),
     store_all_eigenvalues_to_file(0),
     try_eigv_on_next_iteration_if_fail(0),
+    puri_compute_eigv_in_each_iteration(0), 
+    run_shift_and_square_method_on_F(0),
+    save_permuted_F_matrix_in_bin(0),
     write_overlap_matrix(0),
     save_full_matrices_for_matlab(0),
     analyze_result_after_scf(0),
@@ -240,7 +242,7 @@ Options() : calculation_identifier("N/A"),
     scan_do_invcholfactor_transf(1),
     scan_no_of_steps(16),
     scan_start_thresh(1e-9),
-    scan_step_factor(sqrt((ergo_real)10)),
+    scan_step_factor(template_blas_sqrt((ergo_real)10)),
     create_mtx_file_S(0),
     create_mtx_file_H_core(0),
     create_mtx_files_F(0),
@@ -251,17 +253,17 @@ Options() : calculation_identifier("N/A"),
     create_basis_func_coord_file(0),
     use_prev_vector_as_initial_guess(0),
     output_homo_and_lumo_eigenvectors(0),
+    number_of_occupied_eigenvectors(1),
+    number_of_unoccupied_eigenvectors(1),
+    jump_over_X_iter_proj_method(3),
+    go_back_X_iter_proj_method(10),
     eigenvectors_method("square"),
     eigenvectors_iterative_method("lanczos"),
-    eigensolver_accuracy(1e-12),
+    eigensolver_accuracy(1e-4*template_blas_sqrt(mat::getMachineEpsilon<ergo_real>())),
     eigensolver_maxiter(200),
     output_mulliken_pop(0),
     compute_gradient_fixeddens(0),
-    verify_gradient_fixeddens(0),
-    use_stochastic_orbs(0),
-    stochastic_orbs_no_of_vectors(100),
-    stochastic_orbs_use_unit_vectors(0),
-    stochastic_orbs_rand_param(1)
+    verify_gradient_fixeddens(0)
   { 
     memset(disturbedElementIndexVector, 0,
            sizeof(disturbedElementIndexVector));
@@ -285,10 +287,11 @@ struct MatOptions {
   int threads;
   int parallelLevel;
   int no_of_buffers_per_allocator;
+  int use_allocator_manager;
 
   MatOptions() :
     sparse_threshold(1e-8),
-    threshold_inch(0),
+    threshold_inch(1e-10),
     sparse_matrix_block_size(32),
     sparse_matrix_block_factor_3(8),
     sparse_matrix_block_factor_2(8),
@@ -297,7 +300,8 @@ struct MatOptions {
     parallelLevel(1),
     /* FIXME: there should be a param to set no_of_buffers_per_allocator, for large calculations it needs to be larger, e.g. 10 x larger seems to give much better performance of matrix operations for large cases. 
        This is also connected to blocksize, maybe the best solution would be to have a param determining the number of MegaBytes per allocator or something like that.  */
-    no_of_buffers_per_allocator(20000) 
+    no_of_buffers_per_allocator(20000),
+    use_allocator_manager(1)
   {};
   ~MatOptions() {
   }
