@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,22 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
 /** @file scf_utils.cc
- Various utilities used by SCF code. 
 
- For example, interface routines converting between HML matrix format
- used in main SCF code and elementwise format used by integral code.
+    @brief Various utilities used by self-consistent field (SCF)
+    code. For example, interface routines converting between HML
+    matrix format used in main SCF code and elementwise format used by
+    integral code.
 
-    @author: Elias Rudberg <em>responsible</em>.
+    @author: Elias Rudberg <em>responsible</em>
 */
 
 #include <pthread.h>
@@ -275,8 +277,8 @@ public:
     invCholFactor(invCholFactor_),
     doInvCholFactorTransformation(doInvCholFactorTransformation_),
     matrix_size_block_info(matrix_size_block_info_),
-    permutationHML(permutationHML_),
-    inversePermutationHML(inversePermutationHML_)
+    permutationHML(permutationHML_)
+    //inversePermutationHML(inversePermutationHML_
   {
   }
   void ComputeMatrix(ergo_real param,
@@ -292,7 +294,7 @@ private:
   bool doInvCholFactorTransformation;
   mat::SizesAndBlocks const & matrix_size_block_info;
   std::vector<int> const & permutationHML;
-  std::vector<int> const & inversePermutationHML;
+  //std::vector<int> const & inversePermutationHML;
 };
 
 void Vxc_worker::ComputeMatrix(ergo_real param,
@@ -403,59 +405,6 @@ add_square_matrices(int n, const ergo_real* A, const ergo_real* B, ergo_real* C)
 }
 
 
-
-
-
-#if 0
-static int
-assign_from_full_via_vectors_symm(int n, 
-				  ergo_real* full,
-				  symmMatrix & result,
-				  std::vector<int> const & permutationHML)
-{
-  // Count nonzero elements
-  const ergo_real threshold = 1e-14;
-  int nvalues = 0;
-  for(int i = 0; i < n; i++)
-    for(int j = i; j < n; j++)
-      {
-	if(std::fabs(full[i*n+j]) > threshold)
-	  nvalues++;
-      }
-  
-  // allocate vectors
-  std::vector<int> rowind(nvalues);
-  std::vector<int> colind(nvalues);
-  std::vector<ergo_real> values(nvalues);
-
-  // populate vectors
-  int count = 0;
-  for(int i = 0; i < n; i++)
-    for(int j = i; j < n; j++)
-      {
-	if(std::fabs(full[i*n+j]) > threshold)
-	  {
-	    if(count >= nvalues)
-	      return -1;
-	    rowind[count] = i;
-	    colind[count] = j;
-	    values[count] = full[i*n+j];
-	    count++;
-	  }
-      }
-
-  result.assign_from_sparse(rowind,
-			    colind,
-			    values,
-			    permutationHML,
-			    permutationHML);
-  
-  return 0;
-}
-#endif
-
-
-
 int
 compute_h_core_matrix_simple_dense(const IntegralInfo& integralInfo,
 				   const Molecule& molecule,
@@ -464,9 +413,11 @@ compute_h_core_matrix_simple_dense(const IntegralInfo& integralInfo,
 				   ergo_real threshold_integrals_1el,
 				   int noOfThreadsForV,
 				   mat::SizesAndBlocks const & matrix_size_block_info,
-				   std::vector<int> const & permutationHML)
+				   std::vector<int> const & permutationHML,
+				   ergo_real & result_nuclearRepulsionEnergy)
 {
   int n = basisInfo.noOfBasisFuncs;
+  result_nuclearRepulsionEnergy = molecule.getNuclearRepulsionEnergyQuadratic();
   std::vector<ergo_real> A(n*n);
   if(compute_h_core_matrix_full(integralInfo,
 				basisInfo,
@@ -513,6 +464,8 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 			     symmMatrix & H_core_Matrix_sparse,
 			     ergo_real threshold_integrals_1el,
 			     int noOfThreadsForV,
+			     ergo_real boxSizeForVT,
+			     ergo_real & result_nuclearRepulsionEnergy,
 			     mat::SizesAndBlocks const & matrix_size_block_info,
 			     std::vector<int> const & permutationHML,
 			     int const create_dipole_mtx,
@@ -533,30 +486,31 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
   symmMatrix T(H_core_Matrix_sparse);
   T.clear();
 
-  if(compute_T_sparse(basisInfo,
-		      integralInfo,
-		      threshold_integrals_1el,
-		      T,
-		      permutationHML) != 0) {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_T_sparse");
-      return -1;
-    }
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "compute_T_sparse returned OK.");
-  timeMeterT.print(LOG_AREA_SCF, "compute_T_sparse");
+  if(compute_T_sparse_linear(basisInfo,
+			     integralInfo,
+			     threshold_integrals_1el,
+			     boxSizeForVT,
+			     T,
+			     permutationHML) != 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_T_sparse");
+    return -1;
+  }
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "compute_T_sparse_linear returned OK.");
+  timeMeterT.print(LOG_AREA_SCF, "compute_T_sparse_linear");
 
   // Get V
   Util::TimeMeter timeMeterV;
   symmMatrix V;
   V.resetSizesAndBlocks(matrix_size_block_info,
 			matrix_size_block_info);
-  ergo_real boxSize = 10.0;
   if(compute_V_sparse(basisInfo,
 		      integralInfo, 
 		      molecule,
 		      threshold_integrals_1el,
-		      boxSize,
+		      boxSizeForVT,
 		      V,
-		      permutationHML) != 0)
+		      permutationHML,
+		      result_nuclearRepulsionEnergy) != 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_V_sparse");
       return -1;
@@ -698,7 +652,7 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
     }
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Using 'extra charges', sum = %8.4f, min = %8.4f, max = %8.4f", 
 	      (double)sumCharge, (double)minCharge, (double)maxCharge);
-    ergo_real minDist = std::numeric_limits<ergo_real>::max();
+    ergo_real minDist = template_blas_get_num_limit_max<ergo_real>();
     ergo_real minDistCharge1 = 0;
     ergo_real minDistCharge2 = 0;
     for(int i = 0; i < extraCharges.getNoOfAtoms(); i++)
@@ -706,7 +660,7 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 	ergo_real dx = extraCharges.getAtom(i).coords[0] - molecule.getAtom(j).coords[0];
 	ergo_real dy = extraCharges.getAtom(i).coords[1] - molecule.getAtom(j).coords[1];
 	ergo_real dz = extraCharges.getAtom(i).coords[2] - molecule.getAtom(j).coords[2];
-	ergo_real dist = std::sqrt(dx*dx+dy*dy+dz*dz);
+	ergo_real dist = template_blas_sqrt(dx*dx+dy*dy+dz*dz);
 	if(dist < minDist) {
 	  minDist = dist;
 	  minDistCharge1 = molecule.getAtom(j).charge;
@@ -720,14 +674,15 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
     symmMatrix V_extra;
     V_extra.resetSizesAndBlocks(matrix_size_block_info,
 				matrix_size_block_info);
-    ergo_real boxSize = 10.0;
+    ergo_real nuclearRepulsionEnergy_dummy = 0;
     if(compute_V_sparse(basisInfo,
 			integralInfo, 
 			extraCharges,
 			threshold_integrals_1el,
-			boxSize,
+			boxSizeForVT,
 			V_extra,
-			permutationHML) != 0) {
+			permutationHML,
+			nuclearRepulsionEnergy_dummy) != 0) {
       do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_V_sparse");
       return -1;
     }    
@@ -1201,8 +1156,7 @@ get_2e_matrices_and_energy_simple_HF_sparse_unrestricted(const BasisInfoStruct&
 
 /** 
 Routine for computing the two-electron part of the Fock/KS matrix, in
-sparse form.  This routine is only used when FMM is enabled.  This
-routine does not do density fitting, obviously.
+sparse form. This routine is only used when FMM is enabled.
 */
 static int
 get_2e_matrix_and_energy_simple_sparse(const BasisInfoStruct& basisInfo,
@@ -1534,12 +1488,10 @@ get_2e_matrices_and_energy_simple_sparse_unrestricted(const BasisInfoStruct& bas
 /** 
 General routine for computing the two-electron part of the Fock/KS
 matrix.  Both input and output matrices are in sparse form, but full
-matrix form may be used in intermediate steps. If FMM is not used, or
-if CAM is used, or if density fitting is used, full matrix format is
-applied.
+matrix form may be used in intermediate steps. If FMM is not used,
+full matrix format is applied.
 
 @param basisInfo the used basis set.
-@param basisInfoDensFit the auxiliary basis set (may be NULL).
 @param molecule position of atoms (used for eg. XC grid).
 @param integralInfo - the integral evaluation object.
 @param twoelMatrix_sparse - the evaluation result.
@@ -1559,11 +1511,15 @@ added. 1 means that the traditional full matrix code should be called,
 
 @param CAM_params a structure containing parameters needed when functionals like CAMB3LYP are used.
 @param gridParams a structure containing parameters for the grid.
-@param df_data parameters related to density fitting.
 @param matrix_size_block_info block sizes etc for hierarchic matrix library.
 
 @param get_J_K_Fxc_matrices flag saying if matrices should be saved for statistics/testing purposes.
-If that feature is active, matrices are saved in parameters @param J_matrix @param K_matrix @param Fxc_matrix .
+If that feature is active, matrices are saved in parameters J_matrix K_matrix Fxc_matrix .
+
+@param J_matrix resulting J matrix, if requested.
+@param K_matrix resulting K matrix, if requested.
+@param Fxc_matrix resulting Fxc matrix, if requested.
+
 @param stats a structure holding SCF statistics.
 
 @param permutationHML - the permutation of basis functions, needed
@@ -1573,7 +1529,6 @@ for transformation between the dense and sparse formats.
 */
 int
 get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
-                                const BasisInfoStruct & basisInfoDensFit,
 				const Molecule& molecule,
 				const IntegralInfo& integralInfo, 
 				symmMatrix & twoelMatrix_sparse, 
@@ -1584,7 +1539,6 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
 				int do_xc,
 				ergo_real* energy_2el,
 				int noOfElectrons,
-				DensfitData* df_data,
 				mat::SizesAndBlocks const & matrix_size_block_info,
 				std::vector<int> const & permutationHML,
 				std::vector<int> const & inversePermutationHML,
@@ -1594,7 +1548,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
 				symmMatrix & Fxc_matrix,
 				SCF_statistics & stats)
 {
-  if(J_K_params.use_naive_fockmatrix_construction == 1)
+  if(J_K_params.use_naive_fockmat_constr == 1)
     {
       int n = basisInfo.noOfBasisFuncs;
       do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_2e_matrix_and_energy_sparse naive impl allocating full n*n matrices.");
@@ -1628,7 +1582,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
       return 0;
     }
   
-  if(J_K_params.use_densfit_for_J == 0 && do_xc == 0 && J_K_params.use_fmm == 1)
+  if(do_xc == 0 && J_K_params.use_fmm == 1)
     {
       // Simple Hartree-Fock case, with FMM. 
       // In this case we use a special function to get away with less memory usage.
@@ -1653,8 +1607,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
       return 0;
     }
 
-  if(J_K_params.use_densfit_for_J == 0 &&
-     (J_K_params.use_fmm == 1 || CAM_params.computeRangeSeparatedExchange) )
+  if(J_K_params.use_fmm == 1 || CAM_params.computeRangeSeparatedExchange)
     {
       // Simple case, with FMM. 
       if(get_2e_matrix_and_energy_simple_sparse(basisInfo,
@@ -1694,71 +1647,35 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
 				    inversePermutationHML);
     std::copy (fullTmp.begin(), fullTmp.end(), densityMatrix_full);
   }
-  if( (CAM_params.alpha != ergo_real(0.0) || CAM_params.computeRangeSeparatedExchange) 
-      && J_K_params.use_densfit_for_J == 0)
-    {
-      // get both J and K at the same time
-      if(J_K_params.use_differential_density == 1)
-	{
-	  if(compute_2e_matrix_list_difden(basisInfo,
-					   integralInfo,
-					   CAM_params,
-					   &twoelMatrix_full,
-					   &densityMatrix_full,
-					   1,
-					   J_K_params) != 0)
-	    {
-	      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_2e_matrix_list");
-	      return -1;
-	    }
-	}
-      else
-	{
-	  if(compute_2e_matrix_list(basisInfo,
-				    integralInfo,
-				    CAM_params,
-				    &twoelMatrix_full,
-				    &densityMatrix_full,
-				    1,
-				    J_K_params) != 0)
-	    {
-	      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_2e_matrix_list");
-	      return -1;
-	    }
-	}
-    }
-  else
-    {
-      // Get J
-      if(compute_2e_matrix_coulomb(basisInfo, basisInfoDensFit,
-                                   integralInfo, 
-				   twoelMatrix_full, 
-				   densityMatrix_full, 
-				   J_K_params,
-				   df_data) != 0)
-	{
-	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_2e_matrix_coulomb");
-	  return -1;
-	}
-
-      if(CAM_params.alpha != ergo_real(0.0) || CAM_params.computeRangeSeparatedExchange)
-	{
-	  ergo_real* K   = new ergo_real[n*n];
-	  // Get K
-	  if(compute_2e_matrix_exchange(basisInfo, 
-					integralInfo, 
-					CAM_params,
-					K, 
-					densityMatrix_full,
-					J_K_params.threshold_K) != 0)
-	    {
-	      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_2e_matrix_exchange");
-	      return -1;
-	    }
-	  add_square_matrices(n, twoelMatrix_full, K, twoelMatrix_full);
-	  delete []K;
-	}
-    } // END ELSE
+  {
+    // Get J
+    if(compute_2e_matrix_coulomb(basisInfo,
+				 integralInfo, 
+				 twoelMatrix_full, 
+				 densityMatrix_full, 
+				 J_K_params) != 0)
+      {
+	do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_2e_matrix_coulomb");
+	return -1;
+      }
+    if(CAM_params.alpha != ergo_real(0.0) || CAM_params.computeRangeSeparatedExchange)
+      {
+	ergo_real* K   = new ergo_real[n*n];
+	// Get K
+	if(compute_2e_matrix_exchange(basisInfo, 
+				      integralInfo, 
+				      CAM_params,
+				      K, 
+				      densityMatrix_full,
+				      J_K_params.threshold_K) != 0)
+	  {
+	    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_2e_matrix_exchange");
+	    return -1;
+	  }
+	add_square_matrices(n, twoelMatrix_full, K, twoelMatrix_full);
+	delete []K;
+      }
+  }
   
   *energy_2el = get_trace(n, densityMatrix_full, twoelMatrix_full)*0.5;
       
@@ -1802,7 +1719,6 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
 
 int
 get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo, 
-					       const BasisInfoStruct & basisInfoDensFit,
 					       const Molecule& molecule,
 					       const IntegralInfo& integralInfo, 
 					       const JK::ExchWeights & CAM_params,
@@ -1815,18 +1731,17 @@ get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo
 					       int do_xc,
 					       ergo_real* energy_2el,
 					       int noOfElectrons,
-					       DensfitData* df_data,
 					       mat::SizesAndBlocks const & matrix_size_block_info,
 					       std::vector<int> const & permutationHML,
 					       std::vector<int> const & inversePermutationHML)
 {
-  if(J_K_params.use_naive_fockmatrix_construction == 1)
+  if(J_K_params.use_naive_fockmat_constr == 1)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error: use_naive_fockmatrix_construction not implemented for unrestricted case.");
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error: use_naive_fockmat_constr not implemented for unrestricted case.");
       return -1;
     }
 
-  if(J_K_params.use_densfit_for_J == 0 && do_xc == 0 && J_K_params.use_fmm == 1)
+  if(do_xc == 0 && J_K_params.use_fmm == 1)
     {
       // Simple Hartree-Fock case, with FMM. 
       // In this case we use a special function to get away with less memory usage.
@@ -1849,8 +1764,7 @@ get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo
 	}
       return 0;
     }
-  if(J_K_params.use_densfit_for_J == 0 &&
-     (J_K_params.use_fmm == 1 || CAM_params.computeRangeSeparatedExchange))
+  if(J_K_params.use_fmm == 1 || CAM_params.computeRangeSeparatedExchange)
     {
       // Simple case, with FMM. 
       if(get_2e_matrices_and_energy_simple_sparse_unrestricted(basisInfo,
@@ -1894,78 +1808,70 @@ get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo
     std::copy (fullTmp.begin(), fullTmp.end(), densityMatrix_full_beta.begin());
   }
 
-  if(J_K_params.use_densfit_for_J == 0)
+  // get both J and K at the same time
+  if(J_K_params.use_differential_density == 1)
     {
-      // get both J and K at the same time
-      if(J_K_params.use_differential_density == 1)
-	{
-          do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error: use_differential_density not impl for unrestricted case.");
-          return -1;
-	}
-      else
-	{
-	  if(CAM_params.computeRangeSeparatedExchange)
-	    {
-	      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, 
-			"error in get_2e_matrices_and_energy_simple_sparse_unrestricted: "
-			"cannot use compute_JK_single_box when CAM params are used.");
-	      return -1;
-	    }
-
-	  // Get both J and K at once
-	  ergo_real* J_list[2];
-          ergo_real* K_list[2];
-	  J_list[0] = new ergo_real[n*n];
-	  J_list[1] = new ergo_real[n*n];
-          K_list[0] = new ergo_real[n*n];
-          K_list[1] = new ergo_real[n*n];
-	  ergo_real* D_list[2];
-	  D_list[0] = &densityMatrix_full_alpha[0];
-	  D_list[1] = &densityMatrix_full_beta[0];
-
-	  // Use smallest of threshold_J and threshold_K.
-	  ergo_real threshold_JK = J_K_params.threshold_J;
-	  if(J_K_params.threshold_K < threshold_JK)
-	    threshold_JK = J_K_params.threshold_K;
-
-	  if(compute_JK_single_box(basisInfo,
-				   integralInfo,
-				   J_list[0],
-				   K_list[0],
-				   D_list[0],
-				   threshold_JK) != 0)
-	    {
-	      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_JK_single_box");
-	      return -1;
-	    }
-	  if(compute_JK_single_box(basisInfo,
-				   integralInfo,
-				   J_list[1],
-				   K_list[1],
-				   D_list[1],
-				   threshold_JK) != 0)
-	    {
-	      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_JK_single_box");
-	      return -1;
-	    }
-
-          ergo_real hf_weight = CAM_params.alpha;
-	  for(int i = 0; i < n*n; i++)
-	    {
-	      twoelMatrix_full_alpha[i] = J_list[0][i] + J_list[1][i] + 2 * K_list[0][i] * hf_weight;
-	      twoelMatrix_full_beta [i] = J_list[0][i] + J_list[1][i] + 2 * K_list[1][i] * hf_weight;
-	    }
-	  delete [] J_list[0];
-	  delete [] J_list[1];
-          delete [] K_list[0];
-          delete [] K_list[1];
-	}
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error: use_differential_density not impl for unrestricted case.");
+      return -1;
     }
   else
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error: case not implemented.");
-      return -1;
-    } // END ELSE
+      if(CAM_params.computeRangeSeparatedExchange)
+	{
+	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, 
+		    "error in get_2e_matrices_and_energy_simple_sparse_unrestricted: "
+		    "cannot use compute_JK_single_box when CAM params are used.");
+	  return -1;
+	}
+
+      // Get both J and K at once
+      ergo_real* J_list[2];
+      ergo_real* K_list[2];
+      J_list[0] = new ergo_real[n*n];
+      J_list[1] = new ergo_real[n*n];
+      K_list[0] = new ergo_real[n*n];
+      K_list[1] = new ergo_real[n*n];
+      ergo_real* D_list[2];
+      D_list[0] = &densityMatrix_full_alpha[0];
+      D_list[1] = &densityMatrix_full_beta[0];
+
+      // Use smallest of threshold_J and threshold_K.
+      ergo_real threshold_JK = J_K_params.threshold_J;
+      if(J_K_params.threshold_K < threshold_JK)
+	threshold_JK = J_K_params.threshold_K;
+
+      if(compute_JK_single_box(basisInfo,
+			       integralInfo,
+			       J_list[0],
+			       K_list[0],
+			       D_list[0],
+			       threshold_JK) != 0)
+	{
+	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_JK_single_box");
+	  return -1;
+	}
+      if(compute_JK_single_box(basisInfo,
+			       integralInfo,
+			       J_list[1],
+			       K_list[1],
+			       D_list[1],
+			       threshold_JK) != 0)
+	{
+	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_JK_single_box");
+	  return -1;
+	}
+
+      ergo_real hf_weight = CAM_params.alpha;
+      for(int i = 0; i < n*n; i++)
+	{
+	  twoelMatrix_full_alpha[i] = J_list[0][i] + J_list[1][i] + 2 * K_list[0][i] * hf_weight;
+	  twoelMatrix_full_beta [i] = J_list[0][i] + J_list[1][i] + 2 * K_list[1][i] * hf_weight;
+	}
+      delete [] J_list[0];
+      delete [] J_list[1];
+      delete [] K_list[0];
+      delete [] K_list[1];
+    }
 
 
   *energy_2el = 0;
@@ -2021,7 +1927,6 @@ get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo
 */
 int
 get_2e_matrices_and_energy_restricted_open(const BasisInfoStruct & basisInfo, 
-					   const BasisInfoStruct & basisInfoDensFit,
 					   const Molecule& molecule,
 					   const IntegralInfo& integralInfo, 
 					   const JK::ExchWeights & CAM_params,
@@ -2034,22 +1939,14 @@ get_2e_matrices_and_energy_restricted_open(const BasisInfoStruct & basisInfo,
 					   int do_xc,
 					   ergo_real* energy_2el,
 					   int noOfElectrons,
-					   DensfitData* df_data,
 					   mat::SizesAndBlocks const & matrix_size_block_info,
 					   std::vector<int> const & permutationHML,
 					   std::vector<int> const & inversePermutationHML)
 {
-  if(J_K_params.use_naive_fockmatrix_construction)
+  if(J_K_params.use_naive_fockmat_constr)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_SCF,
-		"error: use_naive_fockmatrix_construction not implemented for unrestricted case.");
-      return -1;
-    }
-
-  if(J_K_params.use_densfit_for_J)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF,
-		"error: density fitting not implemented for restricted-open shell case.");
+		"error: use_naive_fockmat_constr not implemented for unrestricted case.");
       return -1;
     }
 
@@ -2352,7 +2249,7 @@ get_mulliken_charges(const symmMatrix & densityMatrix,
       ergo_real dx = molecule.getAtom(k).coords[0] - basisInfo.basisFuncList[i].centerCoords[0];
       ergo_real dy = molecule.getAtom(k).coords[1] - basisInfo.basisFuncList[i].centerCoords[1];
       ergo_real dz = molecule.getAtom(k).coords[2] - basisInfo.basisFuncList[i].centerCoords[2];
-      ergo_real distance = std::sqrt(dx*dx + dy*dy + dz*dz);
+      ergo_real distance = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
       if(distance < 1e-4)
 	foundAtomIndex = k;
     }
@@ -2464,112 +2361,51 @@ static int write_gcube_file_header(FILE* f,
 
 
 
-
 void get_exp_value_pos_operator(const BasisInfoStruct & basisInfo,
 				const Molecule& molecule,
-				const ergo_real* densityMatrixFull, 
+				const symmMatrix & densityMatrix,
+				mat::SizesAndBlocks const & matrix_size_block_info,
+				std::vector<int> const & permutationHML,
 				std::vector<ergo_real> &mean,
 				std::vector<ergo_real> &std)
+
 {
-  
-  int nPrims1 = 0;
-  int nPrims2 = 0;
-  
-  DistributionSpecStruct* rho1 = NULL;
-  ergo_real cutoff = 1e-7;
-
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_exp_value_pos_operator, calling get_density() with cutoff = %8.4g.", cutoff);
-
-  nPrims1 = get_no_of_primitives_for_density(cutoff, densityMatrixFull, basisInfo);
-  rho1 = new DistributionSpecStruct[nPrims1];
-
-  Util::TimeMeter timeMeterGetDens;
-  int n1 = get_density(basisInfo,
-		       densityMatrixFull,
-		       cutoff,
-		       nPrims1,
-		       rho1);    
-  if(n1 < 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "aborting get_exp_value_pos_operator(): (n1 < 0)");
-      throw "aborting get_exp_value_pos_operator(): (n1 < 0)";
-    }
-
-  nPrims1 = n1;
-  timeMeterGetDens.print(LOG_AREA_SCF, "get_density() calls");
-
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_exp_value_pos_operator, after get_density() calls: nPrims1 = %d, nPrims2 = %d.", nPrims1, nPrims2);
-
-  // get bounds for the integral
-  ergo_real minlist[3];
-  ergo_real maxlist[3];
-  int coordno;
-  for(coordno = 0; coordno < 3; coordno++)
-    {
-      minlist[coordno] = std::numeric_limits<ergo_real>::max();
-      maxlist[coordno] = std::numeric_limits<ergo_real>::min(); 
-    }
-  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
-    {
-      for(coordno = 0; coordno < 3; coordno++)
-	{
-	  ergo_real curr = molecule.getAtom(i).coords[coordno];
-	  if(curr < minlist[coordno])
-	    minlist[coordno] = curr;
-	  if(curr > maxlist[coordno])
-	    maxlist[coordno] = curr;
-	}
-    }
-
-
-  // OK, now we have min max coords for molecule.
-  // Add some margin.
-  ergo_real margin, length;
-  for(coordno = 0; coordno < 3; coordno++)
-    {
-      length = maxlist[coordno] - minlist[coordno];
-      margin = (length+1)*10; //  sizes of molecule
-      minlist[coordno] -= margin;
-      maxlist[coordno] += margin;
-    }
-
-  Util::TimeMeter timeMeterGetInt;
-
+  ergo_real expval; 
+  int xi = 0, yi = 0, zi = 0;
   mean.clear(); mean.resize(3);
   std.clear(); std.resize(3);
-  std::vector<int> monomialIntsAdd;
-  ergo_real tmp;
 
+  symmMatrix opMatrix;
+  opMatrix.resetSizesAndBlocks(matrix_size_block_info, matrix_size_block_info);
 
-  for(coordno = 0; coordno < 3; coordno++)
+  for(int coord = 0; coord < 3; ++coord)
     {
-      monomialIntsAdd.clear(); monomialIntsAdd.resize(3);
-      monomialIntsAdd[coordno] = 1;
-      mean[coordno] = integrate_density_in_box_2(nPrims1,
-					rho1,
-					minlist, 
-					maxlist,
-					monomialIntsAdd);
+      switch(coord) 
+	{
+	case 0: xi = 1; break;
+	case 1: yi = 1; break;
+	case 2: zi = 1; break;
+	default: throw "Error in get_exp_value_pos_operator: wrong value of coord";
+	}
+      // compute expectated value of position operator
+      if(compute_operator_matrix_sparse_symm(basisInfo, xi, yi, zi, opMatrix, permutationHML) != 0)
+	throw "Error in compute_operator_matrix_sparse_symm";
+      expval = symmMatrix::trace_ab(densityMatrix, opMatrix); // E[X]
+      mean[coord] = expval;
 
-      monomialIntsAdd[coordno] = 2;
-      // get mean value of x^2
-      tmp = integrate_density_in_box_2(nPrims1,
-					rho1,
-					minlist, 
-					maxlist,
-					monomialIntsAdd);
-      std[coordno] = std::sqrt(tmp - mean[coordno]*mean[coordno]);
+      // compute standard deviation
+      if(compute_operator_matrix_sparse_symm(basisInfo, 2*xi, 2*yi, 2*zi, opMatrix, permutationHML) != 0)
+	throw "Error in compute_operator_matrix_sparse_symm";
+      expval = symmMatrix::trace_ab(densityMatrix, opMatrix); // E[X^2]
+      std[coord] = template_blas_sqrt(expval - mean[coord]*mean[coord]); // sqrt[ E[X^2] - (E[X])^2 ]
+
+      xi = 0; yi = 0; zi = 0;
     }
-
-
-  timeMeterGetInt.print(LOG_AREA_SCF, "integrate_density_in_box_2() calls");
-
 }
 
 
 
 
-
 void 
 do_density_images(const BasisInfoStruct & basisInfo,
 		  const Molecule& molecule,
@@ -2621,8 +2457,8 @@ do_density_images(const BasisInfoStruct & basisInfo,
   int coordno;
   for(coordno = 0; coordno < 3; coordno++)
     {
-      minlist[coordno] = std::numeric_limits<ergo_real>::max();
-      maxlist[coordno] = std::numeric_limits<ergo_real>::min(); 
+      minlist[coordno] = template_blas_get_num_limit_max<ergo_real>();
+      maxlist[coordno] = template_blas_get_num_limit_min<ergo_real>();
     }
   for(int i = 0; i < molecule.getNoOfAtoms(); i++)
     {
