@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,26 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file scf_utils.h
+
+    @brief Various utilities used by self-consistent field (SCF)
+    code. For example, interface routines converting between HML
+    matrix format used in main SCF code and elementwise format used by
+    integral code.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef SCF_UTILS_HEADER
 #define SCF_UTILS_HEADER
 
@@ -33,7 +44,6 @@
 #include "basisinfo.h"
 #include "integrals_2el.h"
 #include "matrix_typedefs.h"
-#include "densityfitting.h"
 #include "grid_stream.h"
 #include "SCF_statistics.h"
 
@@ -53,6 +63,8 @@ compute_h_core_matrix_sparse(const IntegralInfo& integralInfo,
 			     symmMatrix & H_core_Matrix_sparse,
 			     ergo_real threshold_integrals_1el,
 			     int noOfThreadsForV,
+			     ergo_real boxSizeForVT,
+			     ergo_real & result_nuclearRepulsionEnergy,
 			     mat::SizesAndBlocks const & matrix_size_block_info,
 			     std::vector<int> const & permutationHML,
 			     int const create_dipole_mtx = 0,
@@ -68,7 +80,8 @@ compute_h_core_matrix_simple_dense(const IntegralInfo& integralInfo,
 				   ergo_real threshold_integrals_1el,
 				   int noOfThreadsForV,
 				   mat::SizesAndBlocks const & matrix_size_block_info,
-				   std::vector<int> const & permutationHML);
+				   std::vector<int> const & permutationHML,
+				   ergo_real & result_nuclearRepulsionEnergy);
 
 int
 get_gradient_for_given_mol_and_dens(const IntegralInfo& integralInfo,
@@ -124,7 +137,6 @@ write_2el_integral_m_file(const BasisInfoStruct & basisInfo, const IntegralInfo
 
 int
 get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
-                                const BasisInfoStruct & basisInfoDensFit,
 				const Molecule& molecule,
 				const IntegralInfo& integralInfo, 
 				symmMatrix & twoelMatrix_sparse, 
@@ -135,9 +147,7 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
 				int do_xc,
 				ergo_real* energy_2el,
 				int noOfElectrons,
-				DensfitData* df_data,
 				mat::SizesAndBlocks const & matrix_size_block_info,
-
 				std::vector<int> const & permutationHML,
 				std::vector<int> const & inversePermutationHML,
 				int get_J_K_Fxc_matrices,
@@ -148,7 +158,6 @@ get_2e_matrix_and_energy_sparse(const BasisInfoStruct & basisInfo,
 
 int
 get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo, 
-					       const BasisInfoStruct & basisInfoDensFit,
 					       const Molecule& molecule,
 					       const IntegralInfo& integralInfo, 
 					       const JK::ExchWeights & CAM_params,
@@ -161,14 +170,12 @@ get_2e_matrices_and_energy_sparse_unrestricted(const BasisInfoStruct & basisInfo
 					       int do_xc,
 					       ergo_real* energy_2el,
 					       int noOfElectrons,
-					       DensfitData* df_data,
 					       mat::SizesAndBlocks const & matrix_size_block_info,
 					       std::vector<int> const & permutationHML,
 					       std::vector<int> const & inversePermutationHML);
 
 int
 get_2e_matrices_and_energy_restricted_open(const BasisInfoStruct & basisInfo, 
-					   const BasisInfoStruct & basisInfoDensFit,
 					   const Molecule& molecule,
 					   const IntegralInfo& integralInfo, 
 					   const JK::ExchWeights & CAM_params,
@@ -181,7 +188,6 @@ get_2e_matrices_and_energy_restricted_open(const BasisInfoStruct & basisInfo,
 					   int do_xc,
 					   ergo_real* energy_2el,
 					   int noOfElectrons,
-					   DensfitData* df_data,
 					   mat::SizesAndBlocks const & matrix_size_block_info,
 					   std::vector<int> const & permutationHML,
 					   std::vector<int> const & inversePermutationHML);
@@ -230,10 +236,11 @@ do_mulliken_spin_densities(const symmMatrix & spinDensityMatrix,
 			   std::vector<int> const & inversePermutationHML,
 			   const Molecule& molecule);
 
-
 void get_exp_value_pos_operator(const BasisInfoStruct & basisInfo,
 				const Molecule& molecule,
-				const ergo_real* densityMatrixFull, 
+				const symmMatrix & densityMatrix,
+				mat::SizesAndBlocks const & matrix_size_block_info,
+				std::vector<int> const & permutationHML,
 				std::vector<ergo_real> &mean,
 				std::vector<ergo_real> &std);
 
