@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -29,6 +30,8 @@
 /** @file slr.cc contains a Simple Linear Response implementation
     based on the MO orbital picture. Copyright(c) Pawel Salek 2006.
 
+    @author: Pawel Salek <em>responsible</em>
+
     The solver is generic: it can handle both linear sets of equations
     and eigenvalue problems. The variable parts are implemented by
     subclasses. A subclass must implement following functions:
@@ -466,9 +469,9 @@ void VarVector::symorth(void)
   ergo_real ovl= 2*dot(nvar, v, v+nvar )/nrm;
   ergo_real x1 = 1+ovl;
   ergo_real x2 = 1-ovl;
-  if(fabs(x1)>1e-10 && fabs(x2)>1e-10) {
-    x1 = 0.5 / sqrt(x1*nrm);
-    x2 = 0.5 / sqrt(x2*nrm);
+  if(template_blas_fabs(x1)>1e-10 && template_blas_fabs(x2)>1e-10) {
+    x1 = 0.5 / template_blas_sqrt(x1*nrm);
+    x2 = 0.5 / template_blas_sqrt(x2*nrm);
     ergo_real c1 = (x1 + x2);
     ergo_real c2 = (x1 - x2);
     for(int i=0; i<nvar; i++) {
@@ -479,7 +482,7 @@ void VarVector::symorth(void)
     }
   } else {
     do_output(LOG_CAT_INFO, LOG_AREA_LR, "Removing Y part, normfactor = %g", (double)nrm);
-    ergo_real f = sqrt(2.0/nrm);
+    ergo_real f = template_blas_sqrt(2.0/nrm);
     for(int i=0; i<nvar; i++) {
       v[i]      = v[i]*f;
       v[i+nvar] = 0;
@@ -498,9 +501,9 @@ precondition(VarVector&v, const VarVector& e2diag, ergo_real shift)
   /* Factors 2 here is shaky - compare against exact E2 diagonal. */
   for(int i=0; i<v.nvar; i++) {
     denom = e2diag.x()[i] - shift;
-    if(fabs(denom)>0) v.x()[i] /= denom;
+    if(template_blas_fabs(denom)>0) v.x()[i] /= denom;
     denom = e2diag.y()[i] + shift;
-    if(fabs(denom)>0) v.y()[i] /= denom;
+    if(template_blas_fabs(denom)>0) v.y()[i] /= denom;
   }
   v.onDisk = 0;
 }
@@ -649,9 +652,9 @@ LRSolver::addToSpace(VarVectorCollection& v, E2Evaluator& e2)
     /* orthogonalize */
     for(i=0; i<subspaceSize; i+=2) {
       ergo_real prod = v[ivec] * vects[i];
-      if(fabs(prod)>1e-10) v[ivec] += -prod*vects[i];
+      if(template_blas_fabs(prod)>1e-10) v[ivec] += -prod*vects[i];
       prod = v[ivec] * swapXY(vects[i]);
-      if(fabs(prod)>1e-10) v[ivec] += -prod*swapXY(vects[i]);
+      if(template_blas_fabs(prod)>1e-10) v[ivec] += -prod*swapXY(vects[i]);
     }
     v[ivec].symorth();
     v.releaseAll();
@@ -769,7 +772,7 @@ LRSolver::solve(E2Evaluator& e2, bool diskMode)
     /* residue estimation phase */
     if(!getResidual(v))
       break;
-    ergo_real nr = sqrt(v[0] * v[0]);
+    ergo_real nr = template_blas_sqrt(v[0] * v[0]);
 
     printf("Step %2d: ssize: %3d |r|=%13.6g shift=%10.6f\n",
            step+1, subspaceSize, (double)nr, (double)getPreconditionerShift(0));
@@ -833,7 +836,7 @@ LRSolver::setE2diag(int nbast, int nocc,
     ret = fdiag[nocc]-fdiag[nocc-1];
     
     do_output(LOG_CAT_INFO, LOG_AREA_LR, "HOMO-LUMO gap: %20.15f eV",
-              ret*27.2114);
+              (double)ret / UNIT_one_eV);
 
     return ret;
   } else {
@@ -981,11 +984,11 @@ SetOfEqSolver::getResidual(VarVectorCollection& residualv)
   getAvMinusFreqSv(frequency, xSub, residualv[0]);
   for(int i=0; i<2*rhs.nvar; i++)
     residualv[0][i] -= rhs[i];
-  ergo_real n = sqrt(residualv[0]*residualv[0]);
+  ergo_real n = template_blas_sqrt(residualv[0]*residualv[0]);
   xTimesRHS = multiplyXtimesVec(rhs);
   do_output(LOG_CAT_INFO, LOG_AREA_LR, 
 	    "SetOfEqSolver::getResidual: norm of the residual: %g X*RHS: %g",
-            n, xTimesRHS);
+            (double)n, (double)xTimesRHS);
   return n >= convThreshold;
 }
 
@@ -1117,7 +1120,7 @@ EigenSolver::getResidual(VarVectorCollection& residualv)
         tmp[j] = s;
       }
       ergo_real n = 
-        1.0/sqrt(dot(subspaceSize, tmp, last_ev + i*subspaceSize));
+        1.0/template_blas_sqrt(dot(subspaceSize, tmp, last_ev + i*subspaceSize));
       for(int j=0; j<subspaceSize; j++)
         last_ev[j + i*subspaceSize] *=n;
     }
@@ -1134,17 +1137,17 @@ EigenSolver::getResidual(VarVectorCollection& residualv)
       }
       getAvMinusFreqSv(ritzVals[i], last_ev + i*subspaceSize,
                        residualv[nadded]);
-      //ergo_real n = sqrt(residualv[0]*residualv[0]);
-      ergo_real n = sqrt(residualv[nadded]*residualv[nadded]);
+      //ergo_real n = template_blas_sqrt(residualv[0]*residualv[0]);
+      ergo_real n = template_blas_sqrt(residualv[nadded]*residualv[nadded]);
       if(n<convThreshold) {
         nConverged++;
         do_output(LOG_CAT_INFO, LOG_AREA_LR, "Converged %d root %g with n=%g",
-		  nConverged, ritzVals[i], n);
+		  nConverged, (double)ritzVals[i], (double)n);
       } else {
         do_output(LOG_CAT_INFO, LOG_AREA_LR,
                   "Trial vector for %2i th root needed orig %3i, "
                   "n=%g ritz=%f",
-		  i+1, idx[i], n, ritzVals[i]);
+		  i+1, idx[i], (double)n, (double)ritzVals[i]);
         nadded++;
       }
     }
