@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -37,6 +38,7 @@
 #include <vector>
 
 #include "integrals_2el_explicit.h"
+#include "matInclude.h"
 
 int try_integral_diffs(const IntegralInfo & integralInfo, ergo_real h) {
 
@@ -69,7 +71,7 @@ int try_integral_diffs(const IntegralInfo & integralInfo, ergo_real h) {
 
   ergo_real moleculeBaseCoord = m.getAtom(0).coords[0];
 
-  ergo_real machine_epsilon = std::numeric_limits<ergo_real>::epsilon();
+  ergo_real machine_epsilon = mat::getMachineEpsilon<ergo_real>();
 
   // OK, now we have a molecule. Now do a loop where we move one atom
   // by a small distance and save resulting 2-el integrals for each
@@ -120,7 +122,7 @@ int try_integral_diffs(const IntegralInfo & integralInfo, ergo_real h) {
 	  // linearly dependent on the modufied atomic coordinate.
 	  ergo_real totDiff  = integralList[noOfCases-1][i][j][k][l] - integralList[0][i][j][k][l];
 	  ergo_real halfDiff = integralList[noOfCases/2][i][j][k][l] - integralList[0][i][j][k][l];
-	  if(halfDiff != 0 && std::fabs(totDiff) > machine_epsilon) {
+	  if(halfDiff != 0 && template_blas_fabs(totDiff) > machine_epsilon) {
 	    ergo_real kvot = totDiff / halfDiff;
 	    // Now if kvot is approximately 2 we think this integral behaves linearly.
 	    if(kvot > 1.95 && kvot < 2.05) {
@@ -153,7 +155,7 @@ int main(int argc, char *argv[]) {
   IntegralInfo integralInfo(true);
   integralInfo.init();
 
-  ergo_real machine_epsilon = std::numeric_limits<ergo_real>::epsilon();
+  ergo_real machine_epsilon = mat::getMachineEpsilon<ergo_real>();
   ergo_real h = 10000*machine_epsilon;
   printf("machine_epsilon = %9.5g, using h = %9.5g\n", (double)machine_epsilon, (double)h);
 
