@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,16 +17,22 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
-/** @file gradient_test.cc Tests the gradient computation.  */
+/** @file gradient_test.cc
+
+    @brief Tests the gradient computation.
+
+    @author Elias Rudberg <em>responsible</em>
+*/
 
 #include "integrals_1el_potential.h"
 #include "integrals_1el_single.h"
@@ -65,7 +71,7 @@ static ergo_real get_nucl_energy_for_given_mol_and_dens(const IntegralInfo& inte
 							ergo_real threshold_integrals_1el,
 							mat::SizesAndBlocks const & matrix_size_block_info,
 							std::vector<int> const & permutationHML) {
-  ergo_real nuclearRepulsionEnergy = molecule.getNuclearRepulsionEnergy();
+  ergo_real nuclearRepulsionEnergy = molecule.getNuclearRepulsionEnergyQuadratic();
   ergo_real elecNuclEnergy = get_electron_nuclear_attraction_energy(integralInfo,
 								    molecule,
 								    basisInfo,
@@ -152,7 +158,7 @@ static int get_gradient_using_explicit_integrals(const IntegralInfo& integralInf
 	  }
 	}
     }
-  molecule.getNuclearRepulsionEnergyGradientContrib(resultGradient);
+  molecule.getNuclearRepulsionEnergyGradientContribQuadratic(resultGradient);
   return 0;
 }
 
@@ -289,9 +295,9 @@ test_gradient_by_explicit_comparison(const IntegralInfo & integralInfo)
   for(int i = 0; i < nAtoms; i++) 
     for(int coordIdx = 0; coordIdx < 3; coordIdx++) {
       int idx = i*3+coordIdx;
-      ergo_real absdiff_1_2 = std::fabs(gradient1[idx] - gradient2[idx]);
-      ergo_real absdiff_1_3 = std::fabs(gradient1[idx] - gradient3[idx]);
-      ergo_real absdiff_2_3 = std::fabs(gradient2[idx] - gradient3[idx]);
+      ergo_real absdiff_1_2 = template_blas_fabs(gradient1[idx] - gradient2[idx]);
+      ergo_real absdiff_1_3 = template_blas_fabs(gradient1[idx] - gradient3[idx]);
+      ergo_real absdiff_2_3 = template_blas_fabs(gradient2[idx] - gradient3[idx]);
       if(absdiff_1_2 > maxabsdiff_1_2)
 	maxabsdiff_1_2 = absdiff_1_2;
       if(absdiff_1_3 > maxabsdiff_1_3)
