@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -53,7 +54,7 @@ int main(int argc, char *argv[])
 	   <<mat::Params::getMatrixParallelLevel()<<"."<<std::endl;
 #endif
 
-  IntegralInfo biBasic(true);
+  IntegralInfo integralInfo(true);
   BasisInfoStruct bis;
 
   Molecule m;
@@ -94,7 +95,7 @@ int main(int argc, char *argv[])
 	}
 	
   if(bis.addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/6-31Gss",
-                                   0, NULL, biBasic, 0, 0, 0) != 0) {
+                                   0, NULL, integralInfo, 0, 0, 0) != 0) {
     printf("bis.addBasisfuncsForMolecule failed.\n");
     return 1;
   }
@@ -153,13 +154,13 @@ int main(int argc, char *argv[])
 
   JK::Params J_K_params;
   J_K_params. noOfThreads_K = defThreads;
-  static const ergo_real EPS = std::numeric_limits<ergo_real>::epsilon();
-  J_K_params.threshold_K = sqrt(EPS);
+  static const ergo_real EPS = mat::getMachineEpsilon<ergo_real>();
+  J_K_params.threshold_K = template_blas_sqrt(EPS);
 
   JK::ExchWeights CAM_params_not_used;
 
   J_K_params.exchange_box_size = 10;
-  if(compute_K_by_boxes_sparse(bis, biBasic, CAM_params_not_used, J_K_params, K_1, D,
+  if(compute_K_by_boxes_sparse(bis, integralInfo, CAM_params_not_used, J_K_params, K_1, D,
 			       permutationHML,
 			       inversePermutationHML) != 0)
     {
@@ -168,7 +169,7 @@ int main(int argc, char *argv[])
     }
   
   J_K_params.exchange_box_size = 4;
-  if(compute_K_by_boxes_sparse(bis, biBasic, CAM_params_not_used, J_K_params, K_2, D,
+  if(compute_K_by_boxes_sparse(bis, integralInfo, CAM_params_not_used, J_K_params, K_2, D,
 			       permutationHML,
 			       inversePermutationHML) != 0)
     {
@@ -179,7 +180,7 @@ int main(int argc, char *argv[])
   K_diff = K_1;
   K_diff += (ergo_real)(-1.0) * K_2;
 
-  ergo_real acc = sqrt(EPS);
+  ergo_real acc = template_blas_sqrt(EPS);
   ergo_real diffNorm = K_diff.eucl(acc);
   ergo_real requestedAcc;
   if(getenv("RUN_BENCHMARK")) 
