@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -80,7 +81,7 @@ static void verify_mmul_result(const std::vector<ergo_real> & A,
       ergo_real sum = 0;
       for(int k = 0; k < m; k++)
 	sum += A[i*m+k] * B[k*p+j];
-      double absdiff = fabs(C[j*n+i] - sum);
+      double absdiff = template_blas_fabs(C[j*n+i] - sum);
       if(absdiff > maxabsdiff)
 	maxabsdiff = absdiff;
       count++;
@@ -93,7 +94,7 @@ ergo_real compare_matrices(const std::vector<ergo_real> & A,
 			   int n, int m) {
   ergo_real maxabsdiff = 0;
   for(int i = 0; i < n*m; i++) {
-    ergo_real absdiff = fabs(A[i] - B[i]);
+    ergo_real absdiff = template_blas_fabs(A[i] - B[i]);
     if(absdiff > maxabsdiff)
       maxabsdiff = absdiff;
   }
