@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -38,6 +39,7 @@
 #include "realtype.h"
 #include "utilities.h"
 #include "mat_gblas.h"
+#include "config.h" // Needed to get the PRECISION_SINGLE macro
 
 static void fill_matrix_with_random_numbers(int n, std::vector<ergo_real> & A) {
   for(int i = 0; i < n; i++)
@@ -65,7 +67,7 @@ ergo_real compare_matrices(const std::vector<ergo_real> & A,
 			   int n) {
   ergo_real maxabsdiff = 0;
   for(int i = 0; i < n*n; i++) {
-    ergo_real absdiff = fabs(A[i] - B[i]);
+    ergo_real absdiff = template_blas_fabs(A[i] - B[i]);
     if(absdiff > maxabsdiff)
       maxabsdiff = absdiff;
   }
@@ -144,7 +146,10 @@ int main(int argc, char *argv[])
   }
   printf("Max abs diff (elementwise) between naive and BLAS gemm results: %6.3g\n", (double)maxdiff2);
 
-  ergo_real tol = 2e-4;
+  ergo_real tol = 1e-6;
+#ifdef PRECISION_SINGLE
+  tol = 5e-4;
+#endif
   if(maxdiff1 > tol || maxdiff2 > tol) {
     printf("Error: too large diff between naive mmul and BLAS gemm results.\n");
     return -1;
