@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,23 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file naive_response_test.cc
+
+    @brief Tests naive implementation of linear response calculation.
+
+    @author Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdio.h>
 #include <unistd.h>
 #include <memory>
@@ -74,7 +82,7 @@ static void report_timing(const Util::TimeMeter & tm, const char* s) {
 }
 
 void get_HML_J(const mat::SizesAndBlocks & sizeBlockInfo,
-	       const IntegralInfo & biBasic,
+	       const IntegralInfo & integralInfo,
 	       const BasisInfoStruct & bis,
 	       const std::vector<int> & permutationHML,
 	       const symmMatrix & D,
@@ -82,7 +90,7 @@ void get_HML_J(const mat::SizesAndBlocks & sizeBlockInfo,
   J.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo); 
   JK::Params J_K_params;
   if(compute_J_by_boxes_sparse(bis, 
-			       biBasic,
+			       integralInfo,
 			       J_K_params,
 			       J,
 			       D, // use S as "density matrix" for J matrix test
@@ -91,7 +99,7 @@ void get_HML_J(const mat::SizesAndBlocks & sizeBlockInfo,
 }
 
 void get_HML_K(const mat::SizesAndBlocks & sizeBlockInfo,
-	       const IntegralInfo & biBasic,
+	       const IntegralInfo & integralInfo,
 	       const BasisInfoStruct & bis,
 	       const std::vector<int> & permutationHML,
 	       const std::vector<int> & inversePermutationHML,
@@ -101,7 +109,7 @@ void get_HML_K(const mat::SizesAndBlocks & sizeBlockInfo,
   JK::Params J_K_params;
   JK::ExchWeights CAM_params;
   if(compute_K_by_boxes_sparse(bis,
-			       biBasic,
+			       integralInfo,
 			       CAM_params,
 			       J_K_params,
 			       K,
@@ -112,15 +120,15 @@ void get_HML_K(const mat::SizesAndBlocks & sizeBlockInfo,
 }
 
 void get_HML_G_matrix(const mat::SizesAndBlocks & sizeBlockInfo,
-		      const IntegralInfo & biBasic,
+		      const IntegralInfo & integralInfo,
 		      const BasisInfoStruct & bis,
 		      const std::vector<int> & permutationHML,
 		      const std::vector<int> & inversePermutationHML,
 		      symmMatrix & D,
 		      symmMatrix & G) {
   symmMatrix J, K;
-  get_HML_J(sizeBlockInfo, biBasic, bis, permutationHML, D, J);
-  get_HML_K(sizeBlockInfo, biBasic, bis, permutationHML, inversePermutationHML, D, K);
+  get_HML_J(sizeBlockInfo, integralInfo, bis, permutationHML, D, J);
+  get_HML_K(sizeBlockInfo, integralInfo, bis, permutationHML, inversePermutationHML, D, K);
   G = J;
   G += K;
 }
@@ -148,7 +156,6 @@ void get_HML_dens_matrix(int noOfBasisFuncs,
   DensFromFock.set_no_occupied_orbs(noOfOccupiedOrbitals);
   DensFromFock.set_use_diagonalization();
   DensFromFock.set_use_diag_on_error();
-  DensFromFock.unset_use_stochastic_orbitals();
   DensFromFock.unset_use_purification();
   DensFromFock.unset_store_all_eigenvalues_to_file();
 
@@ -182,10 +189,6 @@ void get_HML_dens_matrix(int noOfBasisFuncs,
   /**********/
 
 
-  DensFromFock.unset_do_sparsity_investigation();
-  // DensFromFock.set_sparsity_plots_resolution_m(10); 
-  DensFromFock.unset_do_comparison_to_simple_purification();
-  DensFromFock.unset_do_puri_mmul_tests();
   DensFromFock.set_purification_maxmul(88);
   DensFromFock.unset_purification_ignore_failure();
   DensFromFock.unset_purification_use_rand_perturbation_for_alleigsint();
@@ -217,7 +220,7 @@ void get_HML_dens_matrix(int noOfBasisFuncs,
 void do_HF_HML(int noOfBasisFuncs,
 	       int noOfOccupiedOrbitals,
 	       const mat::SizesAndBlocks & sizeBlockInfo,
-	       const IntegralInfo & biBasic,
+	       const IntegralInfo & integralInfo,
 	       const BasisInfoStruct & bis,
 	       const std::vector<int> & permutationHML,
 	       const std::vector<int> & inversePermutationHML,
@@ -236,7 +239,7 @@ void do_HF_HML(int noOfBasisFuncs,
   while(1) {
     // Get new Fock matrix
     symmMatrix G;
-    get_HML_G_matrix(sizeBlockInfo, biBasic, bis, permutationHML, inversePermutationHML, D, G);
+    get_HML_G_matrix(sizeBlockInfo, integralInfo, bis, permutationHML, inversePermutationHML, D, G);
     double E = symmMatrix::trace_ab(D, Hcore) + 0.5 * symmMatrix::trace_ab(D, G) + nuclearRepulsionEnergy;
     printf("Energy %3d: %22.11f  (symmMatrix::trace_ab(D, G) = %12.6f, symmMatrix::trace_ab(D, Hcore) = %12.6f)\n",
 	   count, E, (double)symmMatrix::trace_ab(D, G), (double)symmMatrix::trace_ab(D, Hcore));
@@ -354,7 +357,7 @@ int main(int argc, char *argv[])
   printf("moleculeStr = '%s'\n", moleculeStr.c_str());
   printf("===== End of parameters ======================================================\n");
 
-  IntegralInfo biBasic(true);
+  IntegralInfo integralInfo(true);
   BasisInfoStruct bis;
   Molecule m;
 
@@ -405,14 +408,14 @@ int main(int argc, char *argv[])
   if(read_basisset_file(basissetDef, basisSetName, 6, dirv, 0) != 0)
     throw std::runtime_error("Error in read_basisset_file().");
 
-  ergo_real nuclearRepulsionEnergy = m.getNuclearRepulsionEnergy();
+  ergo_real nuclearRepulsionEnergy = m.getNuclearRepulsionEnergyQuadratic();
 
   std::vector<Atom> atomList(m.getNoOfAtoms());
   for(int i = 0; i < m.getNoOfAtoms(); i++)
     atomList[i] = m.getAtom(i);
 
   if(bis.addBasisfuncsForMolecule(m, basisSetName,
-				  0, NULL, biBasic, 0, 0, 0) != 0)
+				  0, NULL, integralInfo, 0, 0, 0) != 0)
     throw std::runtime_error("bis.addBasisfuncsForMolecule failed.");
   int nBasisFuncs = bis.noOfBasisFuncs;
   printf("nBasisFuncs = %d\n", nBasisFuncs);
@@ -436,24 +439,28 @@ int main(int argc, char *argv[])
   // Get HML T matrix
   T_notrunc.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo); 
   ergo_real threshold_for_T = 1e-12;
-  if(compute_T_sparse(bis, 
-		      biBasic,
-		      threshold_for_T,
-		      T_notrunc,
-		      permutationHML) != 0)
+  ergo_real box_size_for_T = 5.8;
+  if(compute_T_sparse_linear(bis,
+			     integralInfo,
+			     threshold_for_T,
+			     box_size_for_T,
+			     T_notrunc,
+			     permutationHML) != 0)
     throw std::runtime_error("Error in compute_T_sparse.");
 
   // Get HML V matrix
   V_notrunc.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo); 
   ergo_real threshold_for_V = 1e-12;
   ergo_real boxSize_for_V = 8.8;
+  ergo_real nuclearRepulsionEnergy_dummy = 0;
   if(compute_V_sparse(bis, 
-		      biBasic,
+		      integralInfo,
 		      m,
 		      threshold_for_V,
 		      boxSize_for_V,
 		      V_notrunc,
-		      permutationHML) != 0)
+		      permutationHML,
+		      nuclearRepulsionEnergy_dummy) != 0)
     throw std::runtime_error("Error in compute_V_sparse.");
 
   symmMatrix FockMatrix;
@@ -461,7 +468,7 @@ int main(int argc, char *argv[])
   do_HF_HML(nBasisFuncs,
 	    noOfOccupiedOrbitals,
 	    sizeBlockInfo,
-	    biBasic,
+	    integralInfo,
 	    bis,
 	    permutationHML,
 	    inversePermutationHML,
@@ -491,7 +498,7 @@ int main(int argc, char *argv[])
     for(int q = 0; q < n; q++)
       for(int r = 0; r < n; r++)
 	for(int s = 0; s < n; s++)
-	  g_AO->x[p][q][r][s] = do_2e_integral(p, q, r, s, bis, biBasic);
+	  g_AO->x[p][q][r][s] = do_2e_integral(p, q, r, s, bis, integralInfo);
   printf("Two-electron integrals done.\n");
   
   // Get two-electron integrals in MO basis
@@ -557,7 +564,7 @@ int main(int argc, char *argv[])
 
   printf("Nonzero eigenvalues of M and II:\n");
   for(int i = 0; i < n2b; i++) {
-    if(fabs(eigvalList[i]) > 1e-6)
+    if(template_blas_fabs(eigvalList[i]) > 1e-6)
       printf("Eigenvalue %5d: %12.8f\n", i, (double)eigvalList[i]);
   }
 
@@ -566,7 +573,7 @@ int main(int argc, char *argv[])
     ergo_real maxabsdiff = 0;
     ergo_real refValues[] = {0.483554699, 0.556648356, 0.612541560, 0.702754741};
     for(int i = 0; i < 4; i++) {
-      ergo_real absdiff = std::fabs(eigvalList[88+i] - refValues[i]);
+      ergo_real absdiff = template_blas_fabs(eigvalList[88+i] - refValues[i]);
       if(absdiff > maxabsdiff)
 	maxabsdiff = absdiff;
     }
