@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -37,10 +38,11 @@
 
 #include "integrals_1el_kinetic.h"
 #include "memorymanag.h"
+#include "matInclude.h"
 
 int main(int argc, char *argv[])
 {
-  IntegralInfo biBasic(true);
+  IntegralInfo integralInfo(true);
   BasisInfoStruct bis;
   Molecule m;
   static const long double T[2][2] = {
@@ -53,17 +55,17 @@ int main(int argc, char *argv[])
   m.addAtom(1, 0,0,1);
 
   if(bis.addBasisfuncsForMolecule(m, ERGO_SPREFIX "/basis/STO-3G",
-                                   0, NULL, biBasic, 0, 0, 0) != 0) {
+                                   0, NULL, integralInfo, 0, 0, 0) != 0) {
     printf("bis.addBasisfuncsForMolecule failed.\n");
     return 1;
   }
   int n = bis.noOfBasisFuncs;
   ergo_real *mat= ergo_new(n*n, ergo_real);
-  ergo_real EPS = std::numeric_limits<ergo_real>::epsilon()*10;
+  ergo_real EPS = mat::getMachineEpsilon<ergo_real>()*10;
   /* Ugly fix because the reference values are only accurate ato
      double precision so we cannot compare long double to anything
      more accurate than that. */
-  ergo_real EPS_double = std::numeric_limits<double>::epsilon()*10;
+  ergo_real EPS_double = mat::getMachineEpsilon<double>()*10;
   if(EPS < EPS_double)
     EPS = EPS_double;
   if (compute_T_matrix_full(bis, EPS, mat)) {
@@ -73,7 +75,7 @@ int main(int argc, char *argv[])
   int failed = 0;
   for(int i=0; i<n; i++) {
     for(int j=0; j<n; j++) 
-      if ( std::fabs(mat[j+i*n]-T[j][i]) > EPS) {
+      if ( template_blas_fabs(mat[j+i*n]-T[j][i]) > EPS) {
         printf("(%d %d): reference: %27.24Lf got: %27.24Lf diff: %12g\n", j, i,
 	       static_cast<long double>(T[j][i]),
                static_cast<long double>(mat[j+i*n]),
