@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -113,17 +114,24 @@ test_S_V_comparison(const IntegralInfo & integralInfo)
   ergo_real boxSize = 3.3; // Use small box size to provoke errors.
   symmMatrix V;
   V.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo);
+  ergo_real nuclearRepulsionEnergy = 0;
   if(compute_V_sparse(basisInfo,
 		      integralInfo,
 		      molecule2,
 		      threshold,
 		      boxSize,
 		      V,
-		      permutationHML) != 0)
+		      permutationHML,
+		      nuclearRepulsionEnergy) != 0)
     {
       puts("error in compute_V_sparse");
       return -1;
     }
+  ergo_real nuclearRepulsionEnergyDiff = template_blas_fabs(nuclearRepulsionEnergy - molecule2.getNuclearRepulsionEnergyQuadratic());
+  if(nuclearRepulsionEnergyDiff > 1e-9) {
+    puts("Error: nuclearRepulsionEnergyDiff too large");
+    return -1;
+  }
 
   // Now V matrix should be approximately equal to -1 * (1/R) * (sum of charges) * S
   ergo_real sumOfCharges = 14.0;
@@ -187,18 +195,51 @@ test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
   ergo_real boxSize = 3.3; // Use small box size to provoke errors.
   symmMatrix V;
   V.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo);
+  ergo_real nuclearRepulsionEnergy_1 = 0;
   if(compute_V_sparse(basisInfo,
 		      integralInfo,
 		      molecule,
 		      threshold,
 		      boxSize,
 		      V,
-		      permutationHML) != 0)
+		      permutationHML,
+		      nuclearRepulsionEnergy_1) != 0)
     {
       puts("error in compute_V_sparse");
       return -1;
     }
 
+  {
+    // Test also compute_V_sparse_hierarchical
+    symmMatrix V2;
+    V2.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo);
+    ergo_real nuclearRepulsionEnergy_2 = 0;
+    if(compute_V_sparse_hierarchical(basisInfo,
+				     integralInfo,
+				     molecule,
+				     threshold,
+				     boxSize,
+				     V2,
+				     permutationHML,
+				     nuclearRepulsionEnergy_2) != 0)
+      {
+	puts("error in compute_V_sparse_hierarchical");
+	return -1;
+      }
+    ergo_real diff = symmMatrix::frob_diff(V, V2);
+    ergo_real tolerance = 2e-9;
+    printf("compute_V_sparse_hierarchical comparison gave diff = %g\n", (double)diff);
+    if(diff > tolerance) {
+      printf("Error in test_V_by_explicit_comparison, compute_V_sparse_hierarchical comparison gave diff = %g which is greater than tolerance %g\n", (double)diff, (double)tolerance);
+      return -1;
+    }
+    ergo_real nuclRepDiff = template_blas_fabs(nuclearRepulsionEnergy_2 - nuclearRepulsionEnergy_1);
+    if(nuclRepDiff > tolerance) {
+      printf("Error in test_V_by_explicit_comparison, compute_V_sparse_hierarchical comparison gave nuclRepDiff = %g which is greater than tolerance %g\n", (double)nuclRepDiff, (double)tolerance);
+      return -1;
+    }
+  }
+
   // Convert V to full matrix format
   int n = basisInfo.noOfBasisFuncs;
   std::vector<ergo_real> V_full(n*n);
@@ -221,10 +262,10 @@ test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
 	ergo_real sum = 0;
 	for(int m = 0; m < nPrims; m++)
 	  for(int k = 0; k < molecule.getNoOfAtoms(); k++)
-	    sum += -1 * do_1e_repulsion_integral_using_symb_info(&list[m], molecule.getAtom(k).charge,
+	    sum += -1 * do_1e_repulsion_integral_using_symb_info(list[m], molecule.getAtom(k).charge,
 								 molecule.getAtom(k).coords,
 								 integralInfo);
-	ergo_real absDiff = fabs(V_full[i*n+j] - sum);
+	ergo_real absDiff = template_blas_fabs(V_full[i*n+j] - sum);
 	if(absDiff > maxAbsDiff) {
 	  maxAbsDiff = absDiff;
 	  sum_for_maxAbsDiff = sum;
@@ -253,7 +294,7 @@ test_V_by_explicit_comparison(const IntegralInfo & integralInfo)
   ergo_real value_for_maxAbsDiff2 = 0;
   for(int i = 0; i < n; i++)
     for(int j = 0; j < n; j++) {
-      ergo_real absDiff = fabs(V_full[i*n+j] - V_full_other[i*n+j]);
+      ergo_real absDiff = template_blas_fabs(V_full[i*n+j] - V_full_other[i*n+j]);
       if(absDiff > maxAbsDiff2) {
 	maxAbsDiff2 = absDiff;
 	value_for_maxAbsDiff2 = V_full[i*n+j];
@@ -323,13 +364,15 @@ test_V_by_explicit_comparison_tight(const IntegralInfo & integralInfo)
   ergo_real boxSize = 3.3; // Use small box size to provoke errors.
   symmMatrix V;
   V.resetSizesAndBlocks(sizeBlockInfo, sizeBlockInfo);
+  ergo_real nuclearRepulsionEnergy_dummy = 0;
   if(compute_V_sparse(basisInfo,
 		      integralInfo,
 		      molecule,
 		      threshold,
 		      boxSize,
 		      V,
-		      permutationHML) != 0)
+		      permutationHML,
+		      nuclearRepulsionEnergy_dummy) != 0)
     {
       puts("error in compute_V_sparse");
       return -1;
@@ -354,10 +397,10 @@ test_V_by_explicit_comparison_tight(const IntegralInfo & integralInfo)
 	ergo_real sum = 0;
 	for(int m = 0; m < nPrims; m++)
 	  for(int k = 0; k < molecule.getNoOfAtoms(); k++)
-	    sum += -1 * do_1e_repulsion_integral_using_symb_info(&list[m], molecule.getAtom(k).charge,
+	    sum += -1 * do_1e_repulsion_integral_using_symb_info(list[m], molecule.getAtom(k).charge,
 								 molecule.getAtom(k).coords,
 								 integralInfo);
-	ergo_real absDiff = fabs(V_full[i*n+j] - sum);
+	ergo_real absDiff = template_blas_fabs(V_full[i*n+j] - sum);
 	if(absDiff > maxAbsDiff)
 	  maxAbsDiff = absDiff;
       }
