@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -42,6 +43,7 @@
 #include "grid_reader.h"
 #include "xc_matrix_sparse.h"
 #include "matrix_utilities.h"
+#include "config.h" // Needed to get the PRECISION_SINGLE macro
 
 static const bool PRINT_TIME = false;
 
@@ -126,10 +128,20 @@ small_calculation_core(const IntegralInfo& ii,
   /* We give some room to accumulation error. For long double calculation,
    * accumulation factor for xc matrix element equal to 200 would suffice.
    * Energy comparison need to be looser, this why we take 2100. */
-  static const ergo_real EPS_accurate = 
-      std::numeric_limits<ergo_real>::epsilon()*
-      (sizeof(ergo_real) == sizeof(ergo_long_real) ? 2100 : 100);
+  /* Since the reference values are computed using long double we
+     cannot check more accurately than that. */
+  ergo_real EPS_accurate = mat::getMachineEpsilon<ergo_real>();
+  if(EPS_accurate < mat::getMachineEpsilon<long double>())
+    EPS_accurate = mat::getMachineEpsilon<long double>();
+  ergo_real extraFactor = 100;
+  if(EPS_accurate <= mat::getMachineEpsilon<long double>())
+    extraFactor = 2100;
+  EPS_accurate *= extraFactor;
+#ifdef PRECISION_SINGLE
+  static const ergo_real EPS_sloppy = 3e-5;
+#else
   static const ergo_real EPS_sloppy = 2e-5;
+#endif
   static ergo_real EPS = EPS_accurate;
   /* Allow larger error for HiCu grid. */
   if(useHiCu)
@@ -143,12 +155,13 @@ small_calculation_core(const IntegralInfo& ii,
       rowind[0] = row;
       colind[0] = col;
       xcMat.get_values(rowind, colind, values, permutationHML, permutationHML);
-      if (std::fabs(values[0] - xcRef[row][col])>EPS) {
-        printf(" (%d,%d): ref: %28.25Lf got: %28.25Lf diff: %12g\n",
+      if (template_blas_fabs(values[0] - xcRef[row][col])>EPS) {
+        printf(" (%d,%d): ref: %28.25Lf got: %28.25Lf diff: %g eps: %g\n",
                row, col,
                static_cast<long double>(xcRef[row][col]),
                static_cast<long double>(values[0]), 
-               double(values[0] - xcRef[row][col]));
+               double(template_blas_fabs(values[0] - xcRef[row][col])),
+	       double(EPS));
         failed = true;
       }
     }
@@ -156,7 +169,7 @@ small_calculation_core(const IntegralInfo& ii,
   std::string gridStr = "std ";
   if(useHiCu)
     gridStr = "HiCu";
-  if(std::fabs(xcERef - dftEnergy) > EPS)
+  if(template_blas_fabs(xcERef - dftEnergy) > EPS)
     {
       printf("Sparse XC %s (grid %s) test failed: could not reproduce the same energy.\n"
 	     "Computed: %25.22Lf diff: %g eps: %g\n", functionalName, gridStr.c_str(),
@@ -259,7 +272,7 @@ benchmark_calculation(const IntegralInfo& ii, int sideLength)
 #if 1
   /* We give some room to accumulation error. */
   static const ergo_real EPS = 1e-5;
-  if(std::fabs(REF_XC_ENERGY - dftEnergy) > EPS*sideLength*sideLength)
+  if(template_blas_fabs(REF_XC_ENERGY - dftEnergy) > EPS*sideLength*sideLength)
     {
       printf("DFT XC test failed: could not reproduce the same energy.\n");
       printf("Computed: %25.22Lf Reference: %25.2Lf diff: %g\n",
@@ -302,27 +315,40 @@ mol_calculation(const IntegralInfo& ii, const char *fname)
 
 int main(int argc, char *argv[])
 {
-  static const int PROBLEM_SQUARE_SIDE_LENGTH = 60;
-  IntegralInfo ii(true);
-  dft_init();
-  const char *tmpdir = getenv("TMPDIR");
-  tmpdir = tmpdir ? tmpdir : "/tmp";
-  grid_set_tmpdir(tmpdir);
-  bool success;
-  if(getenv("RUN_BENCHMARK")) {
-    printf("Running an XC benchmark, tmpdir=%s.\n", tmpdir);
-    success = benchmark_calculation(ii, PROBLEM_SQUARE_SIDE_LENGTH);
-  } else {
-    if(argc>1) {
-      int side = strtol(argv[1], NULL, 10);
-      if(side)
-        success = benchmark_calculation(ii, side);
-      else
-        success = mol_calculation(ii, argv[1]);
-    } else 
-      success = small_calculation(ii);
+  bool success = false;
+  try {
+    static const int PROBLEM_SQUARE_SIDE_LENGTH = 60;
+    IntegralInfo ii(true);
+    dft_init();
+    const char *tmpdir = getenv("TMPDIR");
+    tmpdir = tmpdir ? tmpdir : "/tmp";
+    grid_set_tmpdir(tmpdir);
+    if(getenv("RUN_BENCHMARK")) {
+      printf("Running an XC benchmark, tmpdir=%s.\n", tmpdir);
+      success = benchmark_calculation(ii, PROBLEM_SQUARE_SIDE_LENGTH);
+    } else {
+      if(argc>1) {
+	int side = strtol(argv[1], NULL, 10);
+	if(side)
+	  success = benchmark_calculation(ii, side);
+	else
+	  success = mol_calculation(ii, argv[1]);
+      } else 
+	success = small_calculation(ii);
+    }
+  }
+  catch(const std::exception& e) {
+    std::cerr << "Error: exception caught, e.what() = " << e.what() << std::endl;
+    return EXIT_FAILURE;
+  } 
+  catch(char const* e) {
+    std::cerr << "Error: (char*) exception caught: " << e << std::endl;
+    return EXIT_FAILURE;
+  }
+  catch(...) {
+    printf("Error: excpetion caught.\n");
+    return EXIT_FAILURE;
   }
-
   printf("Success: %s\n", success ? "YES" : "NO");
   return  success ? 0 : 1;
 }
