@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,11 +17,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
@@ -51,18 +52,19 @@ compare_matrices(char mat_name,
                  ergo_real eps)
 {
   bool failed = false;
-
   for(int row=0; row<sz; row++) {
-    for(int col=0; col<sz; col++)
-      if (std::fabs(computed[row + col*sz]- ref[row+col*sz])>eps) {
+    for(int col=0; col<sz; col++) {
+      real currdiff = computed[row + col*sz]- ref[row+col*sz];
+      if (template_blas_fabs(currdiff)>eps) {
         printf("%c (%d,%d): ref: %28.25Lf got: %28.25Lf diff: %12g eps: %g\n",
                mat_name, row, col,
                (long double)ref[row + col*sz],
                (long double)computed[row + col*sz],
-               (double)(computed[row + col*sz]- ref[row+col*sz]),
+               (double)currdiff,
                (double)eps);
         failed = true;
       }
+    }
   }
   return failed;
 }
@@ -117,8 +119,20 @@ test_small(const IntegralInfo& ii, const char *functional,
   ergo_real dftEnergy = 0;
   dft_get_xc_mt(noOfElectrons, dmat, bis, &m, gridParams, xcmat, &dftEnergy);
   /* We give some room to accumulation error. */
-  static const ergo_real EPS = std::numeric_limits<ergo_real>::epsilon()*
-    (sizeof(ergo_real) == sizeof(ergo_long_real) ? 230 : 20);
+  /* Since the reference values are computed using long double we
+     cannot check more accurately than that. */
+  ergo_real EPS = mat::getMachineEpsilon<ergo_real>();
+  if(EPS < mat::getMachineEpsilon<long double>())
+    EPS = mat::getMachineEpsilon<long double>();
+  ergo_real extraFactor = 20;
+  if(EPS <= mat::getMachineEpsilon<long double>())
+    extraFactor = 230;
+  /* ELIAS NOTE 2016-09-06: earlier the factor used here was selected like this:
+     (sizeof(ergo_real) == sizeof(ergo_long_real) ? 230 : 20)
+     unclear why that would be needed.
+     FIXME: find out why long double does not give as high accuracy as
+     expected here. */
+  EPS *= extraFactor;
   int nrepeat = 2;
   bool failed = false;
   for(int i = 0; i < nrepeat; i++)
@@ -129,7 +143,7 @@ test_small(const IntegralInfo& ii, const char *functional,
       electronsR = dft_get_xc_mt(noOfElectrons, dmat, bis, &m, gridParams,
                                  xcmat, &dftEnergyAgain);
       failed = compare_matrices('R', xcmat, &XCRef[0][0], n, EPS);
-      if(std::fabs(dftEnergyAgain - dftEnergy) > EPS)
+      if(template_blas_fabs(dftEnergyAgain - dftEnergy) > EPS)
 	{
 	  printf("%s/%s energy repeatability test failed.\n",
 		 selected_func->is_gga() ? "GGA" : "LDA", functional);
@@ -150,7 +164,7 @@ test_small(const IntegralInfo& ii, const char *functional,
                                   xca, xcb, &dftEnergyU);
       failed = compare_matrices('A', xca, &XCRef[0][0], n, EPS)
         || compare_matrices('B', xcb, &XCRef[0][0], n, EPS);
-      if (std::fabs(electronsU - electronsR) > EPS) {
+      if (template_blas_fabs(electronsU - electronsR) > EPS) {
           printf("%s/%s Electrons restricted %28.25Lg unrestricted %28.25Lg\n",
                  selected_func->is_gga() ? "GGA" : "LDA", functional,
                  (long double)electronsR,
@@ -184,37 +198,46 @@ test_small_both()
 
   static const int sys1Z[2] = { 2, 1 };
   static const ergo_real sys1C[2][3] = { { 0, 0, 0 }, { 0, 0, 1.5 } };
+  static const int sys2Z[2] = { 2, 1 };
+  static const ergo_real sys2C[2][3] = { { 0, 0, 0 }, { 0, 0, 20.0 } };
 
-#if 0
-  /* these used to work at some point in time */
+  /* ELIAS NOTE 2016-09-06: new reference values were computed (using
+     gcc 6.1.1) because the test failed after the previous fix adding
+     the L suffix for the pi constant in pi.h which is apparently used
+     by the XC computations. So the old reference values were wrong
+     because they were computed using an inaccurate pi value. Of
+     course it's a bad idea to have this kind of test, it just
+     verifies that we get the same result as earlier, it does not
+     really check if it is correct. But now it is like this, and it is
+     still better than no test at all. */
   static const long double XCRefSys1Svwn5_GC2[2][2] = {
-    { -0.4684689023709603356666539L, -0.2774104719817796799879745L },
-    { -0.2774104719817796799879745L, -0.6305248743996174761393923L }
+    { -0.46846890237096027839L, -0.27741047198177963827L },
+    { -0.27741047198177963827L, -0.63052487439961731318L }
   };
   static const long double XCRefSys1Svwn5_Turbo[2][2] = {
-    { -0.4683770126432161890915259L, -0.2773048168955870736141454L },
-    { -0.2773048168955870736141454L, -0.6306586344434581195149588L }
+    { -0.46837701264321619278L, -0.27730481689558706597L },
+    { -0.27730481689558706597L, -0.63065863444345805105L }
   };
   static const long double XCRefSys1Svwn5_LMG[2][2] = {
-    { -0.4684708993939939889676171L, -0.2773291661077663829268383L },
-    { -0.2773291661077663829268383L, -0.6307497031343927392844396L }
+    { -0.46847089939399400585L, -0.27732916610776639027L },
+    { -0.27732916610776639027L, -0.63074970313439271847L }
   };
-#else
-  /* gcc version 4.5.1 20100924 (Red Hat 4.5.1-4) (GCC) on x86_64
-     yields different long double results! */
-  static const long double XCRefSys1Svwn5_GC2[2][2] = {
-    { -0.4684689023709603142536610L, -0.2774104719817796595236585L },
-    { -0.2774104719817796595236585L, -0.6305248743996173615934328L }
+  static const long double XCRefSys1BP86_LMG[2][2] = {
+    { -0.48456321202299733965L, -0.28477354319527885414L },
+    { -0.28477354319527885414L, -0.65856888979121373437L }
   };
-  static const long double XCRefSys1Svwn5_Turbo[2][2] = {
-    { -0.4683770126432162286649052L, -0.2773048168955870872208827L },
-    { -0.2773048168955870872208827L, -0.6306586344434580992403781L }
+  static const long double XCRefSys1BP86_TURBO[2][2] = {
+    { -0.48447235314731952526L, -0.28476089225530221456L },
+    { -0.28476089225530221456L, -0.65847904553389222668L }
   };
-  static const long double XCRefSys1Svwn5_LMG[2][2] = {
-    { -0.4684708993939940416869477L, -0.2773291661077664115226706L },
-    { -0.2773291661077664115226706L, -0.6307497031343927669315950L }
+  static const long double XCRefSys2Combine[2][2] = {
+    { -0.41581586869051081036L,  0.00000000000000000000L },
+    {  0.00000000000000000000L, -0.58371746046633447966L }
+  };
+  static const long double XCRefSys2Blyp[2][2] = {
+    { -0.43551596812630800643L,  0.00000000000000000000L },
+    {  0.00000000000000000000L, -0.61589810422545549677L }
   };
-#endif
 
   res += test_small(ii, "SVWN5", Dft::GridParams::GC2, "GC2  ",
                     sys1Z, sys1C, &XCRefSys1Svwn5_GC2[0]);
@@ -224,64 +247,13 @@ test_small_both()
 
   res += test_small(ii, "SVWN5", Dft::GridParams::LMG, "LMG  ",
                     sys1Z, sys1C, &XCRefSys1Svwn5_LMG[0]);
-#if 0
-  /* these used to work at some point in time */
-  static const long double XCRefSys1BP86_LMG[2][2] = {
-    { -0.4845632120229973237648946L, -0.2847735431952788480978751L },
-    { -0.2847735431952788480978751L, -0.6585688897912137575196313L }
-  };
 
-  static const long double XCRefSys1BP86_TURBO[2][2] = {
-    { -0.4844723531473195265333910L, -0.2847608922553022243432004L },
-    { -0.2847608922553022243432004L, -0.6584790455338923006230883L }
-  };
-#else
-  /* gcc version 4.5.1 20100924 (Red Hat 4.5.1-4) (GCC) on x86_64
-     yields different long double results! */
-  static const long double XCRefSys1BP86_LMG[2][2] = {
-    { -0.4845632120229973761047545L, -0.2847735431952788756637153L },
-    { -0.2847735431952788756637153,  -0.6585688897912137838657441L }
-  };
-
-  static const long double XCRefSys1BP86_TURBO[2][2] = {
-    { -0.4844723531473195618241717L, -0.2847608922553022361067940L },
-    { -0.2847608922553022361067940L, -0.6584790455338922763911698L }
-  };
-#endif
   res += test_small(ii, "BP86", Dft::GridParams::LMG, "LMG  ",
                     sys1Z, sys1C, &XCRefSys1BP86_LMG[0]);
+
   res += test_small(ii, "BP86", Dft::GridParams::TURBO, "Turbo",
                     sys1Z, sys1C, &XCRefSys1BP86_TURBO[0]);
 
-
-  static const int sys2Z[2] = { 2, 1 };
-  static const ergo_real sys2C[2][3] = { { 0, 0, 0 }, { 0, 0, 20.0 } };
-
-#if 0  
-  /* these used to work at some point in time */
-  static const long double XCRefSys2Combine[2][2] = {
-    { -0.4158158686905108041562736L,  0.0                          },
-    {  0.0,                          -0.5837174604663345164109328L }
-  };
-
-  static const long double XCRefSys2Blyp[2][2] = {
-    { -0.4355159681263079985444418L,  0.0                          },
-    {  0.0,                          -0.6158981042254555345592283L }
-  };
-#else
-  /* gcc version 4.5.1 20100924 (Red Hat 4.5.1-4) (GCC) on x86_64
-     yields different long double results! */
-  /* these used to work at some point in time */
-  static const long double XCRefSys2Combine[2][2] = {
-    { -0.4158158686905108421575598L,  0.0                          },
-    {  0.0,                          -0.5837174604663345243256087L }
-  };
-
-  static const long double XCRefSys2Blyp[2][2] = {
-    { -0.4355159681263080390122878L,  0.0                          },
-    {  0.0,                          -0.6158981042254555427991648L }
-  };
-#endif
   res += test_small(ii, "Combine Slater=1 PZ81=1",
                     Dft::GridParams::LMG, "LMG  ",
 		    sys2Z, sys2C, &XCRefSys2Combine[0]);
@@ -304,11 +276,11 @@ test_mol(const char *mol_fname, const char *basisSet, const char *xcFunc)
     return 1;
   }
 
-  IntegralInfo biBasic(true);
+  IntegralInfo integralInfo(true);
   BasisInfoStruct *bis = new BasisInfoStruct();
 
   if(bis->addBasisfuncsForMolecule(m, basisSet, 0, NULL,
-                                   biBasic, 0, 1, 0) != 0) {
+                                   integralInfo, 0, 1, 0) != 0) {
     printf("bis->addBasisfuncsForMolecule failed.\n");
     delete bis;
     return 1;
