@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,17 +17,22 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
-/* An interface file for writing and reading density matrices from a
-   file. 
+/** @file density_description_file.cc
+
+    @brief An interface file for writing and reading density matrices
+    to/from a file, including basis set information.
+
+    @author: Elias Rudberg <em>responsible</em>
 */
 
 #define _LARGEFILE_SOURCE 1
@@ -173,7 +178,7 @@ ddf_get_nvalues_symm_matrix(int n, const ergo_real* matrix)
   for(long i = 0; i < n; i++)
     for(long j = i; j < n; j++)
       {
-	if(std::fabs(matrix[i*n+j]) > THRESHOLD_FOR_VECTOR_STORAGE)
+	if(template_blas_fabs(matrix[i*n+j]) > THRESHOLD_FOR_VECTOR_STORAGE)
 	  nvalues++;
       }
   return nvalues;
@@ -187,18 +192,19 @@ ddf_store_matrix_as_vectors(char* p, int n, const ergo_real* matrix, size_t size
     return -1;
   int* rowind = (int*)p;
   int* colind = &rowind[nvalues];
-  ergo_real* values = (ergo_real*)(&rowind[2*nvalues]);
+  // The values follow as a sequence of ergo_real after rowind and colind, but we use a char ptr and memcpy to avoid memory alignment problems e.g. in quad precision case.
+  char* valuesPtr = (char*)(&rowind[2*nvalues]);
   long count = 0;
   for(long i = 0; i < n; i++)
     for(long j = i; j < n; j++)
       {
-	if(std::fabs(matrix[i*n+j]) > THRESHOLD_FOR_VECTOR_STORAGE)
+	if(template_blas_fabs(matrix[i*n+j]) > THRESHOLD_FOR_VECTOR_STORAGE)
 	  {
 	    if(count >= nvalues)
 	      return -1;
 	    rowind[count] = i;
 	    colind[count] = j;
-	    values[count] = matrix[i*n+j];
+	    memcpy(valuesPtr+count*sizeof(ergo_real), &matrix[i*n+j], sizeof(ergo_real)); // Use memcpy to avoid memory alignment problems e.g. in quad precision case.
 	    count++;
 	  }
       } // END FOR i j
@@ -234,13 +240,15 @@ ddf_get_matrix_from_vector_storage(const char* p, size_t sizeInBytes, int n, erg
   long nvalues = sizeInBytes / nBytesPerElement;
   int* rowind = (int*)p;
   int* colind = &rowind[nvalues];
-  ergo_real* values = (ergo_real*)(&rowind[2*nvalues]);
+  // The values follow as a sequence of ergo_real after rowind and colind, but we use a char ptr and memcpy to avoid memory alignment problems e.g. in quad precision case.
+  char* valuesPtr = (char*)(&rowind[2*nvalues]);
   memset(resultMatrix, 0, n*n*sizeof(ergo_real));
   for(long i = 0; i < nvalues; i++)
     {
       long row = rowind[i];
       long col = colind[i];
-      ergo_real value = values[i];
+      ergo_real value = 0;
+      memcpy(&value, &valuesPtr[i*sizeof(ergo_real)], sizeof(ergo_real)); // Use memcpy to avoid memory alignment problems e.g. in quad precision case.
       resultMatrix[row*n+col] = value;
       resultMatrix[col*n+row] = value;
     }
@@ -521,6 +529,7 @@ ddf_writeShellListAndDensityMatricesToFile(const BasisInfoStruct & basisInfo,
 					   const char* fileName) {
   int n = basisInfo.noOfBasisFuncs;
   int nShells = basisInfo.noOfShells;
+  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "ddf_writeShellListAndDensityMatricesToFile, n = %d, nShells = %d", n, nShells);
   if(n <= 0 || nShells <= 0) {
     do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "error in writeShellListAndDensityMatricesToFile: "
 	      "(n <= 0 || nShells <= 0)");
@@ -568,7 +577,7 @@ ddf_writeShellListAndDensityMatricesToFile(const BasisInfoStruct & basisInfo,
     }
     matrixStorageSizeList[i] = sizeInBytes;
   }  
-  
+
   size_t matrixStorageSizeTotal = 0;
   for(int i = 0; i < noOfDensityMatrices; i++)
     matrixStorageSizeTotal += matrixStorageSizeList[i];
@@ -1152,7 +1161,7 @@ ddf_load_density(const char *densityFileName,
   for(int i = 0; i < noOfDensityMatrices; i++)
     densityMatrix[i] = 
       ergo_new(noOfBasisFuncs_sg*noOfBasisFuncs_sg,ergo_real);
-  
+
   if(ddf_read_shells_and_density_matrices(*basisInfo,
                                           noOfDensityMatrices,
                                           densityMatrix,
