@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file densitymanager.cc
+
+    @brief Functionality for working with the electron density as a
+    function of space, for a given basis set and density matrix.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdlib.h>
 #include <memory.h>
 #include <math.h>
@@ -48,13 +57,13 @@ compute_1d_gaussian_integral_recursive(ergo_real a, ergo_real b, int n, ergo_rea
   ergo_real aToPowerNminus1, bToPowerNminus1;
   if(n == 0)
     {
-      sqrtalpha = std::sqrt(alpha);
-      result = std::sqrt(pi/(4*alpha)) * (template_blas_erf(sqrtalpha*b) - template_blas_erf(sqrtalpha*a));
+      sqrtalpha = template_blas_sqrt(alpha);
+      result = template_blas_sqrt(pi/(4*alpha)) * (template_blas_erf(sqrtalpha*b) - template_blas_erf(sqrtalpha*a));
       return result;
     }
   if(n == 1)
     {
-      result = -(1 / (2*alpha)) * (std::exp(-alpha*b*b) - std::exp(-alpha*a*a));
+      result = -(1 / (2*alpha)) * (template_blas_exp(-alpha*b*b) - template_blas_exp(-alpha*a*a));
       return result;
     }
   if(n < 0)
@@ -64,11 +73,11 @@ compute_1d_gaussian_integral_recursive(ergo_real a, ergo_real b, int n, ergo_rea
     }
   /* now we know that n >= 2 */
   term1 = (n - 1) * compute_1d_gaussian_integral_recursive(a, b, n-2, alpha);
-  aToPowerNminus1 = std::pow(a, n-1);
-  bToPowerNminus1 = std::pow(b, n-1);
+  aToPowerNminus1 = template_blas_pow(a, (ergo_real)n-1);
+  bToPowerNminus1 = template_blas_pow(b, (ergo_real)n-1);
   term2  = 
-    bToPowerNminus1 * std::exp(-alpha*b*b) - 
-    aToPowerNminus1 * std::exp(-alpha*a*a);
+    bToPowerNminus1 * template_blas_exp(-alpha*b*b) - 
+    aToPowerNminus1 * template_blas_exp(-alpha*a*a);
   result = (term1 - term2) / (2 * alpha);
   /*  return 0; */
   return result;
@@ -83,14 +92,15 @@ compute_integral_over_box(DistributionSpecStruct* distr,
 {
   ergo_real result, a, b, alpha;
   int i, n;
+
   result = distr->coeff;
   alpha = distr->exponent;
   for(i = 0; i < 3; i++) // run over the coordinates
     {
-      n = distr->monomialInts[i] + monomialIntsAdd[i];
+      n = distr->monomialInts[i];
       a = minVect[i] - distr->centerCoords[i];
       b = maxVect[i] - distr->centerCoords[i];
-      result *= compute_1d_gaussian_integral_recursive(a, b, n, alpha);
+      result *= compute_1d_gaussian_integral_recursive(a, b, n, alpha); 
     } /* END FOR i */
   return result;
 } /* END compute_integral_over_box */
@@ -99,6 +109,8 @@ compute_integral_over_box(DistributionSpecStruct* distr,
 
 
 
+
+
 ergo_real
 integrate_density_in_box(int nPrims,
 			 DistributionSpecStruct* rho,
@@ -184,7 +196,7 @@ get_no_of_primitives_for_density(ergo_real cutoff,
 	      DistributionSpecStruct* currDistr = &tempList[k];
 	      ergo_real Mij = dmat[i*nBasisFuncs+j];
 	      ergo_real newCoeff = currDistr->coeff * Mij * symmetryFactor;
-	      if(std::fabs(newCoeff) > cutoff)
+	      if(template_blas_fabs(newCoeff) > cutoff)
 		nn++;
 	    }
 	}
@@ -379,7 +391,7 @@ int get_density(const BasisInfoStruct & basisInfo,
 	      DistributionSpecStruct* currDistr = &tempList[k];
 	      ergo_real Mij = dmat[i*nBasisFuncs+j];
 	      ergo_real newCoeff = currDistr->coeff * Mij * symmetryFactor;
-	      if(std::fabs(newCoeff) > cutoff)
+	      if(template_blas_fabs(newCoeff) > cutoff)
 		{
 		  /* add to final list */
 		  if(nn > nNeededForRho)
@@ -453,7 +465,7 @@ int get_density(const BasisInfoStruct & basisInfo,
 	    sameYesNo = 0;
 	} /* END FOR j */
       /* check exponent */
-      absdiff = std::fabs(rho[i].exponent - rho[firstIndex].exponent);
+      absdiff = template_blas_fabs(rho[i].exponent - rho[firstIndex].exponent);
       if(absdiff > EXPONENT_DIFF_LIMIT)
 	sameYesNo = 0;
       if(sameYesNo == 0)
@@ -471,7 +483,7 @@ int get_density(const BasisInfoStruct & basisInfo,
 		      withinLimit = 1;
 		      for(kk = 0; kk < 3; kk++)
 			{
-			  absdiff = std::fabs(rho[j].centerCoords[kk] - 
+			  absdiff = template_blas_fabs(rho[j].centerCoords[kk] - 
 					 rho[k].centerCoords[kk]);
 			  if(absdiff > DISTR_CENTER_DIST_LIMIT)
 			    withinLimit = 0;
@@ -511,7 +523,7 @@ int get_density(const BasisInfoStruct & basisInfo,
 	      withinLimit = 1;
 	      for(kk = 0; kk < 3; kk++)
 		{
-		  absdiff = std::fabs(rho[j].centerCoords[kk] - 
+		  absdiff = template_blas_fabs(rho[j].centerCoords[kk] - 
 				 rho[k].centerCoords[kk]);
 		  if(absdiff > DISTR_CENTER_DIST_LIMIT)
 		    withinLimit = 0;
@@ -543,7 +555,7 @@ int get_density(const BasisInfoStruct & basisInfo,
   resultCount = 0;
   for(i = 0; i < count; i++)
     {
-      sqrtValue = std::sqrt(pi / workList[i].exponent);
+      sqrtValue = template_blas_sqrt(pi / workList[i].exponent);
       absvalue = workList[i].coeff * sqrtValue * sqrtValue * sqrtValue;
       if(absvalue < 0) absvalue *= -1;      
       if(absvalue > cutoff)
