@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integral_matrix_wrappers.cc
+
+    @brief Wrapper routines for different parts of the integral code,
+    including conversion of matrices from/to the hierarchical matrix
+    library (HML) format.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <string.h>
 
 #include "integral_matrix_wrappers.h"
@@ -36,9 +46,9 @@
 #include "operator_matrix.h"
 #include "basis_func_pair_list.h"
 #include "integrals_2el_boxed.h"
-#include "integrals_2el_coulomb.h"
-#include "integrals_2el_exchange.h"
-#include "integrals_2el_exchange_prep.h"
+#include "integrals_2el_J.h"
+#include "integrals_2el_K.h"
+#include "integrals_2el_K_prep.h"
 #include "utilities.h"
 #include "memorymanag.h"
 
@@ -54,6 +64,24 @@ static ergo_real get_max_charge(const Molecule& molecule) {
 }
 
 
+static void convert_symm_CSR_to_HML_and_destroy_CSR(csr_matrix_struct & M_CSR,
+						    symmMatrix & M_HML,
+						    std::vector<int> const & permutationHML) {
+  // Convert matrix from CSR to HML format.
+  int nvalues = ergo_CSR_get_nvalues(&M_CSR);
+  std::vector<int> rowind(nvalues);
+  std::vector<int> colind(nvalues);
+  std::vector<ergo_real> values(nvalues);
+  if(ergo_CSR_get_values(&M_CSR, rowind, colind, values, nvalues) != 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_get_values.");
+    throw std::runtime_error("Error in convert_symm_CSR_to_HML_and_destroy_CSR(): ergo_CSR_get_values() failed.");
+  }
+  // now the information is in the vectors rowind colind values, we can free memory for M_CSR.
+  ergo_CSR_destroy(&M_CSR);
+  M_HML.assign_from_sparse(rowind, colind, values, permutationHML, permutationHML);
+}
+
+
 int
 compute_V_sparse(const BasisInfoStruct& basisInfo,
 		 const IntegralInfo& integralInfo,
@@ -61,7 +89,8 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 		 ergo_real threshold,
 		 ergo_real boxSize,
 		 symmMatrix & V,
-		 std::vector<int> const & permutationHML)
+		 std::vector<int> const & permutationHML,
+		 ergo_real & result_nuclearRepulsionEnergy)
 {
   int n = basisInfo.noOfBasisFuncs;
 
@@ -71,11 +100,12 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 	    "compute_V_sparse, no of atoms = %5i, maxCharge = %6.2f", 
 	    molecule.getNoOfAtoms(), (double)maxCharge);
 
-  int noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo,
-							     threshold,
-							     maxCharge,
-							     NULL,
-							     2000000000);
+  int noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el_for_V(basisInfo,
+								   threshold,
+								   boxSize,
+								   maxCharge,
+								   NULL,
+								   2000000000);
   if(noOfBasisFuncIndexPairs <= 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
@@ -83,11 +113,12 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
     }
   std::vector<basis_func_index_pair_struct_1el> basisFuncIndexPairList(noOfBasisFuncIndexPairs);
   std::vector<ergo_real> V_list(noOfBasisFuncIndexPairs);
-  noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo,
-							 threshold,
-							 maxCharge,
-							 &basisFuncIndexPairList[0],
-							 noOfBasisFuncIndexPairs);
+  noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el_for_V(basisInfo,
+							       threshold,
+							       boxSize,
+							       maxCharge,
+							       &basisFuncIndexPairList[0],
+							       noOfBasisFuncIndexPairs);
   if(noOfBasisFuncIndexPairs <= 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
@@ -106,7 +137,8 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 				   &basisFuncIndexPairList[0],
 				   &V_list[0],
 				   noOfBasisFuncIndexPairs,
-				   false, NULL, NULL) != 0)
+				   false, NULL, NULL,
+				   result_nuclearRepulsionEnergy) != 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_V_and_gradient_linear");
       return -1;
@@ -116,7 +148,6 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
   {
     std::vector<int> rowind(noOfBasisFuncIndexPairs);
     std::vector<int> colind(noOfBasisFuncIndexPairs);
-    std::vector<ergo_real> V_list_Tmp(&V_list[0], &V_list[noOfBasisFuncIndexPairs]);
     for(int i = 0; i < noOfBasisFuncIndexPairs; i++)
       {
 	rowind[i] = basisFuncIndexPairList[i].index_1;
@@ -124,7 +155,7 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
       }
     V.assign_from_sparse(rowind,
 			 colind,
-			 V_list_Tmp,
+			 V_list,
 			 permutationHML,
 			 permutationHML);
   }
@@ -133,6 +164,75 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 }
 
 
+int
+compute_V_sparse_hierarchical(const BasisInfoStruct& basisInfo,
+			      const IntegralInfo& integralInfo,
+			      const Molecule& molecule,
+			      ergo_real threshold,
+			      ergo_real boxSize,
+			      symmMatrix & V,
+			      std::vector<int> const & permutationHML,
+			      ergo_real & result_nuclearRepulsionEnergy) {
+  //
+  // Do "get_basis_func_pair_list_1el_hierarchical" or similar
+  //
+
+  int n = basisInfo.noOfBasisFuncs;
+  ergo_real maxCharge = get_max_charge(molecule);
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "compute_V_sparse_hierarchical, no of atoms = %5i, maxCharge = %6.2f",
+	    molecule.getNoOfAtoms(), (double)maxCharge);
+  int noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el_for_V(basisInfo, threshold, boxSize, maxCharge, NULL, 2000000000);
+  if(noOfBasisFuncIndexPairs <= 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+    return -1;
+  }
+  std::vector<basis_func_index_pair_struct_1el> basisFuncIndexPairList(noOfBasisFuncIndexPairs);
+  noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el_for_V(basisInfo, threshold, boxSize, maxCharge, &basisFuncIndexPairList[0], noOfBasisFuncIndexPairs);
+  if(noOfBasisFuncIndexPairs <= 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+    return -1;
+  }
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "noOfBasisFuncIndexPairs = %i ==> storing %6.2f %% of a full matrix", 
+	    noOfBasisFuncIndexPairs, (double)100*noOfBasisFuncIndexPairs/((double)n*n));
+
+  //
+  // Set up CSR matrix for storing resulting V matrix (similar to compute_K_by_boxes_sparse)
+  //
+  csr_matrix_struct V_CSR;
+  {
+    memset(&V_CSR, 0, sizeof(csr_matrix_struct));
+    int symmetryFlagForCSR = 1;
+    std::vector<int> rowind(noOfBasisFuncIndexPairs);
+    std::vector<int> colind(noOfBasisFuncIndexPairs);
+    for(int k = 0; k < noOfBasisFuncIndexPairs; k++) {
+      rowind[k] = basisFuncIndexPairList[k].index_1;
+      colind[k] = basisFuncIndexPairList[k].index_2;
+    }
+    if(ergo_CSR_create(&V_CSR, symmetryFlagForCSR, n, noOfBasisFuncIndexPairs, rowind, colind) != 0) {
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_create for V_CSR");
+      return -1;
+    }
+  }
+
+  //
+  // Call "compute_V_hierarchical" or similar
+  //
+  if(compute_V_hierarchical(basisInfo, integralInfo, molecule, threshold, boxSize,
+			    &basisFuncIndexPairList[0], noOfBasisFuncIndexPairs, &V_CSR,
+			    result_nuclearRepulsionEnergy) != 0) {
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_V_hierarchical");
+    return -1;
+  }
+
+  //
+  // Convert result V matrix from CSR form to symmMatrix (similar to compute_K_by_boxes_sparse)
+  //
+  convert_symm_CSR_to_HML_and_destroy_CSR(V_CSR, V, permutationHML);
+  
+  return 0;
+}
+
+
 int
 compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
 				  const IntegralInfo& integralInfo,
@@ -150,14 +250,14 @@ compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
 	    "compute_gradient_of_tr_D_V, no of atoms = %5i, maxCharge = %6.2f", 
 	    molecule.getNoOfAtoms(), (double)maxCharge);
 
-  int noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo, threshold, maxCharge, NULL, 2000000000);
+  int noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el_for_V(basisInfo, threshold, boxSize, maxCharge, NULL, 2000000000);
   if(noOfBasisFuncIndexPairs <= 0) {
     do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
     return -1;
   }
   std::vector<basis_func_index_pair_struct_1el> basisFuncIndexPairList(noOfBasisFuncIndexPairs);
   std::vector<ergo_real> V_list(noOfBasisFuncIndexPairs);
-  noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo, threshold, maxCharge, &basisFuncIndexPairList[0], noOfBasisFuncIndexPairs);
+  noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el_for_V(basisInfo, threshold, boxSize, maxCharge, &basisFuncIndexPairList[0], noOfBasisFuncIndexPairs);
   if(noOfBasisFuncIndexPairs <= 0) {
     do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
     return -1;
@@ -179,6 +279,7 @@ compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
 	    "noOfBasisFuncIndexPairs = %i ==> storing %6.2f %% of a full matrix", 
 	    noOfBasisFuncIndexPairs, (double)100*noOfBasisFuncIndexPairs/((double)n*n));
 
+  ergo_real nuclRepEnergyDummy = 0;
   if(compute_V_and_gradient_linear(basisInfo,
 				   integralInfo,
 				   molecule,
@@ -188,7 +289,8 @@ compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
 				   &V_list[0],
 				   noOfBasisFuncIndexPairs,
 				   true, &D_list[0],
-				   result_gradient_list) != 0)
+				   result_gradient_list,
+				   nuclRepEnergyDummy) != 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_V_and_gradient_linear");
       return -1;
@@ -211,34 +313,37 @@ get_electron_nuclear_attraction_energy(const IntegralInfo& integralInfo,
   symmMatrix V;
   V.resetSizesAndBlocks(matrix_size_block_info, matrix_size_block_info);
   ergo_real boxSize = 10.0;
+  ergo_real nuclRepEnergyDummy = 0;
   if(compute_V_sparse(basisInfo,
 		      integralInfo, 
 		      molecule,
 		      threshold_integrals_1el,
 		      boxSize,
 		      V,
-		      permutationHML) != 0)
+		      permutationHML,
+		      nuclRepEnergyDummy) != 0)
     throw "error in compute_V_sparse";
   return symmMatrix::trace_ab(D, V);
 }
 
 
 int
-compute_T_sparse(const BasisInfoStruct& basisInfo,
-		 const IntegralInfo& integralInfo,
-		 ergo_real threshold,
-		 symmMatrix & T,
-		 std::vector<int> const & permutationHML) {
+compute_T_sparse_linear(const BasisInfoStruct& basisInfo,
+			const IntegralInfo& integralInfo,
+			ergo_real threshold,
+			ergo_real boxSize,
+			symmMatrix & T,
+			std::vector<int> const & permutationHML) {
   int n = basisInfo.noOfBasisFuncs;
   std::vector<int> nvaluesList(n);
   std::vector<int*> colindList(n);
   std::vector<ergo_real*> valuesList(n);
-  if(compute_T_matrix_sparse(basisInfo,
-			     threshold,
-			     n,
-			     &nvaluesList[0],
-			     &colindList[0],
-			     &valuesList[0]) != 0) {
+  if(compute_T_matrix_sparse_linear(basisInfo,
+				    threshold,
+				    boxSize,
+				    &nvaluesList[0],
+				    &colindList[0],
+				    &valuesList[0]) != 0) {
     do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_T_matrix_sparse");
     return -1;
   }
@@ -295,7 +400,7 @@ check_diagonal_elements_of_overlap_matrix(int n, const symmMatrix & S_symm)
   ergo_real maxabsdiff = 0;
   for(int i = 0; i < n; i++)
     {
-      ergo_real absdiff = std::fabs(values[i] - 1);
+      ergo_real absdiff = template_blas_fabs(values[i] - 1);
       if(absdiff > maxabsdiff)
 	maxabsdiff = absdiff;
     }
@@ -337,6 +442,78 @@ compute_overlap_matrix_sparse(const BasisInfoStruct& basisInfo,
 }
 
 
+int
+compute_R_matrix_sparse(const BasisInfoStruct & basisInfo_A,
+			const BasisInfoStruct & basisInfo_B,
+			normalMatrix & result_R,
+			ergo_real sparse_threshold,
+                        std::vector<int> const & matrixPermutationVec_A,
+                        std::vector<int> const & matrixPermutationVec_B)
+{
+  int n_A = basisInfo_A.noOfBasisFuncs;
+  int n_B = basisInfo_B.noOfBasisFuncs;
+
+  std::vector<int> nvaluesList(n_A);
+  std::vector< std::vector<int> > colindList(n_A);
+  std::vector< std::vector<ergo_real> > valuesList(n_A);
+  
+  if(compute_operator_matrix_sparse(basisInfo_A, 
+				    basisInfo_B,
+				    0,
+				    0,
+				    0,
+				    n_A,
+				    n_B,
+				    nvaluesList,
+				    colindList,
+				    valuesList) != 0)
+    {
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_operator_matrix_sparse");
+      return -1;
+    }
+  
+  // Now convert result to three vectors to prepare for HML storage.
+  long nvalues = 0;
+  for(int i = 0; i < n_A; i++)
+    nvalues += nvaluesList[i];
+  // allocate vectors
+  std::vector<int> rowind(nvalues);
+  std::vector<int> colind(nvalues);
+  std::vector<ergo_real> values(nvalues);
+  // populate vectors
+  long count = 0;
+  for(long i = 0; i < n_A; i++)
+    {
+      for(long j = 0; j < nvaluesList[i]; j++)
+	{
+	  rowind[count] = i;
+	  colind[count] = colindList[i][j];
+	  values[count] = valuesList[i][j];
+	  count++;
+	} // END FOR j
+    } // END FOR i
+
+  // Now the information is in rowind colind values.
+
+  /* FIXME: there is confusion about the meaning of "rows" and "columns".
+     Here it is reversed, otherwise it did not work.
+     We should use the same convention for rows and columns everywhere. */
+  result_R.assign_from_sparse(colind,
+			      rowind,
+			      values,
+			      matrixPermutationVec_B,
+                              matrixPermutationVec_A);
+  
+  result_R.eucl_thresh(sparse_threshold);
+
+  // Write to file and read again to reduce memory fragmentation.
+  result_R.writeToFile();
+  result_R.readFromFile();
+
+  return 0;
+}
+
+
 
 typedef int* int_ptr;
 typedef ergo_real* ergo_real_ptr;
@@ -524,9 +701,9 @@ get_CSR_from_symmMatrix(int n,
 		   values,
 		   inversePermutationHML,
 		   inversePermutationHML);
-  int nvalues = values.size();      
+  size_t nvalues = values.size();      
   // switch rows and columns if necessary
-  for(int i = 0; i < nvalues; i++)
+  for(size_t i = 0; i < nvalues; i++)
     {
       int row = rowind[i];
       int col = colind[i];
@@ -540,13 +717,13 @@ get_CSR_from_symmMatrix(int n,
 		     1,
 		     n,
 		     nvalues,
-		     &rowind[0],
-		     &colind[0]) != 0)
+		     rowind,
+		     colind) != 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_create.");
       return -1;
     }
-  for(int i = 0; i < nvalues; i++)
+  for(size_t i = 0; i < nvalues; i++)
     {
       ergo_CSR_add_to_element(&CSR, 
 			      rowind[i],
@@ -554,7 +731,7 @@ get_CSR_from_symmMatrix(int n,
 			      values[i]);
     }
   // check that CSR matrix is correct.
-  for(int i = 0; i < nvalues; i++)
+  for(size_t i = 0; i < nvalues; i++)
     {
       if(ergo_CSR_get_element(&CSR, rowind[i], colind[i]) != values[i])
 	{
@@ -580,25 +757,25 @@ get_CSR_from_normalMatrix(int n,
 		   values,
 		   inversePermutationHML,
 		   inversePermutationHML);
-  int nvalues = values.size();      
+  size_t nvalues = values.size();      
   if(ergo_CSR_create(&CSR, 
 		     0,
 		     n,
 		     nvalues,
-		     &rowind[0],
-		     &colind[0]) != 0)
+		     rowind,
+		     colind) != 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_create.");
       return -1;
     }
-  for(int i = 0; i < nvalues; i++) {
+  for(size_t i = 0; i < nvalues; i++) {
     ergo_CSR_add_to_element(&CSR, 
 			    rowind[i],
 			    colind[i],
 			    values[i]);
   }
   // check that CSR matrix is correct.
-  for(int i = 0; i < nvalues; i++) {
+  for(size_t i = 0; i < nvalues; i++) {
     if(ergo_CSR_get_element(&CSR, rowind[i], colind[i]) != values[i])
       {
 	do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "Error in get_CSR_from_normalMatrix, error checking CSR matrix.");
@@ -662,9 +839,7 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 			integralInfo,
 			CAM_params,
 			J_K_params,
-			NULL,
 			&K_CSR,
-			NULL,
 			&dens_CSR,
 			1) != 0)
     {
@@ -674,28 +849,7 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
   output_current_memory_usage(LOG_AREA_UNDEFINED, "After calling compute_K_by_boxes");
 
   // collect result
-  // Convert K matrix from CSR to HML format.
-  int nvalues = ergo_CSR_get_nvalues(&K_CSR);
-  std::vector<int> rowind(nvalues);
-  std::vector<int> colind(nvalues);
-  std::vector<ergo_real> values(nvalues);
-  if(ergo_CSR_get_values(&K_CSR,
-			 &rowind[0], 
-			 &colind[0], 
-			 &values[0],
-			 nvalues) != 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_get_values.");
-      return -1;
-    }
-  // now the information is in the vectors rowind colind values, we can free memory for K_CSR.
-
-  ergo_CSR_destroy(&K_CSR);
-  K.assign_from_sparse(rowind,
-		       colind,
-		       values,
-		       permutationHML,
-		       permutationHML);
+  convert_symm_CSR_to_HML_and_destroy_CSR(K_CSR, K, permutationHML);
 
   ergo_CSR_destroy(&dens_CSR);
   
@@ -722,10 +876,10 @@ get_fullmatrix_from_normalMatrix(int n,
                    values,
                    inversePermutationHML,
                    inversePermutationHML);
-  int nvalues = values.size();
-  for(int i = 0; i < n*n; i++)
+  size_t nvalues = values.size();
+  for(size_t i = 0; i < n*n; i++)
     full[i] = 0;
-  for(int i = 0; i < nvalues; i++) {
+  for(size_t i = 0; i < nvalues; i++) {
     int p = rowind[i];
     int q = colind[i];
     full[p*n+q] = values[i];
@@ -754,9 +908,6 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
   csr_matrix_struct K_CSR;
   memset(&K_CSR, 0, sizeof(csr_matrix_struct));
 
-  csr_matrix_struct K_CSR_2;
-  memset(&K_CSR, 0, sizeof(csr_matrix_struct));
-
   if(get_CSR_from_normalMatrix(n, 
 			       densityMatrix_sparse,
 			       inversePermutationHML,
@@ -778,16 +929,6 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
       do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_K_by_boxes");
       return -1;
     }
-  if(create_CSR_for_K(basisInfo,
-		      integralInfo,
-		      J_K_params,
-		      &dens_CSR,
-		      &K_CSR_2,
-		      0) != 0)
-    {
-      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_K_by_boxes");
-      return -1;
-    }
 
   densityMatrix_sparse.writeToFile();
 
@@ -796,9 +937,7 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 			integralInfo,
 			CAM_params,
 			J_K_params,
-			NULL,
 			&K_CSR,
-			NULL,
 			&dens_CSR,
 			0) != 0)
     {
@@ -809,14 +948,14 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 
   // collect result
   // Convert K matrix from CSR to HML format.
-  int nvalues = ergo_CSR_get_nvalues(&K_CSR);
+  size_t nvalues = ergo_CSR_get_nvalues(&K_CSR);
   std::vector<int> rowind(nvalues);
   std::vector<int> colind(nvalues);
   std::vector<ergo_real> values(nvalues);
   if(ergo_CSR_get_values(&K_CSR,
-			 &rowind[0], 
-			 &colind[0], 
-			 &values[0],
+			 rowind,
+			 colind,
+			 values,
 			 nvalues) != 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_get_values.");
