@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,25 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file integral_matrix_wrappers.h
+
+    @brief Wrapper routines for different parts of the integral code,
+    including conversion of matrices from/to the hierarchical matrix
+    library (HML) format.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef INTEGRAL_MATRIX_WRAPPERS_HEADER
 #define INTEGRAL_MATRIX_WRAPPERS_HEADER
 
@@ -41,7 +51,18 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 		 ergo_real threshold,
 		 ergo_real boxSize,
 		 symmMatrix & V,
-		 std::vector<int> const & permutationHML);
+		 std::vector<int> const & permutationHML,
+		 ergo_real & result_nuclearRepulsionEnergy);
+
+int
+compute_V_sparse_hierarchical(const BasisInfoStruct& basisInfo,
+			      const IntegralInfo& integralInfo,
+			      const Molecule& molecule,
+			      ergo_real threshold,
+			      ergo_real boxSize,
+			      symmMatrix & V,
+			      std::vector<int> const & permutationHML,
+			      ergo_real & result_nuclearRepulsionEnergy);
 
 
 int
@@ -66,11 +87,12 @@ get_electron_nuclear_attraction_energy(const IntegralInfo& integralInfo,
 
 
 int
-compute_T_sparse(const BasisInfoStruct& basisInfo,
-		 const IntegralInfo& integralInfo,
-		 ergo_real threshold,
-		 symmMatrix & T,
-		 std::vector<int> const & permutationHML);
+compute_T_sparse_linear(const BasisInfoStruct& basisInfo,
+			const IntegralInfo& integralInfo,
+			ergo_real threshold,
+			ergo_real boxSize,
+			symmMatrix & T,
+			std::vector<int> const & permutationHML);
 
 
 int
@@ -79,6 +101,18 @@ compute_overlap_matrix_sparse(const BasisInfoStruct& basisInfo,
 			      std::vector<int> const & permutationHML);
 
 
+/** compute_R_matrix_sparse computes the overlap matrix between two
+    different basis sets.
+*/
+int
+compute_R_matrix_sparse(const BasisInfoStruct & basisInfo_A,
+			const BasisInfoStruct & basisInfo_B,
+			normalMatrix & result_R,
+			ergo_real sparse_threshold,
+                        std::vector<int> const & matrixPermutationVec_A,
+                        std::vector<int> const & matrixPermutationVec_B);
+
+
 int
 compute_operator_matrix_sparse_symm(const BasisInfoStruct& basisInfo,
 				    int pow_x,
