@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file matrix_utilities.cc
+
+    @brief Utilities related to the hierarchical matrix library (HML),
+    including functions for setting up permutations of basis functions
+    to increase data locality in the hierarchical matrix data
+    structure.
+*/
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -73,7 +82,7 @@ void sortCoord(std::vector<Treal> const & xpos,
 	       std::vector<Treal> const & ypos, 
 	       std::vector<Treal> const & zpos,
 	       TIndexIterator first, 
-	       TIndexIterator last) throw(std::exception) {
+	       TIndexIterator last) {
   CompareClass<typename std::vector<Treal>::const_iterator> 
     compareX(xpos.begin());
   CompareClass<typename std::vector<Treal>::const_iterator> 
@@ -131,6 +140,31 @@ void permuteAndRecurse(std::vector<Treal> const & xpos,
   }
 }
 
+
+/* Added by Anastasia, neeeded for creation of
+ large artificial hamiltonian matrix. */
+template<typename Treal>
+void permuteAndRecurseStart(std::vector<Treal> const & xpos, 
+		       std::vector<Treal> const & ypos, 
+		       std::vector<Treal> const & zpos,
+		       std::vector<int> & index,
+		       int const first, 
+		       int const last,
+		       std::vector<int> const & blockSizes,
+		       int bSizeIndex) {
+
+    sortCoord(xpos, ypos, zpos, 
+	      index.begin() + first, 
+	      index.begin() + last);
+
+		int sizeBox1 = index.size()/7;
+		for(int i = 0; i < 7; ++i)
+    permuteAndRecurse(xpos, ypos, zpos, 
+		      index, first+sizeBox1*i, first + sizeBox1*(i+1),
+		      blockSizes, bSizeIndex);   
+}
+
+
 template<typename Treal>
 void getPermutation(std::vector<Treal> const & xpos, 
 		    std::vector<Treal> const & ypos, 
@@ -228,14 +262,14 @@ void getMatrixPermutationOnlyFactor2(const std::vector<ergo_real> & xcoords,
     nTmp /= currFactor;
     nLevels++;
   }
-  int bSizeVecTmp[nLevels];
+  std::vector<int> bSizeVecTmp(nLevels);
   bSizeVecTmp[nLevels-1] = 1;
   bSizeVecTmp[nLevels-2] = sparse_block_size_lowest;
   if(nLevels >= 3)
     bSizeVecTmp[nLevels-3] = sparse_block_size_lowest * first_factor;
   for(int i = nLevels-4; i >= 0; i--)
     bSizeVecTmp[i] = bSizeVecTmp[i+1] * 2;
-  std::vector<int> blockSizeVector(bSizeVecTmp, bSizeVecTmp + nLevels);
+  std::vector<int> blockSizeVector(&bSizeVecTmp[0], &bSizeVecTmp[nLevels]);
   do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "creating matrix permutation using blockSizeVector (%2d levels):", nLevels);
   for(int i = 0; i < nLevels; i++)
     do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "blockSizeVector[%i] = %12i", i, blockSizeVector[i]);
@@ -340,10 +374,10 @@ check_if_matrix_contains_strange_elements(const symmMatrix & M,
   for(int i = 0; i < nvalues; i++) {
     ergo_real x = values[i];
     bool ok1 = false;
-    if(x > -std::numeric_limits<ergo_real>::max())
+    if(x > -template_blas_get_num_limit_max<ergo_real>())
       ok1 = true;
     bool ok2 = false;
-    if(x < std::numeric_limits<ergo_real>::max())
+    if(x < template_blas_get_num_limit_max<ergo_real>())
       ok2 = true;
     if( ! (ok1 && ok2) )
       return true;
