@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,23 +17,33 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file csr_matrix.cc
+
+    @brief Compressed sparse row (CSR) matrix functionality.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <cmath>
 #include <cstdlib>
 #include <cstdio>
+#include <cassert>
 #include <stdexcept>
 
 #include "csr_matrix.h"
 #include "output.h"
 #include "memorymanag.h"
+#include "mat_gblas.h"
 
 
 typedef struct
@@ -65,9 +75,9 @@ int
 ergo_CSR_create(csr_matrix_struct* csr, 
 		int symmetryFlag,
 		int n,
-		int nnz,
-		int* rowind,
-		int* colind)
+		long nnz,
+		const std::vector<int> & rowind,
+		const std::vector<int> & colind)
 {
   csr_index_pair_struct* list;
 
@@ -84,7 +94,7 @@ ergo_CSR_create(csr_matrix_struct* csr,
   /* Create list of index pairs, and sort it. */
   list = new csr_index_pair_struct[nnz];
 
-  for(int i = 0; i < nnz; i++)
+  for(long i = 0; i < nnz; i++)
     {
       int row = rowind[i];
       int col = colind[i];
@@ -103,10 +113,9 @@ ergo_CSR_create(csr_matrix_struct* csr,
   
   /* Now the list of index pairs is sorted by rows and within each row by columns. */
   /* Create CSR structure one row at a time. */
-  int currIndex = 0;
+  long currIndex = 0;
   for(int row = 0; row < n; row++)
     {
-      int count;
       csr->rowList[row].firstElementIndex = currIndex;
       if(currIndex == nnz)
 	{
@@ -114,7 +123,7 @@ ergo_CSR_create(csr_matrix_struct* csr,
 	  csr->rowList[row].noOfElementsInRow = 0;
 	  continue;
 	}
-      count = 0;
+      int count = 0;
       while(list[currIndex].row == row)
 	{
 	  csr->columnIndexList[currIndex] = list[currIndex].col;
@@ -146,7 +155,7 @@ ergo_CSR_create(csr_matrix_struct* csr,
     {
       for(int col = 1; col < csr->rowList[row].noOfElementsInRow; col++)
 	{
-	  int ii = csr->rowList[row].firstElementIndex;
+	  long ii = csr->rowList[row].firstElementIndex;
 	  if(csr->columnIndexList[ii+col] <= csr->columnIndexList[ii+col-1])
 	    {
 	      do_output(LOG_CAT_ERROR, LOG_AREA_LOWLEVEL, "error in CSR_create: list not sorted within row.");
@@ -172,7 +181,7 @@ int ergo_CSR_destroy(csr_matrix_struct* csr)
 int ergo_CSR_copy(csr_matrix_struct* csrDest, const csr_matrix_struct* csrSource)
 {
   int n = csrSource->n;
-  int nnz = csrSource->nnz;
+  long nnz = csrSource->nnz;
 
   csrDest->n = n;
   csrDest->nnz = nnz;
@@ -193,7 +202,7 @@ int ergo_CSR_copy(csr_matrix_struct* csrDest, const csr_matrix_struct* csrSource
 int ergo_CSR_add_equal_structure(csr_matrix_struct* csrDest, const csr_matrix_struct* csrSource)
 {
   int n = csrSource->n;
-  int nnz = csrSource->nnz;
+  long nnz = csrSource->nnz;
   
   // Check that matrices have identical structure
   if(csrDest->symmetryFlag != csrSource->symmetryFlag)
@@ -204,14 +213,14 @@ int ergo_CSR_add_equal_structure(csr_matrix_struct* csrDest, const csr_matrix_st
     return -1;
 
   // OK, add elements of source to elements of dest, store in dest.
-  for(int i = 0; i < nnz; i++)
+  for(long i = 0; i < nnz; i++)
     csrDest->elementList[i] += csrSource->elementList[i];
 
   return 0;
 }
 
 
-static int
+static long
 ergo_csr_find_index(const csr_matrix_struct* csr, int row, int col)
 {
   if(row < 0 || row >= csr->n)
@@ -219,7 +228,7 @@ ergo_csr_find_index(const csr_matrix_struct* csr, int row, int col)
   int n = csr->rowList[row].noOfElementsInRow;
   if(n <= 0) /* If row is empty we can return already here. */
     return -1;
-  int baseIndex = csr->rowList[row].firstElementIndex;
+  long baseIndex = csr->rowList[row].firstElementIndex;
   int* colList = &csr->columnIndexList[baseIndex];
   int lo = 0;
   int hi = n-1;
@@ -257,7 +266,7 @@ ergo_CSR_add_to_element(csr_matrix_struct* csr,
 	  col2 = row;
 	}
     }
-  int i = ergo_csr_find_index(csr, row2, col2);
+  long i = ergo_csr_find_index(csr, row2, col2);
   if(i < 0)
     return 0;
   csr->elementList[i] += value;
@@ -280,7 +289,7 @@ ergo_CSR_get_element(const csr_matrix_struct* csr,
 	  col2 = row;
 	}
     }
-  int i = ergo_csr_find_index(csr, row2, col2);
+  long i = ergo_csr_find_index(csr, row2, col2);
   if(i < 0)
     return 0;
   return csr->elementList[i];
@@ -291,9 +300,9 @@ ergo_real
 ergo_CSR_get_max_abs_element(const csr_matrix_struct* csr)
 {
   ergo_real maxabs = 0;
-  for(int i = 0; i < csr->nnz; i++)
+  for(long i = 0; i < csr->nnz; i++)
     {
-      ergo_real absval = std::fabs(csr->elementList[i]);
+      ergo_real absval = template_blas_fabs(csr->elementList[i]);
       if(absval > maxabs)
 	maxabs = absval;
     }
@@ -301,7 +310,7 @@ ergo_CSR_get_max_abs_element(const csr_matrix_struct* csr)
 }
 
 
-int 
+long
 ergo_CSR_get_nvalues(const csr_matrix_struct* csr)
 {
   return csr->nnz;
@@ -310,10 +319,10 @@ ergo_CSR_get_nvalues(const csr_matrix_struct* csr)
 
 int 
 ergo_CSR_get_values(const csr_matrix_struct* csr,
-		    int* rowind,
-		    int* colind,
-		    ergo_real* values,
-		    int nvalues)
+		    std::vector<int> & rowind,
+		    std::vector<int> & colind,
+		    std::vector<ergo_real> & values,
+		    long nvalues)
 {
   int count = 0;
   
@@ -321,7 +330,7 @@ ergo_CSR_get_values(const csr_matrix_struct* csr,
     return -1;
   for(int i = 0; i < csr->n; i++)
     {
-      int baseIndex = csr->rowList[i].firstElementIndex;
+      long baseIndex = csr->rowList[i].firstElementIndex;
       int* colList = &csr->columnIndexList[baseIndex];
       ergo_real* valueList = &csr->elementList[baseIndex];
       for(int j = 0; j < csr->rowList[i].noOfElementsInRow; j++)
@@ -348,13 +357,15 @@ ergo_CSR_get_nvalues_singlerow(const csr_matrix_struct* csr,
 int 
 ergo_CSR_get_values_singlerow(const csr_matrix_struct* csr,
 			      int row,
-			      int* colind,
-			      ergo_real* values,
+			      std::vector<int> & colind,
+			      std::vector<ergo_real> & values,
 			      int nvalues)
 {
   if(nvalues != csr->rowList[row].noOfElementsInRow)
     return -1;
-  int baseIndex = csr->rowList[row].firstElementIndex;
+  assert(colind.size() >= (size_t)nvalues);
+  assert(values.size() >= (size_t)nvalues);
+  long baseIndex = csr->rowList[row].firstElementIndex;
   int* colList = &csr->columnIndexList[baseIndex];
   ergo_real* valueList = &csr->elementList[baseIndex];
   for(int i = 0; i < nvalues; i++) {
