@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,31 +17,40 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file csr_matrix.h
+
+    @brief Compressed sparse row (CSR) matrix functionality.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef CSR_MATRIX_HEADER
 #define CSR_MATRIX_HEADER
 
 #include "realtype.h"
+#include <vector>
 
 
 typedef struct
 {
-  int noOfElementsInRow;
-  int firstElementIndex;
+  long noOfElementsInRow;
+  long firstElementIndex;
 } csr_matrix_row_struct;
 
 struct csr_matrix_struct
 {
   int n;
-  int nnz;
+  long nnz;
   int symmetryFlag;
   csr_matrix_row_struct* rowList;
   ergo_real* elementList;
@@ -50,12 +59,12 @@ struct csr_matrix_struct
 
 
 
-int ergo_CSR_create(csr_matrix_struct* csr, 
+int ergo_CSR_create(csr_matrix_struct* csr,
 		    int symmetryFlag,
 		    int n,
-		    int nnz,
-		    int* rowind,
-		    int* colind);
+		    long nnz,
+		    const std::vector<int> & rowind,
+		    const std::vector<int> & colind);
 
 int ergo_CSR_destroy(csr_matrix_struct* csr);
 
@@ -74,21 +83,21 @@ ergo_real ergo_CSR_get_element(const csr_matrix_struct* csr,
 
 ergo_real ergo_CSR_get_max_abs_element(const csr_matrix_struct* csr);
 
-int ergo_CSR_get_nvalues(const csr_matrix_struct* csr);
+long ergo_CSR_get_nvalues(const csr_matrix_struct* csr);
 
 int ergo_CSR_get_values(const csr_matrix_struct* csr,
-			int* rowind,
-			int* colind,
-			ergo_real* values,
-			int nvalues);
+			std::vector<int> & rowind,
+			std::vector<int> & colind,
+			std::vector<ergo_real> & values,
+			long nvalues);
 
 int ergo_CSR_get_nvalues_singlerow(const csr_matrix_struct* csr,
 				   int row);
 
 int ergo_CSR_get_values_singlerow(const csr_matrix_struct* csr,
 				  int row,
-				  int* colind,
-				  ergo_real* values,
+				  std::vector<int> & colind,
+				  std::vector<ergo_real> & values,
 				  int nvalues);
 
 
