@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file molecule.cc
+
+    @brief Class representing a molecule as a set of atoms with
+    assiciated coordinates and charges of the atomic nuclei.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <memory.h>
@@ -50,11 +59,11 @@ get_distance_between_atoms(const Atom& atomA, const Atom& atomB)
       ergo_real dx = atomB.coords[i] - atomA.coords[i];
       sum += dx*dx;
     }
-  return std::sqrt(sum);
+  return template_blas_sqrt(sum);
 }
 
 void
-Molecule::getExtremeInternuclearDistances(ergo_real & minDist, ergo_real & maxDist) const
+Molecule::getExtremeInternuclearDistancesQuadratic(ergo_real & minDist, ergo_real & maxDist) const
 {
   minDist = maxDist = 0; // This matters if there is only one atom.
   bool firstTime = true;
@@ -75,7 +84,7 @@ Molecule::getExtremeInternuclearDistances(ergo_real & minDist, ergo_real & maxDi
 }
 
 ergo_real
-Molecule::getNuclearRepulsionEnergy() const
+Molecule::getNuclearRepulsionEnergyQuadratic() const
 {
   int A, B;
   ergo_real sum;
@@ -99,7 +108,7 @@ Molecule::getNuclearRepulsionEnergy() const
 }
 
 void
-Molecule::getNuclearRepulsionEnergyGradientContrib(ergo_real* resultGradient) const 
+Molecule::getNuclearRepulsionEnergyGradientContribQuadratic(ergo_real* resultGradient) const 
 {
   int N = noOfAtoms;
   int A, B;
@@ -112,7 +121,7 @@ Molecule::getNuclearRepulsionEnergyGradientContrib(ergo_real* resultGradient) co
       ergo_real dx = atomB.coords[0] - atomA.coords[0];
       ergo_real dy = atomB.coords[1] - atomA.coords[1];
       ergo_real dz = atomB.coords[2] - atomA.coords[2];
-      ergo_real r = std::sqrt(dx*dx + dy*dy + dz*dz);
+      ergo_real r = template_blas_sqrt(dx*dx + dy*dy + dz*dz);
       ergo_real derivative_x = ZA * ZB * (-0.5) * 2.0 * dx / (r*r*r);
       ergo_real derivative_y = ZA * ZB * (-0.5) * 2.0 * dy / (r*r*r);
       ergo_real derivative_z = ZA * ZB * (-0.5) * 2.0 * dz / (r*r*r);
