@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file molecule.h
+
+    @brief Class representing a molecule as a set of atoms with
+    assiciated coordinates and charges of the atomic nuclei.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef MOLECULE_HEADER
 #define MOLECULE_HEADER
 
@@ -34,6 +43,7 @@
 #include <cassert>
 
 #include "realtype.h"
+#include "mat_gblas.h"
 
 /** Simple atom representation by its charge and cartesian coordinates.
  *
@@ -65,9 +75,9 @@ struct Vector3D {
   }
   /** compute distance between two points. */
   ergo_real dist(const Vector3D& b) const 
-  { return std::sqrt(dist2(b.v)); }
+  { return template_blas_sqrt(dist2(b.v)); }
   ergo_real dist(const ergo_real b[]) const 
-  { return std::sqrt(dist2(b)); }
+  { return template_blas_sqrt(dist2(b)); }
 };
 
 /**
@@ -105,16 +115,16 @@ class Molecule {
   ergo_real getNetCharge() const { return netCharge; }
 
   /** Compute smallest and largest internuclear distances. */
-  void getExtremeInternuclearDistances(ergo_real & minDist, ergo_real & maxDist) const;
+  void getExtremeInternuclearDistancesQuadratic(ergo_real & minDist, ergo_real & maxDist) const;
   /** Compute nuclear repulsion energy. */
-  ergo_real getNuclearRepulsionEnergy() const;
+  ergo_real getNuclearRepulsionEnergyQuadratic() const;
   /** Compute nuclear energy in given electric field. */
   ergo_real getNuclearElectricFieldEnergy(const Vector3D& electricField) const;
   /** Compute total number of electrons. The result is sum of atomic
       charges plus netCharge. */
   int getNumberOfElectrons() const;
   /** Compute gradient of nuclear repulsion energy w.r.t. changes in nuclear coordinates. Result is added to resultGradient vector.  */
-  void getNuclearRepulsionEnergyGradientContrib(ergo_real* resultGradient) const;
+  void getNuclearRepulsionEnergyGradientContribQuadratic(ergo_real* resultGradient) const;
 
   /** Loads molecule from a given file name, assuming given net
       charge. basissetFile will be set if the file contains basis set
