@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,15 +17,24 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file realtype.h
+
+    @brief Definition of the main floating-point datatype used; the
+    ergo_real type.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #ifndef REALTYPEHEADER
 #define REALTYPEHEADER
 
@@ -49,6 +58,12 @@ typedef long double ergo_long_real;
 #define REALTYPE_DEFINED_OK
 #endif
 
+#ifdef PRECISION_QUAD_FLT128
+typedef __float128 ergo_real;
+typedef __float128 ergo_long_real;
+#define REALTYPE_DEFINED_OK
+#endif
+
 /* if precision not specified, use double as default */
 #ifndef REALTYPE_DEFINED_OK
 typedef double ergo_real;
