@@ -1,6 +1,6 @@
-/* Ergo, version 3.5, a program for linear scaling electronic structure
+/* Ergo, version 3.8, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * Copyright (C) 2019 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
  * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
@@ -17,21 +17,28 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * Primary academic reference:
- * Kohn−Sham Density Functional Theory Electronic Structure Calculations 
- * with Linearly Scaling Computational Time and Memory Usage,
- * Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek,
- * J. Chem. Theory Comput. 7, 340 (2011),
- * <http://dx.doi.org/10.1021/ct100611z>
+ * Ergo: An open-source program for linear-scaling electronic structure
+ * calculations,
+ * Elias Rudberg, Emanuel H. Rubensson, Pawel Salek, and Anastasia
+ * Kruchinina,
+ * SoftwareX 7, 107 (2018),
+ * <http://dx.doi.org/10.1016/j.softx.2018.03.005>
  * 
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file solve_lin_eq_syst.cc
+
+    @brief Functionality for solving linear equation systems.
+
+    @author: Elias Rudberg <em>responsible</em>
+*/
+
 #include <memory.h>
 #include "solve_lin_eq_syst.h"
 #include "mat_gblas.h"
 #include "output.h"
 
-
 int 
 solve_linear_equation_system(int n, 
 			     const ergo_real* matrix, 
@@ -59,58 +66,3 @@ solve_linear_equation_system(int n,
 
   return 0;
 }
-
-
-
-
-
-#if 0
-
-int 
-main()
-{
-  printf("hello\n");
-
-  integer n = 3;
-
-  ergo_real A[n*n];
-
-  A[0] = 1;
-  A[1] = 2;
-  A[2] = 0;
-
-  A[3] = 0;
-  A[4] = 3;
-  A[5] = 3;
-
-  A[6] = 1;
-  A[7] = 1;
-  A[8] = 0;
-
-  ergo_real RHS[n];
-  RHS[0] = 1;
-  RHS[1] = 0;
-  RHS[2] = 1;
-
-  integer IPIV[n];
-  integer NRHS = 1;
-
-  integer info = -1;
-
-  dgesv_(&n, &NRHS, A, &n, IPIV, RHS, &n, &info);
-  if(info != 0)
-    {
-      printf("ERROR!\n");
-      return -1;
-    }
-  printf("OK\n");
-
-  printf("solution:\n");
-  int i;
-  for(i = 0; i < n; i++)
-    printf("%22.11f\n", (double)RHS[i]);
-  
-  return 0;
-}
-
-#endif
