@@ -16,6 +16,7 @@ fi
 
 if test `"$top_builddir"/source/ergo -e precision` = single; then
     echo SKIPPED
+    cd .. ; rm -r $currtestdirname
     exit 0
 fi
 
@@ -27,7 +28,7 @@ echo
 
 echo Testing h2o_1000 HF/6-31G**
 rm -f ergoscf.out
-../source/ergo -m ../mol/h2o_1000.xyz <<EOINPUT > /dev/null
+"$top_builddir"/source/ergo -m "$top_srcdir"/mol/h2o_1000.xyz <<EOINPUT > /dev/null
 enable_memory_usage_output = 1
 basis = "6-31Gss"
 use_simple_starting_guess=1
@@ -40,7 +41,7 @@ if
 check_final_energy -76022.6431000 1e-4 ; 
 then
 echo OK
-grep CONVERGED ergoscf.out
+grep "RESC CONVERGED" ergoscf.out
 mv ergoscf.out ergoscf.out.h2o_1000
 else
 echo ERROR
